@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: asmlibrarian.ml,v 1.12 2000/04/21 08:10:26 weis Exp $ *)
+(* $Id: asmlibrarian.ml,v 1.13 2002/04/04 09:00:16 garrigue Exp $ *)
 
 (* Build libraries of .cmx files *)
 
@@ -46,6 +46,10 @@ let create_archive file_list lib_name =
     output_string outchan cmxa_magic_number;
     let (objfile_list, descr_list) =
       List.split (List.map read_info file_list) in
+    List.iter2
+      (fun file_name (unit, crc) ->
+        Asmlink.check_consistency file_name unit crc)
+      file_list descr_list;
     let infos =
       { lib_units = descr_list;
         lib_ccobjs = !Clflags.ccobjs;
