@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: reload.ml,v 1.4 1999/11/17 18:56:43 xleroy Exp $ *)
+(* $Id: reload.ml,v 1.5 2002/11/22 15:09:18 xleroy Exp $ *)
 
 open Cmm
 open Arch
@@ -39,15 +39,26 @@ method makereg r =
 
 method reload_operation op arg res =
   match op with
-    Iintop(Iadd|Isub|Imul|Iand|Ior|Ixor|Icomp _|Icheckbound) ->
+    Iintop(Iadd|Isub|Iand|Ior|Ixor|Icomp _|Icheckbound) ->
       (* One of the two arguments can reside in the stack *)
       if stackp arg.(0) && stackp arg.(1)
       then ([|arg.(0); self#makereg arg.(1)|], res)
       else (arg, res)
+  | Iintop(Imul) ->
+      (* First argument (and destination) must be in register,
+         second arg can reside in stack *)
+      if stackp arg.(0)
+      then let r = self#makereg arg.(0) in ([|r; arg.(1)|], [|r|])
+      else (arg, res)
   | Iintop_imm(Iadd, _) when arg.(0).loc <> res.(0).loc ->
       (* This add will be turned into a lea; args and results must be
          in registers *)
       super#reload_operation op arg res
+  | Iintop_imm(Imul, _) ->
+      (* First argument and destination must be in register *)
+      if stackp arg.(0)
+      then let r = self#makereg arg.(0) in ([|r|], [|r|])
+      else (arg, res)
   | Iintop(Ilsl|Ilsr|Iasr) | Iintop_imm(_, _) | Ifloatofint | Iintoffloat |
     Ispecific(Ipush) ->
       (* The argument(s) can be either in register or on stack *)
