@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: linearize.ml,v 1.23 2001/02/05 08:49:10 xleroy Exp $ *)
+(* $Id: linearize.ml,v 1.24 2002/01/09 19:40:48 xleroy Exp $ *)
 
 (* Transformation of Mach code into a list of pseudo-instructions. *)
 
@@ -25,7 +25,7 @@ let new_label() = incr label_counter; !label_counter
 
 type instruction =
   { mutable desc: instruction_desc;
-    next: instruction;
+    mutable next: instruction;
     arg: Reg.t array;
     res: Reg.t array;
     live: Reg.Set.t }
