@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: proc.ml,v 1.8 2002/07/22 16:37:54 doligez Exp $ *)
+(* $Id: proc.ml,v 1.12 2004/06/19 17:39:35 xleroy Exp $ *)
 
 (* Description of the Power PC *)
 
@@ -128,11 +128,8 @@ let calling_conventions
           ofs := !ofs + size_float
         end
   done;
-  let final_ofs = if toc && !ofs > 0 then !ofs + 24 else !ofs in
-  (loc, Misc.align final_ofs 8)
-  (* Keep stack 8-aligned. 
-     Under PowerOpen, keep a free 24 byte linkage area at the bottom
-     if we need to stack-allocate some arguments. *)
+  (loc, Misc.align !ofs 16)
+  (* Keep stack 16-aligned. *)
 
 let incoming ofs = Incoming ofs
 let outgoing ofs = Outgoing ofs
@@ -185,12 +182,12 @@ let poweropen_external_conventions first_int last_int
         end;
         int := !int + 2
   done;
-  (loc, Misc.align !ofs 8) (* Keep stack 8-aligned *)
+  (loc, Misc.align !ofs 16) (* Keep stack 16-aligned *)
 
 let loc_external_arguments =
   match Config.system with
-    "aix" | "rhapsody" -> poweropen_external_conventions 0 7 100 112
-  | "elf" -> calling_conventions 0 7 100 107 outgoing 8
+  | "rhapsody" -> poweropen_external_conventions 0 7 100 112
+  | "elf" | "bsd" -> calling_conventions 0 7 100 107 outgoing 8
   | _ -> assert false
 
 let extcall_use_push = false
@@ -239,12 +236,9 @@ let assemble_file infile outfile =
   let infile = Filename.quote infile
   and outfile = Filename.quote outfile in
   match Config.system with
-    "aix" ->
-      let proc = if powerpc then "ppc" else "pwr" in
-      Ccomp.command ("as -u -m " ^ proc ^ " -o " ^ outfile ^ " " ^ infile)
   | "elf" ->
       Ccomp.command ("as -u -m ppc -o " ^ outfile ^ " " ^ infile)
-  | "rhapsody" ->
+  | "rhapsody" | "bsd" ->
       Ccomp.command ("as -o " ^ outfile ^ " " ^ infile)
   | _ -> assert false
 
