@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: schedgen.ml,v 1.9 1999/11/17 18:56:36 xleroy Exp $ *)
+(* $Id: schedgen.ml,v 1.10 2000/12/28 13:02:54 weis Exp $ *)
 
 (* Instruction scheduling *)
 
@@ -87,7 +87,7 @@ let rec longest_path critical_outputs node =
       [] ->
         node.length <-
           if is_critical critical_outputs node.instr.res
-          or node.instr.desc = Lreloadretaddr (* alway critical *)
+          || node.instr.desc = Lreloadretaddr (* alway critical *)
           then node.delay
           else 0
     | sons ->
