@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: bytelink.ml,v 1.79 2002/07/02 16:13:11 weis Exp $ *)
+(* $Id: bytelink.ml,v 1.83.4.1 2004/07/02 09:10:50 xleroy Exp $ *)
 
 (* Link a set of .cmo files and produce a bytecode executable. *)
 
@@ -158,29 +158,21 @@ let scan_file obj_name tolink =
 
 (* Consistency check between interfaces *)
 
-let crc_interfaces =
-  (Hashtbl.create 17 : (string, string * Digest.t) Hashtbl.t)
+let crc_interfaces = Consistbl.create ()
 
 let check_consistency file_name cu =
-  List.iter
-    (fun (name, crc) ->
-      if name = cu.cu_name then begin
-        Hashtbl.add crc_interfaces name (file_name, crc)
-      end else begin
-        try
-          let (auth_name, auth_crc) = Hashtbl.find crc_interfaces name in
-          if crc <> auth_crc then
-            raise(Error(Inconsistent_import(name, file_name, auth_name)))
-        with Not_found ->
-          (* Can only happen for unit for which only a .cmi file was used,
-             but no .cmo is provided *)
-          Hashtbl.add crc_interfaces name (file_name, crc)
-      end)
-    cu.cu_imports
+  try
+    List.iter
+      (fun (name, crc) ->
+        if name = cu.cu_name
+        then Consistbl.set crc_interfaces name crc file_name
+        else Consistbl.check crc_interfaces name crc file_name)
+      cu.cu_imports
+  with Consistbl.Inconsistency(name, user, auth) ->
+    raise(Error(Inconsistent_import(name, user, auth)))
 
 let extract_crc_interfaces () =
-  Hashtbl.fold (fun name (file_name, crc) accu -> (name, crc) :: accu)
-               crc_interfaces []
+  Consistbl.extract crc_interfaces
 
 (* Record compilation events *)
 
@@ -272,13 +264,10 @@ let make_absolute file =
 (* Create a bytecode executable file *)
 
 let link_bytecode tolink exec_name standalone =
-  if Sys.os_type = "MacOS" then begin
-    (* Create it as a text file for bytecode scripts *)
-    let c = open_out_gen [Open_wronly; Open_creat] 0o777 exec_name in
-    close_out c
-  end;
-  let outchan = open_out_gen [Open_wronly; Open_trunc; Open_creat; Open_binary]
-                             0o777 exec_name in
+  Misc.remove_file exec_name; (* avoid permission problems, cf PR#1911 *)
+  let outchan =
+    open_out_gen [Open_wronly; Open_trunc; Open_creat; Open_binary]
+                 0o777 exec_name in
   try
     if standalone then begin
       (* Copy the header *)
@@ -301,7 +290,7 @@ let link_bytecode tolink exec_name standalone =
     (* The bytecode *)
     let start_code = pos_out outchan in
     Symtable.init();
-    Hashtbl.clear crc_interfaces;
+    Consistbl.clear crc_interfaces;
     let sharedobjs = List.map Dll.extract_dll_name !Clflags.dllibs in
     if standalone then begin
       (* Initialize the DLL machinery *)
@@ -336,6 +325,9 @@ let link_bytecode tolink exec_name standalone =
     (* The map of global identifiers *)
     Symtable.output_global_map outchan;
     Bytesections.record outchan "SYMB";
+    (* CRCs for modules *)
+    output_value outchan (extract_crc_interfaces());
+    Bytesections.record outchan "CRCS";
     (* Debug info *)
     if !Clflags.debug then begin
       output_debug_info outchan;
@@ -389,9 +381,16 @@ let link_bytecode_as_c tolink outfile =
   let outchan = open_out outfile in
   try
     (* The bytecode *)
+    output_string outchan "#include <caml/mlvalues.h>\n";
+    output_string outchan "\
+CAMLextern void caml_startup_code(
+           code_t code, asize_t code_size,
+           char *data, asize_t data_size,
+           char *section_table, asize_t section_table_size,
+           char **argv);\n";
     output_string outchan "static int caml_code[] = {\n";
     Symtable.init();
-    Hashtbl.clear crc_interfaces;
+    Consistbl.clear crc_interfaces;
     let output_fun = output_code_string outchan
     and currpos_fun () = 0 in
     List.iter (link_file output_fun currpos_fun) tolink;
@@ -401,15 +400,26 @@ let link_bytecode_as_c tolink outfile =
     output_string outchan "static char caml_data[] = {\n";
     output_data_string outchan
       (Marshal.to_string (Symtable.initial_global_table()) []);
-    Printf.fprintf outchan "\n};\n\n";
+    output_string outchan "\n};\n\n";
+    (* The sections *)
+    let sections =
+      [ "SYMB", Symtable.data_global_map();
+        "PRIM", Obj.repr(Symtable.data_primitive_names());
+        "CRCS", Obj.repr(extract_crc_interfaces()) ] in
+    output_string outchan "static char caml_sections[] = {\n";
+    output_data_string outchan
+      (Marshal.to_string sections []);
+    output_string outchan "\n};\n\n";
     (* The table of primitives *)
     Symtable.output_primitive_table outchan;
     (* The entry point *)
     output_string outchan "\n
-void caml_startup(argv)
-        char ** argv;
+void caml_startup(char ** argv)
 {
-  caml_startup_code(caml_code, sizeof(caml_code), caml_data, argv);
+  caml_startup_code(caml_code, sizeof(caml_code),
+                    caml_data, sizeof(caml_data),
+                    caml_sections, sizeof(caml_sections),
+                    argv);
 }\n";
     close_out outchan
   with x ->
@@ -460,35 +470,6 @@ let build_custom_runtime prim_name exec_name =
       remove_file
         (Filename.chop_suffix (Filename.basename prim_name) ".c" ^ ".obj");
       retcode
-  | "mrc" ->
-      let cppc = "mrc"
-      and libsppc = "\"{sharedlibraries}MathLib\" \
-                     \"{ppclibraries}PPCCRuntime.o\" \
-                     \"{ppclibraries}PPCToolLibs.o\" \
-                     \"{sharedlibraries}StdCLib\" \
-                     \"{ppclibraries}StdCRuntime.o\" \
-                     \"{sharedlibraries}InterfaceLib\""
-      and linkppc = "ppclink -d"
-      and objsppc = extract ".x" (List.rev !Clflags.ccobjs)
-      and q_prim_name = Filename.quote prim_name
-      and q_exec_name = Filename.quote exec_name
-      in
-      Ccomp.run_command (Printf.sprintf "%s %s %s %s -o %s.x"
-        cppc
-        (Clflags.std_include_flag "-i ")
-        (String.concat " " (List.rev_map Filename.quote !Clflags.ccopts))
-        q_prim_name
-        q_prim_name);
-      Ccomp.run_command ("delete -i " ^ q_exec_name);
-      Ccomp.command (Printf.sprintf
-        "%s -t MPST -c 'MPS ' -o %s %s.x %s %s %s"
-        linkppc
-        q_exec_name
-        q_prim_name
-        (String.concat " " (List.map Filename.quote objsppc))
-        (Filename.quote
-            (Filename.concat Config.standard_library "libcamlrun.x"))
-        libsppc)
   | _ -> assert false
 
 let append_bytecode_and_cleanup bytecode_name exec_name prim_name =
