@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: instruct.mli,v 1.18 2001/11/26 12:47:01 xleroy Exp $ *)
+(* $Id: instruct.mli,v 1.20 2004/05/26 11:10:50 garrigue Exp $ *)
 
 (* The type of the instructions of the abstract machine *)
 
@@ -39,7 +39,7 @@ type compilation_env =
 type debug_event =
   { mutable ev_pos: int;                (* Position in bytecode *)
     ev_module: string;                  (* Name of defining module *)
-    ev_char: int;                       (* Location in source file *)
+    ev_char: Lexing.position;           (* Position in source file *)
     ev_kind: debug_event_kind;          (* Before/after event *)
     ev_info: debug_event_info;          (* Extra information *)
     ev_typenv: Env.summary;             (* Typing environment *)
@@ -116,6 +116,8 @@ type instruction =
   | Kisint
   | Kisout
   | Kgetmethod
+  | Kgetpubmet of int
+  | Kgetdynmet
   | Kevent of debug_event
   | Kstop
 
