@@ -10,16 +10,19 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: meta.mli,v 1.9 2001/08/28 14:47:07 xleroy Exp $ *)
+(* $Id: meta.mli,v 1.13 2004/04/16 13:46:27 starynke Exp $ *)
 
 (* To control the runtime system and bytecode interpreter *)
 
-external global_data : unit -> Obj.t array = "get_global_data"
-external realloc_global_data : int -> unit = "realloc_global"
-external static_alloc : int -> string = "static_alloc"
-external static_free : string -> unit = "static_free"
-external static_resize : string -> int -> string = "static_resize"
+external global_data : unit -> Obj.t array = "caml_get_global_data"
+external realloc_global_data : int -> unit = "caml_realloc_global"
+external static_alloc : int -> string = "caml_static_alloc"
+external static_free : string -> unit = "caml_static_free"
+external static_release_bytecode : string -> int -> unit = "caml_static_release_bytecode" 
+external static_resize : string -> int -> string = "caml_static_resize"
 type closure = unit -> Obj.t
-external reify_bytecode : string -> int -> closure = "reify_bytecode"
+external reify_bytecode : string -> int -> closure = "caml_reify_bytecode"
 external invoke_traced_function : Obj.t -> Obj.t -> Obj.t -> Obj.t
-                                = "invoke_traced_function"
+                                = "caml_invoke_traced_function"
+external get_section_table : unit -> (string * Obj.t) list
+                                = "caml_get_section_table"
