@@ -10,16 +10,16 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: translclass.mli,v 1.7 2000/03/06 22:11:12 weis Exp $ *)
+(* $Id: translclass.mli,v 1.10 2004/05/26 11:10:50 garrigue Exp $ *)
 
 open Typedtree
 open Lambda
 
-val class_stub : lambda
+val dummy_class : lambda -> lambda
 val transl_class :
   Ident.t list -> Ident.t -> int -> string list -> class_expr -> lambda;;
 
-type error = Illegal_class_expr
+type error = Illegal_class_expr | Tags of string * string
 
 exception Error of Location.t * error
 
