@@ -11,7 +11,7 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: backtrace.c,v 1.11 2002/07/22 08:22:27 xleroy Exp $ */
+/* $Id: backtrace.c,v 1.20 2004/01/02 19:23:19 doligez Exp $ */
 
 /* Stack backtrace for uncaught exceptions */
 
@@ -34,10 +34,10 @@
 #include "sys.h"
 #include "backtrace.h"
 
-CAMLexport int backtrace_active = 0;
-CAMLexport int backtrace_pos = 0;
-CAMLexport code_t * backtrace_buffer = NULL;
-CAMLexport value backtrace_last_exn = Val_unit;
+CAMLexport int caml_backtrace_active = 0;
+CAMLexport int caml_backtrace_pos = 0;
+CAMLexport code_t * caml_backtrace_buffer = NULL;
+CAMLexport value caml_backtrace_last_exn = Val_unit;
 #define BACKTRACE_BUFFER_SIZE 1024
 
 /* Location of fields in the Instruct.debug_event record */
@@ -46,38 +46,48 @@ enum { EV_POS = 0,
        EV_CHAR = 2,
        EV_KIND = 3 };
 
+/* Location of fields in the Lexing.position record. */
+enum {
+  POS_FNAME = 0,
+  POS_LNUM = 1,
+  POS_BOL = 2,
+  POS_CNUM = 3
+};
+
 /* Initialize the backtrace machinery */
 
-void init_backtrace(void)
+void caml_init_backtrace(void)
 {
-  backtrace_active = 1;
-  register_global_root(&backtrace_last_exn);
-  /* Note: lazy initialization of backtrace_buffer in stash_backtrace
+  caml_backtrace_active = 1;
+  caml_register_global_root(&caml_backtrace_last_exn);
+  /* Note: lazy initialization of caml_backtrace_buffer in caml_stash_backtrace
      to simplify the interface with the thread libraries */
 }
 
 /* Store the return addresses contained in the given stack fragment
    into the backtrace array */
 
-void stash_backtrace(value exn, code_t pc, value * sp)
+void caml_stash_backtrace(value exn, code_t pc, value * sp)
 {
-  code_t end_code = (code_t) ((char *) start_code + code_size);
+  code_t end_code = (code_t) ((char *) caml_start_code + caml_code_size);
   if (pc != NULL) pc = pc - 1;
-  if (exn != backtrace_last_exn) {
-    backtrace_pos = 0;
-    backtrace_last_exn = exn;
+  if (exn != caml_backtrace_last_exn) {
+    caml_backtrace_pos = 0;
+    caml_backtrace_last_exn = exn;
   }
-  if (backtrace_buffer == NULL) {
-    backtrace_buffer = malloc(BACKTRACE_BUFFER_SIZE * sizeof(code_t));
-    if (backtrace_buffer == NULL) return;
+  if (caml_backtrace_buffer == NULL) {
+    caml_backtrace_buffer = malloc(BACKTRACE_BUFFER_SIZE * sizeof(code_t));
+    if (caml_backtrace_buffer == NULL) return;
   }
-  if (backtrace_pos >= BACKTRACE_BUFFER_SIZE) return;
-  backtrace_buffer[backtrace_pos++] = pc;
-  for (/*nothing*/; sp < trapsp; sp++) {
+  if (caml_backtrace_pos >= BACKTRACE_BUFFER_SIZE) return;
+  if (pc >= caml_start_code && pc < end_code){
+    caml_backtrace_buffer[caml_backtrace_pos++] = pc;
+  }
+  for (/*nothing*/; sp < caml_trapsp; sp++) {
     code_t p = (code_t) *sp;
-    if (p >= start_code && p < end_code) {
-      if (backtrace_pos >= BACKTRACE_BUFFER_SIZE) break;
-      backtrace_buffer[backtrace_pos++] = p;
+    if (p >= caml_start_code && p < end_code) {
+      if (caml_backtrace_pos >= BACKTRACE_BUFFER_SIZE) break;
+      caml_backtrace_buffer[caml_backtrace_pos++] = p;
     }
   }
 }
@@ -102,19 +112,19 @@ static value read_debug_info(void)
   value evl, l;
 
   exec_name = caml_exe_name;
-  fd = attempt_open(&exec_name, &trail, 1);
+  fd = caml_attempt_open(&exec_name, &trail, 1);
   if (fd < 0) CAMLreturn(Val_false);
-  read_section_descriptors(fd, &trail);
-  if (seek_optional_section(fd, &trail, "DBUG") == -1) {
+  caml_read_section_descriptors(fd, &trail);
+  if (caml_seek_optional_section(fd, &trail, "DBUG") == -1) {
     close(fd);
     CAMLreturn(Val_false);
   }
-  chan = open_descriptor_in(fd);
-  num_events = getword(chan);
-  events = alloc(num_events, 0);
+  chan = caml_open_descriptor_in(fd);
+  num_events = caml_getword(chan);
+  events = caml_alloc(num_events, 0);
   for (i = 0; i < num_events; i++) {
-    orig = getword(chan);
-    evl = input_val(chan);
+    orig = caml_getword(chan);
+    evl = caml_input_val(chan);
     /* Relocate events in event list */
     for (l = evl; l != Val_int(0); l = Field(l, 1)) {
       value ev = Field(l, 0);
@@ -123,7 +133,7 @@ static value read_debug_info(void)
     /* Record event list */
     Store_field(events, i, evl);
   }
-  close_channel(chan);
+  caml_close_channel(chan);
   CAMLreturn(events);
 }
 
@@ -132,15 +142,17 @@ static value read_debug_info(void)
 static value event_for_location(value events, code_t pc)
 {
   mlsize_t i;
-  value pos, l, ev;
+  value pos, l, ev, ev_pos;
 
-  Assert(pc >= start_code && pc < start_code + code_size);
-  pos = Val_long((char *) pc - (char *) start_code);
+  Assert(pc >= caml_start_code && pc < caml_start_code + caml_code_size);
+  pos = Val_long((char *) pc - (char *) caml_start_code);
   for (i = 0; i < Wosize_val(events); i++) {
     for (l = Field(events, i); l != Val_int(0); l = Field(l, 1)) {
       ev = Field(l, 0);
-      if (Field(ev, EV_POS) == pos /* && Is_block(Field(ev, EV_KIND)) */)
-        return ev;
+      ev_pos = Field(ev, EV_POS);
+      /* ocamlc sometimes moves an event past a following PUSH instruction;
+         allow mismatch by 1 instruction. */
+      if (ev_pos == pos || ev_pos == pos + 8) return ev;
     }
   }
   return Val_false;
@@ -150,16 +162,12 @@ static value event_for_location(value events, code_t pc)
 
 static void print_location(value events, int index)
 {
-  code_t pc = backtrace_buffer[index];
+  code_t pc = caml_backtrace_buffer[index];
   char * info;
   value ev;
 
-  if (pc == NULL) {
-    fprintf(stderr, "Raised from a C function\n");
-    return;
-  }
   ev = event_for_location(events, pc);
-  if (is_instruction(*pc, RAISE)) {
+  if (caml_is_instruction(*pc, RAISE)) {
     /* Ignore compiler-inserted raise */
     if (ev == Val_false) return;
     /* Initial raise if index == 0, re-raise otherwise */
@@ -168,20 +176,27 @@ static void print_location(value events, int index)
     else
       info = "Re-raised at";
   } else {
-    info = "Called from";
+    if (index == 0)
+      info = "Raised by primitive operation at";
+    else
+      info = "Called from";
   }
   if (ev == Val_false) {
     fprintf(stderr, "%s unknown location\n", info);
   } else {
-    fprintf(stderr, "%s module %s, character %d\n", info,
-            String_val(Field(ev, EV_MODULE)),
-            Int_val(Field(ev, EV_CHAR)));
+    value ev_char = Field (ev, EV_CHAR);
+    char *fname = String_val (Field (ev_char, POS_FNAME));
+    int lnum = Int_val (Field (ev_char, POS_LNUM));
+    int chr = Int_val (Field (ev_char, POS_CNUM))
+              - Int_val (Field (ev_char, POS_BOL));
+    fprintf (stderr, "%s file \"%s\", line %d, character %d\n", info, fname,
+             lnum, chr);
   }
 }
 
 /* Print a backtrace */
 
-CAMLexport void print_exception_backtrace(void)
+CAMLexport void caml_print_exception_backtrace(void)
 {
   value events;
   int i;
@@ -192,6 +207,6 @@ CAMLexport void print_exception_backtrace(void)
             "(Program not linked with -g, cannot print stack backtrace)\n");
     return;
   }
-  for (i = 0; i < backtrace_pos; i++)
+  for (i = 0; i < caml_backtrace_pos; i++)
     print_location(events, i);
 }
