@@ -11,7 +11,7 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: callback.c,v 1.15 2002/07/11 15:37:18 xleroy Exp $ */
+/* $Id: callback.c,v 1.22 2004/04/26 12:02:07 basile Exp $ */
 
 /* Callbacks from C to Caml */
 
@@ -30,9 +30,12 @@
 #include "fix_code.h"
 #include "stacks.h"
 
-int callback_depth = 0;
+CAMLexport int caml_callback_depth = 0;
 
+#ifndef LOCAL_CALLBACK_BYTECODE
 static opcode_t callback_code[] = { ACC, 0, APPLY, 0, POP, 1, STOP };
+#endif 
+
 
 #ifdef THREADED_CODE
 
@@ -40,7 +43,7 @@ static int callback_code_threaded = 0;
 
 static void thread_callback(void)
 {
-  thread_code(callback_code, sizeof(callback_code));
+  caml_thread_code(callback_code, sizeof(callback_code));
   callback_code_threaded = 1;
 }
 
@@ -52,56 +55,83 @@ static void thread_callback(void)
 
 #endif
 
-CAMLexport value callbackN_exn(value closure, int narg, value args[])
+CAMLexport value caml_callbackN_exn(value closure, int narg, value args[])
 {
   int i;
   value res;
 
+  /* some alternate bytecode implementations (e.g. a JIT translator)
+     might require that the bytecode is kept in a local variable on
+     the C stack */
+#ifdef LOCAL_CALLBACK_BYTECODE
+  opcode_t local_callback_code[7];
+#endif
+
   Assert(narg + 4 <= 256);
+
+  caml_extern_sp -= narg + 4;
+  for (i = 0; i < narg; i++) caml_extern_sp[i] = args[i]; /* arguments */
+#ifndef LOCAL_CALLBACK_BYTECODE
+  caml_extern_sp[narg] = (value) (callback_code + 4); /* return address */
+  caml_extern_sp[narg + 1] = Val_unit;    /* environment */
+  caml_extern_sp[narg + 2] = Val_long(0); /* extra args */
+  caml_extern_sp[narg + 3] = closure;
   Init_callback();
-  extern_sp -= narg + 4;
-  for (i = 0; i < narg; i++) extern_sp[i] = args[i]; /* arguments */
-  extern_sp[narg] = (value) (callback_code + 4); /* return address */
-  extern_sp[narg + 1] = Val_unit;    /* environment */
-  extern_sp[narg + 2] = Val_long(0); /* extra args */
-  extern_sp[narg + 3] = closure;
   callback_code[1] = narg + 3;
   callback_code[3] = narg;
-  res = interprete(callback_code, sizeof(callback_code));
-  if (Is_exception_result(res)) extern_sp += narg + 4; /* PR#1228 */
+  res = caml_interprete(callback_code, sizeof(callback_code));
+#else /*have LOCAL_CALLBACK_BYTECODE*/
+  caml_extern_sp[narg] = (value) (local_callback_code + 4); /* return address */
+  caml_extern_sp[narg + 1] = Val_unit;    /* environment */
+  caml_extern_sp[narg + 2] = Val_long(0); /* extra args */
+  caml_extern_sp[narg + 3] = closure;
+  local_callback_code[0] = ACC;
+  local_callback_code[1] = narg + 3;
+  local_callback_code[2] = APPLY;
+  local_callback_code[3] = narg;
+  local_callback_code[4] = POP;
+  local_callback_code[5] =  1;
+  local_callback_code[6] = STOP;
+#ifdef THREADED_CODE
+  caml_thread_code(local_callback_code, sizeof(local_callback_code));
+#endif /*THREADED_CODE*/
+  res = caml_interprete(local_callback_code, sizeof(local_callback_code));
+  caml_release_bytecode(local_callback_code, sizeof(local_callback_code));
+#endif /*LOCAL_CALLBACK_BYTECODE*/
+  if (Is_exception_result(res)) caml_extern_sp += narg + 4; /* PR#1228 */
   return res;
 }
 
-CAMLexport value callback_exn(value closure, value arg1)
+CAMLexport value caml_callback_exn(value closure, value arg1)
 {
   value arg[1];
   arg[0] = arg1;
-  return callbackN_exn(closure, 1, arg);
+  return caml_callbackN_exn(closure, 1, arg);
 }
 
-CAMLexport value callback2_exn(value closure, value arg1, value arg2)
+CAMLexport value caml_callback2_exn(value closure, value arg1, value arg2)
 {
   value arg[2];
   arg[0] = arg1;
   arg[1] = arg2;
-  return callbackN_exn(closure, 2, arg);
+  return caml_callbackN_exn(closure, 2, arg);
 }
 
-CAMLexport value callback3_exn(value closure,
+CAMLexport value caml_callback3_exn(value closure,
                                value arg1, value arg2, value arg3)
 {
   value arg[3];
   arg[0] = arg1;
   arg[1] = arg2;
   arg[2] = arg3;
-  return callbackN_exn(closure, 3, arg);
+  return caml_callbackN_exn(closure, 3, arg);
 }
 
 #else
 
-/* Native-code callbacks.  callback[123]_exn are implemented in asm. */
+/* Native-code callbacks.  caml_callback[123]_exn are implemented in asm. */
 
-CAMLexport value callbackN_exn(value closure, int narg, value args[])
+CAMLexport value caml_callbackN_exn(value closure, int narg, value args[])
 {
   CAMLparam1 (closure);
   CAMLxparamN (args, narg);
@@ -113,17 +143,17 @@ CAMLexport value callbackN_exn(value closure, int narg, value args[])
     /* Pass as many arguments as possible */
     switch (narg - i) {
     case 1:
-      res = callback_exn(res, args[i]);
+      res = caml_callback_exn(res, args[i]);
       if (Is_exception_result(res)) CAMLreturn (res);
       i += 1;
       break;
     case 2:
-      res = callback2_exn(res, args[i], args[i + 1]);
+      res = caml_callback2_exn(res, args[i], args[i + 1]);
       if (Is_exception_result(res)) CAMLreturn (res);
       i += 2;
       break;
     default:
-      res = callback3_exn(res, args[i], args[i + 1], args[i + 2]);
+      res = caml_callback3_exn(res, args[i], args[i + 1], args[i + 2]);
       if (Is_exception_result(res)) CAMLreturn (res);
       i += 3;
       break;
@@ -136,31 +166,32 @@ CAMLexport value callbackN_exn(value closure, int narg, value args[])
 
 /* Exception-propagating variants of the above */
 
-CAMLexport value callback (value closure, value arg)
+CAMLexport value caml_callback (value closure, value arg)
 {
-  value res = callback_exn(closure, arg);
-  if (Is_exception_result(res)) mlraise(Extract_exception(res));
+  value res = caml_callback_exn(closure, arg);
+  if (Is_exception_result(res)) caml_raise(Extract_exception(res));
   return res;
 }
 
-CAMLexport value callback2 (value closure, value arg1, value arg2)
+CAMLexport value caml_callback2 (value closure, value arg1, value arg2)
 {
-  value res = callback2_exn(closure, arg1, arg2);
-  if (Is_exception_result(res)) mlraise(Extract_exception(res));
+  value res = caml_callback2_exn(closure, arg1, arg2);
+  if (Is_exception_result(res)) caml_raise(Extract_exception(res));
   return res;
 }
 
-CAMLexport value callback3 (value closure, value arg1, value arg2, value arg3)
+CAMLexport value caml_callback3 (value closure, value arg1, value arg2,
+                                 value arg3)
 {
-  value res = callback3_exn(closure, arg1, arg2, arg3);
-  if (Is_exception_result(res)) mlraise(Extract_exception(res));
+  value res = caml_callback3_exn(closure, arg1, arg2, arg3);
+  if (Is_exception_result(res)) caml_raise(Extract_exception(res));
   return res;
 }
 
-CAMLexport value callbackN (value closure, int narg, value args[])
+CAMLexport value caml_callbackN (value closure, int narg, value args[])
 {
-  value res = callbackN_exn(closure, narg, args);
-  if (Is_exception_result(res)) mlraise(Extract_exception(res));
+  value res = caml_callbackN_exn(closure, narg, args);
+  if (Is_exception_result(res)) caml_raise(Extract_exception(res));
   return res;
 }
 
@@ -183,19 +214,19 @@ static unsigned int hash_value_name(char *name)
   return h % Named_value_size;
 }
 
-CAMLprim value register_named_value(value vname, value val)
+CAMLprim value caml_register_named_value(value vname, value val)
 {
   struct named_value * nv;
   char * name = String_val(vname);
   unsigned int h = hash_value_name(name);
 
   nv = (struct named_value *)
-         stat_alloc(sizeof(struct named_value) + strlen(name));
+         caml_stat_alloc(sizeof(struct named_value) + strlen(name));
   strcpy(nv->name, name);
   nv->val = val;
   nv->next = named_value_table[h];
   named_value_table[h] = nv;
-  register_global_root(&nv->val);
+  caml_register_global_root(&nv->val);
   return Val_unit;
 }
 
