@@ -11,23 +11,22 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: config.h,v 1.30 2002/08/01 14:14:10 doligez Exp $ */
+/* $Id: config.h,v 1.36 2004/04/19 07:55:28 starynke Exp $ */
 
-#ifndef _config_
-#define _config_
+#ifndef CAML_CONFIG_H
+#define CAML_CONFIG_H
 
 /* <include ../config/m.h> */
 /* <include ../config/s.h> */
 /* <private> */
-#if !macintosh
 #include "../config/m.h"
 #include "../config/s.h"
-#else
-#include <m.h>
-#include <s.h>
-#endif
 /* </private> */
 
+#ifndef CAML_NAME_SPACE
+#include "compatibility.h"
+#endif
+
 /* Types for signed chars, 16-bit integers, 32-bit integers, 64-bit integers */
 
 typedef signed char schar;
@@ -75,7 +74,7 @@ typedef struct { uint32 l, h; } uint64, int64;
 /* We use threaded code interpretation if the compiler provides labels
    as first-class values (GCC 2.x). */
 
-#if defined(__GNUC__) && __GNUC__ >= 2 && !defined(DEBUG) && !defined (SHRINKED_GNUC)
+#if defined(__GNUC__) && __GNUC__ >= 2 && !defined(DEBUG) && !defined (SHRINKED_GNUC) && !defined(CAML_JIT)
 #define THREADED_CODE
 #endif
 
@@ -121,18 +120,13 @@ typedef struct { uint32 l, h; } uint64, int64;
    Must be a multiple of [Page_size / sizeof (value)]. */
 #define Heap_chunk_min (2 * Page_size / sizeof (value))
 
-/* Maximum size of a contiguous piece of the heap (words).
-   Must be greater than or equal to [Heap_chunk_min].
-   Must be greater than or equal to [Bhsize_wosize (Max_wosize)]. */
-#define Heap_chunk_max (Bhsize_wosize (Max_wosize))
-
 /* Default size increment when growing the heap. (words)
    Must be a multiple of [Page_size / sizeof (value)]. */
-#define Heap_chunk_def (62 * 1024)
+#define Heap_chunk_def (15 * Page_size)
 
 /* Default initial size of the major heap (words);
    same constraints as for Heap_chunk_def. */
-#define Init_heap_def (62 * 1024)
+#define Init_heap_def (15 * Page_size)
 
 
 /* Default speed setting for the major GC.  The heap will grow until
@@ -148,4 +142,4 @@ typedef struct { uint32 l, h; } uint64, int64;
 #define Max_percent_free_def 500
 
 
-#endif /* _config_ */
+#endif /* CAML_CONFIG_H */
