@@ -11,14 +11,19 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: finalise.h,v 1.3 2001/12/07 13:39:27 xleroy Exp $ */
+/* $Id: finalise.h,v 1.5 2004/01/02 19:23:21 doligez Exp $ */
+
+#ifndef CAML_FINALISE_H
+#define CAML_FINALISE_H
 
 #include "roots.h"
 
-void final_update (void);
-void final_do_calls (void);
-void final_do_strong_roots (scanning_action f);
-void final_do_weak_roots (scanning_action f);
-void final_do_young_roots (scanning_action f);
-void final_empty_young (void);
-value final_register (value f, value v);
+void caml_final_update (void);
+void caml_final_do_calls (void);
+void caml_final_do_strong_roots (scanning_action f);
+void caml_final_do_weak_roots (scanning_action f);
+void caml_final_do_young_roots (scanning_action f);
+void caml_final_empty_young (void);
+value caml_final_register (value f, value v);
+
+#endif /* CAML_FINALISE_H */
