@@ -11,10 +11,10 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: gc.h,v 1.11 2002/04/18 07:27:37 garrigue Exp $ */
+/* $Id: gc.h,v 1.14 2003/12/15 18:10:46 doligez Exp $ */
 
-#ifndef _gc_
-#define _gc_
+#ifndef CAML_GC_H
+#define CAML_GC_H
 
 
 #include "mlvalues.h"
@@ -40,9 +40,11 @@
 
 /* This depends on the layout of the header.  See [mlvalues.h]. */
 #define Make_header(wosize, tag, color)                                       \
+      (/*Assert ((wosize) <= Max_wosize),*/                                   \
        ((header_t) (((header_t) (wosize) << 10)                               \
                     + (color)                                                 \
-                    + (tag_t) (tag)))
+                    + (tag_t) (tag)))                                         \
+      )
 
 #define Is_white_val(val) (Color_val(val) == Caml_white)
 #define Is_gray_val(val) (Color_val(val) == Caml_gray)
@@ -50,4 +52,4 @@
 #define Is_black_val(val) (Color_val(val) == Caml_black)
 
 
-#endif /* _gc_ */
+#endif /* CAML_GC_H */
