@@ -6,11 +6,12 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../LICENSE.     */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: instrtrace.h,v 1.5 1999/11/17 18:57:09 xleroy Exp $ */
+/* $Id: instrtrace.h,v 1.8 2004/04/22 09:48:04 basile Exp $ */
 
 /* Trace the instructions executed */
 
@@ -21,10 +22,10 @@
 #include "mlvalues.h"
 #include "misc.h"
 
-extern int trace_flag;
-extern long icount;
-void stop_here (void);
-void disasm_instr (code_t pc);
-
-
+extern int caml_trace_flag;
+extern long caml_icount;
+void caml_stop_here (void);
+void caml_disasm_instr (code_t pc);
+void caml_trace_value_file (value v, code_t prog, int proglen, FILE * f);
+void caml_trace_accu_sp_file(value accu, value * sp, code_t prog, int proglen, FILE * f);
 #endif
