@@ -11,7 +11,7 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: interp.c,v 1.73 2002/05/25 08:32:53 xleroy Exp $ */
+/* $Id: interp.c,v 1.90 2004/06/12 10:40:52 xleroy Exp $ */
 
 /* The bytecode interpreter */
 #include <stdio.h>
@@ -37,10 +37,10 @@
         sp         the stack pointer (grows downward)
         accu       the accumulator
         env        heap-allocated environment
-        trapsp     pointer to the current trap frame
+        caml_trapsp pointer to the current trap frame
         extra_args number of extra arguments provided by the caller
 
-sp is a local copy of the global variable extern_sp. */
+sp is a local copy of the global variable caml_extern_sp. */
 
 /* Instruction decoding */
 
@@ -68,10 +68,11 @@ sp is a local copy of the global variable extern_sp. */
 
 /* GC interface */
 
-#define Setup_for_gc { sp -= 2; sp[0] = accu; sp[1] = env; extern_sp = sp; }
+#define Setup_for_gc \
+  { sp -= 2; sp[0] = accu; sp[1] = env; caml_extern_sp = sp; }
 #define Restore_after_gc { accu = sp[0]; env = sp[1]; sp += 2; }
-#define Setup_for_c_call { *--sp = env; extern_sp = sp; }
-#define Restore_after_c_call { sp = extern_sp; env = *sp++; }
+#define Setup_for_c_call { saved_pc = pc; *--sp = env; caml_extern_sp = sp; }
+#define Restore_after_c_call { sp = caml_extern_sp; env = *sp++; }
 
 /* An event frame must look like accu + a C_CALL frame + a RETURN 1 frame */
 #define Setup_for_event \
@@ -82,9 +83,9 @@ sp is a local copy of the global variable extern_sp. */
     sp[3] = (value) pc; /* RETURN frame: saved return address */ \
     sp[4] = env; /* RETURN frame: saved environment */ \
     sp[5] = Val_long(extra_args); /* RETURN frame: saved extra args */ \
-    extern_sp = sp; }
+    caml_extern_sp = sp; }
 #define Restore_after_event \
-  { sp = extern_sp; accu = sp[0]; \
+  { sp = caml_extern_sp; accu = sp[0]; \
     pc = (code_t) sp[3]; env = sp[4]; extra_args = Long_val(sp[5]); \
     sp += 6; }
 
@@ -94,15 +95,15 @@ sp is a local copy of the global variable extern_sp. */
    { sp -= 4; \
      sp[0] = accu; sp[1] = (value)(pc - 1); \
      sp[2] = env; sp[3] = Val_long(extra_args); \
-     extern_sp = sp; }
+     caml_extern_sp = sp; }
 #define Restore_after_debugger { sp += 4; }
 
 #ifdef THREADED_CODE
 #define Restart_curr_instr \
-  goto *(jumptable[saved_code[pc - 1 - start_code]])
+  goto *(jumptable[caml_saved_code[pc - 1 - caml_start_code]])
 #else
 #define Restart_curr_instr \
-  curr_instr = saved_code[pc - 1 - start_code]; \
+  curr_instr = caml_saved_code[pc - 1 - caml_start_code]; \
   goto dispatch_instr
 #endif
 
@@ -113,7 +114,7 @@ sp is a local copy of the global variable extern_sp. */
    For GCC, I have hand-assigned hardware registers for several architectures.
 */
 
-#if defined(__GNUC__) && !defined(DEBUG)
+#if defined(__GNUC__) && !defined(__INTEL_COMPILER) && !defined(DEBUG)
 #ifdef __mips__
 #define PC_REG asm("$16")
 #define SP_REG asm("$17")
@@ -168,6 +169,11 @@ sp is a local copy of the global variable extern_sp. */
 #define ACCU_REG asm("38")
 #define JUMPTBL_BASE_REG asm("39")
 #endif
+#ifdef __x86_64__
+#define PC_REG asm("%r15")
+#define SP_REG asm("%r14")
+#define ACCU_REG asm("%r13")
+#endif
 #endif
 
 /* Division and modulus madness */
@@ -177,9 +183,14 @@ extern long caml_safe_div(long p, long q);
 extern long caml_safe_mod(long p, long q);
 #endif
 
+
+#ifdef DEBUG
+static long caml_bcodcount;
+#endif
+
 /* The interpreter itself */
 
-value interprete(code_t prog, asize_t prog_size)
+value caml_interprete(code_t prog, asize_t prog_size)
 {
 #ifdef PC_REG
   register code_t pc PC_REG;
@@ -201,9 +212,10 @@ value interprete(code_t prog, asize_t prog_size)
   long extra_args;
   struct longjmp_buffer * initial_external_raise;
   int initial_sp_offset;
-  /* volatile prevents collapsing initial_local_roots with another
-     local variable, like Digital Unix 4.0 C compiler does (wrongly) */
+  /* volatile ensures that initial_local_roots and saved_pc
+     will keep correct value across longjmp */
   struct caml__roots_block * volatile initial_local_roots;
+  volatile code_t saved_pc;
   struct longjmp_buffer raise_buf;
   value * modify_dest, modify_newval;
 #ifndef THREADED_CODE
@@ -218,8 +230,8 @@ value interprete(code_t prog, asize_t prog_size)
 
   if (prog == NULL) {           /* Interpreter is initializing */
 #ifdef THREADED_CODE
-    instr_table = (char **) jumptable;
-    instr_base = Jumptbl_base;
+    caml_instr_table = (char **) jumptable;
+    caml_instr_base = Jumptbl_base;
 #endif
     return Val_unit;
   }
@@ -227,21 +239,22 @@ value interprete(code_t prog, asize_t prog_size)
 #if defined(THREADED_CODE) && defined(ARCH_SIXTYFOUR) && !defined(ARCH_CODE32)
   jumptbl_base = Jumptbl_base;
 #endif
-  initial_local_roots = local_roots;
-  initial_sp_offset = (char *) stack_high - (char *) extern_sp;
-  initial_external_raise = external_raise;
-  callback_depth++;
+  initial_local_roots = caml_local_roots;
+  initial_sp_offset = (char *) caml_stack_high - (char *) caml_extern_sp;
+  initial_external_raise = caml_external_raise;
+  caml_callback_depth++;
+  saved_pc = NULL;
 
   if (sigsetjmp(raise_buf.buf, 0)) {
-    local_roots = initial_local_roots;
-    sp = extern_sp;
-    accu = exn_bucket;
-    pc = NULL;
+    caml_local_roots = initial_local_roots;
+    sp = caml_extern_sp;
+    accu = caml_exn_bucket;
+    pc = saved_pc + 2; /* +2 adjustement for the sole purpose of backtraces */
     goto raise_exception;
   }
-  external_raise = &raise_buf;
+  caml_external_raise = &raise_buf;
 
-  sp = extern_sp;
+  sp = caml_extern_sp;
   pc = prog;
   extra_args = 0;
   env = Atom(0);
@@ -250,18 +263,27 @@ value interprete(code_t prog, asize_t prog_size)
 #ifdef THREADED_CODE
 #ifdef DEBUG
  next_instr:
-  if (icount-- == 0) stop_here ();
-  Assert(sp >= stack_low);
-  Assert(sp <= stack_high);
+  if (caml_icount-- == 0) caml_stop_here ();
+  Assert(sp >= caml_stack_low);
+  Assert(sp <= caml_stack_high);
 #endif
   goto *(void *)(jumptbl_base + *pc++); /* Jump to the first instruction */
 #else
   while(1) {
 #ifdef DEBUG
-    if (icount-- == 0) stop_here ();
-    if (trace_flag) disasm_instr(pc);
-    Assert(sp >= stack_low);
-    Assert(sp <= stack_high);
+    caml_bcodcount++;
+    if (caml_icount-- == 0) caml_stop_here ();
+    if (caml_trace_flag>1) printf("\n##%ld\n", caml_bcodcount);
+    if (caml_trace_flag) caml_disasm_instr(pc);
+    if (caml_trace_flag>1) {
+      printf("env=");
+      caml_trace_value_file(env,prog,prog_size,stdout);
+      putchar('\n');
+      caml_trace_accu_sp_file(accu,sp,prog,prog_size,stdout);
+      fflush(stdout);
+    };
+    Assert(sp >= caml_stack_low);
+    Assert(sp <= caml_stack_high);
 #endif
     curr_instr = *pc++;
 
@@ -563,7 +585,7 @@ value interprete(code_t prog, asize_t prog_size)
       *--sp = accu;
       /* Fallthrough */
     Instruct(GETGLOBAL):
-      accu = Field(global_data, *pc);
+      accu = Field(caml_global_data, *pc);
       pc++;
       Next;
 
@@ -571,7 +593,7 @@ value interprete(code_t prog, asize_t prog_size)
       *--sp = accu;
       /* Fallthrough */
     Instruct(GETGLOBALFIELD): {
-      accu = Field(global_data, *pc);
+      accu = Field(caml_global_data, *pc);
       pc++;
       accu = Field(accu, *pc);
       pc++;
@@ -579,7 +601,7 @@ value interprete(code_t prog, asize_t prog_size)
     }
 
     Instruct(SETGLOBAL):
-      modify(&Field(global_data, *pc), accu);
+      caml_modify(&Field(caml_global_data, *pc), accu);
       accu = Val_unit;
       pc++;
       Next;
@@ -603,9 +625,15 @@ value interprete(code_t prog, asize_t prog_size)
       tag_t tag = *pc++;
       mlsize_t i;
       value block;
-      Alloc_small(block, wosize, tag);
-      Field(block, 0) = accu;
-      for (i = 1; i < wosize; i++) Field(block, i) = *sp++;
+      if (wosize <= Max_young_wosize) {
+        Alloc_small(block, wosize, tag);
+        Field(block, 0) = accu;
+        for (i = 1; i < wosize; i++) Field(block, i) = *sp++;
+      } else {
+        block = caml_alloc_shr(wosize, tag);
+        caml_initialize(&Field(block, 0), accu);
+        for (i = 1; i < wosize; i++) caml_initialize(&Field(block, i), *sp++);
+      }
       accu = block;
       Next;
     }
@@ -642,7 +670,11 @@ value interprete(code_t prog, asize_t prog_size)
       mlsize_t size = *pc++;
       mlsize_t i;
       value block;
-      Alloc_small(block, size * Double_wosize, Double_array_tag);
+      if (size <= Max_young_wosize / Double_wosize) {
+        Alloc_small(block, size * Double_wosize, Double_array_tag);
+      } else {
+        block = caml_alloc_shr(size * Double_wosize, Double_array_tag);
+      }
       Store_double_field(block, 0, Double_val(accu));
       for (i = 1; i < size; i++){
         Store_double_field(block, i, Double_val(*sp));
@@ -767,38 +799,40 @@ value interprete(code_t prog, asize_t prog_size)
     Instruct(PUSHTRAP):
       sp -= 4;
       Trap_pc(sp) = pc + *pc;
-      Trap_link(sp) = trapsp;
+      Trap_link(sp) = caml_trapsp;
       sp[2] = env;
       sp[3] = Val_long(extra_args);
-      trapsp = sp;
+      caml_trapsp = sp;
       pc++;
       Next;
 
     Instruct(POPTRAP):
-      if (something_to_do) {
+      if (caml_something_to_do) {
         /* We must check here so that if a signal is pending and its
            handler triggers an exception, the exception is trapped
            by the current try...with, not the enclosing one. */
         pc--; /* restart the POPTRAP after processing the signal */
         goto process_signal;
       }
-      trapsp = Trap_link(sp);
+      caml_trapsp = Trap_link(sp);
       sp += 4;
       Next;
 
     Instruct(RAISE):
     raise_exception:
-      if (trapsp >= trap_barrier) debugger(TRAP_BARRIER);
-      if (backtrace_active) stash_backtrace(accu, pc, sp);
-      if ((char *) trapsp >= (char *) stack_high - initial_sp_offset) {
-        external_raise = initial_external_raise;
-        extern_sp = (value *) ((char *) stack_high - initial_sp_offset);
-        callback_depth--;
+      if (caml_trapsp >= caml_trap_barrier) caml_debugger(TRAP_BARRIER);
+      if (caml_backtrace_active) caml_stash_backtrace(accu, pc, sp);
+      if ((char *) caml_trapsp
+          >= (char *) caml_stack_high - initial_sp_offset) {
+        caml_external_raise = initial_external_raise;
+        caml_extern_sp = (value *) ((char *) caml_stack_high
+                                    - initial_sp_offset);
+        caml_callback_depth--;
         return Make_exception_result(accu);
       }
-      sp = trapsp;
+      sp = caml_trapsp;
       pc = Trap_pc(sp);
-      trapsp = Trap_link(sp);
+      caml_trapsp = Trap_link(sp);
       env = sp[2];
       extra_args = Long_val(sp[3]);
       sp += 4;
@@ -807,23 +841,23 @@ value interprete(code_t prog, asize_t prog_size)
 /* Stack checks */
 
     check_stacks:
-      if (sp < stack_threshold) {
-        extern_sp = sp;
-        realloc_stack(Stack_threshold / sizeof(value));
-        sp = extern_sp;
+      if (sp < caml_stack_threshold) {
+        caml_extern_sp = sp;
+        caml_realloc_stack(Stack_threshold / sizeof(value));
+        sp = caml_extern_sp;
       }
       /* Fall through CHECK_SIGNALS */
 
 /* Signal handling */
 
     Instruct(CHECK_SIGNALS):    /* accu not preserved */
-      if (something_to_do) goto process_signal;
+      if (caml_something_to_do) goto process_signal;
       Next;
 
     process_signal:
-      something_to_do = 0;
+      caml_something_to_do = 0;
       Setup_for_event;
-      process_event();
+      caml_process_event();
       Restore_after_event;
       Next;
 
@@ -915,7 +949,7 @@ value interprete(code_t prog, asize_t prog_size)
 
     Instruct(DIVINT): {
       long divisor = Long_val(*sp++);
-      if (divisor == 0) { Setup_for_c_call; raise_zero_divide(); }
+      if (divisor == 0) { Setup_for_c_call; caml_raise_zero_divide(); }
 #ifdef NONSTANDARD_DIV_MOD
       accu = Val_long(caml_safe_div(Long_val(accu), divisor));
 #else
@@ -925,7 +959,7 @@ value interprete(code_t prog, asize_t prog_size)
     }
     Instruct(MODINT): {
       long divisor = Long_val(*sp++);
-      if (divisor == 0) { Setup_for_c_call; raise_zero_divide(); }
+      if (divisor == 0) { Setup_for_c_call; caml_raise_zero_divide(); }
 #ifdef NONSTANDARD_DIV_MOD
       accu = Val_long(caml_safe_mod(Long_val(accu), divisor));
 #else
@@ -992,33 +1026,92 @@ value interprete(code_t prog, asize_t prog_size)
     
 /* Object-oriented operations */
 
-#define Lookup(obj, lab) \
-  Field (Field (Field (obj, 0), ((lab) >> 16) / sizeof (value)), \
-         ((lab) / sizeof (value)) & 0xFF)
+#define Lookup(obj, lab) Field (Field (obj, 0), Int_val(lab))
+
+      /* please don't forget to keep below code in sync with the
+	 functions caml_cache_public_method and
+	 caml_cache_public_method2 in obj.c */
 
     Instruct(GETMETHOD):
       accu = Lookup(sp[0], accu);
       Next;
 
+#define CAML_METHOD_CACHE
+#ifdef CAML_METHOD_CACHE
+    Instruct(GETPUBMET): {
+      /* accu == object, pc[0] == tag, pc[1] == cache */
+      value meths = Field (accu, 0);
+      value ofs;
+#ifdef CAML_TEST_CACHE
+      static int calls = 0, hits = 0;
+      if (calls >= 10000000) {
+        fprintf(stderr, "cache hit = %d%%\n", hits / 100000);
+        calls = 0; hits = 0;
+      }
+      calls++;
+#endif
+      *--sp = accu;
+      accu = Val_int(*pc++);
+      ofs = *pc & Field(meths,1);
+      if (*(value*)(((char*)&Field(meths,3)) + ofs) == accu) {
+#ifdef CAML_TEST_CACHE
+        hits++;
+#endif
+        accu = *(value*)(((char*)&Field(meths,2)) + ofs);
+      }
+      else
+      {
+        int li = 3, hi = Field(meths,0), mi;
+        while (li < hi) {
+          mi = ((li+hi) >> 1) | 1;
+          if (accu < Field(meths,mi)) hi = mi-2;
+          else li = mi;
+        }
+        *pc = (li-3)*sizeof(value);
+        accu = Field (meths, li-1);
+      }
+      pc++;
+      Next;
+    }
+#else
+    Instruct(GETPUBMET):
+      *--sp = accu;
+      accu = Val_int(*pc);
+      pc += 2;
+      /* Fallthrough */
+#endif
+    Instruct(GETDYNMET): {
+      /* accu == tag, sp[0] == object, *pc == cache */
+      value meths = Field (sp[0], 0);
+      int li = 3, hi = Field(meths,0), mi;
+      while (li < hi) {
+        mi = ((li+hi) >> 1) | 1;
+        if (accu < Field(meths,mi)) hi = mi-2;
+        else li = mi;
+      }
+      accu = Field (meths, li-1);
+      Next;
+    }
+
 /* Debugging and machine control */
 
     Instruct(STOP):
-      external_raise = initial_external_raise;
-      extern_sp = sp;
-      callback_depth--;
+      caml_external_raise = initial_external_raise;
+      caml_extern_sp = sp;
+      caml_callback_depth--;
       return accu;
 
     Instruct(EVENT):
-      if (--event_count == 0) {
+      if (--caml_event_count == 0) {
         Setup_for_debugger;
-        debugger(EVENT_COUNT);
+        caml_debugger(EVENT_COUNT);
         Restore_after_debugger;
       }
       Restart_curr_instr;
 
     Instruct(BREAK):
       Setup_for_debugger;
-      debugger(BREAKPOINT);
+      caml_debugger(BREAKPOINT);
       Restore_after_debugger;
       Restart_curr_instr;
 
@@ -1027,10 +1120,29 @@ value interprete(code_t prog, asize_t prog_size)
 #if _MSC_VER >= 1200
       __assume(0);
 #else
-      fatal_error_arg("Fatal error: bad opcode (%lx)\n",
-                      (char *)(long)(*(pc-1)));
+      caml_fatal_error_arg("Fatal error: bad opcode (%lx)\n",
+                           (char *)(long)(*(pc-1)));
 #endif
     }
   }
 #endif
 }
+
+void caml_prepare_bytecode(code_t prog, asize_t prog_size) {
+  /* other implementations of the interpreter (such as an hypothetical
+     JIT translator) might want to do something with a bytecode before
+     running it */
+  Assert(prog);
+  Assert(prog_size>0);
+  /* actually, the threading of the bytecode might be done here */
+} 
+
+void caml_release_bytecode(code_t prog, asize_t prog_size) {
+  /* other implementations of the interpreter (such as an hypothetical
+     JIT translator) might want to know when a bytecode is removed */
+  /* check that we have a program */
+  Assert(prog);
+  Assert(prog_size>0);
+}
+
+/* eof $Id: interp.c,v 1.90 2004/06/12 10:40:52 xleroy Exp $ */
