@@ -11,19 +11,19 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: md5.h,v 1.8 2001/12/07 13:39:32 xleroy Exp $ */
+/* $Id: md5.h,v 1.12 2003/12/31 14:20:37 doligez Exp $ */
 
 /* MD5 message digest */
 
-#ifndef _md5
-#define _md5
+#ifndef CAML_MD5_H
+#define CAML_MD5_H
 
 
 #include "mlvalues.h"
 #include "io.h"
 
-CAMLextern value md5_string (value str, value ofs, value len);
-CAMLextern value md5_chan (value vchan, value len);
+CAMLextern value caml_md5_string (value str, value ofs, value len);
+CAMLextern value caml_md5_chan (value vchan, value len);
 
 struct MD5Context {
         uint32 buf[4];
@@ -31,10 +31,11 @@ struct MD5Context {
         unsigned char in[64];
 };
 
-void MD5Init (struct MD5Context *context);
-void MD5Update (struct MD5Context *context, unsigned char *buf, unsigned len);
-void MD5Final (unsigned char *digest, struct MD5Context *ctx);
-void MD5Transform (uint32 *buf, uint32 *in);
+CAMLextern void caml_MD5Init (struct MD5Context *context);
+CAMLextern void caml_MD5Update (struct MD5Context *context, unsigned char *buf, 
+                                unsigned long len);
+CAMLextern void caml_MD5Final (unsigned char *digest, struct MD5Context *ctx);
+CAMLextern void caml_MD5Transform (uint32 *buf, uint32 *in);
 
 
-#endif
+#endif /* CAML_MD5_H */
