@@ -11,7 +11,7 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: minor_gc.c,v 1.35 2002/07/23 14:11:48 doligez Exp $ */
+/* $Id: minor_gc.c,v 1.42 2004/01/05 20:25:59 doligez Exp $ */
 
 #include <string.h>
 #include "config.h"
@@ -27,15 +27,15 @@
 #include "roots.h"
 #include "signals.h"
 
-asize_t minor_heap_size;
-char *young_start = NULL, *young_end = NULL;
-char *young_ptr = NULL, *young_limit = NULL;
+asize_t caml_minor_heap_size;
+CAMLexport char *caml_young_start = NULL, *caml_young_end = NULL;
+CAMLexport char *caml_young_ptr = NULL, *caml_young_limit = NULL;
 static value **ref_table = NULL, **ref_table_end, **ref_table_threshold;
-value **ref_table_ptr = NULL, **ref_table_limit;
+CAMLexport value **caml_ref_table_ptr = NULL, **caml_ref_table_limit;
 static asize_t ref_table_size, ref_table_reserve;
-int in_minor_collection = 0;
+int caml_in_minor_collection = 0;
 
-void set_minor_heap_size (asize_t size)
+void caml_set_minor_heap_size (asize_t size)
 {
   char *new_heap;
   value **new_table;
@@ -43,27 +43,27 @@ void set_minor_heap_size (asize_t size)
   Assert (size >= Minor_heap_min);
   Assert (size <= Minor_heap_max);
   Assert (size % sizeof (value) == 0);
-  if (young_ptr != young_end) minor_collection ();
-                                           Assert (young_ptr == young_end);
-  new_heap = (char *) stat_alloc (size);
-  if (young_start != NULL){
-    stat_free (young_start);
+  if (caml_young_ptr != caml_young_end) caml_minor_collection ();
+                                    Assert (caml_young_ptr == caml_young_end);
+  new_heap = (char *) caml_stat_alloc (size);
+  if (caml_young_start != NULL){
+    caml_stat_free (caml_young_start);
   }
-  young_start = new_heap;
-  young_end = new_heap + size;
-  young_limit = young_start;
-  young_ptr = young_end;
-  minor_heap_size = size;
+  caml_young_start = new_heap;
+  caml_young_end = new_heap + size;
+  caml_young_limit = caml_young_start;
+  caml_young_ptr = caml_young_end;
+  caml_minor_heap_size = size;
 
-  ref_table_size = minor_heap_size / sizeof (value) / 8;
+  ref_table_size = caml_minor_heap_size / sizeof (value) / 8;
   ref_table_reserve = 256;
-  new_table = (value **) stat_alloc ((ref_table_size + ref_table_reserve)
-                                     * sizeof (value *));
-  if (ref_table != NULL) stat_free (ref_table);
+  new_table = (value **) caml_stat_alloc ((ref_table_size + ref_table_reserve)
+                                          * sizeof (value *));
+  if (ref_table != NULL) caml_stat_free (ref_table);
   ref_table = new_table;
-  ref_table_ptr = ref_table;
+  caml_ref_table_ptr = ref_table;
   ref_table_threshold = ref_table + ref_table_size;
-  ref_table_limit = ref_table_threshold;
+  caml_ref_table_limit = ref_table_threshold;
   ref_table_end = ref_table + ref_table_size + ref_table_reserve;
 }
 
@@ -72,24 +72,26 @@ static value oldify_todo_list = 0;
 /* Note that the tests on the tag depend on the fact that Infix_tag,
    Forward_tag, and No_scan_tag are contiguous. */
 
-void oldify_one (value v, value *p)
+void caml_oldify_one (value v, value *p)
 {
-  value result, field0;
+  value result;
   header_t hd;
   mlsize_t sz, i;
   tag_t tag;
 
  tail_call:
   if (Is_block (v) && Is_young (v)){
-    Assert (Hp_val (v) >= young_ptr);
+    Assert (Hp_val (v) >= caml_young_ptr);
     hd = Hd_val (v);
     if (hd == 0){         /* If already forwarded */
       *p = Field (v, 0);  /*  then forward pointer is first field. */
     }else{
       tag = Tag_hd (hd);
       if (tag < Infix_tag){
+        value field0;
+
         sz = Wosize_hd (hd);
-        result = alloc_shr (sz, tag);
+        result = caml_alloc_shr (sz, tag);
         *p = result;
         field0 = Field (v, 0);
         Hd_val (v) = 0;            /* Set forward flag */
@@ -106,19 +108,37 @@ void oldify_one (value v, value *p)
         }
       }else if (tag >= No_scan_tag){
         sz = Wosize_hd (hd);
-        result = alloc_shr (sz, tag);
+        result = caml_alloc_shr (sz, tag);
         for (i = 0; i < sz; i++) Field (result, i) = Field (v, i);
         Hd_val (v) = 0;            /* Set forward flag */
         Field (v, 0) = result;     /*  and forward pointer. */
         *p = result;
       }else if (tag == Infix_tag){
         mlsize_t offset = Infix_offset_hd (hd);
-        oldify_one (v - offset, p);   /* This cannot recurse deeper than 1. */
+        caml_oldify_one (v - offset, p);   /* Cannot recurse deeper than 1. */
         *p += offset;
       }else{
+        value f = Forward_val (v);
+        tag_t ft = 0;
+
         Assert (tag == Forward_tag);
-        v = Forward_val (v);          /* Follow the forwarding */
-        goto tail_call;               /*  then oldify. */
+        if (Is_block (f) && (Is_young (f) || Is_in_heap (f))){
+          ft = Tag_val (Hd_val (f) == 0 ? Field (f, 0) : f);
+        }
+        if (ft == Forward_tag || ft == Lazy_tag || ft == Double_tag){
+          /* Do not short-circuit the pointer.  Copy as a normal block. */
+          Assert (Wosize_hd (hd) == 1);
+          result = caml_alloc_shr (1, Forward_tag);
+          *p = result;
+          Hd_val (v) = 0;             /* Set (GC) forward flag */
+          Field (v, 0) = result;      /*  and forward pointer. */
+          p = &Field (result, 0);
+          v = f;
+          goto tail_call;
+        }else{
+          v = f;                        /* Follow the forwarding */
+          goto tail_call;               /*  then oldify. */
+        }
       }
     }
   }else{
@@ -126,11 +146,11 @@ void oldify_one (value v, value *p)
   }
 }
 
-/* Finish the work that was put off by oldify_one.
-   Note that oldify_one itself is called by oldify_mopup, so we
+/* Finish the work that was put off by [caml_oldify_one].
+   Note that [caml_oldify_one] itself is called by oldify_mopup, so we
    have to be careful to remove the first entry from the list before
    oldifying its fields. */
-void oldify_mopup (void)
+void caml_oldify_mopup (void)
 {
   value v, new_v, f;
   mlsize_t i;
@@ -143,12 +163,12 @@ void oldify_mopup (void)
 
     f = Field (new_v, 0);
     if (Is_block (f) && Is_young (f)){
-      oldify_one (f, &Field (new_v, 0));
+      caml_oldify_one (f, &Field (new_v, 0));
     }
     for (i = 1; i < Wosize_val (new_v); i++){
       f = Field (v, i);
       if (Is_block (f) && Is_young (f)){
-        oldify_one (f, &Field (new_v, i));
+        caml_oldify_one (f, &Field (new_v, i));
       }else{
         Field (new_v, i) = f;
       }
@@ -159,32 +179,32 @@ void oldify_mopup (void)
 /* Make sure the minor heap is empty by performing a minor collection
    if needed.
 */
-void empty_minor_heap (void)
+void caml_empty_minor_heap (void)
 {
   value **r;
 
-  if (young_ptr != young_end){
-    in_minor_collection = 1;
-    gc_message (0x02, "<", 0);
-    oldify_local_roots();
-    for (r = ref_table; r < ref_table_ptr; r++){
-      oldify_one (**r, *r);
+  if (caml_young_ptr != caml_young_end){
+    caml_in_minor_collection = 1;
+    caml_gc_message (0x02, "<", 0);
+    caml_oldify_local_roots();
+    for (r = ref_table; r < caml_ref_table_ptr; r++){
+      caml_oldify_one (**r, *r);
     }
-    oldify_mopup ();
-    if (young_ptr < young_start) young_ptr = young_start;
-    stat_minor_words += Wsize_bsize (young_end - young_ptr);
-    young_ptr = young_end;
-    young_limit = young_start;
-    ref_table_ptr = ref_table;
-    ref_table_limit = ref_table_threshold;
-    gc_message (0x02, ">", 0);
-    in_minor_collection = 0;
+    caml_oldify_mopup ();
+    if (caml_young_ptr < caml_young_start) caml_young_ptr = caml_young_start;
+    caml_stat_minor_words += Wsize_bsize (caml_young_end - caml_young_ptr);
+    caml_young_ptr = caml_young_end;
+    caml_young_limit = caml_young_start;
+    caml_ref_table_ptr = ref_table;
+    caml_ref_table_limit = ref_table_threshold;
+    caml_gc_message (0x02, ">", 0);
+    caml_in_minor_collection = 0;
   }
-  final_empty_young ();
+  caml_final_empty_young ();
 #ifdef DEBUG
   {
     value *p;
-    for (p = (value *) young_start; p < (value *) young_end; ++p){
+    for (p = (value *) caml_young_start; p < (value *) caml_young_end; ++p){
       *p = Debug_free_minor;
     }
   }
@@ -195,51 +215,53 @@ void empty_minor_heap (void)
    functions, etc.
    Leave the minor heap empty.
 */
-void minor_collection (void)
+CAMLexport void caml_minor_collection (void)
 {
-  long prev_alloc_words = allocated_words;
+  long prev_alloc_words = caml_allocated_words;
 
-  empty_minor_heap ();
+  caml_empty_minor_heap ();
 
-  stat_promoted_words += allocated_words - prev_alloc_words;
-  ++ stat_minor_collections;
-  major_collection_slice (0);
-  force_major_slice = 0;
+  caml_stat_promoted_words += caml_allocated_words - prev_alloc_words;
+  ++ caml_stat_minor_collections;
+  caml_major_collection_slice (0);
+  caml_force_major_slice = 0;
 
-  final_do_calls ();
+  caml_final_do_calls ();
 
-  empty_minor_heap ();
+  caml_empty_minor_heap ();
 }
 
-value check_urgent_gc (value extra_root)
+CAMLexport value caml_check_urgent_gc (value extra_root)
 {
   CAMLparam1 (extra_root);
-  if (force_major_slice) minor_collection();
+  if (caml_force_major_slice) caml_minor_collection();
   CAMLreturn (extra_root);
 }
 
-void realloc_ref_table (void)
-{                                 Assert (ref_table_ptr == ref_table_limit);
-                                  Assert (ref_table_limit <= ref_table_end);
-                            Assert (ref_table_limit >= ref_table_threshold);
+void caml_realloc_ref_table (void)
+{                       Assert (caml_ref_table_ptr == caml_ref_table_limit);
+                             Assert (caml_ref_table_limit <= ref_table_end);
+                       Assert (caml_ref_table_limit >= ref_table_threshold);
 
-  if (ref_table_limit == ref_table_threshold){
-    gc_message (0x08, "ref_table threshold crossed\n", 0);
-    ref_table_limit = ref_table_end;
-    urge_major_slice ();
+  if (caml_ref_table_limit == ref_table_threshold){
+    caml_gc_message (0x08, "ref_table threshold crossed\n", 0);
+    caml_ref_table_limit = ref_table_end;
+    caml_urge_major_slice ();
   }else{ /* This will almost never happen with the bytecode interpreter. */
     asize_t sz;
-    asize_t cur_ptr = ref_table_ptr - ref_table;
-                                                  Assert (force_major_slice);
+    asize_t cur_ptr = caml_ref_table_ptr - ref_table;
+                                             Assert (caml_force_major_slice);
 
     ref_table_size *= 2;
     sz = (ref_table_size + ref_table_reserve) * sizeof (value *);
-    gc_message (0x08, "Growing ref_table to %ldk bytes\n", (long) sz / 1024);
+    caml_gc_message (0x08, "Growing ref_table to %ldk bytes\n", (long) sz/1024);
     ref_table = (value **) realloc ((char *) ref_table, sz);
-    if (ref_table == NULL) fatal_error ("Fatal error: ref_table overflow\n");
+    if (ref_table == NULL){
+      caml_fatal_error ("Fatal error: ref_table overflow\n");
+    }
     ref_table_end = ref_table + ref_table_size + ref_table_reserve;
     ref_table_threshold = ref_table + ref_table_size;
-    ref_table_ptr = ref_table + cur_ptr;
-    ref_table_limit = ref_table_end;
+    caml_ref_table_ptr = ref_table + cur_ptr;
+    caml_ref_table_limit = ref_table_end;
   }
 }
