@@ -6,32 +6,33 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../LICENSE.     */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: roots.h,v 1.14 2000/02/14 14:30:40 weis Exp $ */
+/* $Id: roots.h,v 1.18 2004/01/01 16:42:37 doligez Exp $ */
 
-#ifndef _roots_
-#define _roots_
+#ifndef CAML_ROOTS_H
+#define CAML_ROOTS_H
 
 #include "misc.h"
 #include "memory.h"
 
 typedef void (*scanning_action) (value, value *);
 
-void oldify_local_roots (void);
-void darken_all_roots (void);
-void do_roots (scanning_action);
+void caml_oldify_local_roots (void);
+void caml_darken_all_roots (void);
+void caml_do_roots (scanning_action);
 #ifndef NATIVE_CODE
-void do_local_roots (scanning_action, value *, value *,
-                     struct caml__roots_block *);
+CAMLextern void caml_do_local_roots (scanning_action, value *, value *,
+                                     struct caml__roots_block *);
 #else
-void do_local_roots(scanning_action f, char * bottom_of_stack,
-                    unsigned long last_retaddr, value * gc_regs,
-                    struct caml__roots_block * local_roots);
+CAMLextern void caml_do_local_roots(scanning_action f, char * bottom_of_stack,
+                                    unsigned long last_retaddr, value * gc_regs,
+                                    struct caml__roots_block * local_roots);
 #endif
 
-extern void (*scan_roots_hook) (scanning_action);
+CAMLextern void (*caml_scan_roots_hook) (scanning_action);
 
-#endif /* _roots_ */
+#endif /* CAML_ROOTS_H */
