@@ -11,7 +11,7 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: startup.c,v 1.54 2002/06/05 12:26:08 doligez Exp $ */
+/* $Id: startup.c,v 1.64.4.1 2004/07/03 10:01:00 doligez Exp $ */
 
 /* Start-up code */
 
@@ -61,16 +61,16 @@
 #define SEEK_END 2
 #endif
 
-extern int parser_trace;
+extern int caml_parser_trace;
 
-CAMLexport header_t atom_table[256];
+CAMLexport header_t caml_atom_table[256];
 
 /* Initialize the atom table */
 
 static void init_atoms(void)
 {
   int i;
-  for(i = 0; i < 256; i++) atom_table[i] = Make_header(0, i, Caml_white);
+  for(i = 0; i < 256; i++) caml_atom_table[i] = Make_header(0, i, Caml_white);
 }
 
 /* Read the trailer of a bytecode file */
@@ -94,35 +94,35 @@ static int read_trailer(int fd, struct exec_trailer *trail)
     return BAD_BYTECODE;
 }
 
-int attempt_open(char **name, struct exec_trailer *trail,
-                 int do_open_script)
+int caml_attempt_open(char **name, struct exec_trailer *trail,
+                      int do_open_script)
 {
   char * truename;
   int fd;
   int err;
   char buf [2];
 
-  truename = search_exe_in_path(*name);
+  truename = caml_search_exe_in_path(*name);
   *name = truename;
-  gc_message(0x100, "Opening bytecode executable %s\n",
-             (unsigned long) truename);
+  caml_gc_message(0x100, "Opening bytecode executable %s\n",
+                  (unsigned long) truename);
   fd = open(truename, O_RDONLY | O_BINARY);
   if (fd == -1) {
-    gc_message(0x100, "Cannot open file\n", 0);
+    caml_gc_message(0x100, "Cannot open file\n", 0);
     return FILE_NOT_FOUND;
   }
   if (!do_open_script) {
     err = read (fd, buf, 2);
     if (err < 2 || (buf [0] == '#' && buf [1] == '!')) {
       close(fd);
-      gc_message(0x100, "Rejected #! script\n", 0);
+      caml_gc_message(0x100, "Rejected #! script\n", 0);
       return BAD_BYTECODE;
     }
   }
   err = read_trailer(fd, trail);
   if (err != 0) {
     close(fd);
-    gc_message(0x100, "Not a bytecode executable\n", 0);
+    caml_gc_message(0x100, "Not a bytecode executable\n", 0);
     return err;
   }
   return fd;
@@ -130,15 +130,15 @@ int attempt_open(char **name, struct exec_trailer *trail,
 
 /* Read the section descriptors */
 
-void read_section_descriptors(int fd, struct exec_trailer *trail)
+void caml_read_section_descriptors(int fd, struct exec_trailer *trail)
 {
   int toc_size, i;
 
   toc_size = trail->num_sections * 8;
-  trail->section = stat_alloc(toc_size);
+  trail->section = caml_stat_alloc(toc_size);
   lseek(fd, - (long) (TRAILER_SIZE + toc_size), SEEK_END);
   if (read(fd, (char *) trail->section, toc_size) != toc_size)
-    fatal_error("Fatal error: cannot read section table\n");
+    caml_fatal_error("Fatal error: cannot read section table\n");
   /* Fixup endianness of lengths */
   for (i = 0; i < trail->num_sections; i++)
     fixup_endianness_trailer(&(trail->section[i].len));
@@ -148,7 +148,7 @@ void read_section_descriptors(int fd, struct exec_trailer *trail)
    Return the length of the section data in bytes, or -1 if no section
    found with that name. */
 
-int32 seek_optional_section(int fd, struct exec_trailer *trail, char *name)
+int32 caml_seek_optional_section(int fd, struct exec_trailer *trail, char *name)
 {
   long ofs;
   int i;
@@ -167,11 +167,11 @@ int32 seek_optional_section(int fd, struct exec_trailer *trail, char *name)
 /* Position fd at the beginning of the section having the given name.
    Return the length of the section data in bytes. */
 
-int32 seek_section(int fd, struct exec_trailer *trail, char *name)
+int32 caml_seek_section(int fd, struct exec_trailer *trail, char *name)
 {
-  int32 len = seek_optional_section(fd, trail, name);
+  int32 len = caml_seek_optional_section(fd, trail, name);
   if (len == -1) 
-    fatal_error_arg("Fatal_error: section `%s' is missing\n", name);
+    caml_fatal_error_arg("Fatal_error: section `%s' is missing\n", name);
   return len;
 }
 
@@ -183,11 +183,11 @@ static char * read_section(int fd, struct exec_trailer *trail, char *name)
   int32 len;
   char * data;
 
-  len = seek_optional_section(fd, trail, name);
+  len = caml_seek_optional_section(fd, trail, name);
   if (len == -1) return NULL;
-  data = stat_alloc(len + 1);
+  data = caml_stat_alloc(len + 1);
   if (read(fd, data, len) != len)
-    fatal_error_arg("Fatal error: error reading section %s\n", name);
+    caml_fatal_error_arg("Fatal error: error reading section %s\n", name);
   data[len] = 0;
   return data;
 }
@@ -236,34 +236,34 @@ static int parse_command_line(char **argv)
     switch(argv[i][1]) {
 #ifdef DEBUG
     case 't':
-      trace_flag = 1;
+      caml_trace_flag++;
       break;
 #endif
     case 'v':
-      verb_gc = 0x001+0x004+0x008+0x010+0x020;
+      caml_verb_gc = 0x001+0x004+0x008+0x010+0x020;
       break;
     case 'p':
-      for (j = 0; names_of_builtin_cprim[j] != NULL; j++)
-        printf("%s\n", names_of_builtin_cprim[j]);
+      for (j = 0; caml_names_of_builtin_cprim[j] != NULL; j++)
+        printf("%s\n", caml_names_of_builtin_cprim[j]);
       exit(0);
       break;
     case 'b':
-      init_backtrace();
+      caml_init_backtrace();
       break;
     case 'I':
       if (argv[i + 1] != NULL) {
-        ext_table_add(&shared_libs_path, argv[i + 1]);
+        caml_ext_table_add(&caml_shared_libs_path, argv[i + 1]);
         i++;
       }
       break;
     default:
-      fatal_error_arg("Unknown option %s.\n", argv[i]);
+      caml_fatal_error_arg("Unknown option %s.\n", argv[i]);
     }
   }
   return i;
 }
 
-/* Parse the CAMLRUNPARAM variable */
+/* Parse the OCAMLRUNPARAM variable */
 /* The option letter for each runtime option is the first letter of the
    last word of the ML name of the option (see [stdlib/gc.mli]).
    Except for l (maximum stack size) and h (initial heap size).
@@ -296,15 +296,15 @@ static void parse_camlrunparam(void)
       case 'l': scanmult (opt, &max_stack_init); break;
       case 'o': scanmult (opt, &percent_free_init); break;
       case 'O': scanmult (opt, &max_percent_free_init); break;
-      case 'v': scanmult (opt, &verb_gc); break;
-      case 'b': init_backtrace(); break;
-      case 'p': parser_trace = 1; break;
+      case 'v': scanmult (opt, &caml_verb_gc); break;
+      case 'b': caml_init_backtrace(); break;
+      case 'p': caml_parser_trace = 1; break;
       }
     }
   }
 }
 
-extern void init_ieee_floats (void);
+extern void caml_init_ieee_floats (void);
 
 #ifdef _WIN32
 extern void caml_signal_thread(void * lpParam);
@@ -322,139 +322,141 @@ CAMLexport void caml_main(char **argv)
   char * exe_name;
 #ifdef __linux__
   static char proc_self_exe[256];
-  int retcode;
 #endif
 
   /* Machine-dependent initialization of the floating-point hardware
      so that it behaves as much as possible as specified in IEEE */
-  init_ieee_floats();
-  init_custom_operations();
-  ext_table_init(&shared_libs_path, 8);
-  external_raise = NULL;
+  caml_init_ieee_floats();
+  caml_init_custom_operations();
+  caml_ext_table_init(&caml_shared_libs_path, 8);
+  caml_external_raise = NULL;
   /* Determine options and position of bytecode file */
 #ifdef DEBUG
-  verb_gc = 63;
+  caml_verb_gc = 0xBF;
 #endif
   parse_camlrunparam();
   pos = 0;
   exe_name = argv[0];
 #ifdef __linux__
-  /* Recover executable name from /proc/self/exe, much more reliable */
-  retcode = readlink("/proc/self/exe", proc_self_exe, sizeof(proc_self_exe));
-  if (retcode != -1 && retcode < sizeof(proc_self_exe)) {
-    proc_self_exe[retcode] = 0;
+  if (caml_executable_name(proc_self_exe, sizeof(proc_self_exe)) == 0)
     exe_name = proc_self_exe;
-  }
 #endif
-  fd = attempt_open(&exe_name, &trail, 0);
+  fd = caml_attempt_open(&exe_name, &trail, 0);
   if (fd < 0) {
     pos = parse_command_line(argv);
     if (argv[pos] == 0)
-      fatal_error("No bytecode file specified.\n");
+      caml_fatal_error("No bytecode file specified.\n");
     exe_name = argv[pos];
-    fd = attempt_open(&exe_name, &trail, 1);
+    fd = caml_attempt_open(&exe_name, &trail, 1);
     switch(fd) {
     case FILE_NOT_FOUND:
-      fatal_error_arg("Fatal error: cannot find file %s\n", argv[pos]);
+      caml_fatal_error_arg("Fatal error: cannot find file %s\n", argv[pos]);
       break;
     case BAD_BYTECODE:
-      fatal_error_arg(
+      caml_fatal_error_arg(
         "Fatal error: the file %s is not a bytecode executable file\n",
         argv[pos]);
       break;
     }
   }
   /* Read the table of contents (section descriptors) */
-  read_section_descriptors(fd, &trail);
+  caml_read_section_descriptors(fd, &trail);
   /* Initialize the abstract machine */
-  init_gc (minor_heap_init, heap_size_init, heap_chunk_init,
-           percent_free_init, max_percent_free_init);
-  init_stack (max_stack_init);
+  caml_init_gc (minor_heap_init, heap_size_init, heap_chunk_init,
+                percent_free_init, max_percent_free_init);
+  caml_init_stack (max_stack_init);
   init_atoms();
   /* Initialize the interpreter */
-  interprete(NULL, 0);
+  caml_interprete(NULL, 0);
   /* Initialize the debugger, if needed */
-  debugger_init();
+  caml_debugger_init();
   /* Load the code */
-  code_size = seek_section(fd, &trail, "CODE");
-  load_code(fd, code_size);
+  caml_code_size = caml_seek_section(fd, &trail, "CODE");
+  caml_load_code(fd, caml_code_size);
   /* Build the table of primitives */
   shared_lib_path = read_section(fd, &trail, "DLPT");
   shared_libs = read_section(fd, &trail, "DLLS");
   req_prims = read_section(fd, &trail, "PRIM");
-  if (req_prims == NULL) fatal_error("Fatal error: no PRIM section\n");
-  build_primitive_table(shared_lib_path, shared_libs, req_prims);
-  stat_free(shared_lib_path);
-  stat_free(shared_libs);
-  stat_free(req_prims);
+  if (req_prims == NULL) caml_fatal_error("Fatal error: no PRIM section\n");
+  caml_build_primitive_table(shared_lib_path, shared_libs, req_prims);
+  caml_stat_free(shared_lib_path);
+  caml_stat_free(shared_libs);
+  caml_stat_free(req_prims);
   /* Load the globals */
-  seek_section(fd, &trail, "DATA");
-  chan = open_descriptor_in(fd);
-  global_data = input_val(chan);
-  close_channel(chan); /* this also closes fd */
-  stat_free(trail.section);
+  caml_seek_section(fd, &trail, "DATA");
+  chan = caml_open_descriptor_in(fd);
+  caml_global_data = caml_input_val(chan);
+  caml_close_channel(chan); /* this also closes fd */
+  caml_stat_free(trail.section);
   /* Ensure that the globals are in the major heap. */
-  oldify_one (global_data, &global_data);
-  oldify_mopup ();
+  caml_oldify_one (caml_global_data, &caml_global_data);
+  caml_oldify_mopup ();
   /* Initialize system libraries */
-  init_exceptions();
-  sys_init(exe_name, argv + pos);
+  caml_init_exceptions();
+  caml_sys_init(exe_name, argv + pos);
 #ifdef _WIN32
   /* Start a thread to handle signals */
   if (getenv("CAMLSIGPIPE"))
     _beginthread(caml_signal_thread, 4096, NULL);
 #endif
   /* Execute the program */
-  debugger(PROGRAM_START);
-  res = interprete(start_code, code_size);
+  caml_debugger(PROGRAM_START);
+  res = caml_interprete(caml_start_code, caml_code_size);
   if (Is_exception_result(res)) {
-    exn_bucket = Extract_exception(res);
-    if (debugger_in_use) {
-      extern_sp = &exn_bucket; /* The debugger needs the exception value. */
-      debugger(UNCAUGHT_EXC);
+    caml_exn_bucket = Extract_exception(res);
+    if (caml_debugger_in_use) {
+      caml_extern_sp = &caml_exn_bucket; /* The debugger needs the
+                                            exception value.*/
+      caml_debugger(UNCAUGHT_EXC);
     }
-    fatal_uncaught_exception(exn_bucket);
+    caml_fatal_uncaught_exception(caml_exn_bucket);
   }
 }
 
 /* Main entry point when code is linked in as initialized data */
 
-CAMLexport void caml_startup_code(code_t code, asize_t code_size,
-                                  char *data, char **argv)
+CAMLexport void caml_startup_code(
+           code_t code, asize_t code_size,
+           char *data, asize_t data_size,
+           char *section_table, asize_t section_table_size,
+           char **argv)
 {
   value res;
 
-  init_ieee_floats();
+  caml_init_ieee_floats();
+  caml_init_custom_operations();
 #ifdef DEBUG
-  verb_gc = 63;
+  caml_verb_gc = 63;
 #endif
   parse_camlrunparam();
-  external_raise = NULL;
+  caml_external_raise = NULL;
   /* Initialize the abstract machine */
-  init_gc (minor_heap_init, heap_size_init, heap_chunk_init,
-           percent_free_init, max_percent_free_init);
-  init_stack (max_stack_init);
+  caml_init_gc (minor_heap_init, heap_size_init, heap_chunk_init,
+                percent_free_init, max_percent_free_init);
+  caml_init_stack (max_stack_init);
   init_atoms();
   /* Initialize the interpreter */
-  interprete(NULL, 0);
+  caml_interprete(NULL, 0);
   /* Load the code */
-  start_code = code;
+  caml_start_code = code;
 #ifdef THREADED_CODE
-  thread_code(start_code, code_size);
+  caml_thread_code(caml_start_code, code_size);
 #endif
   /* Use the builtin table of primitives */
-  prim_table.size = prim_table.capacity = -1;
-  prim_table.contents = (void **) builtin_cprim;
+  caml_build_primitive_table_builtin();
   /* Load the globals */
-  global_data = input_val_from_string((value)data, 0);
+  caml_global_data = caml_input_value_from_block(data, data_size);
   /* Ensure that the globals are in the major heap. */
-  oldify_one (global_data, &global_data);
-  oldify_mopup ();
+  caml_oldify_one (caml_global_data, &caml_global_data);
+  caml_oldify_mopup ();
+  /* Record the sections (for caml_get_section_table in meta.c) */
+  caml_section_table = section_table;
+  caml_section_table_size = section_table_size;
   /* Run the code */
-  init_exceptions();
-  sys_init("", argv);
-  res = interprete(start_code, code_size);
+  caml_init_exceptions();
+  caml_sys_init("", argv);
+  res = caml_interprete(caml_start_code, code_size);
   if (Is_exception_result(res))
-    fatal_uncaught_exception(Extract_exception(res));
+    caml_fatal_uncaught_exception(Extract_exception(res));
 }
 
