@@ -6,14 +6,21 @@
 /*                                                                     */
 /*  Copyright 1997 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../LICENSE.     */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: weak.h,v 1.3 1999/11/17 18:57:15 xleroy Exp $ */
+/* $Id: weak.h,v 1.7 2004/01/01 16:42:38 doligez Exp $ */
 
 /* Operations on weak arrays */
 
+#ifndef CAML_WEAK_H
+#define CAML_WEAK_H
+
 #include "mlvalues.h"
 
-extern value weak_list_head;
+extern value caml_weak_list_head;
+extern value caml_weak_none;
+
+#endif /* CAML_WEAK_H */
