@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: ast2pt.ml,v 1.15 2002/07/19 14:53:43 mauny Exp $ *)
+(* $Id: ast2pt.ml,v 1.31 2004/05/25 11:38:31 mauny Exp $ *)
 
 open Stdpp;
 open MLast;
@@ -19,7 +19,7 @@ open Longident;
 open Asttypes;
 
 value fast = ref False;
-value no_constructors_arity = ref False;
+value no_constructors_arity = Pcaml.no_constructors_arity;
 
 value get_tag x =
   if Obj.is_block (Obj.repr x) then Obj.tag (Obj.repr x) else Obj.magic x
@@ -32,15 +32,52 @@ value char_of_char_token loc s =
 ;
 
 value string_of_string_token loc s =
-  try Token.eval_string s with [ Failure _ as exn -> raise_with_loc loc exn ]
+  try Token.eval_string loc s
+  with [ Failure _ as exn -> raise_with_loc loc exn ]
 ;
 
+value glob_fname = ref "";
+
 value mkloc (bp, ep) =
-  {Location.loc_start = bp; Location.loc_end = ep; Location.loc_ghost = False}
+  let loc_at n = 
+    { (n) with
+        Lexing.pos_fname = 
+          if n.Lexing.pos_fname = "" then
+            if glob_fname.val = "" then
+              Pcaml.input_file.val
+            else
+              glob_fname.val
+          else
+            n.Lexing.pos_fname
+      }
+  in
+  {Location.loc_start = loc_at bp;
+   Location.loc_end = loc_at ep;
+   Location.loc_ghost =
+     bp.Lexing.pos_cnum = 0 && ep.Lexing.pos_cnum = 0}
+;
+
+value mkghloc (bp, ep) =
+  let loc_at n = 
+    { (n) with
+        Lexing.pos_fname = 
+          if n.Lexing.pos_fname = "" then
+            if glob_fname.val = "" then
+              Pcaml.input_file.val
+            else
+              glob_fname.val
+          else
+            n.Lexing.pos_fname
+      }
+  in
+  {Location.loc_start = loc_at bp;
+   Location.loc_end = loc_at ep;
+   Location.loc_ghost = True}
 ;
 
 value mktyp loc d = {ptyp_desc = d; ptyp_loc = mkloc loc};
 value mkpat loc d = {ppat_desc = d; ppat_loc = mkloc loc};
+value mkghpat loc d = {ppat_desc = d; ppat_loc = mkghloc loc};
 value mkexp loc d = {pexp_desc = d; pexp_loc = mkloc loc};
 value mkmty loc d = {pmty_desc = d; pmty_loc = mkloc loc};
 value mksig loc d = {psig_desc = d; psig_loc = mkloc loc};
@@ -105,22 +142,37 @@ value rec ctyp_fa al =
   | f -> (f, al) ]
 ;
 
-value rec ctyp_long_id =
-  fun
+value rec ctyp_long_id_prefix t =
+  match t with
   [ TyAcc _ m (TyLid _ s) ->
-      let (is_cls, li) = ctyp_long_id m in
-      (is_cls, ldot li s)
+      error (loc_of_ctyp t) "invalid module expression"
   | TyAcc _ m (TyUid _ s) ->
-      let (is_cls, li) = ctyp_long_id m in
+      let (is_cls, li) = ctyp_long_id_prefix m in
       (is_cls, ldot li s)
   | TyApp _ m1 m2 ->
-      let (is_cls, li1) = ctyp_long_id m1 in
-      let (_, li2) = ctyp_long_id m2 in
+      let (is_cls, li1) = ctyp_long_id_prefix m1 in
+      let (_, li2) = ctyp_long_id_prefix m2 in
       (is_cls, Lapply li1 li2)
   | TyUid _ s -> (False, lident s)
+  | TyLid _ s ->
+      error (loc_of_ctyp t) "invalid module expression"
+  | t -> error (loc_of_ctyp t) "invalid module expression" ]
+;
+
+value ctyp_long_id t =
+  match t with
+  [ TyAcc _ m (TyLid _ s) ->
+      let (is_cls, li) = ctyp_long_id_prefix m in
+      (is_cls, ldot li s)
+  | TyAcc _ m (TyUid _ s as t) ->
+      error (loc_of_ctyp t) "invalid type name"
+  | TyApp _ m1 m2 ->
+      error (loc_of_ctyp t) "invalid type name"
+  | TyUid _ s ->
+      error (loc_of_ctyp t) "invalid type name"
   | TyLid _ s -> (False, lident s)
   | TyCls loc sl -> (True, long_id_of_string_list loc sl)
-  | t -> error (loc_of_ctyp t) "incorrect type" ]
+  | t -> error (loc_of_ctyp t) "invalid type" ]
 ;
 
 value rec ctyp =
@@ -134,7 +186,7 @@ value rec ctyp =
         match (t1, t2) with
         [ (t, TyQuo _ s) -> (t, s)
         | (TyQuo _ s, t) -> (t, s)
-        | _ -> error loc "incorrect alias type" ]
+        | _ -> error loc "invalid alias type" ]
       in
       mktyp loc (Ptyp_alias (ctyp t) i)
   | TyAny loc -> mktyp loc Ptyp_any
@@ -152,16 +204,16 @@ value rec ctyp =
   | TyObj loc fl v -> mktyp loc (Ptyp_object (meth_list loc fl v))
   | TyCls loc id ->
       mktyp loc (Ptyp_class (long_id_of_string_list loc id) [] [])
-  | TyLab loc _ _ -> error loc "labeled type not allowed here"
+  | TyLab loc _ _ -> error loc "labelled type not allowed here"
   | TyLid loc s -> mktyp loc (Ptyp_constr (lident s) [])
-  | TyMan loc _ _ -> error loc "type manifest not allowed here"
-  | TyOlb loc lab _ -> error loc "labeled type not allowed here"
+  | TyMan loc _ _ -> error loc "manifest type not allowed here"
+  | TyOlb loc lab _ -> error loc "labelled type not allowed here"
   | TyPol loc pl t -> mktyp loc (Ptyp_poly pl (ctyp t))
   | TyQuo loc s -> mktyp loc (Ptyp_var s)
-  | TyRec loc _ -> error loc "record type not allowed here"
-  | TySum loc _ -> error loc "sum type not allowed here"
+  | TyRec loc _ _ -> error loc "record type not allowed here"
+  | TySum loc _ _ -> error loc "sum type not allowed here"
   | TyTup loc tl -> mktyp loc (Ptyp_tuple (List.map ctyp tl))
-  | TyUid loc s -> mktyp loc (Ptyp_constr (lident s) [])
+  | TyUid loc s as t -> error (loc_of_ctyp t) "invalid type"
   | TyVrn loc catl ool ->
       let catl =
         List.map
@@ -195,15 +247,16 @@ value mktrecord (_, n, m, t) = (n, mkmutable m, ctyp (mkpolytype t));
 value mkvariant (_, c, tl) = (c, List.map ctyp tl);
 value type_decl tl cl =
   fun
-  [ TyMan loc t (TyRec _ ltl) ->
-      mktype loc tl cl (Ptype_record (List.map mktrecord ltl)) (Some (ctyp t))
-  | TyMan loc t (TySum _ ctl) ->
-      mktype loc tl cl (Ptype_variant (List.map mkvariant ctl))
+  [ TyMan loc t (TyRec _ pflag ltl) ->
+      mktype loc tl cl (Ptype_record (List.map mktrecord ltl) (mkprivate pflag))
         (Some (ctyp t))
-  | TyRec loc ltl ->
-      mktype loc tl cl (Ptype_record (List.map mktrecord ltl)) None
-  | TySum loc ctl ->
-      mktype loc tl cl (Ptype_variant (List.map mkvariant ctl)) None
+  | TyMan loc t (TySum _ pflag ctl) ->
+      mktype loc tl cl (Ptype_variant (List.map mkvariant ctl) (mkprivate pflag))
+        (Some (ctyp t))
+  | TyRec loc pflag ltl ->
+      mktype loc tl cl (Ptype_record (List.map mktrecord ltl) (mkprivate pflag)) None
+  | TySum loc pflag ctl ->
+      mktype loc tl cl (Ptype_variant (List.map mkvariant ctl) (mkprivate pflag)) None
   | t ->
       let m =
         match t with
@@ -221,6 +274,33 @@ value option f =
   | None -> None ]
 ;
 
+value expr_of_lab loc lab =
+  fun
+  [ Some e -> e
+  | None -> ExLid loc lab ]
+;
+
+value patt_of_lab loc lab =
+  fun
+  [ Some p -> p
+  | None -> PaLid loc lab ]
+;
+
+value paolab loc lab peoo =
+  let lab =
+    match (lab, peoo) with
+    [ ("", Some (PaLid _ i | PaTyc _ (PaLid _ i) _, _)) -> i
+    | ("", _) -> error loc "bad ast"
+    | _ -> lab ]
+  in
+  let (p, eo) =
+    match peoo with
+    [ Some peo -> peo
+    | None -> (PaLid loc lab, None) ]
+  in
+  (lab, p, eo)
+;
+
 value rec same_type_expr ct ce =
   match (ct, ce) with
   [ (TyLid _ s1, ExLid _ s2) -> s1 = s2
@@ -286,13 +366,21 @@ value rec patt_fa al =
   | f -> (f, al) ]
 ;
 
+value rec deep_mkrangepat loc c1 c2 =
+  if c1 = c2 then mkghpat loc (Ppat_constant (Const_char c1))
+  else
+    mkghpat loc
+      (Ppat_or (mkghpat loc (Ppat_constant (Const_char c1)))
+         (deep_mkrangepat loc (Char.chr (Char.code c1 + 1)) c2))
+;
+
 value rec mkrangepat loc c1 c2 =
   if c1 > c2 then mkrangepat loc c2 c1
   else if c1 = c2 then mkpat loc (Ppat_constant (Const_char c1))
   else
     mkpat loc
-      (Ppat_or (mkpat loc (Ppat_constant (Const_char c1)))
-         (mkrangepat loc (Char.chr (Char.code c1 + 1)) c2))
+      (Ppat_or (mkghpat loc (Ppat_constant (Const_char c1)))
+         (deep_mkrangepat loc (Char.chr (Char.code c1 + 1)) c2))
 ;
 
 value rec patt_long_id il =
@@ -329,7 +417,7 @@ value rec patt =
         match (p1, p2) with
         [ (p, PaLid _ s) -> (p, s)
         | (PaLid _ s, p) -> (p, s)
-        | _ -> error loc "incorrect alias pattern" ]
+        | _ -> error loc "invalid alias pattern" ]
       in
       mkpat loc (Ppat_alias (patt p) i)
   | PaAnt _ p -> patt p
@@ -363,10 +451,13 @@ value rec patt =
   | PaChr loc s ->
       mkpat loc (Ppat_constant (Const_char (char_of_char_token loc s)))
   | PaInt loc s -> mkpat loc (Ppat_constant (Const_int (int_of_string s)))
+  | PaInt32 loc s -> mkpat loc (Ppat_constant (Const_int32 (Int32.of_string s)))
+  | PaInt64 loc s -> mkpat loc (Ppat_constant (Const_int64 (Int64.of_string s)))
+  | PaNativeInt loc s -> mkpat loc (Ppat_constant (Const_nativeint (Nativeint.of_string s)))
   | PaFlo loc s -> mkpat loc (Ppat_constant (Const_float s))
   | PaLab loc _ _ -> error loc "labeled pattern not allowed here"
   | PaLid loc s -> mkpat loc (Ppat_var s)
-  | PaOlb loc _ _ _ -> error loc "labeled pattern not allowed here"
+  | PaOlb loc _ _ -> error loc "labeled pattern not allowed here"
   | PaOrp loc p1 p2 -> mkpat loc (Ppat_or (patt p1) (patt p2))
   | PaRng loc p1 p2 ->
       match (p1, p2) with
@@ -425,6 +516,18 @@ value class_info class_expr ci =
    pci_variance = variance}
 ;
 
+value apply_with_var v x f =
+  let vx = v.val in
+  try
+    do {
+      v.val := x;
+      let r = f ();
+      v.val := vx;
+      r
+    }
+  with e -> do { v.val := vx; raise e }
+;
+
 value rec expr =
   fun
   [ ExAcc loc x (ExLid _ "val") ->
@@ -483,6 +586,7 @@ value rec expr =
         (Pexp_apply (mkexp loc (Pexp_ident (array_function "Array" "get")))
            [("", expr e1); ("", expr e2)])
   | ExArr loc el -> mkexp loc (Pexp_array (List.map expr el))
+  | ExAsf loc -> mkexp loc Pexp_assertfalse
   | ExAss loc e v ->
       let e =
         match e with
@@ -504,6 +608,7 @@ value rec expr =
         | _ -> error loc "bad left part of assignment" ]
       in
       mkexp loc e
+  | ExAsr loc e -> mkexp loc (Pexp_assert (expr e))
   | ExChr loc s ->
       mkexp loc (Pexp_constant (Const_char (char_of_char_token loc s)))
   | ExCoe loc e t1 t2 ->
@@ -513,15 +618,21 @@ value rec expr =
       let e3 = ExSeq loc el in
       let df = if df then Upto else Downto in
       mkexp loc (Pexp_for i (expr e1) (expr e2) df (expr e3))
-  | ExFun loc [(PaLab _ lab p, w, e)] ->
-      mkexp loc (Pexp_function lab None [(patt p, when_expr e w)])
-  | ExFun loc [(PaOlb _ lab p eo, w, e)] ->
+  | ExFun loc [(PaLab _ lab po, w, e)] ->
+      mkexp loc
+        (Pexp_function lab None
+           [(patt (patt_of_lab loc lab po), when_expr e w)])
+  | ExFun loc [(PaOlb _ lab peoo, w, e)] ->
+      let (lab, p, eo) = paolab loc lab peoo in
       mkexp loc
         (Pexp_function ("?" ^ lab) (option expr eo) [(patt p, when_expr e w)])
   | ExFun loc pel -> mkexp loc (Pexp_function "" None (List.map mkpwe pel))
   | ExIfe loc e1 e2 e3 ->
       mkexp loc (Pexp_ifthenelse (expr e1) (expr e2) (Some (expr e3)))
   | ExInt loc s -> mkexp loc (Pexp_constant (Const_int (int_of_string s)))
+  | ExInt32 loc s -> mkexp loc (Pexp_constant (Const_int32 (Int32.of_string s)))
+  | ExInt64 loc s -> mkexp loc (Pexp_constant (Const_int64 (Int64.of_string s)))
+  | ExNativeInt loc s -> mkexp loc (Pexp_constant (Const_nativeint (Nativeint.of_string s)))
   | ExLab loc _ _ -> error loc "labeled expression not allowed here"
   | ExLaz loc e -> mkexp loc (Pexp_lazy (expr e))
   | ExLet loc rf pel e ->
@@ -530,15 +641,25 @@ value rec expr =
   | ExLmd loc i me e -> mkexp loc (Pexp_letmodule i (module_expr me) (expr e))
   | ExMat loc e pel -> mkexp loc (Pexp_match (expr e) (List.map mkpwe pel))
   | ExNew loc id -> mkexp loc (Pexp_new (long_id_of_string_list loc id))
+  | ExObj loc po cfl -> 
+      let p =
+        match po with
+        [ Some p -> p
+        | None -> PaAny loc ]
+      in
+      let cil = List.fold_right class_str_item cfl [] in
+      mkexp loc (Pexp_object (patt p, cil))
   | ExOlb loc _ _ -> error loc "labeled expression not allowed here"
   | ExOvr loc iel -> mkexp loc (Pexp_override (List.map mkideexp iel))
   | ExRec loc lel eo ->
-      let eo =
-        match eo with
-        [ Some e -> Some (expr e)
-        | None -> None ]
-      in
-      mkexp loc (Pexp_record (List.map mklabexp lel) eo)
+      if lel = [] then error loc "empty record"
+      else
+        let eo =
+          match eo with
+          [ Some e -> Some (expr e)
+          | None -> None ]
+        in
+        mkexp loc (Pexp_record (List.map mklabexp lel) eo)
   | ExSeq loc el ->
       let rec loop =
         fun
@@ -568,8 +689,8 @@ value rec expr =
       mkexp loc (Pexp_while (expr e1) (expr e2)) ]
 and label_expr =
   fun
-  [ ExLab loc lab e -> (lab, expr e)
-  | ExOlb loc lab e -> ("?" ^ lab, expr e)
+  [ ExLab loc lab eo -> (lab, expr (expr_of_lab loc lab eo))
+  | ExOlb loc lab eo -> ("?" ^ lab, expr (expr_of_lab loc lab eo))
   | e -> ("", expr e) ]
 and mkpe (p, e) = (patt p, expr e)
 and mkpwe (p, w, e) = (patt p, when_expr e w)
@@ -614,6 +735,8 @@ and sig_item s l =
   | SgExt loc n t p -> [mksig loc (Psig_value n (mkvalue_desc t p)) :: l]
   | SgInc loc mt -> [mksig loc (Psig_include (module_type mt)) :: l]
   | SgMod loc n mt -> [mksig loc (Psig_module n (module_type mt)) :: l]
+  | SgRecMod loc nmts ->
+      [mksig loc (Psig_recmodule (List.map (fun (n,mt) -> (n, module_type mt)) nmts)) :: l]
   | SgMty loc n mt ->
       let si =
         match mt with
@@ -624,6 +747,9 @@ and sig_item s l =
   | SgOpn loc id ->
       [mksig loc (Psig_open (long_id_of_string_list loc id)) :: l]
   | SgTyp loc tdl -> [mksig loc (Psig_type (List.map mktype_decl tdl)) :: l]
+  | SgUse loc fn sl ->
+      apply_with_var glob_fname fn
+        (fun () -> List.fold_right (fun (si, _) -> sig_item si) sl l)
   | SgVal loc n t -> [mksig loc (Psig_value n (mkvalue_desc t [])) :: l] ]
 and module_expr =
   fun
@@ -658,10 +784,19 @@ and str_item s l =
   | StExt loc n t p -> [mkstr loc (Pstr_primitive n (mkvalue_desc t p)) :: l]
   | StInc loc me -> [mkstr loc (Pstr_include (module_expr me)) :: l]
   | StMod loc n me -> [mkstr loc (Pstr_module n (module_expr me)) :: l]
+  | StRecMod loc nmes ->
+      [mkstr loc
+         (Pstr_recmodule
+            (List.map
+               (fun (n,mt,me) -> (n, module_type mt, module_expr me))
+               nmes)) :: l]
   | StMty loc n mt -> [mkstr loc (Pstr_modtype n (module_type mt)) :: l]
   | StOpn loc id ->
       [mkstr loc (Pstr_open (long_id_of_string_list loc id)) :: l]
   | StTyp loc tdl -> [mkstr loc (Pstr_type (List.map mktype_decl tdl)) :: l]
+  | StUse loc fn sl ->
+      apply_with_var glob_fname fn
+        (fun () -> List.fold_right (fun (si, _) -> str_item si) sl l)
   | StVal loc rf pel ->
       [mkstr loc (Pstr_value (mkrf rf) (List.map mkpe pel)) :: l] ]
 and class_type =
@@ -703,15 +838,13 @@ and class_expr =
   | CeCon loc id tl ->
       mkpcl loc
         (Pcl_constr (long_id_of_string_list loc id) (List.map ctyp tl))
-  | CeFun loc (PaLab _ lab p) ce ->
-      mkpcl loc (Pcl_fun lab None (patt p) (class_expr ce))
-  | CeFun loc (PaOlb _ lab p eo) ce ->
-      let eo =
-        match eo with
-        [ Some e -> Some (expr e)
-        | None -> None ]
-      in
-      mkpcl loc (Pcl_fun ("?" ^ lab) eo (patt p) (class_expr ce))
+  | CeFun loc (PaLab _ lab po) ce ->
+      mkpcl loc
+        (Pcl_fun lab None (patt (patt_of_lab loc lab po)) (class_expr ce))
+  | CeFun loc (PaOlb _ lab peoo) ce ->
+      let (lab, p, eo) = paolab loc lab peoo in
+      mkpcl loc
+        (Pcl_fun ("?" ^ lab) (option expr eo) (patt p) (class_expr ce))
   | CeFun loc p ce -> mkpcl loc (Pcl_fun "" None (patt p) (class_expr ce))
   | CeLet loc rf pel ce ->
       mkpcl loc (Pcl_let (mkrf rf) (List.map mkpe pel) (class_expr ce))
