@@ -10,15 +10,18 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: pcaml.ml,v 1.8 2002/07/22 10:55:31 mauny Exp $ *)
+(* $Id: pcaml.ml,v 1.13.2.4 2004/10/07 09:18:13 mauny Exp $ *)
 
 value version = Sys.ocaml_version;
 
+value syntax_name = ref "";
+
 value gram =
   Grammar.gcreate
     {Token.tok_func _ = failwith "no loaded parsing module";
      Token.tok_using _ = (); Token.tok_removing _ = ();
-     Token.tok_match = fun []; Token.tok_text _ = ""}
+     Token.tok_match = fun []; Token.tok_text _ = "";
+     Token.tok_comm = None}
 ;
 
 value interf = Grammar.Entry.create gram "interf";
@@ -33,6 +36,7 @@ value expr = Grammar.Entry.create gram "expr";
 value patt = Grammar.Entry.create gram "patt";
 value ctyp = Grammar.Entry.create gram "type";
 value let_binding = Grammar.Entry.create gram "let_binding";
+value type_declaration = Grammar.Entry.create gram "type_declaration";
 
 value class_sig_item = Grammar.Entry.create gram "class_sig_item";
 value class_str_item = Grammar.Entry.create gram "class_str_item";
@@ -53,6 +57,27 @@ value sync = ref skip_to_eol;
 value input_file = ref "";
 value output_file = ref None;
 
+value warning_default_function (bp, ep) txt =
+   let c1 = bp.Lexing.pos_cnum - bp.Lexing.pos_bol in
+   let c2 = ep.Lexing.pos_cnum - bp.Lexing.pos_bol in
+  do { Printf.eprintf "<W> File \"%s\", line %d, chars %d-%d: %s\n"
+         bp.Lexing.pos_fname bp.Lexing.pos_lnum c1 c2 txt; flush stderr }
+;
+
+value warning = ref warning_default_function;
+
+value apply_with_var v x f =
+  let vx = v.val in
+  try
+    do {
+      v.val := x;
+      let r = f ();
+      v.val := vx;
+      r
+    }
+  with e -> do { v.val := vx; raise e }
+;
+
 List.iter (fun (n, f) -> Quotation.add n f)
   [("id", Quotation.ExStr (fun _ s -> "$0:" ^ s ^ "$"));
    ("string", Quotation.ExStr (fun _ s -> "\"" ^ String.escaped s ^ "\""))];
@@ -60,25 +85,31 @@ List.iter (fun (n, f) -> Quotation.add n f)
 value quotation_dump_file = ref (None : option string);
 
 type err_ctx =
-  [ Finding | Expanding | ParsingResult of (int * int) and string | Locating ]
+  [ Finding | Expanding | ParsingResult of MLast.loc and string | Locating ]
 ;
 exception Qerror of string and err_ctx and exn;
 
 value expand_quotation loc expander shift name str =
-  try expander str with
-  [ Stdpp.Exc_located (p1, p2) exc ->
-      let exc1 = Qerror name Expanding exc in
-      raise (Stdpp.Exc_located (shift + p1, shift + p2) exc1)
-  | exc ->
-      let exc1 = Qerror name Expanding exc in
-      raise (Stdpp.Exc_located loc exc1) ]
+  let new_warning =
+    let warn = warning.val in
+    fun (bp, ep) txt -> warn (Reloc.adjust_loc shift (bp, ep)) txt
+  in
+  apply_with_var warning new_warning
+    (fun () ->
+       try expander str with
+       [ Stdpp.Exc_located loc exc ->
+           let exc1 = Qerror name Expanding exc in
+           raise (Stdpp.Exc_located (Reloc.adjust_loc shift (Reloc.linearize loc)) exc1)
+       | exc ->
+           let exc1 = Qerror name Expanding exc in
+           raise (Stdpp.Exc_located loc exc1) ])
 ;
 
 value parse_quotation_result entry loc shift name str =
   let cs = Stream.of_string str in
   try Grammar.Entry.parse entry cs with
   [ Stdpp.Exc_located iloc (Qerror _ Locating _ as exc) ->
-      raise (Stdpp.Exc_located (shift + fst iloc, shift + snd iloc) exc)
+      raise (Stdpp.Exc_located (Reloc.adjust_loc shift iloc) exc)
   | Stdpp.Exc_located iloc (Qerror _ Expanding exc) ->
       let ctx = ParsingResult iloc str in
       let exc1 = Qerror name ctx exc in
@@ -91,18 +122,22 @@ value parse_quotation_result entry loc shift name str =
       raise (Stdpp.Exc_located loc exc1) ]
 ;
 
+value ghostify (bp, ep) =
+    let ghost p = { (p) with Lexing.pos_cnum = 0 } in
+    (ghost bp, ghost ep)
+;
+
 value handle_quotation loc proj in_expr entry reloc (name, str) =
   let shift =
     match name with
     [ "" -> String.length "<<"
     | _ -> String.length "<:" + String.length name + String.length "<" ]
   in
-  let shift = fst loc + shift in
+  let shift = Reloc.shift_pos shift (fst loc) in
   let expander =
     try Quotation.find name with exc ->
       let exc1 = Qerror name Finding exc in
-      let loc = (fst loc, shift) in
-      raise (Stdpp.Exc_located loc exc1)
+      raise (Stdpp.Exc_located (fst loc, shift) exc1)
   in
   let ast =
     match expander with
@@ -112,7 +147,14 @@ value handle_quotation loc proj in_expr entry reloc (name, str) =
     | Quotation.ExAst fe_fp ->
         expand_quotation loc (proj fe_fp) shift name str ]
   in
-  reloc (fun _ -> loc) shift ast
+  (* Warning: below, we use a side-effecting function that produces a real location
+     on its first call, and ghost ones at subsequent calls. *)
+  reloc
+    (let zero = ref None in
+     fun _ -> match zero.val with [
+       None -> do { zero.val := Some (ghostify loc) ; loc }
+     | Some x -> x ])
+    shift ast
 ;
 
 value parse_locate entry shift str =
@@ -121,12 +163,12 @@ value parse_locate entry shift str =
   [ Stdpp.Exc_located (p1, p2) exc ->
       let ctx = Locating in
       let exc1 = Qerror (Grammar.Entry.name entry) ctx exc in
-      raise (Stdpp.Exc_located (shift + p1, shift + p2) exc1) ]
+      raise (Stdpp.Exc_located (Reloc.adjust_loc shift (p1, p2)) exc1) ]
 ;
 
 value handle_locate loc entry ast_f (pos, str) =
   let s = str in
-  let loc = (pos, pos + String.length s) in
+  let loc = (pos, Reloc.shift_pos (String.length s) pos) in
   let x = parse_locate entry (fst loc) s in
   ast_f loc x
 ;
@@ -159,19 +201,33 @@ value handle_patt_locate loc x = handle_locate loc patt_eoi patt_anti x;
 value expr_reloc = Reloc.expr;
 value patt_reloc = Reloc.patt;
 
+value ctyp_reloc = Reloc.ctyp;
+value row_field_reloc = Reloc.row_field;
+value class_infos_reloc = Reloc.class_infos;
+value module_type_reloc = Reloc.module_type;
+value sig_item_reloc = Reloc.sig_item;
+value with_constr_reloc = Reloc.with_constr;
+value module_expr_reloc = Reloc.module_expr;
+value str_item_reloc = Reloc.str_item;
+value class_type_reloc = Reloc.class_type;
+value class_sig_item_reloc = Reloc.class_sig_item;
+value class_expr_reloc = Reloc.class_expr;
+value class_str_item_reloc = Reloc.class_str_item;
+
+value rename_id = ref (fun x -> x);
+
 value find_line (bp, ep) str =
-  find 0 1 0 where rec find i line col =
-    if i == String.length str then (line, 0, col)
-    else if i == bp then (line, col, col + ep - bp)
-    else if str.[i] == '\n' then find (succ i) (succ line) 0
-    else find (succ i) line (succ col)
+  (bp.Lexing.pos_lnum,
+   bp.Lexing.pos_cnum - bp.Lexing.pos_bol,
+   ep.Lexing.pos_cnum - bp.Lexing.pos_bol)
 ;
 
 value loc_fmt =
   match Sys.os_type with
   [ "MacOS" ->
-      ("File \"%s\"; line %d; characters %d to %d\n### " : format 'a 'b 'c)
-  | _ -> ("File \"%s\", line %d, characters %d-%d:\n" : format 'a 'b 'c) ]
+     format_of_string "File \"%s\"; line %d; characters %d to %d\n### "
+  | _ ->
+     format_of_string "File \"%s\", line %d, characters %d-%d:\n" ]
 ;
 
 value report_quotation_error name ctx =
@@ -250,17 +306,29 @@ value print_format str =
   do { Format.open_box 2; loop 0 0; Format.close_box () }
 ;
 
+value print_file_failed file line char =
+  do {
+    Format.print_string ", file \"";
+    Format.print_string file;
+    Format.print_string "\", line ";
+    Format.print_int line;
+    Format.print_string ", char ";
+    Format.print_int char
+  }
+;
+
 value print_exn =
   fun
   [ Out_of_memory -> Format.print_string "Out of memory\n"
-  | Match_failure (file, first_char, last_char) ->
+  | Assert_failure (file, line, char) ->
       do {
-        Format.print_string "Pattern matching failed, file ";
-        Format.print_string file;
-        Format.print_string ", chars ";
-        Format.print_int first_char;
-        Format.print_char '-';
-        Format.print_int last_char
+        Format.print_string "Assertion failed";
+        print_file_failed file line char;
+      }
+  | Match_failure (file, line, char) ->
+      do {
+        Format.print_string "Pattern matching failed";
+        print_file_failed file line char;
       }
   | Stream.Error str -> print_format ("Parse error: " ^ str)
   | Stream.Failure -> Format.print_string "Parse failure"
@@ -284,7 +352,7 @@ value print_exn =
             let arg = Obj.field (Obj.repr x) i in
             if not (Obj.is_block arg) then
               Format.print_int (Obj.magic arg : int)
-            else if Obj.tag arg = 252 then do {
+            else if Obj.tag arg = Obj.tag (Obj.repr "a") then do {
               Format.print_char '"';
               Format.print_string (Obj.magic arg : string);
               Format.print_char '"'
@@ -312,14 +380,7 @@ value report_error exn =
   | e -> print_exn exn ]
 ;
 
-value warning_default_function (bp, ep) txt =
-  do { Printf.eprintf "<W> loc %d %d: %s\n" bp ep txt; flush stderr }
-;
-
-value warning = ref warning_default_function;
-
-value no_constructors_arity = Ast2pt.no_constructors_arity;
-value no_assert = ref False;
+value no_constructors_arity = ref False;
 
 value arg_spec_list_ref = ref [];
 value arg_spec_list () = arg_spec_list_ref.val;
@@ -347,25 +408,21 @@ and kont = Stream.t pretty
 
 value pr_str_item = {pr_fun = fun []; pr_levels = []};
 value pr_sig_item = {pr_fun = fun []; pr_levels = []};
+value pr_module_type = {pr_fun = fun []; pr_levels = []};
+value pr_module_expr = {pr_fun = fun []; pr_levels = []};
 value pr_expr = {pr_fun = fun []; pr_levels = []};
 value pr_patt = {pr_fun = fun []; pr_levels = []};
 value pr_ctyp = {pr_fun = fun []; pr_levels = []};
+value pr_class_sig_item = {pr_fun = fun []; pr_levels = []};
 value pr_class_str_item = {pr_fun = fun []; pr_levels = []};
+value pr_class_type = {pr_fun = fun []; pr_levels = []};
+value pr_class_expr = {pr_fun = fun []; pr_levels = []};
 value pr_expr_fun_args = ref Extfun.empty;
 
-value not_impl name x =
-  let desc =
-    if Obj.is_block (Obj.repr x) then
-      "tag = " ^ string_of_int (Obj.tag (Obj.repr x))
-    else "int_val = " ^ string_of_int (Obj.magic x)
-  in
-  HVbox [: `S NO ("<pr_fun: not impl: " ^ name ^ "; " ^ desc ^ ">") :]
-;
-
 value pr_fun name pr lab =
   loop False pr.pr_levels where rec loop app =
     fun
-    [ [] -> fun x dg k -> not_impl name x
+    [ [] -> fun x dg k -> failwith ("unable to print " ^ name)
     | [lev :: levl] ->
         if app || lev.pr_label = lab then
           let next = loop True levl in
@@ -376,10 +433,15 @@ value pr_fun name pr lab =
 
 pr_str_item.pr_fun := pr_fun "str_item" pr_str_item;
 pr_sig_item.pr_fun := pr_fun "sig_item" pr_sig_item;
+pr_module_type.pr_fun := pr_fun "module_type" pr_module_type;
+pr_module_expr.pr_fun := pr_fun "module_expr" pr_module_expr;
 pr_expr.pr_fun := pr_fun "expr" pr_expr;
 pr_patt.pr_fun := pr_fun "patt" pr_patt;
 pr_ctyp.pr_fun := pr_fun "ctyp" pr_ctyp;
+pr_class_sig_item.pr_fun := pr_fun "class_sig_item" pr_class_sig_item;
 pr_class_str_item.pr_fun := pr_fun "class_str_item" pr_class_str_item;
+pr_class_type.pr_fun := pr_fun "class_type" pr_class_type;
+pr_class_expr.pr_fun := pr_fun "class_expr" pr_class_expr;
 
 value rec find_pr_level lab =
   fun
@@ -402,4 +464,22 @@ value top_printer pr x =
   }
 ;
 
+value buff = Buffer.create 73;
+value buffer_char = Buffer.add_char buff;
+value buffer_string = Buffer.add_string buff;
+value buffer_newline () = Buffer.add_char buff '\n';
+
+value string_of pr x =
+  do {
+    Buffer.clear buff;
+    Spretty.print_pretty buffer_char buffer_string buffer_newline "" "" 78
+      (fun _ _ -> ("", 0, 0, 0)) 0 (pr.pr_fun "top" x "" [: :]);
+    Buffer.contents buff
+  }
+;
+
 value inter_phrases = ref None;
+
+value position =
+  ref(ref 0, ref 0, ref "")
+;
