@@ -1,5 +1,5 @@
 (* camlp4r *)
-(* $Id: compile.ml,v 1.11 2002/07/19 14:53:45 mauny Exp $ *)
+(* $Id: compile.ml,v 1.13 2004/05/12 15:22:39 mauny Exp $ *)
 
 #load "q_MLast.cmo";
 
@@ -8,7 +8,10 @@ open Gramext;
 value strict_parsing = ref False;
 value keywords = ref [];
 
-value loc = (0, 0);
+value loc =
+  let nowhere = 
+    {(Lexing.dummy_pos) with Lexing.pos_lnum = 1; Lexing.pos_cnum = 0 } in
+  (nowhere,nowhere);
 
 (* Watch the segmentation faults here! the compiled file must have been
    loaded in camlp4 with the option pa_extend.cmo -meta_action. *)
@@ -101,7 +104,7 @@ value nth_patt_of_act (e, n) =
   let patt_list =
     loop e where rec loop =
       fun
-      [ <:expr< fun (loc : (int * int)) -> $_$ >> -> []
+      [ <:expr< fun (loc : (Lexing.position * Lexing.position)) -> $_$ >> -> []
       | <:expr< fun ($p$ : $_$) -> $e$ >> -> [p :: loop e]
       | <:expr< fun $p$ -> $e$ >> -> [p :: loop e]
       | _ -> failwith "nth_patt_of_act" ]
@@ -111,14 +114,14 @@ value nth_patt_of_act (e, n) =
 
 value rec last_patt_of_act =
   fun
-  [ <:expr< fun ($p$ : $_$) (loc : (int * int)) -> $_$ >> -> p
+  [ <:expr< fun ($p$ : $_$) (loc : (Lexing.position * Lexing.position)) -> $_$ >> -> p
   | <:expr< fun $_$ -> $e$ >> -> last_patt_of_act e
   | _ -> failwith "last_patt_of_act" ]
 ;
 
 value rec final_action =
   fun
-  [ <:expr< fun (loc : (int * int)) -> ($e$ : $_$) >> -> e
+  [ <:expr< fun (loc : (Lexing.position * Lexing.position)) -> ($e$ : $_$) >> -> e
   | <:expr< fun $_$ -> $e$ >> -> final_action e
   | _ -> failwith "final_action" ]
 ;
@@ -131,6 +134,15 @@ value parse_standard_symbol e rkont fkont ending_act =
   >>
 ;
 
+value parse_symbol_no_failure e rkont fkont ending_act =
+  <:expr<
+     let $nth_patt_of_act ending_act$ =
+       try $e$ strm__ with [ Stream.Failure -> raise (Stream.Error "") ]
+     in
+     $rkont$
+  >>
+;
+
 value rec contain_loc =
   fun
   [ <:expr< $lid:s$ >> -> s = "loc"
@@ -194,7 +206,7 @@ and parse_symbol entry nlevn s rkont fkont ending_act =
   match s with
   [ Slist0 s ->
       let e = <:expr< P.list0 $symbol_parser entry nlevn s$ >> in
-      parse_standard_symbol e rkont fkont ending_act
+      parse_symbol_no_failure e rkont fkont ending_act
   | Slist1 s ->
       let e = <:expr< P.list1 $symbol_parser entry nlevn s$ >> in
       parse_standard_symbol e rkont fkont ending_act
@@ -204,7 +216,7 @@ and parse_symbol entry nlevn s rkont fkont ending_act =
           P.list0sep $symbol_parser entry nlevn s$
             $symbol_parser entry nlevn sep$ >>
       in
-      parse_standard_symbol e rkont fkont ending_act
+      parse_symbol_no_failure e rkont fkont ending_act
   | Slist1sep s sep ->
       let e =
         <:expr<
@@ -214,7 +226,7 @@ and parse_symbol entry nlevn s rkont fkont ending_act =
       parse_standard_symbol e rkont fkont ending_act
   | Sopt s ->
       let e = <:expr< P.option $symbol_parser entry nlevn s$ >> in
-      parse_standard_symbol e rkont fkont ending_act
+      parse_symbol_no_failure e rkont fkont ending_act
   | Stree tree ->
       let kont = <:expr< raise Stream.Failure >> in
       let act_kont _ act = gen_let_loc loc (final_action act) in
@@ -551,7 +563,10 @@ value compile () =
         $expr_list list$
     >>
   in
-  let loc = (1, 1) in
+  let loc =
+    let l1 = 
+      {(Lexing.dummy_pos) with Lexing.pos_lnum = 1; Lexing.pos_cnum = 1 } in
+    (l1,l1) in
   ([(si1, loc); (si2, loc)], False)
 ;
 
