@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: q_phony.ml,v 1.2 2002/07/19 14:53:47 mauny Exp $ *)
+(* $Id: q_phony.ml,v 1.4 2004/05/12 15:22:41 mauny Exp $ *)
 
 open Pcaml;
 
@@ -23,25 +23,27 @@ Quotation.add ""
           if t.val = "" then "<<" ^ s ^ ">>"
           else "<:" ^ t.val ^ "<" ^ s ^ ">>"
         in
-        let loc = (0, 0) in
+        let loc = (Token.nowhere, Token.nowhere) in
         <:expr< $uid:t$ >>,
       fun s ->
         let t =
           if t.val = "" then "<<" ^ s ^ ">>"
           else "<:" ^ t.val ^ "<" ^ s ^ ">>"
         in
-        let loc = (0, 0) in
+        let loc = (Token.nowhere, Token.nowhere) in
         <:patt< $uid:t$ >>))
 ;
 
 Quotation.default.val := "";
 Quotation.translate.val := fun s -> do { t.val := s; "" };
 
-EXTEND
-  expr: LEVEL "top"
-    [ [ "ifdef"; c = UIDENT; "then"; e1 = expr; "else"; e2 = expr ->
-          <:expr< if def $uid:c$ then $e1$ else $e2$ >>
-      | "ifndef"; c = UIDENT; "then"; e1 = expr; "else"; e2 = expr ->
-          <:expr< if ndef $uid:c$ then $e1$ else $e2$ >> ] ]
-  ;
-END;
+if Pcaml.syntax_name.val <> "Scheme" then
+  EXTEND
+    expr: LEVEL "top"
+      [ [ "IFDEF"; c = UIDENT; "THEN"; e1 = expr; "ELSE"; e2 = expr; "END" ->
+            <:expr< if DEF $uid:c$ then $e1$ else $e2$ >>
+        | "IFNDEF"; c = UIDENT; "THEN"; e1 = expr; "ELSE"; e2 = expr; "END" ->
+            <:expr< if NDEF $uid:c$ then $e1$ else $e2$ >> ] ]
+    ;
+  END
+else ();
