@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: pa_extend.ml,v 1.31 2002/07/19 14:53:50 mauny Exp $ *)
+(* $Id: pa_extend.ml,v 1.33 2004/05/12 15:22:43 mauny Exp $ *)
 
 open Stdpp;
 
@@ -22,9 +22,9 @@ Pcaml.add_option "-split_ext" (Arg.Set split_ext)
 Pcaml.add_option "-split_gext" (Arg.Set split_ext)
   "Old name for the option -split_ext.";
 
-type loc = (int * int);
+type loc = (Lexing.position * Lexing.position);
 
-type name 'e = { expr : 'e; tvar : string; loc : (int * int) };
+type name 'e = { expr : 'e; tvar : string; loc : loc };
 
 type styp =
   [ STlid of loc and string
@@ -163,7 +163,10 @@ module MetaAction =
       in
       failwith (f ^ ", not impl: " ^ desc)
     ;
-    value loc = (0, 0);
+    value loc =
+        let nowhere =
+          { (Lexing.dummy_pos) with Lexing.pos_lnum = 1; Lexing.pos_cnum = 0 } in
+        (nowhere, nowhere);
     value rec mlist mf =
       fun
       [ [] -> <:expr< [] >>
@@ -179,7 +182,10 @@ module MetaAction =
       [ False -> <:expr< False >>
       | True -> <:expr< True >> ]
     ;
-    value mloc = <:expr< (0, 0) >>;
+    value mloc =
+         <:expr< let nowhere =
+          { (Lexing.dummy_pos) with Lexing.pos_lnum = 1; Lexing.pos_cnum = 0 } in
+           (nowhere, nowhere) >>;
     value rec mexpr =
       fun
       [ MLast.ExAcc loc e1 e2 ->
@@ -295,9 +301,12 @@ value rec quot_expr e =
       [ <:expr< $uid:c$ >> ->
           let al = List.map quot_expr al in
           <:expr< Qast.Node $str:c$ $mklistexp loc al$ >>
-      | <:expr< $_$.$uid:c$ >> ->
+      | <:expr< MLast.$uid:c$ >> ->
           let al = List.map quot_expr al in
           <:expr< Qast.Node $str:c$ $mklistexp loc al$ >>
+      | <:expr< $uid:m$.$uid:c$ >> ->
+          let al = List.map quot_expr al in
+          <:expr< Qast.Node $str:m ^ "." ^ c$ $mklistexp loc al$ >>
       | <:expr< $lid:f$ >> ->
           let al = List.map quot_expr al in
           List.fold_left (fun f e -> <:expr< $f$ $e$ >>)
@@ -322,15 +331,16 @@ value rec quot_expr e =
       [ Not_found -> e ]
   | <:expr< $lid:s$ >> ->
       if s = Stdpp.loc_name.val then <:expr< Qast.Loc >> else e
-  | <:expr< $_$.$uid:s$ >> -> <:expr< Qast.Node $str:s$ [] >>
+  | <:expr< MLast.$uid:s$ >> -> <:expr< Qast.Node $str:s$ [] >>
+  | <:expr< $uid:m$.$uid:s$ >> -> <:expr< Qast.Node $str:m ^ "." ^ s$ [] >>
   | <:expr< $uid:s$ >> -> <:expr< Qast.Node $str:s$ [] >>
   | <:expr< $str:s$ >> -> <:expr< Qast.Str $str:s$ >>
   | <:expr< ($list:el$) >> ->
       let el = List.map quot_expr el in
       <:expr< Qast.Tuple $mklistexp loc el$ >>
-  | <:expr< let $rec:r$ $list:pel$ in $e$ >> ->
+  | <:expr< let $opt:r$ $list:pel$ in $e$ >> ->
       let pel = List.map (fun (p, e) -> (p, quot_expr e)) pel in
-      <:expr< let $rec:r$ $list:pel$ in $quot_expr e$ >>
+      <:expr< let $opt:r$ $list:pel$ in $quot_expr e$ >>
   | _ -> e ]
 ;
 
@@ -351,7 +361,10 @@ value quotify_action psl act =
     (fun e ps ->
        match ps.pattern with
        [ Some <:patt< ($list:pl$) >> ->
-           let loc = (0, 0) in
+           let loc =
+             let nowhere =
+               { (Lexing.dummy_pos) with Lexing.pos_lnum = 1; Lexing.pos_cnum = 0 } in
+               (nowhere, nowhere) in
            let pname = pname_of_ptuple pl in
            let (pl1, el1) =
              let (l, _) =
@@ -449,7 +462,7 @@ value text_of_action loc psl rtvar act tvar =
     [ Some act -> if quotify.val then quotify_action psl act else act
     | None -> <:expr< () >> ]
   in
-  let e = <:expr< fun [ ($locid$ : (int * int)) -> ($act$ : '$rtvar$) ] >> in
+  let e = <:expr< fun [ ($locid$ : (Lexing.position * Lexing.position)) -> ($act$ : '$rtvar$) ] >> in
   let txt =
     List.fold_left
       (fun txt ps ->
@@ -720,8 +733,16 @@ value text_of_functorial_extend loc gmod gl el =
   let_in_of_extend loc gmod True gl el args
 ;
 
+value zero_loc = {(Lexing.dummy_pos) with Lexing.pos_cnum = 0};
+
 open Pcaml;
 value symbol = Grammar.Entry.create gram "symbol";
+value semi_sep =
+  if syntax_name.val = "Scheme" then
+    Grammar.Entry.of_parser gram "'/'" (parser [: `("", "/") :] -> ())
+  else
+    Grammar.Entry.of_parser gram "';'" (parser [: `("", ";") :] -> ())
+;
 
 EXTEND
   GLOBAL: expr symbol;
@@ -732,29 +753,30 @@ EXTEND
       | "GDELETE_RULE"; e = gdelete_rule_body; "END" -> e ] ]
   ;
   extend_body:
-    [ [ f = efunction; sl = OPT global; el = LIST1 [ e = entry; ";" -> e ] ->
+    [ [ f = efunction; sl = OPT global;
+        el = LIST1 [ e = entry; semi_sep -> e ] ->
           text_of_extend loc "Grammar" sl el f ] ]
   ;
   gextend_body:
-    [ [ g = UIDENT; sl = OPT global; el = LIST1 [ e = entry; ";" -> e ] ->
+    [ [ g = UIDENT; sl = OPT global; el = LIST1 [ e = entry; semi_sep -> e ] ->
           text_of_functorial_extend loc g sl el ] ]
   ;
   delete_rule_body:
-    [ [ n = name; ":"; sl = LIST1 symbol SEP ";" ->
+    [ [ n = name; ":"; sl = LIST1 symbol SEP semi_sep ->
           let (e, b) = expr_of_delete_rule loc "Grammar" n sl in
           <:expr< Grammar.delete_rule $e$ $b$ >> ] ]
   ;
   gdelete_rule_body:
-    [ [ g = UIDENT; n = name; ":"; sl = LIST1 symbol SEP ";" ->
+    [ [ g = UIDENT; n = name; ":"; sl = LIST1 symbol SEP semi_sep ->
           let (e, b) = expr_of_delete_rule loc g n sl in
           <:expr< $uid:g$.delete_rule $e$ $b$ >> ] ]
   ;
   efunction:
-    [ [ UIDENT "FUNCTION"; ":"; f = qualid; ";" -> f
+    [ [ UIDENT "FUNCTION"; ":"; f = qualid; semi_sep -> f
       | -> <:expr< Grammar.extend >> ] ]
   ;
   global:
-    [ [ UIDENT "GLOBAL"; ":"; sl = LIST1 name; ";" -> sl ] ]
+    [ [ UIDENT "GLOBAL"; ":"; sl = LIST1 name; semi_sep -> sl ] ]
   ;
   entry:
     [ [ n = name; ":"; pos = OPT position; ll = level_list ->
@@ -785,9 +807,9 @@ EXTEND
           retype_rule_list_without_patterns loc rules ] ]
   ;
   rule:
-    [ [ psl = LIST0 psymbol SEP ";"; "->"; act = expr ->
+    [ [ psl = LIST0 psymbol SEP semi_sep; "->"; act = expr ->
           {prod = psl; action = Some act}
-      | psl = LIST0 psymbol SEP ";" ->
+      | psl = LIST0 psymbol SEP semi_sep ->
           {prod = psl; action = None} ] ]
   ;
   psymbol:
@@ -888,13 +910,13 @@ EXTEND
   string:
     [ [ s = STRING -> <:expr< $str:s$ >>
       | i = ANTIQUOT ->
-          let shift = fst loc + String.length "$" in
+          let shift = Reloc.shift_pos (String.length "$") (fst loc) in
           let e =
             try Grammar.Entry.parse Pcaml.expr_eoi (Stream.of_string i) with
             [ Exc_located (bp, ep) exc ->
-                raise_with_loc (shift + bp, shift + ep) exc ]
+                raise_with_loc (Reloc.adjust_loc shift (bp,ep)) exc ]
           in
-          Pcaml.expr_reloc (fun (bp, ep) -> (shift + bp, shift + ep)) 0 e ] ]
+          Pcaml.expr_reloc (fun (bp, ep) -> (Reloc.adjust_loc shift (bp,ep))) zero_loc e ] ]
   ;
 END;
 
