@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: pa_rp.ml,v 1.6 2002/07/19 14:53:50 mauny Exp $ *)
+(* $Id: pa_rp.ml,v 1.7 2003/07/10 12:28:27 michel Exp $ *)
 
 open Pcaml;
 
@@ -90,9 +90,9 @@ value rec subst v e =
   | <:expr< $int:_$ >> -> e
   | <:expr< $chr:_$ >> -> e
   | <:expr< $str:_$ >> -> e
-  | <:expr< $_$ . $_$ >> -> e
-  | <:expr< let $rec:rf$ $list:pel$ in $e$ >> ->
-      <:expr< let $rec:rf$ $list:List.map (subst_pe v) pel$ in $subst v e$ >>
+  | <:expr< $_$.$_$ >> -> e
+  | <:expr< let $opt:rf$ $list:pel$ in $e$ >> ->
+      <:expr< let $opt:rf$ $list:List.map (subst_pe v) pel$ in $subst v e$ >>
   | <:expr< $e1$ $e2$ >> -> <:expr< $subst v e1$ $subst v e2$ >>
   | <:expr< ( $list:el$ ) >> -> <:expr< ( $list:List.map (subst v) el$ ) >>
   | _ -> raise Not_found ]
