@@ -1,4 +1,4 @@
-(* camlp4r pa_ifdef.cmo *)
+(* camlp4r pa_macro.cmo *)
 (***********************************************************************)
 (*                                                                     *)
 (*                             Camlp4                                  *)
@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: odyl_main.ml,v 1.3 2002/07/19 14:53:56 mauny Exp $ *)
+(* $Id: odyl_main.ml,v 1.4.4.1 2004/06/23 14:43:58 mauny Exp $ *)
 
 value go = ref (fun () -> ());
 value name = ref "odyl";
@@ -20,12 +20,12 @@ value first_arg_no_load () =
     if i < Array.length Sys.argv then
       match Sys.argv.(i) with
       [ "-I" -> loop (i + 2)
-      | "-nolib" -> loop (i + 1)
-      | "-where" -> loop (i + 1)
+      | ("-nolib" | "-where" | "-version") -> loop (i + 1)
       | "--" -> i + 1
       | s ->
           if Filename.check_suffix s ".cmo"
-          || Filename.check_suffix s ".cma" then loop (i + 1)
+          || Filename.check_suffix s ".cma"
+          then loop (i + 1)
           else i ]
     else i
 ;
@@ -55,12 +55,13 @@ value initialized = ref False;
 value path = ref ([] : list string);
 
 value loadfile file =
-  ifdef OPT then
+  IFDEF OPT THEN
     raise (Error file "native-code program cannot do a dynamic load")
-  else do {
+  ELSE do {
     if not initialized.val then do {
-      ifdef OPT then ()
-      else do { Dynlink.init (); Dynlink.allow_unsafe_modules True };
+      IFDEF OPT THEN ()
+      ELSE do { Dynlink.init (); Dynlink.allow_unsafe_modules True }
+      END;
       initialized.val := True
     }
     else ();
@@ -75,6 +76,7 @@ value loadfile file =
     try Dynlink.loadfile fname with
     [ Dynlink.Error e -> raise (Error fname (Dynlink.error_message e)) ]
   }
+  END
 ;
 
 value directory d = path.val := [d :: path.val];
