@@ -1,6 +1,6 @@
 #!/bin/sh -e
 if test "`basename $OTOP`" != "ocaml_stuff"; then
-    COMM="$OTOP/boot/ocamlrun$EXE $OTOP/ocamlc -I $OTOP/stdlib"
+    COMM=$OTOP/ocamlcomp.sh
 else
     COMM=ocamlc$OPT
 fi
