@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: camlp4_top.ml,v 1.11 2002/07/19 14:53:56 mauny Exp $ *)
+(* $Id: camlp4_top.ml,v 1.13.2.1 2004/10/07 09:18:13 mauny Exp $ *)
 
 open Parsetree;
 open Lexing;
@@ -59,8 +59,9 @@ value highlight_locations lb loc1 loc2 =
 
 value print_location lb loc =
   if String.length Toploop.input_name.val = 0 then
-    highlight_locations lb loc (-1, -1)
-  else Toploop.print_location Format.err_formatter (Ast2pt.mkloc loc)
+    highlight_locations lb ((fst loc).Lexing.pos_cnum, (snd loc).Lexing.pos_cnum) (-1, -1)
+  else Toploop.print_location Format.err_formatter
+    (Ast2pt.mkloc loc)
 ;
 
 value wrap f shfn lb =
@@ -76,7 +77,7 @@ value wrap f shfn lb =
            do {
              lb.refill_buff lb
            };
-           if lb.lex_eof_reached then None
+           if lb.lex_curr_pos >= lb.lex_buffer_len then None
            else do {
              let c = lb.lex_buffer.[lb.lex_curr_pos] in
              lb.lex_curr_pos := lb.lex_curr_pos + 1;
@@ -123,9 +124,16 @@ value toplevel_phrase cs =
 
 value use_file cs =
   let v = Pcaml.input_file.val in
+  let (bolpos,lnum,fname) = Pcaml.position.val in
+  let restore  =
+    let (bolp,ln,fn) = (bolpos.val, lnum.val, fname.val) in
+    fun () -> do {
+      Pcaml.input_file.val := v;
+      bolpos.val := bolp; lnum.val := ln; fname.val := fn
+    } in
   do {
     Pcaml.input_file.val := Toploop.input_name.val;
-    let restore () = Pcaml.input_file.val := v in
+    bolpos.val := 0; lnum.val := 1; fname.val := Toploop.input_name.val;
     try
       let (pl0, eoi) =
         loop () where rec loop () =
