@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: rprint.ml,v 1.7 2002/07/19 14:53:56 mauny Exp $ *)
+(* $Id: rprint.ml,v 1.14 2004/06/12 08:55:46 xleroy Exp $ *)
 
 open Format;
 open Outcometree;
@@ -53,6 +53,9 @@ value print_out_value ppf tree =
   and print_simple_tree ppf =
     fun
     [ Oval_int i -> fprintf ppf "%i" i
+    | Oval_int32 i -> fprintf ppf "%ldl" i
+    | Oval_int64 i -> fprintf ppf "%LdL" i
+    | Oval_nativeint i -> fprintf ppf "%ndn" i
     | Oval_float f -> fprintf ppf "%.12g" f
     | Oval_char c -> fprintf ppf "'%s'" (Char.escaped c)
     | Oval_string s ->
@@ -112,6 +115,10 @@ value rec print_list pr sep ppf =
   | [a :: l] -> do { pr ppf a; sep ppf; print_list pr sep ppf l } ]
 ;
 
+value pr_vars =
+  print_list (fun ppf s -> fprintf ppf "'%s" s) (fun ppf -> fprintf ppf "@ ")
+;
+
 value pr_present =
   print_list (fun ppf s -> fprintf ppf "`%s" s) (fun ppf -> fprintf ppf "@ ")
 ;
@@ -127,6 +134,10 @@ and print_out_type_1 ppf =
   [ Otyp_arrow lab ty1 ty2 ->
       fprintf ppf "@[%s%a ->@ %a@]" (if lab <> "" then lab ^ ":" else "")
         print_out_type_2 ty1 print_out_type_1 ty2
+  | Otyp_poly sl ty ->
+      fprintf ppf "@[<hov 2>%a.@ %a@]"
+        pr_vars sl
+        print_out_type ty
   | ty -> print_out_type_2 ppf ty ]
 and print_out_type_2 ppf =
   fun
@@ -135,6 +146,7 @@ and print_out_type_2 ppf =
         (print_typlist print_simple_out_type "") tyl
   | ty -> print_simple_out_type ppf ty ]
 and print_simple_out_type ppf =
+  let rec print_tkind ppf =
   fun
   [ Otyp_var ng s -> fprintf ppf "'%s%s" (if ng then "_" else "") s
   | Otyp_constr id [] -> fprintf ppf "@[%a@]" print_ident id
@@ -155,8 +167,8 @@ and print_simple_out_type ppf =
         | Ovar_name id tyl ->
             fprintf ppf "@[%a%a@]" print_typargs tyl print_ident id ]
       in
-      fprintf ppf "%s[|%s@[<hv>@[<hv>%a@]%a|]@]" (if non_gen then "_" else "")
-        (if closed then if tags = None then " " else "< "
+      fprintf ppf "%s[%s@[<hv>@[<hv>%a@]%a ]@]" (if non_gen then "_" else "")
+        (if closed then if tags = None then "= " else "< "
          else if tags = None then "> "
          else "? ")
         print_fields row_fields
@@ -168,16 +180,23 @@ and print_simple_out_type ppf =
         print_ident id
   | Otyp_manifest ty1 ty2 ->
       fprintf ppf "@[<2>%a ==@ %a@]" print_out_type ty1 print_out_type ty2
-  | Otyp_sum constrs ->
-      fprintf ppf "@[<hv>[ %a ]@]"
+  | Otyp_sum constrs priv ->
+      fprintf ppf "@[<hv>%a[ %a ]@]" print_private priv
         (print_list print_out_constr (fun ppf -> fprintf ppf "@ | ")) constrs
-  | Otyp_record lbls ->
-      fprintf ppf "@[<hv 2>{ %a }@]"
+  | Otyp_record lbls priv ->
+      fprintf ppf "@[<hv 2>%a{ %a }@]" print_private priv
         (print_list print_out_label (fun ppf -> fprintf ppf ";@ ")) lbls
   | Otyp_abstract -> fprintf ppf "'abstract"
   | Otyp_alias _ _ | Otyp_poly _ _
   | Otyp_arrow _ _ _ | Otyp_constr _ [_ :: _] as ty ->
       fprintf ppf "@[<1>(%a)@]" print_out_type ty ]
+  and print_private ppf =
+  fun
+  [ Asttypes.Public -> ()
+  | Asttypes.Private -> fprintf ppf "private "
+  ]
+  in
+  print_tkind ppf
 and print_out_constr ppf (name, tyl) =
   match tyl with
   [ [] -> fprintf ppf "%s" name
@@ -294,12 +313,14 @@ and print_out_signature ppf =
         print_out_signature items ]
 and print_out_sig_item ppf =
   fun
-  [ Osig_class vir_flag name params clt ->
-      fprintf ppf "@[<2>class%s@ %a%s@ :@ %a@]"
+  [ Osig_class vir_flag name params clt rs ->
+      fprintf ppf "@[<2>%s%s@ %a%s@ :@ %a@]"
+        (if rs = Orec_next then "and" else "class")
         (if vir_flag then " virtual" else "") print_out_class_params params
         name Toploop.print_out_class_type.val clt
-  | Osig_class_type vir_flag name params clt ->
-      fprintf ppf "@[<2>class type%s@ %a%s@ =@ %a@]"
+  | Osig_class_type vir_flag name params clt rs ->
+      fprintf ppf "@[<2>%s%s@ %a%s@ =@ %a@]"
+        (if rs = Orec_next then "and" else "class type")
         (if vir_flag then " virtual" else "") print_out_class_params params
         name Toploop.print_out_class_type.val clt
   | Osig_exception id tyl ->
@@ -309,10 +330,16 @@ and print_out_sig_item ppf =
   | Osig_modtype name mty ->
       fprintf ppf "@[<2>module type %s =@ %a@]" name
         Toploop.print_out_module_type.val mty
-  | Osig_module name mty ->
-      fprintf ppf "@[<2>module %s :@ %a@]" name
+  | Osig_module name mty rs ->
+      fprintf ppf "@[<2>%s %s :@ %a@]" name
+        (match rs with [ Orec_not -> "module"
+                       | Orec_first -> "module rec"
+                       | Orec_next -> "and" ])
         Toploop.print_out_module_type.val mty
-  | Osig_type tdl -> print_out_type_decl_list ppf tdl
+  | Osig_type td rs ->
+      print_out_type_decl
+          (if rs = Orec_next then "and" else "type")
+          ppf td
   | Osig_value name ty prims ->
       let kwd = if prims = [] then "value" else "external" in
       let pr_prims ppf =
@@ -326,16 +353,7 @@ and print_out_sig_item ppf =
       in
       fprintf ppf "@[<2>%s %a :@ %a%a@]" kwd value_ident name
         Toploop.print_out_type.val ty pr_prims prims ]
-and print_out_type_decl_list ppf =
-  fun
-  [ [] -> ()
-  | [x] -> print_out_type_decl "type" ppf x
-  | [x :: l] ->
-      do {
-        print_out_type_decl "type" ppf x;
-        List.iter (fun x -> fprintf ppf "@ %a" (print_out_type_decl "and") x)
-          l
-      } ]
+
 and print_out_type_decl kwd ppf (name, args, ty, constraints) =
   let constrain ppf (ty, ty') =
     fprintf ppf "@ @[<2>constraint %a =@ %a@]" Toploop.print_out_type.val ty
