@@ -11,12 +11,14 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: ia32sse2.c,v 1.1 2003/10/24 09:18:01 xleroy Exp $ */
+/* $Id: ia32sse2.c,v 1.1.6.1 2005/01/31 17:25:42 doligez Exp $ */
 
 /* Test whether IA32 assembler supports SSE2 instructions */
 
 int main()
 {
   asm("pmuludq %mm1, %mm0");
+  asm("paddq %mm1, %mm0");
+  asm("psubq %mm1, %mm0");
   return 0;
 }
