@@ -6,11 +6,12 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../LICENSE.     */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: m-templ.h,v 1.12 2000/03/10 17:36:31 xleroy Exp $ */
+/* $Id: m-templ.h,v 1.14 2001/12/07 13:39:40 xleroy Exp $ */
 
 /* Processor dependencies */
 
@@ -71,3 +72,10 @@
    supports word-aligned 64-bit integers.  Leave undefined if
    64-bit integers are not supported. */
 
+#undef NONSTANDARD_DIV_MOD
+
+/* Leave NONSTANDARD_DIV_MOD undefined if the C operators / and % implement
+   round-towards-zero semantics, as specified by ISO C 9x and implemented
+   by most contemporary processors.  Otherwise, or if you don't know,
+   define NONSTANDARD_DIV_MOD: this will select a slower but correct
+   software emulation of division and modulus. */
