@@ -11,7 +11,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: debugger_config.mli,v 1.3 1999/11/17 18:57:23 xleroy Exp $ *)
+(* $Id: debugger_config.mli,v 1.5 2002/11/17 16:42:10 xleroy Exp $ *)
 
 (********************** Configuration file *****************************)
 
@@ -28,10 +28,8 @@ val history_size : int ref
 
 (*** Time travel paramaters. ***)
 
-val checkpoint_big_step : int ref
-val checkpoint_small_step : int ref
+val checkpoint_big_step : int64 ref
+val checkpoint_small_step : int64 ref
 val checkpoint_max_count : int ref
 val make_checkpoints : bool ref
 
-(*** Dynamic loader ***)
-val stdlib_units : string list
