@@ -11,7 +11,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: events.ml,v 1.4 1999/11/17 18:57:24 xleroy Exp $ *)
+(* $Id: events.ml,v 1.5 2002/11/02 22:36:42 doligez Exp $ *)
 
 (********************************* Events ******************************)
 
@@ -53,7 +53,7 @@ let current_point () =
     None ->
       raise Not_found
   | Some {ev_char = point; ev_module = mdle} ->
-      (mdle, point)
+      (mdle, point.Lexing.pos_cnum)
 
 let current_event_is_before () =
   match !current_event with
