@@ -11,7 +11,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: frames.ml,v 1.8 2000/04/06 09:09:10 xleroy Exp $ *)
+(* $Id: frames.ml,v 1.9 2002/11/02 22:36:42 doligez Exp $ *)
 
 (***************************** Frames **********************************)
 
@@ -35,7 +35,7 @@ let selected_point () =
     None ->
       raise Not_found
   | Some {ev_char = point; ev_module = mdle} ->
-      (mdle, point)
+      (mdle, point.Lexing.pos_cnum)
 
 let selected_event_is_before () =
   match !selected_event with
