@@ -10,18 +10,20 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: loadprinter.mli,v 1.3 1999/11/17 18:57:25 xleroy Exp $ *)
+(* $Id: loadprinter.mli,v 1.4 2000/03/07 18:22:15 weis Exp $ *)
 
 (* Loading and installation of user-defined printer functions *)
 
-val loadfile : string -> unit
-val install_printer : Longident.t -> unit
+open Format
+
+val loadfile : formatter -> string -> unit
+val install_printer : formatter -> Longident.t -> unit
 val remove_printer : Longident.t -> unit
 
 (* Error report *)
 
 type error =
-    Load_failure of Dynlink.error
+  | Load_failure of Dynlink.error
   | Unbound_identifier of Longident.t
   | Unavailable_module of string * Longident.t
   | Wrong_type of Longident.t
@@ -29,4 +31,4 @@ type error =
 
 exception Error of error
 
-val report_error: error -> unit
+val report_error: formatter -> error -> unit
