@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: main_args.ml,v 1.34 2002/08/01 15:18:00 doligez Exp $ *)
+(* $Id: main_args.ml,v 1.42 2003/07/17 08:38:27 xleroy Exp $ *)
 
 module Make_options (F :
    sig
@@ -22,6 +22,7 @@ module Make_options (F :
      val _custom : unit -> unit
      val _dllib : string -> unit
      val _dllpath : string -> unit
+     val _dtypes : unit -> unit
      val _g : unit -> unit
      val _i : unit -> unit
      val _I : string -> unit
@@ -42,6 +43,7 @@ module Make_options (F :
      val _principal : unit -> unit
      val _rectypes : unit -> unit
      val _thread : unit -> unit
+     val _vmthread : unit -> unit
      val _unsafe : unit -> unit
      val _use_prims : string -> unit
      val _use_runtime : string -> unit
@@ -73,8 +75,9 @@ struct
            "<lib>  Use the dynamically-loaded library <lib>";
     "-dllpath", Arg.String F._dllpath,
            "<dir>  Add <dir> to the run-time search path for shared libraries";
+    "-dtypes", Arg.Unit F._dtypes, " Save type information in <filename>.annot";
     "-g", Arg.Unit F._g, " Save debugging information";
-    "-i", Arg.Unit F._i, " Print the types";
+    "-i", Arg.Unit F._i, " Print inferred interface";
     "-I", Arg.String F._I,
            "<dir>  Add <dir> to the list of include directories";
     "-impl", Arg.String F._impl, "<file>  Compile <file> as a .ml file";
@@ -106,7 +109,7 @@ struct
     "-principal", Arg.Unit F._principal,
            " Check principality of type inference";
     "-rectypes", Arg.Unit F._rectypes, " Allow arbitrary recursive types";
-    "-thread", Arg.Unit F._thread, " Use thread-safe standard library";
+    "-thread", Arg.Unit F._thread, " Generate code that supports the system threads library";
     "-unsafe", Arg.Unit F._unsafe,
            " No bounds checking on array and string access";
     "-use-runtime", Arg.String F._use_runtime,
@@ -117,11 +120,13 @@ struct
            " Print compiler version and location of standard library and exit";
     "-version", Arg.Unit F._version, " Print compiler version and exit";
     "-verbose", Arg.Unit F._verbose, " Print calls to external commands";
+    "-vmthread", Arg.Unit F._vmthread, " Generate code that supports the threads library with VM-level scheduling";
     "-w", Arg.String F._w,
       "<flags>  Enable or disable warnings according to <flags>:\n\
       \032    A/a enable/disable all warnings\n\
       \032    C/c enable/disable suspicious comment\n\
       \032    D/d enable/disable deprecated features\n\
+      \032    E/e enable/disable fragile match\n\
       \032    F/f enable/disable partially applied function\n\
       \032    L/l enable/disable labels omitted in application\n\
       \032    M/m enable/disable overriden method\n\
@@ -130,7 +135,8 @@ struct
       \032    U/u enable/disable unused match case\n\
       \032    V/v enable/disable hidden instance variable\n\
       \032    X/x enable/disable all other warnings\n\
-      \032    default setting is \"Al\" (all warnings but labels enabled)";
+      \032    default setting is \"Ale\"\n\
+      \032    (all warnings but labels and fragile match enabled)";
     "-warn-error" , Arg.String F._warn_error,
       "<flags>  Treat the warnings enabled by <flags> as errors.\n\
       \032    See option -w for the list of flags.\n\
