@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: main_args.mli,v 1.22 2002/08/01 15:18:01 doligez Exp $ *)
+(* $Id: main_args.mli,v 1.25 2003/07/17 08:38:27 xleroy Exp $ *)
 
 module Make_options (F :
     sig
@@ -22,6 +22,7 @@ module Make_options (F :
       val _custom : unit -> unit
       val _dllib : string -> unit
       val _dllpath : string -> unit
+      val _dtypes : unit -> unit
       val _g : unit -> unit
       val _i : unit -> unit
       val _I : string -> unit
@@ -42,6 +43,7 @@ module Make_options (F :
       val _principal : unit -> unit
       val _rectypes : unit -> unit
       val _thread : unit -> unit
+      val _vmthread : unit -> unit
       val _unsafe : unit -> unit
       val _use_prims : string -> unit
       val _use_runtime : string -> unit
