@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: optcompile.ml,v 1.43 2002/04/18 07:27:39 garrigue Exp $ *)
+(* $Id: optcompile.ml,v 1.48 2004/06/13 12:46:41 xleroy Exp $ *)
 
 (* The batch compiler *)
 
@@ -26,7 +26,7 @@ open Typedtree
 
 let init_path () =
   let dirs =
-    if !Clflags.thread_safe
+    if !Clflags.use_threads
     then "+threads" :: !Clflags.include_dirs
     else !Clflags.include_dirs in
   let exp_dirs =
@@ -37,7 +37,7 @@ let init_path () =
 (* Return the initial environment in which compilation proceeds. *)
 
 let initial_env () =
-  init_path();
+  Ident.reinit();
   try
     if !Clflags.nopervasives
     then Env.initial
@@ -47,9 +47,10 @@ let initial_env () =
 
 (* Compile a .mli file *)
 
-let interface ppf sourcefile =
-  let prefixname = Misc.chop_extension_if_any sourcefile in
-  let modulename = String.capitalize(Filename.basename prefixname) in
+let interface ppf sourcefile outputprefix =
+  init_path();
+  let modulename =
+    String.capitalize(Filename.basename(chop_extension_if_any sourcefile)) in
   let inputfile = Pparse.preprocess sourcefile in
   try
     let ast =
@@ -60,7 +61,8 @@ let interface ppf sourcefile =
       fprintf std_formatter "%a@." Printtyp.signature
                                    (Typemod.simplify_signature sg);
     Warnings.check_fatal ();
-    Env.save_signature sg modulename (prefixname ^ ".cmi");
+    if not !Clflags.print_types then
+      Env.save_signature sg modulename (outputprefix ^ ".cmi");
     Pparse.remove_preprocessed inputfile
   with e ->
     Pparse.remove_preprocessed_if_ast inputfile;
@@ -75,22 +77,29 @@ let print_if ppf flag printer arg =
 let (++) x f = f x
 let (+++) (x, y) f = (x, f y)
 
-let implementation ppf sourcefile =
-  let prefixname = Misc.chop_extension_if_any sourcefile in
-  let modulename = String.capitalize(Filename.basename prefixname) in
+let implementation ppf sourcefile outputprefix =
+  init_path();
+  let modulename =
+    String.capitalize(Filename.basename(chop_extension_if_any sourcefile)) in
   let inputfile = Pparse.preprocess sourcefile in
   let env = initial_env() in
   Compilenv.reset modulename;
   try
-    Pparse.file ppf inputfile Parse.implementation ast_impl_magic_number
-    ++ print_if ppf Clflags.dump_parsetree Printast.implementation
-    ++ Typemod.type_implementation sourcefile prefixname modulename env
-    ++ Translmod.transl_store_implementation modulename
-    +++ print_if ppf Clflags.dump_rawlambda Printlambda.lambda
-    +++ Simplif.simplify_lambda
-    +++ print_if ppf Clflags.dump_lambda Printlambda.lambda
-    ++ Asmgen.compile_implementation prefixname ppf;
-    Compilenv.save_unit_info (prefixname ^ ".cmx");
+    if !Clflags.print_types then ignore(
+      Pparse.file ppf inputfile Parse.implementation ast_impl_magic_number
+      ++ print_if ppf Clflags.dump_parsetree Printast.implementation
+      ++ Typemod.type_implementation sourcefile outputprefix modulename env)
+    else begin
+      Pparse.file ppf inputfile Parse.implementation ast_impl_magic_number
+      ++ print_if ppf Clflags.dump_parsetree Printast.implementation
+      ++ Typemod.type_implementation sourcefile outputprefix modulename env
+      ++ Translmod.transl_store_implementation modulename
+      +++ print_if ppf Clflags.dump_rawlambda Printlambda.lambda
+      +++ Simplif.simplify_lambda
+      +++ print_if ppf Clflags.dump_lambda Printlambda.lambda
+      ++ Asmgen.compile_implementation outputprefix ppf;
+      Compilenv.save_unit_info (outputprefix ^ ".cmx");
+    end;
     Warnings.check_fatal ();
     Pparse.remove_preprocessed inputfile
   with x ->
