@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: opterrors.ml,v 1.16 2002/02/08 16:55:43 xleroy Exp $ *)
+(* $Id: opterrors.ml,v 1.18 2003/06/19 15:53:49 xleroy Exp $ *)
 
 (* WARNING: if you change something in this file, you must look at
    errors.ml to see if you need to make the same changes there.
@@ -22,10 +22,8 @@ open Format
 
 let report_error ppf exn =
   let report ppf = function
-  | Lexer.Error(err, start, stop) ->
-      Location.print ppf {Location.loc_start = start;
-                          Location.loc_end = stop;
-                          Location.loc_ghost = false};
+  | Lexer.Error(err, l) ->
+      Location.print ppf l;
       Lexer.report_error ppf err
   | Syntaxerr.Error err ->
       Syntaxerr.report_error ppf err
@@ -35,19 +33,25 @@ let report_error ppf exn =
       Env.report_error ppf err
   | Ctype.Tags(l, l') -> fprintf ppf
       "In this program,@ variant constructors@ `%s and `%s@ \
-       have the same hash value." l l'
+       have the same hash value.@ Change one of them." l l'
   | Typecore.Error(loc, err) ->
       Location.print ppf loc; Typecore.report_error ppf err
   | Typetexp.Error(loc, err) ->
       Location.print ppf loc; Typetexp.report_error ppf err
   | Typedecl.Error(loc, err) ->
       Location.print ppf loc; Typedecl.report_error ppf err
+  | Typeclass.Error(loc, err) ->
+      Location.print ppf loc; Typeclass.report_error ppf err
   | Includemod.Error err ->
       Includemod.report_error ppf err
   | Typemod.Error(loc, err) ->
       Location.print ppf loc; Typemod.report_error ppf err
   | Translcore.Error(loc, err) ->
       Location.print ppf loc; Translcore.report_error ppf err
+  | Translclass.Error(loc, err) ->
+      Location.print ppf loc; Translclass.report_error ppf err
+  | Translmod.Error(loc, err) ->
+      Location.print ppf loc; Translmod.report_error ppf err
   | Compilenv.Error code ->
       Compilenv.report_error ppf code
   | Asmgen.Error code ->
@@ -60,10 +64,6 @@ let report_error ppf exn =
       Asmpackager.report_error ppf code
   | Sys_error msg ->
       fprintf ppf "I/O error: %s" msg
-  | Typeclass.Error(loc, err) ->
-      Location.print ppf loc; Typeclass.report_error ppf err
-  | Translclass.Error(loc, err) ->
-      Location.print ppf loc; Translclass.report_error ppf err
   | Warnings.Errors (n) ->
       fprintf ppf "@.Error: %d error-enabled warnings occurred." n
   | x -> fprintf ppf "@]"; raise x in
