@@ -10,36 +10,50 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: optmain.ml,v 1.70 2002/08/01 15:18:01 doligez Exp $ *)
+(* $Id: optmain.ml,v 1.81 2004/06/13 12:46:41 xleroy Exp $ *)
 
 open Config
 open Clflags
 
+let output_prefix name =
+  let oname =
+    match !output_name with
+    | None -> name
+    | Some n -> if !compile_only then (output_name := None; n) else name in
+  Misc.chop_extension_if_any oname
+
 let process_interface_file ppf name =
-  Optcompile.interface ppf name
+  Optcompile.interface ppf name (output_prefix name)
 
 let process_implementation_file ppf name =
-  Optcompile.implementation ppf name;
-  objfiles := (Misc.chop_extension_if_any name ^ ".cmx") :: !objfiles
+  let opref = output_prefix name in
+  Optcompile.implementation ppf name opref;
+  objfiles := (opref ^ ".cmx") :: !objfiles
 
 let process_file ppf name =
   if Filename.check_suffix name ".ml"
   || Filename.check_suffix name ".mlt" then begin
-    Optcompile.implementation ppf name;
-    objfiles := (Misc.chop_extension_if_any name ^ ".cmx") :: !objfiles
+    let opref = output_prefix name in
+    Optcompile.implementation ppf name opref;
+    objfiles := (opref ^ ".cmx") :: !objfiles
+  end
+  else if Filename.check_suffix name !Config.interface_suffix then begin
+    let opref = output_prefix name in
+    Optcompile.interface ppf name opref;
+    if !make_package then objfiles := (opref ^ ".cmi") :: !objfiles
   end
-  else if Filename.check_suffix name !Config.interface_suffix then
-    Optcompile.interface ppf name
   else if Filename.check_suffix name ".cmx" 
        || Filename.check_suffix name ".cmxa" then
     objfiles := name :: !objfiles
+  else if Filename.check_suffix name ".cmi" && !make_package then
+    objfiles := name :: !objfiles
   else if Filename.check_suffix name ext_obj
        || Filename.check_suffix name ext_lib then
     ccobjs := name :: !ccobjs
   else if Filename.check_suffix name ".c" then begin
     Optcompile.c_file name;
     ccobjs := (Filename.chop_suffix (Filename.basename name) ".c" ^ ext_obj)
-    :: !ccobjs
+              :: !ccobjs
   end
   else
     raise(Arg.Bad("don't know what to do with " ^ name))
@@ -76,7 +90,7 @@ let main () =
   c_linker := Config.native_c_linker;
   let ppf = Format.err_formatter in
   try
-    Arg.parse [
+    Arg.parse (Arch.command_line_options @ [
        "-a", Arg.Set make_archive, " Build a library";
        "-c", Arg.Set compile_only, " Compile only (do not link)";
        "-cc", Arg.String(fun s -> c_compiler := s; c_linker := s),
@@ -88,7 +102,10 @@ let main () =
              "<opt>  Pass option <opt> to the C compiler and linker";
        "-compact", Arg.Clear optimize_for_speed,
              " Optimize code size rather than speed";
-       "-i", Arg.Set print_types, " Print the types";
+       "-dtypes", Arg.Set save_types,
+             " Save type information in <filename>.annot";
+       "-i", Arg.Unit (fun () -> print_types := true; compile_only := true),
+             " Print inferred interface";
        "-I", Arg.String(fun dir -> include_dirs := dir :: !include_dirs),
              "<dir>  Add <dir> to the list of include directories";
        "-impl", Arg.String (process_implementation_file ppf),
@@ -126,7 +143,7 @@ let main () =
        "-rectypes", Arg.Set recursive_types,
              " Allow arbitrary recursive types";
        "-S", Arg.Set keep_asm_file, " Keep intermediate assembly file";
-       "-thread", Arg.Set thread_safe, " Use thread-safe standard library";
+       "-thread", Arg.Set use_threads, " Generate code that supports the system threads library";
        "-unsafe", Arg.Set fast,
              " No bounds checking on array and string access";
        "-v", Arg.Unit print_version_and_library,
@@ -139,6 +156,7 @@ let main () =
          \032    A/a enable/disable all warnings\n\
          \032    C/c enable/disable suspicious comment\n\
          \032    D/d enable/disable deprecated features\n\
+         \032    E/e enable/disable fragile match\n\
          \032    F/f enable/disable partially applied function\n\
          \032    L/l enable/disable labels omitted in application\n\
          \032    M/m enable/disable overriden methods\n\
@@ -147,13 +165,14 @@ let main () =
          \032    U/u enable/disable unused match case\n\
          \032    V/v enable/disable hidden instance variables\n\
          \032    X/x enable/disable all other warnings\n\
-         \032    default setting is \"Al\" (all warnings but labels enabled)";
+         \032    default setting is \"Ale\"\n\
+         \032    (all warnings but labels and fragile match enabled)";
        "-warn-error" , Arg.String (Warnings.parse_options true),
-         "<flags>  Enable or disable fatal warnings according to <flags>\n\
-           \032    (see option -w for the list of flags)\n\
-           \032    default setting is a (all warnings are non-fatal)";
-    "-where", Arg.Unit print_standard_library,
-      " Print location of standard library and exit";
+         "<flags>  Treat the warnings enabled by <flags> as errors.\n\
+         \032    See option -w for the list of flags.\n\
+         \032    Default setting is \"a\" (warnings are not errors)";
+       "-where", Arg.Unit print_standard_library,
+         " Print location of standard library and exit";
 
        "-nopervasives", Arg.Set nopervasives, " (undocumented)";
        "-dparsetree", Arg.Set dump_parsetree, " (undocumented)";
@@ -176,7 +195,7 @@ let main () =
        "-dstartup", Arg.Set keep_startup_file, " (undocumented)";
        "-", Arg.String (process_file ppf),
             "<file>  Treat <file> as a file name (even if it starts with `-')"
-      ] (process_file ppf) usage;
+      ]) (process_file ppf) usage;
     if !make_archive then begin
       Optcompile.init_path();
       Asmlibrarian.create_archive (List.rev !objfiles)
