@@ -1,3 +1,17 @@
+;(***********************************************************************)
+;(*                                                                     *)
+;(*                           Objective Caml                            *)
+;(*                                                                     *)
+;(*                Jacques Garrigue and Ian T Zimmerman                 *)
+;(*                                                                     *)
+;(*  Copyright 1997 Institut National de Recherche en Informatique et   *)
+;(*  en Automatique.  All rights reserved.  This file is distributed    *)
+;(*  under the terms of the GNU General Public License.                 *)
+;(*                                                                     *)
+;(***********************************************************************)
+
+;(* $Id: caml-hilit.el,v 1.8.4.1 2004/08/09 16:09:33 doligez Exp $ *)
+
 ; Highlighting patterns for hilit19 under caml-mode
 
 ; defined also in caml.el
@@ -27,7 +41,7 @@
           "\\|ex\\(ception\\|ternal\\)\\|fun\\(ct\\(ion\\|or\\)\\)?"
           "\\|in\\(herit\\)?\\|let\\|m\\(ethod\\|utable\\|odule\\)"
           "\\|of\\|p\\(arser\\|rivate\\)\\|rec\\|type"
-          "\\|v\\(al\\(ue\\)?\\|irtual\\)\\)\\>")
+          "\\|v\\(al\\|irtual\\)\\)\\>")
          nil 'ForestGreen)
 ;blocking
    '("\\<\\(object\\|struct\\|sig\\|begin\\|end\\)\\>" 2 include)
