@@ -9,6 +9,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
+(* $Id: odoc_analyse.ml,v 1.8.6.1 2004/08/06 13:42:16 guesdon Exp $ *)
 
 (** Analysis of source files. This module is strongly inspired from driver/main.ml :-) *)
 
@@ -20,17 +21,14 @@ open Misc
 open Format
 open Typedtree
 
+
 (** Initialize the search path.
    The current directory is always searched first,
    then the directories specified with the -I option (in command-line order),
    then the standard library directory. *)
 let init_path () =
-  let dirs =
-    if !Clflags.thread_safe then
-     Filename.concat Config.standard_library "threads" :: !Clflags.include_dirs
-    else
-     !Clflags.include_dirs in
-  load_path := "" :: List.rev (Config.standard_library :: dirs);
+  load_path :=
+    "" :: List.rev (Config.standard_library :: !Clflags.include_dirs);
   Env.reset_cache()
 
 (** Return the initial environment in which compilation proceeds. *)
@@ -93,7 +91,9 @@ let parse_file inputfile parse_fun ast_magic =
       end else begin
         seek_in ic 0;
         Location.input_name := inputfile;
-        parse_fun (Lexing.from_channel ic)
+        let lexbuf = Lexing.from_channel ic in
+        Location.init lexbuf inputfile;
+        parse_fun lexbuf
       end
     with x -> close_in ic; raise x
   in
@@ -143,6 +143,7 @@ let process_interface_file ppf sourcefile =
 
 (** The module used to analyse the parsetree and signature of an implementation file.*)
 module Ast_analyser = Odoc_ast.Analyser (Odoc_comments.Basic_info_retriever)
+
 (** The module used to analyse the parse tree and typed tree of an interface file.*)
 module Sig_analyser = Odoc_sig.Analyser (Odoc_comments.Basic_info_retriever)
 
@@ -154,10 +155,8 @@ module Sig_analyser = Odoc_sig.Analyser (Odoc_comments.Basic_info_retriever)
    differences only concern code generation (i believe).*)
 let process_error exn =
   let report ppf = function
-  | Lexer.Error(err, start, stop) ->
-      Location.print ppf {Location.loc_start = start;
-                          Location.loc_end = stop;
-                          Location.loc_ghost = false};
+  | Lexer.Error(err, loc) ->
+      Location.print ppf loc;
       Lexer.report_error ppf err
   | Syntaxerr.Error err ->
       Syntaxerr.report_error ppf err
@@ -196,45 +195,49 @@ let process_error exn =
 let process_file ppf sourcefile =
   if !Odoc_args.verbose then 
     (
-     print_string (Odoc_messages.analysing sourcefile) ;
+     let f = match sourcefile with Odoc_args.Impl_file f | Odoc_args.Intf_file f -> f in
+     print_string (Odoc_messages.analysing f) ;
      print_newline ();
     );
-  if Filename.check_suffix sourcefile "ml" then
-    (
-     try
-       let (parsetree_typedtree_opt, input_file) = process_implementation_file ppf sourcefile in
-       match parsetree_typedtree_opt with
-         None ->
-           None
-       | Some (parsetree, typedtree) ->
-           let file_module = Ast_analyser.analyse_typed_tree sourcefile !Location.input_name parsetree typedtree in
-
-           file_module.Odoc_module.m_top_deps <- Odoc_dep.impl_dependencies parsetree ;
-
-           if !Odoc_args.verbose then 
-             (
-              print_string Odoc_messages.ok;
-              print_newline ()
-             );
-           remove_preprocessed input_file;
-           Some file_module
-     with
-     | Sys_error s 
-     | Failure s ->
-         prerr_endline s ;
-         incr Odoc_global.errors ;
-         None
-     | e ->
-         process_error e ;
-         incr Odoc_global.errors ;
-         None
-    )
-  else
-    if Filename.check_suffix sourcefile "mli" then
+  match sourcefile with
+    Odoc_args.Impl_file file ->
       (
        try
-         let (ast, signat, input_file) = process_interface_file ppf sourcefile in
-         let file_module = Sig_analyser.analyse_signature sourcefile !Location.input_name ast signat in
+	 let (parsetree_typedtree_opt, input_file) = process_implementation_file ppf file in
+	 match parsetree_typedtree_opt with
+           None ->
+             None
+	 | Some (parsetree, typedtree) ->
+             let file_module = Ast_analyser.analyse_typed_tree file
+		 !Location.input_name parsetree typedtree 
+	     in
+             file_module.Odoc_module.m_top_deps <- Odoc_dep.impl_dependencies parsetree ;
+
+             if !Odoc_args.verbose then 
+               (
+		print_string Odoc_messages.ok;
+		print_newline ()
+               );
+             remove_preprocessed input_file;
+             Some file_module
+       with
+       | Sys_error s 
+       | Failure s ->
+           prerr_endline s ;
+           incr Odoc_global.errors ;
+           None
+       | e ->
+           process_error e ;
+           incr Odoc_global.errors ;
+           None
+      )
+  | Odoc_args.Intf_file file ->
+      (
+       try
+         let (ast, signat, input_file) = process_interface_file ppf file in
+         let file_module = Sig_analyser.analyse_signature file
+	     !Location.input_name ast signat 
+	 in
 
          file_module.Odoc_module.m_top_deps <- Odoc_dep.intf_dependencies ast ;
 
@@ -256,10 +259,6 @@ let process_file ppf sourcefile =
            incr Odoc_global.errors ;
            None
       )
-    else
-      (
-       raise (Failure (Odoc_messages.unknown_extension sourcefile))
-      )
 
 (** Remove the class elements after the stop special comment. *)
 let rec remove_class_elements_after_stop eles =
@@ -447,3 +446,4 @@ let load_modules file =
       raise (Failure s)
 
 
+(* eof $Id: odoc_analyse.ml,v 1.8.6.1 2004/08/06 13:42:16 guesdon Exp $ *)
