@@ -9,6 +9,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
+(* $Id: odoc_ast.ml,v 1.24 2004/04/17 12:36:14 guesdon Exp $ *)
 
 (** Analysis of implementation files. *)
 open Misc
@@ -66,6 +67,13 @@ module Typedtree_search =
       match tt with
       | Typedtree.Tstr_module (ident, _) -> 
           Hashtbl.add table (M (Name.from_ident ident)) tt
+      | Typedtree.Tstr_recmodule mods ->
+	  List.iter
+	    (fun (ident,mod_expr) -> 
+	      Hashtbl.add table (M (Name.from_ident ident))
+		(Typedtree.Tstr_module (ident,mod_expr))
+	    )
+	    mods
       | Typedtree.Tstr_modtype (ident, _) -> 
           Hashtbl.add table (MT (Name.from_ident ident)) tt
       | Typedtree.Tstr_exception (ident, _) ->
@@ -342,8 +350,8 @@ module Analyser =
              val_type = Odoc_env.subst_type env pat.Typedtree.pat_type ;
              val_recursive = rec_flag = Asttypes.Recursive ;
              val_parameters = tt_analyse_function_parameters env comment_opt pat_exp_list2 ;
-             val_code = Some (get_string_of_file loc.Location.loc_start loc.Location.loc_end) ;
-             val_loc = { loc_impl = Some (!file_name, loc.Location.loc_start) ; loc_inter = None } ;
+             val_code = Some (get_string_of_file loc.Location.loc_start.Lexing.pos_cnum loc.Location.loc_end.Lexing.pos_cnum) ;
+             val_loc = { loc_impl = Some (!file_name, loc.Location.loc_start.Lexing.pos_cnum) ; loc_inter = None } ;
            }    
            in
            [ new_value ]
@@ -359,8 +367,8 @@ module Analyser =
              val_type = Odoc_env.subst_type env pat.Typedtree.pat_type ;
              val_recursive = rec_flag = Asttypes.Recursive ;
              val_parameters = [] ;
-             val_code = Some (get_string_of_file loc.Location.loc_start loc.Location.loc_end) ;
-             val_loc = { loc_impl = Some (!file_name, loc.Location.loc_start) ; loc_inter = None } ;
+             val_code = Some (get_string_of_file loc.Location.loc_start.Lexing.pos_cnum loc.Location.loc_end.Lexing.pos_cnum) ;
+             val_loc = { loc_impl = Some (!file_name, loc.Location.loc_start.Lexing.pos_cnum) ; loc_inter = None } ;
            }    
            in
            [ new_value ]
@@ -377,6 +385,17 @@ module Analyser =
     (** This function takes a Typedtree.class_expr and returns a string which can stand for the class name.
        The name can be "object ... end" if the class expression is not an ident or a class constraint or a class apply. *)
     let rec tt_name_of_class_expr clexp =
+(*
+      (
+       match clexp.Typedtree.cl_desc with
+	 Tclass_ident _ -> prerr_endline "Tclass_ident"
+       | Tclass_structure _ -> prerr_endline "Tclass_structure"
+       | Tclass_fun _ -> prerr_endline "Tclass_fun"
+       | Tclass_apply _ -> prerr_endline "Tclass_apply"
+       | Tclass_let _ -> prerr_endline "Tclass_let"
+       | Tclass_constraint _ -> prerr_endline "Tclass_constraint"
+      );
+*)
       match clexp.Typedtree.cl_desc with
         Typedtree.Tclass_ident p -> Name.from_path p
       | Typedtree.Tclass_constraint (class_expr, _, _, _)
@@ -487,17 +506,26 @@ module Analyser =
               try Typedtree_search.get_nth_inherit_class_expr tt_cls n
               with Not_found -> raise (Failure (Odoc_messages.inherit_classexp_not_found_in_typedtree n))
             in
-            let (info_opt, ele_comments) = get_comments_in_class last_pos p_clexp.Parsetree.pcl_loc.Location.loc_start in
+            let (info_opt, ele_comments) =
+	      get_comments_in_class last_pos 
+		p_clexp.Parsetree.pcl_loc.Location.loc_start.Lexing.pos_cnum 
+	    in
             let text_opt = match info_opt with None -> None | Some i -> i.Odoc_types.i_desc in
             let name = tt_name_of_class_expr tt_clexp in
-            let inher = { ic_name = Odoc_env.full_class_or_class_type_name env name ; ic_class = None ; ic_text = text_opt }  in
+            let inher =
+	      { 
+		ic_name = Odoc_env.full_class_or_class_type_name env name ; 
+		ic_class = None ; 
+		ic_text = text_opt ;
+	      }  
+	    in
             iter (acc_inher @ [ inher ]) (acc_fields @ ele_comments)
-              p_clexp.Parsetree.pcl_loc.Location.loc_end
+              p_clexp.Parsetree.pcl_loc.Location.loc_end.Lexing.pos_cnum
               q
 
         | (Parsetree.Pcf_val (label, mutable_flag, expression, loc)) :: q ->
             let complete_name = Name.concat current_class_name label in
-            let (info_opt, ele_comments) = get_comments_in_class last_pos loc.Location.loc_start in
+            let (info_opt, ele_comments) = get_comments_in_class last_pos loc.Location.loc_start.Lexing.pos_cnum in
             let type_exp =
               try Typedtree_search.search_attribute_type tt_cls label
               with Not_found -> raise (Failure (Odoc_messages.attribute_not_found_in_typedtree complete_name))
@@ -509,17 +537,17 @@ module Analyser =
                               val_type = Odoc_env.subst_type env type_exp ;
                               val_recursive = false ;
                               val_parameters = [] ; 
-                              val_code = Some (get_string_of_file loc.Location.loc_start loc.Location.loc_end) ;
-                              val_loc = { loc_impl = Some (!file_name, loc.Location.loc_start) ; loc_inter = None } ;
+                              val_code = Some (get_string_of_file loc.Location.loc_start.Lexing.pos_cnum loc.Location.loc_end.Lexing.pos_cnum) ;
+                              val_loc = { loc_impl = Some (!file_name, loc.Location.loc_start.Lexing.pos_cnum) ; loc_inter = None } ;
                             } ;
                 att_mutable = mutable_flag = Asttypes.Mutable ;
               } 
             in
-            iter acc_inher (acc_fields @ ele_comments @ [ Class_attribute att ]) loc.Location.loc_end q
+            iter acc_inher (acc_fields @ ele_comments @ [ Class_attribute att ]) loc.Location.loc_end.Lexing.pos_cnum q
       
         | (Parsetree.Pcf_virt  (label, private_flag, _, loc)) :: q ->
             let complete_name = Name.concat current_class_name label in
-            let (info_opt, ele_comments) = get_comments_in_class last_pos loc.Location.loc_start in
+            let (info_opt, ele_comments) = get_comments_in_class last_pos loc.Location.loc_start.Lexing.pos_cnum in
             let met_type = 
               try Odoc_sig.Signature_search.search_method_type label tt_class_sig 
               with Not_found -> raise (Failure (Odoc_messages.method_type_not_found current_class_name label))
@@ -539,8 +567,8 @@ module Analyser =
                               val_type = Odoc_env.subst_type env real_type ;
                               val_recursive = false ;
                               val_parameters = [] ;
-                              val_code = Some (get_string_of_file loc.Location.loc_start loc.Location.loc_end) ;
-                              val_loc = { loc_impl = Some (!file_name, loc.Location.loc_start) ; loc_inter = None } ;
+                              val_code = Some (get_string_of_file loc.Location.loc_start.Lexing.pos_cnum loc.Location.loc_end.Lexing.pos_cnum) ;
+                              val_loc = { loc_impl = Some (!file_name, loc.Location.loc_start.Lexing.pos_cnum) ; loc_inter = None } ;
                             } ;
                 met_private = private_flag = Asttypes.Private ;
                 met_virtual = true ;
@@ -549,11 +577,11 @@ module Analyser =
             (* update the parameter description *)
             Odoc_value.update_value_parameters_text met.met_value;
 
-            iter acc_inher (acc_fields @ ele_comments @ [ Class_method met ]) loc.Location.loc_end q
+            iter acc_inher (acc_fields @ ele_comments @ [ Class_method met ]) loc.Location.loc_end.Lexing.pos_cnum q
 
         | (Parsetree.Pcf_meth  (label, private_flag, _, loc)) :: q ->
             let complete_name = Name.concat current_class_name label in
-            let (info_opt, ele_comments) = get_comments_in_class last_pos loc.Location.loc_start in
+            let (info_opt, ele_comments) = get_comments_in_class last_pos loc.Location.loc_start.Lexing.pos_cnum in
             let exp = 
               try Typedtree_search.search_method_expression tt_cls label
               with Not_found -> raise (Failure (Odoc_messages.method_not_found_in_typedtree complete_name))
@@ -573,8 +601,8 @@ module Analyser =
                               val_type = Odoc_env.subst_type env real_type ;
                               val_recursive = false ;
                               val_parameters = tt_analyse_method_expression env complete_name info_opt exp ;
-                              val_code = Some (get_string_of_file loc.Location.loc_start loc.Location.loc_end) ;
-                              val_loc = { loc_impl = Some (!file_name, loc.Location.loc_start) ; loc_inter = None } ;
+                              val_code = Some (get_string_of_file loc.Location.loc_start.Lexing.pos_cnum loc.Location.loc_end.Lexing.pos_cnum) ;
+                              val_loc = { loc_impl = Some (!file_name, loc.Location.loc_start.Lexing.pos_cnum) ; loc_inter = None } ;
                             } ;
                 met_private = private_flag = Asttypes.Private ;
                 met_virtual = false ;
@@ -583,18 +611,18 @@ module Analyser =
             (* update the parameter description *)
             Odoc_value.update_value_parameters_text met.met_value;
 
-            iter acc_inher (acc_fields @ ele_comments @ [ Class_method met ]) loc.Location.loc_end q
+            iter acc_inher (acc_fields @ ele_comments @ [ Class_method met ]) loc.Location.loc_end.Lexing.pos_cnum q
             
         | Parsetree.Pcf_cstr (_, _, loc) :: q ->
             (* don't give a $*%@ ! *)
-            iter acc_inher acc_fields loc.Location.loc_end q
+            iter acc_inher acc_fields loc.Location.loc_end.Lexing.pos_cnum q
 
         | Parsetree.Pcf_let (_, _, loc) :: q ->
             (* don't give a $*%@ ! *)
-            iter acc_inher acc_fields loc.Location.loc_end q
+            iter acc_inher acc_fields loc.Location.loc_end.Lexing.pos_cnum q
 
         | (Parsetree.Pcf_init exp) :: q ->
-            iter acc_inher acc_fields exp.Parsetree.pexp_loc.Location.loc_end q
+            iter acc_inher acc_fields exp.Parsetree.pexp_loc.Location.loc_end.Lexing.pos_cnum q
       in
       iter [] [] last_pos (snd p_cls)
               
@@ -640,7 +668,7 @@ module Analyser =
               current_class_name 
               tt_class_sig
               last_pos
-              p_class_expr.Parsetree.pcl_loc.Location.loc_end
+              p_class_expr.Parsetree.pcl_loc.Location.loc_end.Lexing.pos_cnum
               p_class_structure
               tt_class_structure
           in
@@ -711,8 +739,8 @@ module Analyser =
           let params_code = 
             List.map 
               (fun e -> get_string_of_file 
-                  e.exp_loc.Location.loc_start
-                  e.exp_loc.Location.loc_end)
+                  e.exp_loc.Location.loc_start.Lexing.pos_cnum
+                  e.exp_loc.Location.loc_end.Lexing.pos_cnum)
               param_exps
           in
           ([],
@@ -735,7 +763,7 @@ module Analyser =
             (*Sig.analyse_class_type_kind
               env
               ""
-              p_class_type2.Parsetree.pcty_loc.Location.loc_start
+              p_class_type2.Parsetree.pcty_loc.Location.loc_start.Lexing.pos_cnum
               p_class_type2
               tt_class_expr2.Typedtree.cl_type
             *)
@@ -751,7 +779,7 @@ module Analyser =
     let analyse_class env current_module_name comment_opt p_class_decl tt_type_params tt_class_exp =
       let name = p_class_decl.Parsetree.pci_name in
       let complete_name = Name.concat current_module_name name in
-      let pos_start = p_class_decl.Parsetree.pci_expr.Parsetree.pcl_loc.Location.loc_start in
+      let pos_start = p_class_decl.Parsetree.pci_expr.Parsetree.pcl_loc.Location.loc_start.Lexing.pos_cnum in
       let type_parameters = tt_type_params in
       let virt = p_class_decl.Parsetree.pci_virt = Asttypes.Virtual in
       let cltype = Odoc_env.subst_class_type env tt_class_exp.Typedtree.cl_type in
@@ -797,6 +825,7 @@ module Analyser =
                   { (* A VOIR : chercher dans les modules et les module types, avec quel env ? *)
                     im_name = tt_name_from_module_expr mod_expr ;
                     im_module = None ;
+		    im_info = None ;
                   } 
                 ] 
         | _ ->
@@ -811,7 +840,8 @@ module Analyser =
         | ([], _) ->
             []
         | ((Element_included_module im) :: q, (im_repl :: im_q)) ->
-            (Element_included_module im_repl) :: (f (q, im_q))
+            (Element_included_module { im_repl with im_info = im.im_info }) 
+	    :: (f (q, im_q))
         | ((Element_included_module im) :: q, []) ->
             (Element_included_module im) :: q
         | (ele :: q, l) ->
@@ -841,12 +871,12 @@ module Analyser =
             ele_comments
         | item :: q -> 
             let (comment_opt, ele_comments) = 
-              get_comments_in_module last_pos item.Parsetree.pstr_loc.Location.loc_start 
+              get_comments_in_module last_pos item.Parsetree.pstr_loc.Location.loc_start.Lexing.pos_cnum 
             in
             let pos_limit2 =
               match q with
                 [] -> pos_limit
-              | item2 :: _ -> item2.Parsetree.pstr_loc.Location.loc_start
+              | item2 :: _ -> item2.Parsetree.pstr_loc.Location.loc_start.Lexing.pos_cnum
             in
             let (maybe_more, new_env, elements) = analyse_structure_item
                 env
@@ -859,7 +889,7 @@ module Analyser =
                 table 
                 table_values
             in
-            ele_comments @ elements @ (iter new_env (item.Parsetree.pstr_loc.Location.loc_end + maybe_more) q)
+            ele_comments @ elements @ (iter new_env (item.Parsetree.pstr_loc.Location.loc_end.Lexing.pos_cnum + maybe_more) q)
       in
       iter env last_pos parsetree
 
@@ -889,7 +919,7 @@ module Analyser =
                 (acc_env, acc)
             | (pat, exp) :: q ->
                 let value_name_opt = iter_pat pat.Parsetree.ppat_desc in
-                let new_last_pos = exp.Parsetree.pexp_loc.Location.loc_end in
+                let new_last_pos = exp.Parsetree.pexp_loc.Location.loc_end.Lexing.pos_cnum in
                 match value_name_opt with
                   None ->
                     iter new_last_pos acc_env acc q
@@ -903,7 +933,7 @@ module Analyser =
                         else
                           get_comments_in_module
                             last_pos 
-                            pat.Parsetree.ppat_loc.Location.loc_start
+                            pat.Parsetree.ppat_loc.Location.loc_start.Lexing.pos_cnum
                       in
                       let l_values = tt_analyse_value 
                           env
@@ -930,7 +960,7 @@ module Analyser =
                       Not_found ->
                         iter new_last_pos acc_env acc q
           in
-          let (new_env, l_ele) = iter ~first: true loc.Location.loc_start env [] pat_exp_list in
+          let (new_env, l_ele) = iter ~first: true loc.Location.loc_start.Lexing.pos_cnum env [] pat_exp_list in
           (0, new_env, l_ele)
 
       | Parsetree.Pstr_primitive (name_pre, val_desc) ->
@@ -945,8 +975,8 @@ module Analyser =
              val_type = Odoc_env.subst_type env typ ;
              val_recursive = false ;
              val_parameters = [] ;
-             val_code = Some (get_string_of_file loc.Location.loc_start loc.Location.loc_end) ;
-             val_loc = { loc_impl = Some (!file_name, loc.Location.loc_start) ; loc_inter = None } ;
+             val_code = Some (get_string_of_file loc.Location.loc_start.Lexing.pos_cnum loc.Location.loc_end.Lexing.pos_cnum) ;
+             val_loc = { loc_impl = Some (!file_name, loc.Location.loc_start.Lexing.pos_cnum) ; loc_inter = None } ;
            }    
            in
           let new_env = Odoc_env.add_value env new_value.val_name in
@@ -969,12 +999,12 @@ module Analyser =
               [] -> (maybe_more_acc, [])
             | (name, type_decl) :: q ->
                 let complete_name = Name.concat current_module_name name in
-                let loc_start = type_decl.Parsetree.ptype_loc.Location.loc_start in
-                let loc_end =  type_decl.Parsetree.ptype_loc.Location.loc_end in
+                let loc_start = type_decl.Parsetree.ptype_loc.Location.loc_start.Lexing.pos_cnum in
+                let loc_end =  type_decl.Parsetree.ptype_loc.Location.loc_end.Lexing.pos_cnum in
                 let pos_limit2 = 
                   match q with 
                     [] -> pos_limit
-                  | (_, td) :: _ -> td.Parsetree.ptype_loc.Location.loc_start
+                  | (_, td) :: _ -> td.Parsetree.ptype_loc.Location.loc_start.Lexing.pos_cnum
                 in
                 let (maybe_more, name_comment_list) = 
                     Sig.name_comment_from_type_kind
@@ -996,22 +1026,34 @@ module Analyser =
                     new_env name_comment_list
                     tt_type_decl.Types.type_kind
                 in
+                let new_end = loc_end + maybe_more in
                 let t =
                   {
                     ty_name = complete_name ;
                     ty_info = com_opt ;
-                    ty_parameters = List.map
-                      (Odoc_env.subst_type new_env) 
-                      tt_type_decl.Types.type_params ;
+                    ty_parameters = 
+		      List.map2
+		        (fun p (co,cn,_) ->
+			  (Odoc_env.subst_type new_env p,
+			   co, cn)
+			)
+		      tt_type_decl.Types.type_params
+		      tt_type_decl.Types.type_variance ;
                     ty_kind = kind ;
                     ty_manifest =
                     (match tt_type_decl.Types.type_manifest with
                       None -> None
                     | Some t -> Some (Odoc_env.subst_type new_env t));
                     ty_loc = { loc_impl = Some (!file_name, loc_start) ; loc_inter = None } ;
+                    ty_code = 
+		      (
+		       if !Odoc_args.keep_code then
+			 Some (get_string_of_file loc_start new_end) 
+		       else
+			 None
+		      ) ;
                   } 
                 in
-                let new_end = loc_end + maybe_more in
                 let (maybe_more2, info_after_opt) = 
                   My_ir.just_after_special
                     !file_name
@@ -1021,7 +1063,7 @@ module Analyser =
                 let (maybe_more3, eles) = f (maybe_more + maybe_more2) (new_end + maybe_more2) q in
                 (maybe_more3, ele_comments @ ((Element_type t) :: eles))
           in
-          let (maybe_more, eles) = f ~first: true 0 loc.Location.loc_start name_typedecl_list in
+          let (maybe_more, eles) = f ~first: true 0 loc.Location.loc_start.Lexing.pos_cnum name_typedecl_list in
           (maybe_more, new_env, eles)
 
       | Parsetree.Pstr_exception (name, excep_decl) ->
@@ -1034,13 +1076,22 @@ module Analyser =
               raise (Failure (Odoc_messages.exception_not_found_in_typedtree complete_name))
           in
           let new_env = Odoc_env.add_exception env complete_name in
+          let loc_start = loc.Location.loc_start.Lexing.pos_cnum in
+          let loc_end =  loc.Location.loc_end.Lexing.pos_cnum in
           let new_ex = 
             {
               ex_name = complete_name ;
               ex_info = comment_opt ;
               ex_args = List.map (Odoc_env.subst_type new_env) tt_excep_decl ;
               ex_alias = None ;
-              ex_loc = { loc_impl = Some (!file_name, loc.Location.loc_start) ; loc_inter = None } ;
+              ex_loc = { loc_impl = Some (!file_name, loc.Location.loc_start.Lexing.pos_cnum) ; loc_inter = None } ;
+	      ex_code = 
+                (
+		 if !Odoc_args.keep_code then
+                   Some (get_string_of_file loc_start loc_end)
+                 else
+                   None
+		) ;
             } 
           in
           (0, new_env, [ Element_exception new_ex ])
@@ -1062,7 +1113,8 @@ module Analyser =
               ex_args = [] ;
               ex_alias = Some { ea_name = (Odoc_env.full_exception_name env (Name.from_path tt_path)) ;
                                 ea_ex = None ; } ;
-              ex_loc = { loc_impl = Some (!file_name, loc.Location.loc_start) ; loc_inter = None } ;
+              ex_loc = { loc_impl = Some (!file_name, loc.Location.loc_start.Lexing.pos_cnum) ; loc_inter = None } ;
+              ex_code = None ;
             } 
           in
           (0, new_env, [ Element_exception new_ex ])
@@ -1072,7 +1124,7 @@ module Analyser =
            (* of string * module_expr *)
            try
              let tt_module_expr = Typedtree_search.search_module table name in
-             let new_module = analyse_module 
+             let new_module_pre = analyse_module 
                  env
                  current_module_name
                  name
@@ -1080,6 +1132,18 @@ module Analyser =
                  module_expr
                  tt_module_expr
              in
+	     let code = 
+	       if !Odoc_args.keep_code then
+		 let loc = module_expr.Parsetree.pmod_loc in
+		 let st = loc.Location.loc_start.Lexing.pos_cnum in
+		 let en = loc.Location.loc_end.Lexing.pos_cnum in
+		 Some (get_string_of_file st en)
+	       else
+		 None
+	     in
+	     let new_module =
+	       { new_module_pre with m_code = code }
+	     in
              let new_env = Odoc_env.add_module env new_module.m_name in
              let new_env2 = 
                match new_module.m_type with
@@ -1097,6 +1161,72 @@ module Analyser =
                raise (Failure (Odoc_messages.module_not_found_in_typedtree complete_name))
           )
 
+      | Parsetree.Pstr_recmodule mods ->
+	  (* A VOIR ICI ca merde avec /work/tmp/graph.ml: pas de lien avec les module type 
+	     dans les contraintes sur les modules *)
+	  let new_env =
+            List.fold_left 
+              (fun acc_env (name, _, mod_exp) ->
+                let complete_name = Name.concat current_module_name name in
+		let e = Odoc_env.add_module acc_env complete_name in
+		let tt_mod_exp = 
+                  try Typedtree_search.search_module table name 
+                  with Not_found -> raise (Failure (Odoc_messages.module_not_found_in_typedtree complete_name))
+                in
+                let new_module = analyse_module 
+                    e
+                    current_module_name
+                    name
+                    None
+                    mod_exp
+                    tt_mod_exp
+		in
+		match new_module.m_type with
+                  Types.Tmty_signature s -> 
+                    Odoc_env.add_signature e new_module.m_name
+		      ~rel: (Name.simple new_module.m_name) s
+		  | _ -> 
+                      e
+              )
+              env
+              mods
+          in
+          let rec f ?(first=false) last_pos name_mod_exp_list =
+            match name_mod_exp_list with
+              [] -> []
+            | (name, _, mod_exp) :: q ->
+                let complete_name = Name.concat current_module_name name in
+                let loc_start = mod_exp.Parsetree.pmod_loc.Location.loc_start.Lexing.pos_cnum in
+                let loc_end =  mod_exp.Parsetree.pmod_loc.Location.loc_end.Lexing.pos_cnum in
+                let pos_limit2 = 
+                  match q with 
+                    [] -> pos_limit
+                  | (_, _, me) :: _ -> me.Parsetree.pmod_loc.Location.loc_start.Lexing.pos_cnum
+                in
+                let tt_mod_exp = 
+                  try Typedtree_search.search_module table name 
+                  with Not_found -> raise (Failure (Odoc_messages.module_not_found_in_typedtree complete_name))
+                in
+                let (com_opt, ele_comments) = (* the comment for the first type was already retrieved *)
+                  if first then
+                    (comment_opt, [])
+                  else
+                    get_comments_in_module last_pos loc_start
+                in
+		let new_module = analyse_module 
+                    new_env
+                    current_module_name
+                    name
+                    com_opt
+                    mod_exp
+                    tt_mod_exp
+		in
+                let eles = f loc_end q in
+                ele_comments @ ((Element_module new_module) :: eles)
+          in
+          let eles = f ~first: true loc.Location.loc_start.Lexing.pos_cnum mods in
+          (0, new_env, eles)
+
       | Parsetree.Pstr_modtype (name, modtype) ->
           let complete_name = Name.concat current_module_name name in
           let tt_module_type =
@@ -1115,7 +1245,7 @@ module Analyser =
               mt_is_interface = false ;
               mt_file = !file_name ;
               mt_kind = Some kind ; 
-              mt_loc = { loc_impl = Some (!file_name, loc.Location.loc_start) ; loc_inter = None } ;
+              mt_loc = { loc_impl = Some (!file_name, loc.Location.loc_start.Lexing.pos_cnum) ; loc_inter = None } ;
             } 
           in
           let new_env = Odoc_env.add_module_type env mt.mt_name in
@@ -1166,9 +1296,9 @@ module Analyser =
                   if first then
                     (comment_opt, [])
                   else
-                    get_comments_in_module last_pos class_decl.Parsetree.pci_loc.Location.loc_start 
+                    get_comments_in_module last_pos class_decl.Parsetree.pci_loc.Location.loc_start.Lexing.pos_cnum 
                 in
-                let last_pos2 = class_decl.Parsetree.pci_loc.Location.loc_end in
+                let last_pos2 = class_decl.Parsetree.pci_loc.Location.loc_end.Lexing.pos_cnum in
                 let new_class = analyse_class 
                     new_env
                     current_module_name
@@ -1179,7 +1309,7 @@ module Analyser =
                 in
                 ele_comments @ ((Element_class new_class) :: (f last_pos2 q))
           in
-          (0, new_env, f ~first: true loc.Location.loc_start class_decl_list)
+          (0, new_env, f ~first: true loc.Location.loc_start.Lexing.pos_cnum class_decl_list)
 
       | Parsetree.Pstr_class_type class_type_decl_list ->
           (* we start by extending the environment *)
@@ -1209,7 +1339,7 @@ module Analyser =
                 let kind = Sig.analyse_class_type_kind
                     new_env
                     complete_name
-                    class_type_decl.Parsetree.pci_loc.Location.loc_start
+                    class_type_decl.Parsetree.pci_loc.Location.loc_start.Lexing.pos_cnum
                     class_type_decl.Parsetree.pci_expr
                     tt_cltype_declaration.Types.clty_type
                 in
@@ -1217,9 +1347,9 @@ module Analyser =
                   if first then
                     (comment_opt, [])
                   else
-                    get_comments_in_module last_pos class_type_decl.Parsetree.pci_loc.Location.loc_start 
+                    get_comments_in_module last_pos class_type_decl.Parsetree.pci_loc.Location.loc_start.Lexing.pos_cnum 
                 in
-                let last_pos2 = class_type_decl.Parsetree.pci_loc.Location.loc_end in
+                let last_pos2 = class_type_decl.Parsetree.pci_loc.Location.loc_end.Lexing.pos_cnum in
                 let new_ele =
                   Element_class_type
                     {
@@ -1229,13 +1359,13 @@ module Analyser =
                       clt_type_parameters = List.map (Odoc_env.subst_type new_env) type_params ;
                       clt_virtual = virt ;
                       clt_kind = kind ;
-                      clt_loc = { loc_impl = Some (!file_name, loc.Location.loc_start) ; 
+                      clt_loc = { loc_impl = Some (!file_name, loc.Location.loc_start.Lexing.pos_cnum) ; 
                                   loc_inter = None } ;
                     } 
                 in
                 ele_comments @ (new_ele :: (f last_pos2 q))
           in
-          (0, new_env, f ~first: true loc.Location.loc_start class_type_decl_list)
+          (0, new_env, f ~first: true loc.Location.loc_start.Lexing.pos_cnum class_type_decl_list)
 
       | Parsetree.Pstr_include module_expr ->
           (* we add a dummy included module which will be replaced by a correct
@@ -1245,6 +1375,7 @@ module Analyser =
             {
               im_name = "dummy" ;
               im_module = None ;
+	      im_info = comment_opt ;
             } 
           in
           (0, env, [ Element_included_module im ]) (* A VOIR : tendre l'environnement ? avec quoi ? *)
@@ -1252,19 +1383,33 @@ module Analyser =
      (** Analysis of a [Parsetree.module_expr] and a name to return a [t_module].*)
      and analyse_module env current_module_name module_name comment_opt p_module_expr tt_module_expr =
       let complete_name = Name.concat current_module_name module_name in
-      let pos_start = p_module_expr.Parsetree.pmod_loc.Location.loc_start in
-      let pos_end = p_module_expr.Parsetree.pmod_loc.Location.loc_end in
-      let modtype = tt_module_expr.Typedtree.mod_type in
+      let pos_start = p_module_expr.Parsetree.pmod_loc.Location.loc_start.Lexing.pos_cnum in
+      let pos_end = p_module_expr.Parsetree.pmod_loc.Location.loc_end.Lexing.pos_cnum in
+      let modtype = 
+        (* A VOIR : Odoc_env.subst_module_type env  ? *)
+	tt_module_expr.Typedtree.mod_type 
+      in
+      let m_code_intf =
+	match p_module_expr.Parsetree.pmod_desc with
+	  Parsetree.Pmod_constraint (_, pmodule_type) ->
+	    let loc_start = pmodule_type.Parsetree.pmty_loc.Location.loc_start.Lexing.pos_cnum in
+            let loc_end = pmodule_type.Parsetree.pmty_loc.Location.loc_end.Lexing.pos_cnum in
+	    Some (get_string_of_file loc_start loc_end)
+	| _ ->
+	    None
+      in
       let m_base =
         {
           m_name = complete_name ;
-          m_type = tt_module_expr.Typedtree.mod_type ;
+          m_type = modtype ;
           m_info = comment_opt ;
           m_is_interface = false ;
           m_file = !file_name ;
           m_kind = Module_struct [] ;
           m_loc = { loc_impl = Some (!file_name, pos_start) ; loc_inter = None } ;
           m_top_deps = [] ;
+	  m_code = None ; (* code is set by the caller, after the module is created *)
+	  m_code_intf = m_code_intf ;
       } 
       in
       match (p_module_expr.Parsetree.pmod_desc, tt_module_expr.Typedtree.mod_desc) with
@@ -1280,33 +1425,45 @@ module Analyser =
           let elements2 = replace_dummy_included_modules elements included_modules_from_tt in
           { m_base with m_kind = Module_struct elements2 }
 
-      | (Parsetree.Pmod_functor (_, _, p_module_expr2), 
+      | (Parsetree.Pmod_functor (_, pmodule_type, p_module_expr2), 
          Typedtree.Tmod_functor (ident, mtyp, tt_module_expr2)) ->
-          let param =
-            {
-              mp_name = Name.from_ident ident ;
-              mp_type = Odoc_env.subst_module_type env mtyp ;              
-            } 
-          in
-          let dummy_complete_name = Name.concat "__" param.mp_name in
-          let new_env = Odoc_env.add_module env dummy_complete_name in
-          let m_base2 = analyse_module 
-              new_env
-              current_module_name
-              module_name
-              None
-              p_module_expr2
-              tt_module_expr2
-          in
-          let kind = 
-            match m_base2.m_kind with
-              Module_functor (params, k) -> Module_functor (param :: params, m_base2.m_kind)
-            | k -> Module_functor ([param], k)
-          in
-          { m_base with m_kind = kind }
+	   let loc_start = pmodule_type.Parsetree.pmty_loc.Location.loc_start.Lexing.pos_cnum in
+           let loc_end = pmodule_type.Parsetree.pmty_loc.Location.loc_end.Lexing.pos_cnum in
+	   let mp_type_code = get_string_of_file loc_start loc_end in
+	   print_DEBUG (Printf.sprintf "mp_type_code=%s" mp_type_code);
+	   let mp_name = Name.from_ident ident in
+	   let mp_kind = Sig.analyse_module_type_kind env 
+	       current_module_name pmodule_type mtyp
+	   in
+           let param =
+             {
+               mp_name = mp_name ;
+               mp_type = Odoc_env.subst_module_type env mtyp ;
+	       mp_type_code = mp_type_code ;
+	       mp_kind = mp_kind ;
+             } 
+           in
+           let dummy_complete_name = (*Name.concat "__"*) param.mp_name in
+	   (* TODO: A VOIR CE __ *)
+           let new_env = Odoc_env.add_module env dummy_complete_name in
+           let m_base2 = analyse_module 
+               new_env
+               current_module_name
+               module_name
+               None
+               p_module_expr2
+               tt_module_expr2
+           in
+           let kind = m_base2.m_kind in
+           { m_base with m_kind = Module_functor (param, kind) }
 
       | (Parsetree.Pmod_apply (p_module_expr1, p_module_expr2), 
-         Typedtree.Tmod_apply (tt_module_expr1, tt_module_expr2, _)) ->
+         Typedtree.Tmod_apply (tt_module_expr1, tt_module_expr2, _))
+      | (Parsetree.Pmod_apply (p_module_expr1, p_module_expr2),
+         Typedtree.Tmod_constraint 
+	   ({ Typedtree.mod_desc = Typedtree.Tmod_apply (tt_module_expr1, tt_module_expr2, _)}, 
+            _, _)
+	) ->
           let m1 = analyse_module 
               env
               current_module_name
@@ -1327,6 +1484,8 @@ module Analyser =
 
       | (Parsetree.Pmod_constraint (p_module_expr2, p_modtype), 
          Typedtree.Tmod_constraint (tt_module_expr2, tt_modtype, _)) ->
+	  print_DEBUG ("Odoc_ast: case Parsetree.Pmod_constraint + Typedtree.Tmod_constraint "^module_name);
+
           (* we create the module with p_module_expr2 and tt_module_expr2 
              but we change its type according to the constraint. 
              A VOIR : est-ce que c'est bien ?
@@ -1346,7 +1505,7 @@ module Analyser =
           in
           { 
             m_base with
-            m_type = tt_modtype ; 
+            m_type = Odoc_env.subst_module_type env tt_modtype ; 
             m_kind = Module_constraint (m_base2.m_kind, 
                                         mtkind)
 
@@ -1355,7 +1514,43 @@ module Analyser =
 *)
           }
  
-      | _ ->
+      | (Parsetree.Pmod_structure p_structure,
+         Typedtree.Tmod_constraint 
+	   ({ Typedtree.mod_desc = Typedtree.Tmod_structure tt_structure}, 
+	    tt_modtype, _)
+	) ->
+	  (* needed for recursive modules *)
+
+	  print_DEBUG ("Odoc_ast: case Parsetree.Pmod_structure + Typedtree.Tmod_constraint "^module_name);
+	  let elements = analyse_structure env complete_name pos_start pos_end p_structure tt_structure in
+          (* we must complete the included modules *)
+          let included_modules_from_tt = tt_get_included_module_list tt_structure in
+          let elements2 = replace_dummy_included_modules elements included_modules_from_tt in
+          { m_base with 
+	    m_type = Odoc_env.subst_module_type env tt_modtype ;
+	    m_kind = Module_struct elements2 ;
+	  }
+
+      | (parsetree, typedtree) ->
+          let s_parse = 
+            match parsetree with
+              Parsetree.Pmod_ident _ -> "Pmod_ident"
+	    | Parsetree.Pmod_structure _ -> "Pmod_structure"
+	    | Parsetree.Pmod_functor _ -> "Pmod_functor"
+	    | Parsetree.Pmod_apply _ -> "Pmod_apply"
+	    | Parsetree.Pmod_constraint _ -> "Pmod_constraint"
+	  in
+	  let s_typed = 
+            match typedtree with
+              Typedtree.Tmod_ident _ -> "Tmod_ident"
+	    | Typedtree.Tmod_structure _ -> "Tmod_structure"
+	    | Typedtree.Tmod_functor _ -> "Tmod_functor"
+	    | Typedtree.Tmod_apply _ -> "Tmod_apply"
+	    | Typedtree.Tmod_constraint _ -> "Tmod_constraint"
+	  in
+	  let code = get_string_of_file pos_start pos_end in
+	  print_DEBUG (Printf.sprintf "code=%s\ns_parse=%s\ns_typed=%s\n" code s_parse s_typed);
+	   
           raise (Failure "analyse_module: parsetree and typedtree don't match.")
 
      let analyse_typed_tree source_file input_file 
@@ -1385,20 +1580,20 @@ module Analyser =
        let included_modules_from_tt = tt_get_included_module_list tree_structure in
        let elements2 = replace_dummy_included_modules elements included_modules_from_tt in
        let kind = Module_struct elements2 in
-       let m =
-         {
-           m_name = mod_name ;
-           m_type = Types.Tmty_signature [] ;
-           m_info = info_opt ;
-           m_is_interface = false ;
-           m_file = !file_name ;
-           m_kind = kind ;
-           m_loc = { loc_impl = Some (!file_name, 0) ; loc_inter = None } ;
-           m_top_deps = [] ;
-         } 
-       in
-       m
+       {
+         m_name = mod_name ;
+         m_type = Types.Tmty_signature [] ;
+         m_info = info_opt ;
+         m_is_interface = false ;
+         m_file = !file_name ;
+         m_kind = kind ;
+         m_loc = { loc_impl = Some (!file_name, 0) ; loc_inter = None } ;
+         m_top_deps = [] ;
+	 m_code = (if !Odoc_args.keep_code then Some !file else None) ;
+	 m_code_intf = None ;
+       } 
   end
 
 
 
+(* eof $Id: odoc_ast.ml,v 1.24 2004/04/17 12:36:14 guesdon Exp $ *)
