@@ -9,6 +9,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
+(* $Id: odoc_merge.ml,v 1.10 2004/01/18 14:26:14 guesdon Exp $ *)
 
 (** Merge of information from [.ml] and [.mli] for a module.*)
 
@@ -189,11 +190,13 @@ let merge_info_opt merge_options mli_opt ml_opt =
 let merge_types merge_options mli ml =
   mli.ty_info <- merge_info_opt merge_options mli.ty_info ml.ty_info;
   mli.ty_loc <- { mli.ty_loc with loc_impl = ml.ty_loc.loc_impl } ;
+  mli.ty_code <- (match mli.ty_code with None -> ml.ty_code | _ -> mli.ty_code) ;
+
   match mli.ty_kind, ml.ty_kind with
     Type_abstract, _ ->
       ()
 
-  | Type_variant l1, Type_variant l2 ->
+  | Type_variant (l1, _), Type_variant (l2, _) ->
       let f cons =
         try
           let cons2 = List.find
@@ -221,7 +224,7 @@ let merge_types merge_options mli ml =
       in
       List.iter f l1
 
-  | Type_record l1, Type_record l2 ->
+  | Type_record (l1, _), Type_record (l2, _) ->
       let f record =
         try
           let record2= List.find
@@ -472,6 +475,7 @@ let rec merge_module_types merge_options mli ml =
                     (
                      ex.ex_info <- merge_info_opt merge_options ex.ex_info ex2.ex_info;
                      ex.ex_loc <- { ex.ex_loc with loc_impl = ex2.ex_loc.loc_impl } ;
+                     ex.ex_code <- (match ex.ex_code with None -> ex2.ex_code | _ -> ex.ex_code) ;
                      true
                     )
                   else
@@ -680,6 +684,28 @@ and merge_modules merge_options mli ml =
   mli.m_loc <- { mli.m_loc with loc_impl = ml.m_loc.loc_impl } ;
   (* More dependencies in the .ml file. *)
   mli.m_top_deps <- ml.m_top_deps ;
+  
+  let code = 
+    if !Odoc_args.keep_code then
+      match mli.m_code, ml.m_code with
+	Some s, _ -> Some s
+      |	_, Some s -> Some s
+      |	_ -> None
+    else
+      None
+  in
+  let code_intf = 
+    if !Odoc_args.keep_code then
+      match mli.m_code_intf, ml.m_code_intf with
+	Some s, _ -> Some s
+      |	_, Some s -> Some s
+      |	_ -> None
+    else
+      None
+  in
+  mli.m_code <- code;
+  mli.m_code_intf <- code_intf;
+
   (* merge exceptions *)
   List.iter
     (fun ex ->
@@ -692,6 +718,7 @@ and merge_modules merge_options mli ml =
                     (
                      ex.ex_info <- merge_info_opt merge_options ex.ex_info ex2.ex_info;
                      ex.ex_loc <- { ex.ex_loc with loc_impl = ex.ex_loc.loc_impl } ;
+                     ex.ex_code <- (match ex.ex_code with None -> ex2.ex_code | _ -> ex.ex_code) ;
                      true
                     )
                   else
@@ -933,3 +960,4 @@ let merge merge_options modules_list =
   in
   iter modules_list
   
+(* eof $Id: odoc_merge.ml,v 1.10 2004/01/18 14:26:14 guesdon Exp $ *)
