@@ -9,6 +9,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
+(* $Id: odoc_search.ml,v 1.6 2004/05/23 10:41:50 guesdon Exp $ *)
 
 (** Research of elements through modules. *)
 
@@ -78,7 +79,9 @@ module Search =
       | T.Link (_, t) -> search_text root t v
       | T.List l 
       | T.Enum l -> List.flatten (List.map (fun t -> search_text root t v) l)
-      | T.Newline -> []
+      | T.Newline 
+      |	T.Module_list _
+      |	T.Index_list -> []
       | T.Title (n, l_opt, t) -> 
           (match l_opt with
             None -> []
@@ -533,3 +536,99 @@ let module_types l =
     | [] -> acc
   in
   iter [] l_ele
+
+let type_exists mods regexp =
+  let l = Search_by_name.search mods regexp in
+  List.exists
+    (function
+	Res_type _ -> true
+      | _ -> false
+    )
+    l
+
+let value_exists mods regexp =
+  let l = Search_by_name.search mods regexp in
+  List.exists
+    (function
+	Res_value _ -> true
+      | _ -> false
+    )
+    l
+
+let class_exists mods regexp =
+  let l = Search_by_name.search mods regexp in
+  List.exists
+    (function
+	Res_class _ -> true
+      | _ -> false
+    )
+    l
+
+let class_type_exists mods regexp =
+  let l = Search_by_name.search mods regexp in
+  List.exists
+    (function
+	Res_class_type _ -> true
+      | _ -> false
+    )
+    l
+
+let module_exists mods regexp =
+  let l = Search_by_name.search mods regexp in
+  List.exists
+    (function
+	Res_module _ -> true
+      | _ -> false
+    )
+    l
+
+let module_type_exists mods regexp =
+  let l = Search_by_name.search mods regexp in
+  List.exists
+    (function
+	Res_module_type _ -> true
+      | _ -> false
+    )
+    l
+
+let exception_exists mods regexp =
+  let l = Search_by_name.search mods regexp in
+  List.exists
+    (function
+	Res_exception _ -> true
+      | _ -> false
+    )
+    l
+
+let attribute_exists mods regexp =
+  let l = Search_by_name.search mods regexp in
+  List.exists
+    (function
+	Res_attribute _ -> true
+      | _ -> false
+    )
+    l
+
+let method_exists mods regexp =
+  let l = Search_by_name.search mods regexp in
+  List.exists
+    (function
+	Res_method _ -> true
+      | _ -> false
+    )
+    l
+
+let find_section mods regexp =
+  let l = Search_by_name.search mods regexp in
+  match 
+    List.find
+      (function
+	  Res_section _ -> true
+	| _ -> false
+      )
+      l
+  with
+    Res_section (_,t) -> t
+  | _ -> assert false
+
+(* eof $Id: odoc_search.ml,v 1.6 2004/05/23 10:41:50 guesdon Exp $ *)
