@@ -11,12 +11,13 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: mmap_win32.c,v 1.3 2001/12/07 13:39:50 xleroy Exp $ */
+/* $Id: mmap_win32.c,v 1.4 2002/06/07 09:49:38 xleroy Exp $ */
 
 #include <stddef.h>
 #include <stdio.h>
 #include <string.h>
 #include "bigarray.h"
+#include "alloc.h"
 #include "custom.h"
 #include "fail.h"
 #include "mlvalues.h"
@@ -110,6 +111,6 @@ static void bigarray_sys_error(void)
                      buffer,
                      sizeof(buffer),
                      NULL))
-    sprintf(buffer, "Unknown error %d\n", errnum);
+    sprintf(buffer, "Unknown error %ld\n", errnum);
   raise_sys_error(copy_string(buffer));
 }
