@@ -12,7 +12,7 @@
 (*                                                                       *)
 (*************************************************************************)
 
-(* $Id: jg_message.ml,v 1.16 2001/12/07 13:39:59 xleroy Exp $ *)
+(* $Id: jg_message.ml,v 1.18 2002/07/26 00:04:05 garrigue Exp $ *)
 
 open StdLabels
 open Tk
@@ -87,13 +87,14 @@ let ask ~title ?master ?(no=true) ?(cancel=true) text =
            ~width:250 ~justify:`Left ~aspect:400 ~anchor:`W
   and fw = Frame.create tl
   and sync = Textvariable.create ~on:tl ()
-  and r = ref (`cancel : [`yes|`no|`cancel]) in
-  let accept = Button.create fw ~text:(if no then "Yes" else "Dismiss")
-      ~command:(fun () -> r := `yes; destroy tl) 
+  and r = ref (`Cancel : [`Yes|`No|`Cancel]) in
+  let accept = Button.create fw
+      ~text:(if no || cancel then "Yes" else "Dismiss")
+      ~command:(fun () -> r := `Yes; destroy tl) 
   and refuse = Button.create fw ~text:"No"
-      ~command:(fun () -> r := `no; destroy tl)
+      ~command:(fun () -> r := `No; destroy tl)
   and cancelB = Button.create fw ~text:"Cancel"
-      ~command:(fun () -> r := `cancel; destroy tl)
+      ~command:(fun () -> r := `Cancel; destroy tl)
   in
   bind tl ~events:[`Destroy] ~extend:true
     ~action:(fun _ -> Textvariable.set sync "1");
@@ -105,3 +106,6 @@ let ask ~title ?master ?(no=true) ?(cancel=true) text =
   Grab.set tl;
   Tkwait.variable sync;
   !r
+
+let info ~title ?master text =
+  ignore (ask ~title ?master ~no:false ~cancel:false text)
