@@ -12,7 +12,7 @@
 (*                                                                       *)
 (*************************************************************************)
 
-(* $Id: lexical.ml,v 1.14 2001/12/07 13:40:00 xleroy Exp $ *)
+(* $Id: lexical.ml,v 1.15 2002/11/01 17:06:45 doligez Exp $ *)
 
 open StdLabels
 open Tk
@@ -140,4 +140,4 @@ let tag ?(start=tstart) ?(stop=tend) tw =
     done
   with
     End_of_file -> ()
-  | Lexer.Error (err, s, e) -> ()
+  | Lexer.Error (err, loc) -> ()
