@@ -12,7 +12,7 @@
 (*                                                                       *)
 (*************************************************************************)
 
-(* $Id: searchpos.ml,v 1.30 2002/07/11 11:09:30 garrigue Exp $ *)
+(* $Id: searchpos.ml,v 1.44.2.1 2004/06/29 01:50:19 garrigue Exp $ *)
 
 open StdLabels
 open Support
@@ -39,11 +39,12 @@ let lines_to_chars n ~text:s =
   in ltc n ~pos:0
 
 let in_loc loc ~pos =
-  loc.loc_ghost || pos >= loc.loc_start && pos < loc.loc_end
+  loc.loc_ghost || pos >= loc.loc_start.Lexing.pos_cnum
+                   && pos < loc.loc_end.Lexing.pos_cnum
 
 let le_loc loc1 loc2 =
-  loc1.loc_start <= loc2.loc_start
-  && loc1.loc_end >= loc2.loc_end
+  loc1.loc_start.Lexing.pos_cnum <= loc2.loc_start.Lexing.pos_cnum
+  && loc1.loc_end.Lexing.pos_cnum >= loc2.loc_end.Lexing.pos_cnum
 
 let add_found ~found sol ~env ~loc =
   if loc.loc_ghost then () else
@@ -164,14 +165,19 @@ let search_pos_type_decl td ~pos ~env =
       Some t -> search_pos_type t ~pos ~env
     | None -> ()
     end;
-    begin match td.ptype_kind with
+    let rec search_tkind = function
       Ptype_abstract -> ()
-    | Ptype_variant dl ->
+    | Ptype_variant (dl, _) ->
         List.iter dl
           ~f:(fun (_, tl) -> List.iter tl ~f:(search_pos_type ~pos ~env))
-    | Ptype_record dl ->
-        List.iter dl ~f:(fun (_, _, t) -> search_pos_type t ~pos ~env)
-    end
+    | Ptype_record (dl, _) ->
+        List.iter dl ~f:(fun (_, _, t) -> search_pos_type t ~pos ~env) in
+    search_tkind td.ptype_kind;
+    List.iter td.ptype_cstrs ~f:
+      begin fun (t1, t2, _) ->
+        search_pos_type t1 ~pos ~env;
+        search_pos_type t2 ~pos ~env
+      end
   end
   
 let rec search_pos_signature l ~pos ~env =
@@ -200,6 +206,8 @@ let rec search_pos_signature l ~pos ~env =
           add_found_sig (`Type, Lident "exn") ~env ~loc:pt.psig_loc
       | Psig_module (_, t) -> 
           search_pos_module t ~pos ~env
+      | Psig_recmodule decls ->
+          List.iter decls ~f:(fun (_, t) -> search_pos_module t ~pos ~env)
       | Psig_modtype (_, Pmodtype_manifest t) ->
           search_pos_module t ~pos ~env
       | Psig_modtype _ -> ()
@@ -282,12 +290,12 @@ let edit_source ~file ~path ~sign =
       let id, kind =
         match item with
           Tsig_value (id, _) -> id, Pvalue
-        | Tsig_type (id, _) -> id, Ptype
+        | Tsig_type (id, _, _) -> id, Ptype
         | Tsig_exception (id, _) -> id, Pconstructor
-        | Tsig_module (id, _) -> id, Pmodule
+        | Tsig_module (id, _, _) -> id, Pmodule
         | Tsig_modtype (id, _) -> id, Pmodtype
-        | Tsig_class (id, _) -> id, Pclass
-        | Tsig_cltype (id, _) -> id, Pcltype
+        | Tsig_class (id, _, _) -> id, Pclass
+        | Tsig_cltype (id, _, _) -> id, Pcltype
       in
       let prefix = List.tl (list_of_path path) and name = Ident.name id in
       let pos =
@@ -308,6 +316,8 @@ let edit_source ~file ~path ~sign =
 (* List of windows to destroy by Close All *)
 let top_widgets = ref []
 
+let dummy_item = Tsig_modtype (Ident.create "dummy", Tmodtype_abstract)
+
 let rec view_signature ?title ?path ?(env = !start_env) ?(detach=false) sign =
   let env =
     match path with None -> env
@@ -321,7 +331,7 @@ let rec view_signature ?title ?path ?(env = !start_env) ?(detach=false) sign =
     try match path, !default_frame with
       None, Some ({mw_title=Some label} as mw) when not detach ->
         Button.configure mw.mw_detach
-          ~command:(fun () -> view_signature sign ~title ~env);
+          ~command:(fun () -> view_signature sign ~title ~env ~detach:true);
         pack [mw.mw_detach] ~side:`Left;
         Pack.forget [mw.mw_edit; mw.mw_intf];
         List.iter ~f:destroy (Winfo.children mw.mw_frame);
@@ -336,7 +346,7 @@ let rec view_signature ?title ?path ?(env = !start_env) ?(detach=false) sign =
             view_module path ~env;
             find_shown_module path
         in
-        !set_path path ~sign;
+        (try !set_path path ~sign with _ -> ());
         begin match mw.mw_title with None -> ()
         | Some label ->
             Label.configure label ~text:title;
@@ -351,8 +361,8 @@ let rec view_signature ?title ?path ?(env = !start_env) ?(detach=false) sign =
             try
               let id = head_id path in
               let file =
-                Misc.find_in_path !Config.load_path
-                  (String.uncapitalize (Ident.name id) ^ ext) in
+                Misc.find_in_path_uncap !Config.load_path
+                  ((Ident.name id) ^ ext) in
               Button.configure button
                 ~command:(fun () -> edit_source ~file ~path ~sign);
               if !repack then Pack.forget [button] else
@@ -385,9 +395,11 @@ let rec view_signature ?title ?path ?(env = !start_env) ?(detach=false) sign =
             Syntaxerr.Unclosed(l,_,_,_) -> l
           | Syntaxerr.Other l -> l
         in
-        Jg_text.tag_and_see  tw ~start:(tpos l.loc_start)
-          ~stop:(tpos l.loc_end) ~tag:"error"; []
-      | Lexer.Error (_, s, e) ->
+        Jg_text.tag_and_see  tw ~start:(tpos l.loc_start.Lexing.pos_cnum)
+          ~stop:(tpos l.loc_end.Lexing.pos_cnum) ~tag:"error"; []
+      | Lexer.Error (_, l) ->
+         let s = l.loc_start.Lexing.pos_cnum in
+         let e = l.loc_end.Lexing.pos_cnum in
          Jg_text.tag_and_see tw ~start:(tpos s) ~stop:(tpos e) ~tag:"error"; []
   in
   Jg_bind.enter_focus tw;
@@ -428,7 +440,7 @@ and view_module path ~env =
       !view_defined_ref (Searchid.longident_of_path path) ~env
   | modtype ->
       let id = ident_of_path path ~default:"M" in
-      view_signature_item [Tsig_module (id, modtype)] ~path ~env
+      view_signature_item [Tsig_module (id, modtype, Trec_not)] ~path ~env
 
 and view_module_id id ~env =
   let path, _ = lookup_module id env in
@@ -441,11 +453,12 @@ and view_type_decl path ~env =
         {desc = Tobject _} ->
           let clt = find_cltype path env in
           view_signature_item ~path ~env
-            [Tsig_cltype(ident_of_path path ~default:"ct", clt)]
+            [Tsig_cltype(ident_of_path path ~default:"ct", clt, Trec_first);
+             dummy_item; dummy_item]
       | _ -> raise Not_found
   with Not_found ->
     view_signature_item ~path ~env
-      [Tsig_type(ident_of_path path ~default:"t", td)]
+      [Tsig_type(ident_of_path path ~default:"t", td, Trec_first)]
 
 and view_type_id li ~env =
   let path, decl = lookup_type li env in
@@ -454,12 +467,14 @@ and view_type_id li ~env =
 and view_class_id li ~env =
   let path, cl = lookup_class li env in
   view_signature_item ~path ~env
-     [Tsig_class(ident_of_path path ~default:"c", cl)]
+     [Tsig_class(ident_of_path path ~default:"c", cl, Trec_first);
+      dummy_item; dummy_item; dummy_item]
 
 and view_cltype_id li ~env =
   let path, clt = lookup_cltype li env in
   view_signature_item ~path ~env
-     [Tsig_cltype(ident_of_path path ~default:"ct", clt)]
+     [Tsig_cltype(ident_of_path path ~default:"ct", clt, Trec_first);
+      dummy_item; dummy_item]
 
 and view_modtype_id li ~env =
   let path, td = lookup_modtype li env in
@@ -566,19 +581,19 @@ let view_type kind ~env =
       | `New path ->
           let cl = find_class path env in
           view_signature_item ~path ~env
-            [Tsig_class(ident_of_path path ~default:"c", cl)]
+            [Tsig_class(ident_of_path path ~default:"c", cl, Trec_first)]
       end
   | `Class (path, cty) ->
       let cld = { cty_params = []; cty_type = cty;
                   cty_path = path; cty_new = None } in
       view_signature_item ~path ~env
-        [Tsig_class(ident_of_path path ~default:"c", cld)]
+        [Tsig_class(ident_of_path path ~default:"c", cld, Trec_first)]
   | `Module (path, mty) ->
       match mty with
         Tmty_signature sign -> view_signature sign ~path ~env
       | modtype ->
           view_signature_item ~path ~env
-            [Tsig_module(ident_of_path path ~default:"M", mty)]
+            [Tsig_module(ident_of_path path ~default:"M", mty, Trec_not)]
 
 let view_type_menu kind ~env ~parent =
   let title =
@@ -655,6 +670,8 @@ let rec search_pos_structure ~pos str =
   | Tstr_exception _ -> ()
   | Tstr_exn_rebind(_, _) -> ()
   | Tstr_module (_, m) -> search_pos_module_expr m ~pos
+  | Tstr_recmodule bindings ->
+      List.iter bindings ~f:(fun (_, m) -> search_pos_module_expr m ~pos)
   | Tstr_modtype _ -> ()
   | Tstr_open _ -> ()
   | Tstr_class l ->
@@ -663,6 +680,23 @@ let rec search_pos_structure ~pos str =
   | Tstr_include (m, _) -> search_pos_module_expr m ~pos
   end
 
+and search_pos_class_structure ~pos cls =
+  List.iter cls.cl_field ~f:
+    begin function
+        Cf_inher (cl, _, _) ->
+          search_pos_class_expr cl ~pos
+      | Cf_val (_, _, exp) -> search_pos_expr exp ~pos
+      | Cf_meth (_, exp) -> search_pos_expr exp ~pos
+      | Cf_let (_, pel, iel) ->
+          List.iter pel ~f:
+            begin fun (pat, exp) ->
+              search_pos_pat pat ~pos ~env:exp.exp_env;
+              search_pos_expr exp ~pos
+            end;
+          List.iter iel ~f:(fun (_,exp) -> search_pos_expr exp ~pos)
+      | Cf_init exp -> search_pos_expr exp ~pos
+    end
+
 and search_pos_class_expr ~pos cl =
   if in_loc cl.cl_loc ~pos then begin
     begin match cl.cl_desc with
@@ -670,21 +704,7 @@ and search_pos_class_expr ~pos cl =
         add_found_str (`Class (path, cl.cl_type))
           ~env:!start_env ~loc:cl.cl_loc
     | Tclass_structure cls ->
-        List.iter cls.cl_field ~f:
-          begin function
-              Cf_inher (cl, _, _) ->
-                search_pos_class_expr cl ~pos
-            | Cf_val (_, _, exp) -> search_pos_expr exp ~pos
-            | Cf_meth (_, exp) -> search_pos_expr exp ~pos
-            | Cf_let (_, pel, iel) ->
-                List.iter pel ~f:
-                  begin fun (pat, exp) ->
-                    search_pos_pat pat ~pos ~env:exp.exp_env;
-                    search_pos_expr exp ~pos
-                  end;
-                List.iter iel ~f:(fun (_,exp) -> search_pos_expr exp ~pos)
-            | Cf_init exp -> search_pos_expr exp ~pos
-          end
+	search_pos_class_structure ~pos cls
     | Tclass_fun (pat, iel, cl, _) ->
         search_pos_pat pat ~pos ~env:pat.pat_env;
         List.iter iel ~f:(fun (_,exp) -> search_pos_expr exp ~pos);
@@ -791,6 +811,8 @@ and search_pos_expr ~pos exp =
       search_pos_expr exp ~pos
   | Texp_lazy exp ->
       search_pos_expr exp ~pos
+  | Texp_object (cls, _, _) ->
+      	search_pos_class_structure ~pos cls
   end;
   add_found_str (`Exp(`Expr, exp.exp_type)) ~env:exp.exp_env ~loc:exp.exp_loc
   end
@@ -841,3 +863,18 @@ and search_pos_module_expr ~pos m =
 
 let search_pos_structure ~pos str =
   observe ~ref:found_str (search_pos_structure ~pos) str
+
+open Stypes
+
+let search_pos_ti ~pos = function
+    Ti_pat p   -> search_pos_pat ~pos ~env:p.pat_env p
+  | Ti_expr e  -> search_pos_expr ~pos e
+  | Ti_class c -> search_pos_class_expr ~pos c
+  | Ti_mod m   -> search_pos_module_expr ~pos m
+
+let rec search_pos_info ~pos = function
+    [] -> []
+  | ti :: l ->
+      if in_loc ~pos (get_location ti)
+      then observe ~ref:found_str (search_pos_ti ~pos) ti
+      else  search_pos_info ~pos l
