@@ -11,12 +11,12 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: str.mli,v 1.18 2001/12/07 13:40:18 xleroy Exp $ *)
+(* $Id: str.mli,v 1.23.2.1 2005/03/10 16:03:11 doligez Exp $ *)
 
 (** Regular expressions and high-level string processing *)
 
 
-(** {2 Regular expressions} *)
+(** {6 Regular expressions} *)
 
 
 type regexp
@@ -24,26 +24,32 @@ type regexp
 
 
 val regexp : string -> regexp
-(** Compile a regular expression. The syntax for regular expressions
-   is the same as in Gnu Emacs. The special characters are
-   [$^.*+?[]]. The following constructs are recognized:
-   - [.     ] matches any character except newline
-   - [*     ] (postfix) matches the previous expression zero, one or
-     several times
-   - [+     ] (postfix) matches the previous expression one or
-     several times
-   - [?     ] (postfix) matches the previous expression once or
-     not at all
-   - [[..]  ] character set; ranges are denoted with [-], as in [[a-z]];
-     an initial [^], as in [[^0-9]], complements the set
-   - [^     ] matches at beginning of line
-   - [$     ] matches at end of line
-   - [\|    ] (infix) alternative between two expressions
-   - [\(..\)] grouping and naming of the enclosed expression
-   - [\1    ] the text matched by the first [\(...\)] expression
-     ([\2] for the second expression, etc)
-   - [\b    ] matches word boundaries
-   - [\     ] quotes special characters. *)
+(** Compile a regular expression. The following constructs are
+    recognized:
+   - [.     ] Matches any character except newline.
+   - [*     ] (postfix) Matches the preceding expression zero, one or
+              several times
+   - [+     ] (postfix) Matches the preceding expression one or
+              several times
+   - [?     ] (postfix) Matches the preceding expression once or
+              not at all
+   - [[..]  ] Character set. Ranges are denoted with [-], as in [[a-z]].
+              An initial [^], as in [[^0-9]], complements the set.
+              To include a [\]] character in a set, make it the first
+              character of the set. To include a [-] character in a set,
+              make it the first or the last character of the set.
+   - [^     ] Matches at beginning of line (either at the beginning of
+              the matched string, or just after a newline character).
+   - [$     ] Matches at end of line (either at the end of the matched
+              string, or just before a newline character).
+   - [\|    ] (infix) Alternative between two expressions.
+   - [\(..\)] Grouping and naming of the enclosed expression.
+   - [\1    ] The text matched by the first [\(...\)] expression
+     ([\2] for the second expression, and so on up to [\9]).
+   - [\b    ] Matches word boundaries.
+   - [\     ] Quotes special characters.  The special characters
+              are [$^.*+?[]].
+*)
 
 val regexp_case_fold : string -> regexp
 (** Same as [regexp], but the compiled expression will match text
@@ -63,32 +69,33 @@ val regexp_string_case_fold : string -> regexp
    but the regexp matches in a case-insensitive way. *)
 
    
-(** {2 String matching and searching} *)
+(** {6 String matching and searching} *)
 
 
-external string_match : regexp -> string -> int -> bool = "str_string_match"
-(** [string_match r s start] tests whether the characters in [s]
-   starting at position [start] match the regular expression [r].
+val string_match : regexp -> string -> int -> bool
+(** [string_match r s start] tests whether a substring of [s] that
+   starts at position [start] matches the regular expression [r].
    The first character of a string has position [0], as usual. *)
 
-external search_forward :
-  regexp -> string -> int -> int = "str_search_forward"
-(** [search_forward r s start] searchs the string [s] for a substring
+val search_forward : regexp -> string -> int -> int
+(** [search_forward r s start] searches the string [s] for a substring
    matching the regular expression [r]. The search starts at position
    [start] and proceeds towards the end of the string.
    Return the position of the first character of the matched
    substring, or raise [Not_found] if no substring matches. *)
 
-external search_backward :
-  regexp -> string -> int -> int = "str_search_backward"
-(** Same as {!Str.search_forward}, but the search proceeds towards the
-   beginning of the string. *)
+val search_backward : regexp -> string -> int -> int
+(** [search_backward r s last] searches the string [s] for a
+  substring matching the regular expression [r]. The search first
+  considers substrings that start at position [last] and proceeds
+  towards the beginning of string. Return the position of the first
+  character of the matched substring; raise [Not_found] if no
+  substring matches. *)
 
-external string_partial_match :
-  regexp -> string -> int -> bool = "str_string_partial_match"
-(** Similar to {!Str.string_match}, but succeeds whenever the argument
-   string is a prefix of a string that matches.  This includes
-   the case of a true complete match. *)
+val string_partial_match : regexp -> string -> int -> bool
+(** Similar to {!Str.string_match}, but also returns true if
+   the argument string is a prefix of a string that matches.
+   This includes the case of a true complete match. *)
 
 val matched_string : string -> string
 (** [matched_string s] returns the substring of [s] that was matched
@@ -126,17 +133,21 @@ val group_beginning : int -> int
    of the substring that was matched by the [n]th group of
    the regular expression. 
    @raise Not_found if the [n]th group of the regular expression
-   was not matched. *)
+   was not matched.
+   @raise Invalid_argument if there are fewer than [n] groups in
+   the regular expression. *)
 
 val group_end : int -> int
 (** [group_end n] returns
    the position of the character following the last character of
    substring that was matched by the [n]th group of the regular expression. 
    @raise Not_found if the [n]th group of the regular expression
-   was not matched. *)
+   was not matched.
+   @raise Invalid_argument if there are fewer than [n] groups in
+   the regular expression. *)
 
 
-(** {2 Replacement} *)
+(** {6 Replacement} *)
 
 
 val global_replace : regexp -> string -> string -> string
@@ -162,15 +173,15 @@ val substitute_first : regexp -> (string -> string) -> string -> string
 (** Same as {!Str.global_substitute}, except that only the first substring
    matching the regular expression is replaced. *)
 
+val replace_matched : string -> string -> string
 (** [replace_matched repl s] returns the replacement text [repl]
    in which [\1], [\2], etc. have been replaced by the text
    matched by the corresponding groups in the most recent matching
    operation.  [s] must be the same string that was matched during
    this matching operation. *)     
-val replace_matched : string -> string -> string
         
 
-(** {2 Splitting} *)
+(** {6 Splitting} *)
 
 
 val split : regexp -> string -> string list
@@ -195,10 +206,7 @@ val split_delim : regexp -> string -> string list
 val bounded_split_delim : regexp -> string -> int -> string list
 (** Same as {!Str.bounded_split}, but occurrences of the
    delimiter at the beginning and at the end of the string are
-   recognized and returned as empty strings in the result.
-   For instance, [split_delim (regexp " ") " abc "]
-   returns [[""; "abc"; ""]], while [split] with the same
-   arguments returns [["abc"]]. *)
+   recognized and returned as empty strings in the result. *)
 
 type split_result = 
     Text of string
@@ -216,12 +224,10 @@ val bounded_full_split : regexp -> string -> int -> split_result list
 (** Same as {!Str.bounded_split_delim}, but returns
    the delimiters as well as the substrings contained between
    delimiters.  The former are tagged [Delim] in the result list;
-   the latter are tagged [Text].  For instance,
-   [full_split (regexp "[{}]") "{ab}"] returns
-   [[Delim "{"; Text "ab"; Delim "}"]]. *)
+   the latter are tagged [Text]. *)
 
 
-(** {2 Extracting substrings} *)
+(** {6 Extracting substrings} *)
 
 
 val string_before : string -> int -> string
