@@ -11,7 +11,7 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: posix.c,v 1.43 2001/12/07 13:40:19 xleroy Exp $ */
+/* $Id: posix.c,v 1.49.2.1 2004/07/01 09:32:38 xleroy Exp $ */
 
 /* Thread interface for POSIX 1003.1c threads */
 
@@ -25,6 +25,10 @@
 #endif
 #include <signal.h>
 #include <sys/time.h>
+#ifdef __linux__
+#include <unistd.h>
+#include <sys/utsname.h>
+#endif
 #include "alloc.h"
 #include "backtrace.h"
 #include "callback.h"
@@ -75,6 +79,7 @@ struct caml_thread_struct {
   value * gc_regs;              /* Saved value of caml_gc_regs */
   char * exception_pointer;     /* Saved value of caml_exception_pointer */
   struct caml__roots_block * local_roots; /* Saved value of local_roots */
+  struct longjmp_buffer * exit_buf; /* For thread exit */
 #else
   value * stack_low;            /* The execution stack for this thread */
   value * stack_high;
@@ -92,12 +97,20 @@ struct caml_thread_struct {
 typedef struct caml_thread_struct * caml_thread_t;
 
 /* The descriptor for the currently executing thread */
-
 static caml_thread_t curr_thread = NULL;
 
-/* The global mutex used to ensure that at most one thread is running
-   Caml code */
-static pthread_mutex_t caml_mutex;
+/* Track whether one thread is running Caml code.  There can be
+   at most one such thread at any time. */
+static volatile int caml_runtime_busy = 1;
+
+/* Number of threads waiting to run Caml code. */
+static volatile int caml_runtime_waiters = 0;
+
+/* Mutex that protects the two variables above. */
+static pthread_mutex_t caml_runtime_mutex = PTHREAD_MUTEX_INITIALIZER;
+
+/* Condition signaled when caml_runtime_busy becomes 0 */
+static pthread_cond_t caml_runtime_is_free = PTHREAD_COND_INITIALIZER;
 
 /* The key used for storing the thread descriptor in the specific data
    of the corresponding Posix thread. */
@@ -109,12 +122,19 @@ static pthread_key_t last_channel_locked_key;
 /* Identifier for next thread creation */
 static long thread_next_ident = 0;
 
-/* Forward declarations */
+/* Whether to use sched_yield() or not */
+static int broken_sched_yield = 0;
 
+/* Forward declarations */
 value caml_threadstatus_new (void);
 void caml_threadstatus_terminate (value);
 int caml_threadstatus_wait (value);
 static void caml_pthread_check (int, char *);
+static void caml_thread_sysdeps_initialize(void);
+
+/* Imports for the native-code compiler */
+extern struct longjmp_buffer caml_termination_jmpbuf;
+extern void (*caml_termination_hook)(void *);
 
 /* Hook for scanning the stacks of the other threads */
 
@@ -175,14 +195,24 @@ static void caml_thread_enter_blocking_section(void)
   curr_thread->backtrace_buffer = backtrace_buffer;
   curr_thread->backtrace_last_exn = backtrace_last_exn;
 #endif
-  /* Release the global mutex */
-  pthread_mutex_unlock(&caml_mutex);
+  /* Tell other threads that the runtime is free */
+  pthread_mutex_lock(&caml_runtime_mutex);
+  caml_runtime_busy = 0;
+  pthread_mutex_unlock(&caml_runtime_mutex);
+  pthread_cond_signal(&caml_runtime_is_free);
 }
 
 static void caml_thread_leave_blocking_section(void)
 {
-  /* Re-acquire the global mutex */
-  pthread_mutex_lock(&caml_mutex);
+  /* Wait until the runtime is free */
+  pthread_mutex_lock(&caml_runtime_mutex);
+  while (caml_runtime_busy) {
+    caml_runtime_waiters++;
+    pthread_cond_wait(&caml_runtime_is_free, &caml_runtime_mutex);
+    caml_runtime_waiters--;
+  }
+  caml_runtime_busy = 1;
+  pthread_mutex_unlock(&caml_runtime_mutex);
   /* Update curr_thread to point to the thread descriptor corresponding
      to the thread currently executing */
   curr_thread = pthread_getspecific(thread_descriptor_key);
@@ -257,8 +287,12 @@ static void * caml_thread_tick(void * arg)
 {
   struct timeval timeout;
   sigset_t mask;
+#ifdef __linux__
+  int tickcount = 0;
+#endif
 
-  /* Block all signals so that we don't try to execute a Caml signal handler */
+  /* Block all signals so that we don't try to execute
+     a Caml signal handler */
   sigfillset(&mask);
   pthread_sigmask(SIG_BLOCK, &mask, NULL);
   while(1) {
@@ -274,6 +308,18 @@ static void * caml_thread_tick(void * arg)
     young_limit = young_end;
 #else
     something_to_do = 1;
+#endif
+#ifdef __linux__
+    /* Hack around LinuxThreads' non-standard signal handling:
+       if program is killed on a signal, e.g. SIGINT, the current
+       thread will not die on this signal (because of the signal blocking
+       above).  Hence, periodically check that the thread manager (our
+       parent process) still exists. */
+    tickcount++;
+    if (tickcount >= 2000000 / Thread_timeout) { /* every 2 secs approx */
+      tickcount = 0;
+      if (getppid() == 1) pthread_exit(NULL);
+    }
 #endif
   }
   return NULL;                  /* prevents compiler warning */
@@ -288,11 +334,11 @@ value caml_thread_initialize(value unit)   /* ML */
   value mu = Val_unit;
   value descr;
 
+  /* Protect against repeated initialization (PR#1325) */
+  if (curr_thread != NULL) return Val_unit;
   Begin_root (mu);
-    /* Initialize the main mutex */
-    caml_pthread_check(pthread_mutex_init(&caml_mutex, NULL),
-                       "Thread.init");
-    pthread_mutex_lock(&caml_mutex);
+    /* OS-specific initialization */
+    caml_thread_sysdeps_initialize();
     /* Initialize the keys */
     pthread_key_create(&thread_descriptor_key, NULL);
     pthread_key_create(&last_channel_locked_key, NULL);
@@ -311,6 +357,9 @@ value caml_thread_initialize(value unit)   /* ML */
     curr_thread->descr = descr;
     curr_thread->next = curr_thread;
     curr_thread->prev = curr_thread;
+#ifdef NATIVE_CODE
+    curr_thread->exit_buf = &caml_termination_jmpbuf;
+#endif
     /* The stack-related fields will be filled in at the next
        enter_blocking_section */
     /* Associate the thread descriptor with the thread */
@@ -322,10 +371,13 @@ value caml_thread_initialize(value unit)   /* ML */
     enter_blocking_section_hook = caml_thread_enter_blocking_section;
     prev_leave_blocking_section_hook = leave_blocking_section_hook;
     leave_blocking_section_hook = caml_thread_leave_blocking_section;
-    channel_mutex_free = caml_io_mutex_free;
-    channel_mutex_lock = caml_io_mutex_lock;
-    channel_mutex_unlock = caml_io_mutex_unlock;
-    channel_mutex_unlock_exn = caml_io_mutex_unlock_exn;
+#ifdef NATIVE_CODE
+    caml_termination_hook = pthread_exit;
+#endif
+    caml_channel_mutex_free = caml_io_mutex_free;
+    caml_channel_mutex_lock = caml_io_mutex_lock;
+    caml_channel_mutex_unlock = caml_io_mutex_unlock;
+    caml_channel_mutex_unlock_exn = caml_io_mutex_unlock_exn;
     /* Fork the tick thread */
     pthread_attr_init(&attr);
     pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
@@ -336,29 +388,23 @@ value caml_thread_initialize(value unit)   /* ML */
   return Val_unit;
 }
 
-/* Create a thread */
+/* Thread cleanup at termination */
 
-static void * caml_thread_start(void * arg)
+static void caml_thread_stop(void)
 {
-  caml_thread_t th = (caml_thread_t) arg;
-  value clos;
+  caml_thread_t th = curr_thread;
 
-  /* Associate the thread descriptor with the thread */
-  pthread_setspecific(thread_descriptor_key, (void *) th);
-  /* Acquire the global mutex and set up the stack variables */
-  leave_blocking_section();
-  /* Callback the closure */
-  clos = Start_closure(th->descr);
-  modify(&(Start_closure(th->descr)), Val_unit);
-  callback_exn(clos, Val_unit);
   /* Signal that the thread has terminated */
   caml_threadstatus_terminate(Terminated(th->descr));
   /* Remove th from the doubly-linked list of threads */
   th->next->prev = th->prev;
   th->prev->next = th->next;
-  /* Release the main mutex (forever) */
+  /* Release the runtime system */
   async_signal_mode = 1;
-  pthread_mutex_unlock(&caml_mutex);
+  pthread_mutex_lock(&caml_runtime_mutex);
+  caml_runtime_busy = 0;
+  pthread_mutex_unlock(&caml_runtime_mutex);
+  pthread_cond_signal(&caml_runtime_is_free);
 #ifndef NATIVE_CODE
   /* Free the memory resources */
   stat_free(th->stack_low);
@@ -366,6 +412,35 @@ static void * caml_thread_start(void * arg)
 #endif
   /* Free the thread descriptor */
   stat_free(th);
+}
+
+/* Create a thread */
+
+static void * caml_thread_start(void * arg)
+{
+  caml_thread_t th = (caml_thread_t) arg;
+  value clos;
+#ifdef NATIVE_CODE
+  struct longjmp_buffer termination_buf;
+#endif
+
+  /* Associate the thread descriptor with the thread */
+  pthread_setspecific(thread_descriptor_key, (void *) th);
+  /* Acquire the global mutex and set up the stack variables */
+  leave_blocking_section();
+#ifdef NATIVE_CODE
+  /* Setup termination handler (for caml_thread_exit) */
+  if (sigsetjmp(termination_buf.buf, 0) == 0) {
+    th->exit_buf = &termination_buf;
+#endif
+    /* Callback the closure */
+    clos = Start_closure(th->descr);
+    modify(&(Start_closure(th->descr)), Val_unit);
+    callback_exn(clos, Val_unit);
+    caml_thread_stop();
+#ifdef NATIVE_CODE
+  }
+#endif
   /* The thread now stops running */
   return NULL;
 }  
@@ -460,12 +535,37 @@ value caml_thread_uncaught_exception(value exn)  /* ML */
   return Val_unit;
 }
 
+/* Terminate current thread */
+
+value caml_thread_exit(value unit)   /* ML */
+{
+#ifdef NATIVE_CODE
+  /* We cannot call pthread_exit here because on some systems this
+     raises a C++ exception, and ocamlopt-generated stack frames
+     cannot be unwound.  Instead, we longjmp to the thread creation
+     point (in caml_thread_start) or to the point in caml_main
+     where caml_termination_hook will be called. */
+  struct longjmp_buffer * exit_buf;
+  if (curr_thread == NULL) invalid_argument("Thread.exit: not initialized");
+  exit_buf = curr_thread->exit_buf;
+  caml_thread_stop();
+  siglongjmp(exit_buf->buf, 1);
+#else
+  /* No such problem in bytecode */
+  if (curr_thread == NULL) invalid_argument("Thread.exit: not initialized");
+  caml_thread_stop();
+  pthread_exit(NULL);
+#endif
+  return Val_unit;  /* not reached */
+}
+
 /* Allow re-scheduling */
 
 value caml_thread_yield(value unit)        /* ML */
 {
+  if (caml_runtime_waiters == 0) return Val_unit;
   enter_blocking_section();
-  sched_yield();
+  if (! broken_sched_yield) sched_yield();
   leave_blocking_section();
   return Val_unit;
 }
@@ -706,6 +806,7 @@ int caml_threadstatus_wait (value wrapper)
 
 value caml_wait_signal(value sigs) /* ML */
 {
+#ifdef HAS_SIGWAIT
   sigset_t set;
   int retcode, signo;
 
@@ -720,6 +821,10 @@ value caml_wait_signal(value sigs) /* ML */
   leave_blocking_section();
   caml_pthread_check(retcode, "Thread.wait_signal");
   return Val_int(signo);
+#else
+  invalid_argument("Thread.wait_signal not implemented");
+  return Val_int(0);		/* not reached */
+#endif
 }
 
 /* Error report */
@@ -740,3 +845,21 @@ static void caml_pthread_check(int retcode, char *msg)
   memmove (&Byte(str, msglen + 2), err, errlen);
   raise_sys_error(str);
 }
+
+/* OS-specific initialization */
+
+static void caml_thread_sysdeps_initialize(void)
+{
+#ifdef __linux__
+  /* sched_yield() doesn't do what we want in kernel 2.6 and up (PR#2663) */
+  struct utsname un;
+  if (uname(&un) == -1) return;
+  broken_sched_yield =
+    un.release[1] != '.' || un.release[0] >= '3'   /* version 3 and up */
+    || (un.release[0] == '2' &&
+        (un.release[3] != '.' || un.release[2] >= '6')); /* 2.6 and up */
+  caml_gc_message(0x100, "POSIX threads.  Avoid sched_yield: %d\n",
+                  broken_sched_yield);
+#endif
+}
+
