@@ -11,7 +11,7 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: win32.c,v 1.36 2002/06/11 13:53:03 xleroy Exp $ */
+/* $Id: win32.c,v 1.38 2003/12/29 22:15:02 doligez Exp $ */
 
 /* Thread interface for Win32 threads */
 
@@ -278,6 +278,8 @@ CAMLprim value caml_thread_initialize(value unit)
   HANDLE tick_thread;
   unsigned long tick_id;
 
+  /* Protect against repeated initialization (PR#1325) */
+  if (curr_thread != NULL) return Val_unit;
   Begin_root (vthread);
     /* Initialize the main mutex and acquire it */
     caml_mutex = CreateMutex(NULL, TRUE, NULL);
@@ -317,10 +319,10 @@ CAMLprim value caml_thread_initialize(value unit)
     enter_blocking_section_hook = caml_thread_enter_blocking_section;
     prev_leave_blocking_section_hook = leave_blocking_section_hook;
     leave_blocking_section_hook = caml_thread_leave_blocking_section;
-    channel_mutex_free = caml_io_mutex_free;
-    channel_mutex_lock = caml_io_mutex_lock;
-    channel_mutex_unlock = caml_io_mutex_unlock;
-    channel_mutex_unlock_exn = caml_io_mutex_unlock_exn;
+    caml_channel_mutex_free = caml_io_mutex_free;
+    caml_channel_mutex_lock = caml_io_mutex_lock;
+    caml_channel_mutex_unlock = caml_io_mutex_unlock;
+    caml_channel_mutex_unlock_exn = caml_io_mutex_unlock_exn;
     /* Fork the tick thread */
     tick_thread = (HANDLE) _beginthread(caml_thread_tick, 0, NULL);
     if (tick_thread == (HANDLE)(-1)) caml_wthread_error("Thread.init");
