@@ -11,7 +11,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: pervasives.ml,v 1.36 2002/07/12 09:47:54 xleroy Exp $ *)
+(* $Id: pervasives.ml,v 1.48.4.1 2004/06/22 12:13:46 xleroy Exp $ *)
 
 (* Same as ../../stdlib/pervasives.ml, except that I/O functions have
    been redefined to not block the whole process, but only the calling
@@ -27,7 +27,6 @@ let failwith s = raise(Failure s)
 let invalid_arg s = raise(Invalid_argument s)
 
 exception Exit
-exception Assert_failure of (string * int * int)
 
 (* Comparisons *)
 
@@ -37,7 +36,7 @@ external (<) : 'a -> 'a -> bool = "%lessthan"
 external (>) : 'a -> 'a -> bool = "%greaterthan"
 external (<=) : 'a -> 'a -> bool = "%lessequal"
 external (>=) : 'a -> 'a -> bool = "%greaterequal"
-external compare: 'a -> 'a -> int = "compare"
+external compare: 'a -> 'a -> int = "%compare"
 
 let min x y = if x <= y then x else y
 let max x y = if x >= y then x else y
@@ -86,65 +85,60 @@ external (+.) : float -> float -> float = "%addfloat"
 external (-.) : float -> float -> float = "%subfloat"
 external ( *. ) : float -> float -> float = "%mulfloat"
 external (/.) : float -> float -> float = "%divfloat"
-external ( ** ) : float -> float -> float = "power_float" "pow" "float"
-external exp : float -> float = "exp_float" "exp" "float"
-external acos : float -> float = "acos_float" "acos" "float"
-external asin : float -> float = "asin_float" "asin" "float"
-external atan : float -> float = "atan_float" "atan" "float"
-external atan2 : float -> float -> float = "atan2_float" "atan2" "float"
-external cos : float -> float = "cos_float" "cos" "float"
-external cosh : float -> float = "cosh_float" "cosh" "float"
-external log : float -> float = "log_float" "log" "float"
-external log10 : float -> float = "log10_float" "log10" "float"
-external sin : float -> float = "sin_float" "sin" "float"
-external sinh : float -> float = "sinh_float" "sinh" "float"
-external sqrt : float -> float = "sqrt_float" "sqrt" "float"
-external tan : float -> float = "tan_float" "tan" "float"
-external tanh : float -> float = "tanh_float" "tanh" "float"
-external ceil : float -> float = "ceil_float" "ceil" "float"
-external floor : float -> float = "floor_float" "floor" "float"
+external ( ** ) : float -> float -> float = "caml_power_float" "pow" "float"
+external exp : float -> float = "caml_exp_float" "exp" "float"
+external acos : float -> float = "caml_acos_float" "acos" "float"
+external asin : float -> float = "caml_asin_float" "asin" "float"
+external atan : float -> float = "caml_atan_float" "atan" "float"
+external atan2 : float -> float -> float = "caml_atan2_float" "atan2" "float"
+external cos : float -> float = "caml_cos_float" "cos" "float"
+external cosh : float -> float = "caml_cosh_float" "cosh" "float"
+external log : float -> float = "caml_log_float" "log" "float"
+external log10 : float -> float = "caml_log10_float" "log10" "float"
+external sin : float -> float = "caml_sin_float" "sin" "float"
+external sinh : float -> float = "caml_sinh_float" "sinh" "float"
+external sqrt : float -> float = "caml_sqrt_float" "sqrt" "float"
+external tan : float -> float = "caml_tan_float" "tan" "float"
+external tanh : float -> float = "caml_tanh_float" "tanh" "float"
+external ceil : float -> float = "caml_ceil_float" "ceil" "float"
+external floor : float -> float = "caml_floor_float" "floor" "float"
 external abs_float : float -> float = "%absfloat"
-external mod_float : float -> float -> float = "fmod_float" "fmod" "float"
-external frexp : float -> float * int = "frexp_float"
-external ldexp : float -> int -> float = "ldexp_float"
-external modf : float -> float * float = "modf_float"
+external mod_float : float -> float -> float = "caml_fmod_float" "fmod" "float"
+external frexp : float -> float * int = "caml_frexp_float"
+external ldexp : float -> int -> float = "caml_ldexp_float"
+external modf : float -> float * float = "caml_modf_float"
 external float : int -> float = "%floatofint"
 external float_of_int : int -> float = "%floatofint"
 external truncate : float -> int = "%intoffloat"
 external int_of_float : float -> int = "%intoffloat"
-external float_of_bytes : string -> float = "float_of_bytes"
+external float_of_bits : int64 -> float = "caml_int64_float_of_bits"
 let infinity =
-  float_of_bytes "\127\240\000\000\000\000\000\000"
-              (* 0x7F F0 00 00 00 00 00 00 *)
+  float_of_bits 0x7F_F0_00_00_00_00_00_00L
 let neg_infinity =
-  float_of_bytes "\255\240\000\000\000\000\000\000"
-              (* 0xFF F0 00 00 00 00 00 00 *)
+  float_of_bits 0xFF_F0_00_00_00_00_00_00L
 let nan =
-  float_of_bytes "\127\240\000\000\000\000\000\001"
-              (* 0x7F F0 00 00 00 00 00 01 *)
+  float_of_bits 0x7F_F0_00_00_00_00_00_01L
 let max_float =
-  float_of_bytes "\127\239\255\255\255\255\255\255"
-              (* 0x7f ef ff ff ff ff ff ff *)
+  float_of_bits 0x7F_EF_FF_FF_FF_FF_FF_FFL
 let min_float =
-  float_of_bytes "\000\016\000\000\000\000\000\000"
-              (* 0x00 10 00 00 00 00 00 00 *)
+  float_of_bits 0x00_10_00_00_00_00_00_00L
 let epsilon_float =
-  float_of_bytes "\060\176\000\000\000\000\000\000"
-              (* 0x3c b0 00 00 00 00 00 00 *)
+  float_of_bits 0x3C_B0_00_00_00_00_00_00L
+
 type fpclass =
     FP_normal
   | FP_subnormal
   | FP_zero
   | FP_infinite
   | FP_nan
-external classify_float: float -> fpclass = "classify_float"
+external classify_float: float -> fpclass = "caml_classify_float"
 
 (* String operations -- more in module String *)
 
 external string_length : string -> int = "%string_length"
-external string_create: int -> string = "create_string"
+external string_create: int -> string = "caml_create_string"
 external string_blit : string -> int -> string -> int -> int -> unit
-                     = "blit_string" "noalloc"
+                     = "caml_blit_string" "noalloc"
 
 let (^) s1 s2 =
   let l1 = string_length s1 and l2 = string_length s2 in
@@ -180,8 +174,8 @@ external decr: int ref -> unit = "%decr"
 
 (* String conversion functions *)
 
-external format_int: string -> int -> string = "format_int"
-external format_float: string -> float -> string = "format_float"
+external format_int: string -> int -> string = "caml_format_int"
+external format_float: string -> float -> string = "caml_format_float"
 
 let string_of_bool b =
   if b then "true" else "false"
@@ -193,12 +187,22 @@ let bool_of_string = function
 let string_of_int n =
   format_int "%d" n
 
-external int_of_string : string -> int = "int_of_string"
+external int_of_string : string -> int = "caml_int_of_string"
 
-let string_of_float f =
-  format_float "%.12g" f
+let valid_float_lexem s =
+  let l = string_length s in
+  let rec loop i =
+    if i >= l then s ^ "." else
+    match s.[i] with
+    | '0' .. '9' | '-' -> loop (i+1)
+    | _ -> s
+  in
+  loop 0
+;;
 
-external float_of_string : string -> float = "float_of_string"
+let string_of_float f = valid_float_lexem (format_float "%.12g" f);;
+
+external float_of_string : string -> float = "caml_float_of_string"
 
 (* List operations -- more in module List *)
 
@@ -212,8 +216,8 @@ let rec (@) l1 l2 =
 type in_channel
 type out_channel
 
-external open_descriptor_out: int -> out_channel = "caml_open_descriptor_out"
-external open_descriptor_in: int -> in_channel = "caml_open_descriptor_in"
+external open_descriptor_out: int -> out_channel = "caml_ml_open_descriptor_out"
+external open_descriptor_in: int -> in_channel = "caml_ml_open_descriptor_in"
 
 let stdin = open_descriptor_in 0
 let stdout = open_descriptor_out 1
@@ -229,8 +233,10 @@ let thread_wait_write fd = thread_wait_write_prim fd
 
 external inchan_ready : in_channel -> bool = "thread_inchan_ready"
 external outchan_ready : out_channel -> int -> bool = "thread_outchan_ready"
-external descr_inchan : in_channel -> Unix.file_descr = "channel_descriptor"
-external descr_outchan : out_channel -> Unix.file_descr = "channel_descriptor"
+external descr_inchan : in_channel -> Unix.file_descr
+                      = "caml_channel_descriptor"
+external descr_outchan : out_channel -> Unix.file_descr
+                       = "caml_channel_descriptor"
 
 let wait_inchan ic =
   if not (inchan_ready ic) then thread_wait_read(descr_inchan ic)
@@ -245,7 +251,7 @@ type open_flag =
   | Open_creat | Open_trunc | Open_excl
   | Open_binary | Open_text | Open_nonblock
 
-external open_desc: string -> open_flag list -> int -> int = "sys_open"
+external open_desc: string -> open_flag list -> int -> int = "caml_sys_open"
 
 let open_out_gen mode perm name =
   open_descriptor_out(open_desc name mode perm)
@@ -256,7 +262,7 @@ let open_out name =
 let open_out_bin name =
   open_out_gen [Open_wronly; Open_creat; Open_trunc; Open_binary] 0o666 name
 
-external flush_partial : out_channel -> bool = "caml_flush_partial"
+external flush_partial : out_channel -> bool = "caml_ml_flush_partial"
 
 let rec flush oc =
   let success =
@@ -267,7 +273,7 @@ let rec flush oc =
   if success then () else flush oc
 
 external out_channels_list : unit -> out_channel list 
-                           = "caml_out_channels_list"
+                           = "caml_ml_out_channels_list"
 
 let flush_all () = 
   let rec iter = function
@@ -282,7 +288,7 @@ let flush_all () =
   in iter (out_channels_list ())
 
 external unsafe_output_partial : out_channel -> string -> int -> int -> int
-                        = "caml_output_partial"
+                        = "caml_ml_output_partial"
 
 let rec unsafe_output oc buf pos len =
   if len > 0 then begin
@@ -295,8 +301,9 @@ let rec unsafe_output oc buf pos len =
   end
 
 external output_char_blocking : out_channel -> char -> unit 
-                              = "caml_output_char"
-external output_byte_blocking : out_channel -> int -> unit = "caml_output_char"
+                              = "caml_ml_output_char"
+external output_byte_blocking : out_channel -> int -> unit
+                              = "caml_ml_output_char"
 
 let rec output_char oc c =
   try
@@ -327,21 +334,24 @@ let output_binary_int oc n =
   output_byte oc n
 
 external marshal_to_string : 'a -> unit list -> string 
-                           = "output_value_to_string"
+                           = "caml_output_value_to_string"
 
 let output_value oc v = output_string oc (marshal_to_string v [])
 
-external seek_out_blocking : out_channel -> int -> unit = "caml_seek_out"
+external seek_out_blocking : out_channel -> int -> unit = "caml_ml_seek_out"
 
 let seek_out oc pos = flush oc; seek_out_blocking oc pos
 
-external pos_out : out_channel -> int = "caml_pos_out"
-external out_channel_length : out_channel -> int = "caml_channel_size"
-external close_out_channel : out_channel -> unit = "caml_close_channel"
+external pos_out : out_channel -> int = "caml_ml_pos_out"
+external out_channel_length : out_channel -> int = "caml_ml_channel_size"
+external close_out_channel : out_channel -> unit = "caml_ml_close_channel"
 
-let close_out oc = flush oc; close_out_channel oc
+let close_out oc = (try flush oc with _ -> ()); close_out_channel oc
+let close_out_noerr oc =
+  (try flush oc with _ -> ());
+  (try close_out_channel oc with _ -> ())
 external set_binary_mode_out : out_channel -> bool -> unit
-                             = "caml_set_binary_mode"
+                             = "caml_ml_set_binary_mode"
 
 (* General input functions *)
 
@@ -354,8 +364,8 @@ let open_in name =
 let open_in_bin name =
   open_in_gen [Open_rdonly; Open_binary] 0 name
 
-external input_char_blocking : in_channel -> char = "caml_input_char"
-external input_byte_blocking : in_channel -> int = "caml_input_char"
+external input_char_blocking : in_channel -> char = "caml_ml_input_char"
+external input_byte_blocking : in_channel -> int = "caml_ml_input_char"
 
 let rec input_char ic =
   try
@@ -364,7 +374,7 @@ let rec input_char ic =
     wait_inchan ic; input_char ic
 
 external unsafe_input_blocking : in_channel -> string -> int -> int -> int
-                               = "caml_input"
+                               = "caml_ml_input"
 
 let rec unsafe_input ic s ofs len =
   try
@@ -377,8 +387,6 @@ let input ic s ofs len =
   then invalid_arg "input"
   else unsafe_input ic s ofs len
 
-let input' ic ~buf ~pos ~len = input ic buf pos len
-
 let rec unsafe_really_input ic s ofs len =
   if len <= 0 then () else begin
     let r = unsafe_input ic s ofs len in
@@ -428,8 +436,8 @@ let input_binary_int ic =
   let b4 = input_byte ic in
   (n1 lsl 24) + (b2 lsl 16) + (b3 lsl 8) + b4
 
-external unmarshal : string -> int -> 'a = "input_value_from_string"
-external marshal_data_size : string -> int -> int = "marshal_data_size"
+external unmarshal : string -> int -> 'a = "caml_input_value_from_string"
+external marshal_data_size : string -> int -> int = "caml_marshal_data_size"
 
 let input_value ic =
   let header = string_create 20 in
@@ -440,12 +448,13 @@ let input_value ic =
   really_input ic buffer 20 bsize;
   unmarshal buffer 0
 
-external seek_in : in_channel -> int -> unit = "caml_seek_in"
-external pos_in : in_channel -> int = "caml_pos_in"
-external in_channel_length : in_channel -> int = "caml_channel_size"
-external close_in : in_channel -> unit = "caml_close_channel"
+external seek_in : in_channel -> int -> unit = "caml_ml_seek_in"
+external pos_in : in_channel -> int = "caml_ml_pos_in"
+external in_channel_length : in_channel -> int = "caml_ml_channel_size"
+external close_in : in_channel -> unit = "caml_ml_close_channel"
+let close_in_noerr ic = (try close_in ic with _ -> ());;
 external set_binary_mode_in : in_channel -> bool -> unit
-                            = "caml_set_binary_mode"
+                            = "caml_ml_set_binary_mode"
 
 (* Output functions on standard output *)
 
@@ -453,7 +462,8 @@ let print_char c = output_char stdout c
 let print_string s = output_string stdout s
 let print_int i = output_string stdout (string_of_int i)
 let print_float f = output_string stdout (string_of_float f)
-let print_endline s = output_string stdout s; output_char stdout '\n'
+let print_endline s =
+  output_string stdout s; output_char stdout '\n'; flush stdout
 let print_newline () = output_char stdout '\n'; flush stdout
 
 (* Output functions on standard error *)
@@ -476,17 +486,37 @@ let read_float () = float_of_string(read_line())
 
 module LargeFile =
   struct
-    external seek_out : out_channel -> int64 -> unit = "caml_seek_out_64"
-    external pos_out : out_channel -> int64 = "caml_pos_out_64"
-    external out_channel_length : out_channel -> int64 = "caml_channel_size_64"
-    external seek_in : in_channel -> int64 -> unit = "caml_seek_in_64"
-    external pos_in : in_channel -> int64 = "caml_pos_in_64"
-    external in_channel_length : in_channel -> int64 = "caml_channel_size_64"
+    external seek_out : out_channel -> int64 -> unit = "caml_ml_seek_out_64"
+    external pos_out : out_channel -> int64 = "caml_ml_pos_out_64"
+    external out_channel_length : out_channel -> int64
+                                = "caml_ml_channel_size_64"
+    external seek_in : in_channel -> int64 -> unit = "caml_ml_seek_in_64"
+    external pos_in : in_channel -> int64 = "caml_ml_pos_in_64"
+    external in_channel_length : in_channel -> int64 = "caml_ml_channel_size_64"
   end
 
+(* Formats *)
+type ('a, 'b, 'c) format = ('a, 'b, 'c, 'c) format4
+external format_of_string :
+ ('a, 'b, 'c, 'd) format4 -> ('a, 'b, 'c, 'd) format4 = "%identity"
+external string_of_format_sys :
+ ('a, 'b, 'c, 'd) format4 -> string = "%identity"
+external string_to_format : string -> ('a, 'b, 'c, 'd) format4 = "%identity"
+
+let (( ^^ ) : ('a, 'b, 'c, 'd) format4 -> ('d, 'b, 'c, 'e) format4 ->
+              ('a, 'b, 'c, 'e) format4) = fun fmt1 fmt2 ->
+  string_to_format (string_of_format_sys fmt1 ^ string_of_format_sys fmt2);;
+
+let string_of_format f =
+  let s = string_of_format_sys f in
+  let l = string_length s in
+  let r = string_create l in
+  string_blit s 0 r 0 l;
+  r
+
 (* Miscellaneous *)
 
-external sys_exit : int -> 'a = "sys_exit"
+external sys_exit : int -> 'a = "caml_sys_exit"
 
 let exit_function = ref flush_all
 
@@ -500,6 +530,7 @@ let exit retcode =
   do_at_exit ();
   sys_exit retcode
 
-external register_named_value: string -> 'a -> unit = "register_named_value"
+external register_named_value : string -> 'a -> unit
+                              = "caml_register_named_value"
 
 let _ = register_named_value "Pervasives.do_at_exit" do_at_exit
