@@ -11,7 +11,7 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: addrofstr.c,v 1.10 2001/12/07 13:40:24 xleroy Exp $ */
+/* $Id: addrofstr.c,v 1.11 2004/04/09 13:25:20 xleroy Exp $ */
 
 #include <mlvalues.h>
 #include <fail.h>
@@ -23,16 +23,25 @@
 
 CAMLprim value unix_inet_addr_of_string(value s)
 {
-#ifdef HAS_INET_ATON
+#if defined(HAS_IPV6)
+  struct in_addr address;
+  struct in6_addr address6;
+  if (inet_pton(AF_INET, String_val(s), &address) > 0)
+    return alloc_inet_addr(&address);
+  else if (inet_pton(AF_INET6, String_val(s), &address6) > 0)
+    return alloc_inet6_addr(&address6);
+  else
+    failwith("inet_addr_of_string");
+#elif defined(HAS_INET_ATON)
   struct in_addr address;
   if (inet_aton(String_val(s), &address) == 0)
     failwith("inet_addr_of_string");
-  return alloc_inet_addr(address.s_addr);
+  return alloc_inet_addr(&address);
 #else
-  unsigned int address;
-  address = inet_addr(String_val(s));
-  if (address == (unsigned int) -1) failwith("inet_addr_of_string");
-  return alloc_inet_addr(address);
+  struct in_addr address;
+  address.s_addr = inet_addr(String_val(s));
+  if (address.s_addr == (uint32) -1) failwith("inet_addr_of_string");
+  return alloc_inet_addr(&address);
 #endif
 }
 
