@@ -6,17 +6,17 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: alarm.c,v 1.5 1999/11/17 18:57:56 xleroy Exp $ */
+/* $Id: alarm.c,v 1.7.6.1 2005/01/17 18:10:36 doligez Exp $ */
 
 #include <mlvalues.h>
 #include "unixsupport.h"
 
-value unix_alarm(t)              /* ML */
-     value t;
+CAMLprim value unix_alarm(value t)
 {
   return Val_int(alarm((unsigned int) Long_val(t)));
 }
