@@ -6,14 +6,16 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: closedir.c,v 1.6 1999/11/17 18:57:58 xleroy Exp $ */
+/* $Id: closedir.c,v 1.9 2004/02/14 10:21:22 xleroy Exp $ */
 
 #include <mlvalues.h>
 #include "unixsupport.h"
+#include <errno.h>
 #include <sys/types.h>
 #ifdef HAS_DIRENT
 #include <dirent.h>
@@ -21,8 +23,11 @@
 #include <sys/dir.h>
 #endif
 
-value unix_closedir(value d)           /* ML */
+CAMLprim value unix_closedir(value vd)
 {
-  closedir((DIR *) d);
+  DIR * d = DIR_Val(vd);
+  if (d == (DIR *) NULL) unix_error(EBADF, "closedir", Nothing);
+  closedir(d);
+  DIR_Val(vd) = (DIR *) NULL;
   return Val_unit;
 }
