@@ -6,18 +6,19 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: dup2.c,v 1.7 2000/04/05 18:30:19 doligez Exp $ */
+/* $Id: dup2.c,v 1.9 2001/12/07 13:40:27 xleroy Exp $ */
 
 #include <mlvalues.h>
 #include "unixsupport.h"
 
 #ifdef HAS_DUP2
 
-value unix_dup2(value fd1, value fd2)        /* ML */
+CAMLprim value unix_dup2(value fd1, value fd2)
 {
   if (dup2(Int_val(fd1), Int_val(fd2)) == -1) uerror("dup2", Nothing);
   return Val_unit;
@@ -38,7 +39,7 @@ static int do_dup2(int fd1, int fd2)
   return res;
 }
 
-value unix_dup2(value fd1, value fd2)
+CAMLprim value unix_dup2(value fd1, value fd2)
 {
   close(Int_val(fd2));
   if (do_dup2(Int_val(fd1), Int_val(fd2)) == -1) uerror("dup2", Nothing);
