@@ -6,18 +6,21 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: envir.c,v 1.7 2001/07/02 12:24:13 xleroy Exp $ */
+/* $Id: envir.c,v 1.9.6.1 2005/01/17 18:10:36 doligez Exp $ */
 
 #include <mlvalues.h>
 #include <alloc.h>
 
+#ifndef _WIN32
 extern char ** environ;
+#endif
 
-value unix_environment(void)   /* ML */
+CAMLprim value unix_environment(value unit)
 {
   return copy_string_array((const char**)environ);
 }
