@@ -6,16 +6,17 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: exit.c,v 1.7 2000/04/05 18:30:19 doligez Exp $ */
+/* $Id: exit.c,v 1.9 2001/12/07 13:40:28 xleroy Exp $ */
 
 #include <mlvalues.h>
 #include "unixsupport.h"
 
-value unix_exit(value n)               /* ML */
+CAMLprim value unix_exit(value n)
 {
   _exit(Int_val(n));
   return Val_unit;                  /* never reached, but suppress warnings */
