@@ -6,20 +6,22 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: fchmod.c,v 1.7 1999/11/17 18:57:59 xleroy Exp $ */
+/* $Id: fchmod.c,v 1.9.6.1 2004/08/23 11:31:44 doligez Exp $ */
 
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <fail.h>
 #include <mlvalues.h>
 #include "unixsupport.h"
 
 #ifdef HAS_FCHMOD
 
-value unix_fchmod(value fd, value perm)      /* ML */
+CAMLprim value unix_fchmod(value fd, value perm)
 {
   if (fchmod(Int_val(fd), Int_val(perm)) == -1) uerror("fchmod", Nothing);
   return Val_unit;
@@ -27,7 +29,7 @@ value unix_fchmod(value fd, value perm)      /* ML */
 
 #else
 
-value unix_fchmod(value fd, value perm)
+CAMLprim value unix_fchmod(value fd, value perm)
 { invalid_argument("fchmod not implemented"); }
   
 #endif
