@@ -6,18 +6,20 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: fchown.c,v 1.6 1999/11/17 18:57:59 xleroy Exp $ */
+/* $Id: fchown.c,v 1.8.6.1 2004/08/23 11:31:44 doligez Exp $ */
 
+#include <fail.h>
 #include <mlvalues.h>
 #include "unixsupport.h"
 
 #ifdef HAS_FCHMOD
 
-value unix_fchown(value fd, value uid, value gid)  /* ML */
+CAMLprim value unix_fchown(value fd, value uid, value gid)
 {
   if (fchown(Int_val(fd), Int_val(uid), Int_val(gid)) == -1)
     uerror("fchown", Nothing);
@@ -26,7 +28,7 @@ value unix_fchown(value fd, value uid, value gid)  /* ML */
 
 #else
 
-value unix_fchown(value fd, value uid, value gid)
+CAMLprim value unix_fchown(value fd, value uid, value gid)
 { invalid_argument("fchown not implemented"); }
   
 #endif
