@@ -6,12 +6,14 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: fcntl.c,v 1.9 2000/04/05 18:30:19 doligez Exp $ */
+/* $Id: fcntl.c,v 1.11.6.1 2004/08/23 11:31:44 doligez Exp $ */
 
+#include <fail.h>
 #include <mlvalues.h>
 #include "unixsupport.h"
 #ifdef HAS_UNISTD
@@ -23,7 +25,7 @@
 #define O_NONBLOCK O_NDELAY
 #endif
 
-value unix_set_nonblock(value fd) /* ML */
+CAMLprim value unix_set_nonblock(value fd)
 {
   int retcode;
   retcode = fcntl(Int_val(fd), F_GETFL, 0);
@@ -33,7 +35,7 @@ value unix_set_nonblock(value fd) /* ML */
   return Val_unit;
 }
 
-value unix_clear_nonblock(value fd) /* ML */
+CAMLprim value unix_clear_nonblock(value fd)
 {
   int retcode;
   retcode = fcntl(Int_val(fd), F_GETFL, 0);
@@ -45,7 +47,7 @@ value unix_clear_nonblock(value fd) /* ML */
 
 #ifdef FD_CLOEXEC
 
-value unix_set_close_on_exec(value fd) /* ML */
+CAMLprim value unix_set_close_on_exec(value fd)
 {
   int retcode;
   retcode = fcntl(Int_val(fd), F_GETFD, 0);
@@ -55,7 +57,7 @@ value unix_set_close_on_exec(value fd) /* ML */
   return Val_unit;
 }
 
-value unix_clear_close_on_exec(value fd) /* ML */
+CAMLprim value unix_clear_close_on_exec(value fd)
 {
   int retcode;
   retcode = fcntl(Int_val(fd), F_GETFD, 0);
@@ -67,10 +69,10 @@ value unix_clear_close_on_exec(value fd) /* ML */
 
 #else
 
-value unix_set_close_on_exec(value fd)
+CAMLprim value unix_set_close_on_exec(value fd)
 { invalid_argument("set_close_on_exec not implemented"); }
 
-value unix_clear_close_on_exec(value fd)
+CAMLprim value unix_clear_close_on_exec(value fd)
 { invalid_argument("clear_close_on_exec not implemented"); }
 
 #endif
