@@ -6,16 +6,17 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: getegid.c,v 1.6 1999/11/17 18:58:00 xleroy Exp $ */
+/* $Id: getegid.c,v 1.8.6.1 2005/01/17 18:10:36 doligez Exp $ */
 
 #include <mlvalues.h>
 #include "unixsupport.h"
 
-value unix_getegid(void)             /* ML */
+CAMLprim value unix_getegid(value unit)
 {
   return Val_int(getegid());
 }
