@@ -6,14 +6,16 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: getgroups.c,v 1.8 2000/02/29 13:55:48 xleroy Exp $ */
+/* $Id: getgroups.c,v 1.10.6.1 2004/08/23 11:31:44 doligez Exp $ */
 
 #include <mlvalues.h>
 #include <alloc.h>
+#include <fail.h>
 
 #ifdef HAS_GETGROUPS
 
@@ -24,7 +26,7 @@
 #include <limits.h>
 #include "unixsupport.h"
 
-value unix_getgroups(value unit)           /* ML */
+CAMLprim value unix_getgroups(value unit)
 {
   gid_t gidset[NGROUPS_MAX];
   int n;
@@ -41,7 +43,7 @@ value unix_getgroups(value unit)           /* ML */
 
 #else
 
-value unix_getgroups(value unit)
+CAMLprim value unix_getgroups(value unit)
 { invalid_argument("getgroups not implemented"); }
 
 #endif
