@@ -11,13 +11,13 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: gethost.c,v 1.20 2001/12/07 13:40:29 xleroy Exp $ */
+/* $Id: gethost.c,v 1.24.2.1 2004/08/23 11:31:44 doligez Exp $ */
 
 #include <string.h>
 #include <mlvalues.h>
 #include <alloc.h>
-#include <memory.h>
 #include <fail.h>
+#include <memory.h>
 #include <signals.h>
 #include "unixsupport.h"
 
@@ -25,9 +25,17 @@
 
 #include "socketaddr.h"
 #ifndef _WIN32
+#include <sys/types.h>
 #include <netdb.h>
 #endif
 
+#define NETDB_BUFFER_SIZE 10000
+
+#ifdef _WIN32
+#define GETHOSTBYADDR_IS_REENTRANT 1
+#define GETHOSTBYNAME_IS_REENTRANT 1
+#endif
+
 static int entry_h_length;
 
 extern int socket_domain_table[];
@@ -35,8 +43,15 @@ extern int socket_domain_table[];
 static value alloc_one_addr(char const *a)
 {
   struct in_addr addr;
-  memmove (&addr, a, entry_h_length);
-  return alloc_inet_addr(addr.s_addr);
+#ifdef HAS_IPV6
+  struct in6_addr addr6;
+  if (entry_h_length == 16) {
+    memmove(&addr6, a, 16);
+    return alloc_inet6_addr(&addr6);
+  }
+#endif
+  memmove (&addr, a, 4);
+  return alloc_inet_addr(&addr);
 }
 
 static value alloc_host_entry(struct hostent *entry)
@@ -67,27 +82,85 @@ static value alloc_host_entry(struct hostent *entry)
 
 CAMLprim value unix_gethostbyaddr(value a)
 {
-  uint32 adr;
-  struct hostent * entry;
-  adr = GET_INET_ADDR(a);
+  struct in_addr adr = GET_INET_ADDR(a);
+  struct hostent * hp;
+#if HAS_GETHOSTBYADDR_R == 7
+  struct hostent h;
+  char buffer[NETDB_BUFFER_SIZE];
+  int h_errnop;
   enter_blocking_section();
-  entry = gethostbyaddr((char *) &adr, 4, AF_INET);
+  hp = gethostbyaddr_r((char *) &adr, 4, AF_INET,
+                       &h, buffer, sizeof(buffer), &h_errnop);
   leave_blocking_section();
-  if (entry == (struct hostent *) NULL) raise_not_found();
-  return alloc_host_entry(entry);
+#elif HAS_GETHOSTBYADDR_R == 8
+  struct hostent h;
+  char buffer[NETDB_BUFFER_SIZE];
+  int h_errnop, rc;
+  enter_blocking_section();
+  rc = gethostbyaddr_r((char *) &adr, 4, AF_INET,
+                       &h, buffer, sizeof(buffer), &hp, &h_errnop);
+  leave_blocking_section();
+  if (rc != 0) hp = NULL;
+#else
+#ifdef GETHOSTBYADDR_IS_REENTRANT
+  enter_blocking_section();
+#endif
+  hp = gethostbyaddr((char *) &adr, 4, AF_INET);
+#ifdef GETHOSTBYADDR_IS_REENTRANT
+  leave_blocking_section();
+#endif
+#endif
+  if (hp == (struct hostent *) NULL) raise_not_found();
+  return alloc_host_entry(hp);
 }
 
 CAMLprim value unix_gethostbyname(value name)
 {
-  char hostname[256];
-  struct hostent * entry;
-  strncpy(hostname, String_val(name), sizeof(hostname) - 1);
-  hostname[sizeof(hostname) - 1] = 0;
+  struct hostent * hp;
+  char * hostname;
+
+#if HAS_GETHOSTBYNAME_R || GETHOSTBYNAME_IS_REENTRANT
+  hostname = stat_alloc(string_length(name) + 1);
+  strcpy(hostname, String_val(name));
+#else
+  hostname = String_val(name);
+#endif
+
+#if HAS_GETHOSTBYNAME_R == 5
+  {
+    struct hostent h;
+    char buffer[NETDB_BUFFER_SIZE];
+    int h_errno;
+    enter_blocking_section();
+    hp = gethostbyname_r(hostname, &h, buffer, sizeof(buffer), &h_errno);
+    leave_blocking_section();
+  }
+#elif HAS_GETHOSTBYNAME_R == 6
+  {
+    struct hostent h;
+    char buffer[NETDB_BUFFER_SIZE];
+    int h_errno, rc;
+    enter_blocking_section();
+    rc = gethostbyname_r(hostname, &h, buffer, sizeof(buffer), &hp, &h_errno);
+    leave_blocking_section();
+    if (rc != 0) hp = NULL;
+  }
+#else
+#ifdef GETHOSTBYNAME_IS_REENTRANT
   enter_blocking_section();
-  entry = gethostbyname(hostname);
+#endif
+  hp = gethostbyname(hostname);
+#ifdef GETHOSTBYNAME_IS_REENTRANT
   leave_blocking_section();
-  if (entry == (struct hostent *) NULL) raise_not_found();
-  return alloc_host_entry(entry);
+#endif
+#endif
+
+#if HAS_GETHOSTBYNAME_R || GETHOSTBYNAME_IS_REENTRANT
+  stat_free(hostname);
+#endif
+
+  if (hp == (struct hostent *) NULL) raise_not_found();
+  return alloc_host_entry(hp);
 }
 
 #else
