@@ -11,13 +11,14 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: gethostname.c,v 1.10 2001/12/07 13:40:29 xleroy Exp $ */
+/* $Id: gethostname.c,v 1.10.6.1 2004/08/23 11:31:44 doligez Exp $ */
 
 #include <mlvalues.h>
 #include <alloc.h>
+#include <fail.h>
 #if defined (_WIN32)
 #include <winsock.h>
-#elif !macintosh
+#else
 #include <sys/param.h>
 #endif
 #include "unixsupport.h"
