@@ -6,16 +6,17 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: getproto.c,v 1.10 2001/07/02 12:24:13 xleroy Exp $ */
+/* $Id: getproto.c,v 1.12.6.1 2004/08/23 11:31:44 doligez Exp $ */
 
 #include <mlvalues.h>
 #include <alloc.h>
-#include <memory.h>
 #include <fail.h>
+#include <memory.h>
 #include "unixsupport.h"
 
 #ifdef HAS_SOCKETS
@@ -42,7 +43,7 @@ static value alloc_proto_entry(struct protoent *entry)
   return res;
 }
 
-value unix_getprotobyname(value name)  /* ML */
+CAMLprim value unix_getprotobyname(value name)
 {
   struct protoent * entry;
   entry = getprotobyname(String_val(name));
@@ -50,7 +51,7 @@ value unix_getprotobyname(value name)  /* ML */
   return alloc_proto_entry(entry);
 }
 
-value unix_getprotobynumber(value proto) /* ML */
+CAMLprim value unix_getprotobynumber(value proto)
 {
   struct protoent * entry;
   entry = getprotobynumber(Int_val(proto));
@@ -60,10 +61,10 @@ value unix_getprotobynumber(value proto) /* ML */
 
 #else
 
-value unix_getprotobynumber(value proto)
+CAMLprim value unix_getprotobynumber(value proto)
 { invalid_argument("getprotobynumber not implemented"); }
   
-value unix_getprotobyname(value name)
+CAMLprim value unix_getprotobyname(value name)
 { invalid_argument("getprotobyname not implemented"); }
 
 #endif
