@@ -6,14 +6,16 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: gettimeofday.c,v 1.5 1999/11/17 18:58:02 xleroy Exp $ */
+/* $Id: gettimeofday.c,v 1.7.6.1 2004/08/23 11:31:44 doligez Exp $ */
 
 #include <mlvalues.h>
 #include <alloc.h>
+#include <fail.h>
 #include "unixsupport.h"
 
 #ifdef HAS_GETTIMEOFDAY
@@ -21,7 +23,7 @@
 #include <sys/types.h>
 #include <sys/time.h>
 
-value unix_gettimeofday(value unit)                /* ML */
+CAMLprim value unix_gettimeofday(value unit)
 {
   struct timeval tp;
   if (gettimeofday(&tp, NULL) == -1) uerror("gettimeofday", Nothing);
@@ -30,7 +32,7 @@ value unix_gettimeofday(value unit)                /* ML */
 
 #else
 
-value unix_gettimeofday(value unit)
+CAMLprim value unix_gettimeofday(value unit)
 { invalid_argument("gettimeofday not implemented"); }
 
 #endif
