@@ -6,11 +6,12 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: kill.c,v 1.8 1999/11/17 18:58:03 xleroy Exp $ */
+/* $Id: kill.c,v 1.10 2001/12/07 13:40:31 xleroy Exp $ */
 
 #include <mlvalues.h>
 #include <fail.h>
@@ -18,7 +19,7 @@
 #include <signal.h>
 #include <signals.h>
 
-value unix_kill(value pid, value signal)     /* ML */
+CAMLprim value unix_kill(value pid, value signal)
 {
   int sig;
   sig = convert_signal_number(Int_val(signal));
