@@ -6,12 +6,14 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: listen.c,v 1.8 2000/04/05 18:30:20 doligez Exp $ */
+/* $Id: listen.c,v 1.10.6.1 2004/08/23 11:31:44 doligez Exp $ */
 
+#include <fail.h>
 #include <mlvalues.h>
 #include "unixsupport.h"
 
@@ -19,7 +21,7 @@
 
 #include <sys/socket.h>
 
-value unix_listen(value sock, value backlog) /* ML */
+CAMLprim value unix_listen(value sock, value backlog)
 {
   if (listen(Int_val(sock), Int_val(backlog)) == -1) uerror("listen", Nothing);
   return Val_unit;
@@ -27,7 +29,7 @@ value unix_listen(value sock, value backlog) /* ML */
 
 #else
 
-value unix_listen(value sock, value backlog)
+CAMLprim value unix_listen(value sock, value backlog)
 { invalid_argument("listen not implemented"); }
 
 #endif
