@@ -11,9 +11,13 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: lseek.c,v 1.9 2001/12/07 13:40:31 xleroy Exp $ */
+/* $Id: lseek.c,v 1.10 2002/03/02 09:16:36 xleroy Exp $ */
 
+#include <errno.h>
+#include <sys/types.h>
 #include <mlvalues.h>
+#include <alloc.h>
+#include <io.h>
 #include "unixsupport.h"
 
 #ifdef HAS_UNISTD
@@ -24,15 +28,30 @@
 #define SEEK_END 2
 #endif
 
+#ifndef EOVERFLOW
+#define EOVERFLOW ERANGE
+#endif
+
 static int seek_command_table[] = {
   SEEK_SET, SEEK_CUR, SEEK_END
 };
 
 CAMLprim value unix_lseek(value fd, value ofs, value cmd)
 {
-  long ret;
+  file_offset ret;
   ret = lseek(Int_val(fd), Long_val(ofs),
                        seek_command_table[Int_val(cmd)]);
   if (ret == -1) uerror("lseek", Nothing);
+  if (ret > Max_long) unix_error(EOVERFLOW, "lseek", Nothing);
   return Val_long(ret);
 }
+
+CAMLprim value unix_lseek_64(value fd, value ofs, value cmd)
+{
+  file_offset ret;
+  ret = lseek(Int_val(fd), File_offset_val(ofs),
+                       seek_command_table[Int_val(cmd)]);
+  if (ret == -1) uerror("lseek", Nothing);
+  return Val_file_offset(ret);
+}
+
