@@ -6,15 +6,17 @@
 /*                                                                     */
 /*  Copyright 1998 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: putenv.c,v 1.6 2000/11/23 13:45:00 doligez Exp $ */
+/* $Id: putenv.c,v 1.8.6.1 2004/08/23 11:31:44 doligez Exp $ */
 
 #include <stdlib.h>
 #include <string.h>
 
+#include <fail.h>
 #include <memory.h>
 #include <mlvalues.h>
 
@@ -22,7 +24,7 @@
 
 #ifdef HAS_PUTENV
 
-value unix_putenv(value name, value val) /* ML */
+CAMLprim value unix_putenv(value name, value val)
 {
   mlsize_t namelen = string_length(name);
   mlsize_t vallen = string_length(val);
@@ -38,7 +40,7 @@ value unix_putenv(value name, value val) /* ML */
 
 #else
 
-value unix_putenv(value name, value val)
+CAMLprim value unix_putenv(value name, value val)
 { invalid_argument("putenv not implemented"); }
 
 #endif
