@@ -6,14 +6,16 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: readlink.c,v 1.8 1999/11/17 18:58:05 xleroy Exp $ */
+/* $Id: readlink.c,v 1.10.6.1 2004/08/23 11:31:44 doligez Exp $ */
 
 #include <mlvalues.h>
 #include <alloc.h>
+#include <fail.h>
 
 #ifdef HAS_SYMLINK
 
@@ -28,7 +30,7 @@
 #endif
 #endif
 
-value unix_readlink(value path)        /* ML */
+CAMLprim value unix_readlink(value path)
 {
   char buffer[PATH_MAX];
   int len;
@@ -40,7 +42,7 @@ value unix_readlink(value path)        /* ML */
 
 #else
 
-value unix_readlink(value path)
+CAMLprim value unix_readlink(value path)
 { invalid_argument("readlink not implemented"); }
 
 #endif
