@@ -6,18 +6,19 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: time.c,v 1.7 1999/11/17 18:58:08 xleroy Exp $ */
+/* $Id: time.c,v 1.9.6.1 2005/01/17 18:10:36 doligez Exp $ */
 
 #include <time.h>
 #include <mlvalues.h>
 #include <alloc.h>
 #include "unixsupport.h"
 
-value unix_time(void)                /* ML */
+CAMLprim value unix_time(value unit)
 {
   return copy_double((double) time((time_t *) NULL));
 }
