@@ -11,7 +11,7 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: write.c,v 1.13 2001/12/07 13:40:39 xleroy Exp $ */
+/* $Id: write.c,v 1.13.6.3 2004/07/08 08:40:47 xleroy Exp $ */
 
 #include <errno.h>
 #include <string.h>
@@ -54,3 +54,34 @@ CAMLprim value unix_write(value fd, value buf, value vofs, value vlen)
   End_roots();
   return Val_long(written);
 }
+
+/* When an error occurs after the first loop, unix_write reports the
+   error and discards the number of already written characters.
+   In this case, it would be better to discard the error and return the
+   number of bytes written, since most likely, unix_write will be call again,
+   and the error will be reproduced and this time will be reported.
+   This problem is avoided in unix_single_write, which is faithful to the
+   Unix system call. */
+
+CAMLprim value unix_single_write(value fd, value buf, value vofs, value vlen) 
+{
+  long ofs, len;
+  int numbytes, ret;
+  char iobuf[UNIX_BUFFER_SIZE];
+
+  Begin_root (buf);
+    ofs = Long_val(vofs);
+    len = Long_val(vlen);
+    ret = 0;
+    if (len > 0) {
+      numbytes = len > UNIX_BUFFER_SIZE ? UNIX_BUFFER_SIZE : len;
+      memmove (iobuf, &Byte(buf, ofs), numbytes);
+      enter_blocking_section();
+      ret = write(Int_val(fd), iobuf, numbytes);
+      leave_blocking_section();
+      if (ret == -1) uerror("single_write", Nothing);
+    }
+  End_roots();
+  return Val_int(ret);
+}
+
