@@ -10,15 +10,16 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: draw.c,v 1.5 2002/07/26 08:37:21 xleroy Exp $ */
+/* $Id: draw.c,v 1.9.2.3 2005/02/03 16:40:12 xleroy Exp $ */
 
 #include <math.h>
 #include "mlvalues.h"
 #include "alloc.h"
+#include "fail.h"
 #include "libgraph.h"
 #include "custom.h"
 #include "memory.h"
-#define UD(y)  (grwindow.height - 1 - y)
+
 HDC gcMetaFile;
 int grdisplay_mode;
 int grremember_mode;
@@ -33,24 +34,20 @@ static void GetCurrentPosition(HDC hDC,POINT *pt)
 static value gr_draw_or_fill_arc(value vx, value vy, value vrx, value vry,
 				 value vstart, value vend, BOOL fill);
 
-CAMLprim value gr_plot(value vx, value vy)
+CAMLprim value caml_gr_plot(value vx, value vy)
 {
         int x = Int_val(vx);
         int y = Int_val(vy);
         gr_check_open();
-        //      gr_moveto(vx,vy);
-        //      gr_lineto(Val_int(Int_val(vx)+1),vy);
-        //      return Val_unit;
         if(grremember_mode)
                 SetPixel(grwindow.gcBitmap, x, Wcvt(y),grwindow.CurrentColor);
         if(grdisplay_mode) {
                 SetPixel(grwindow.gc, x, Wcvt(y),grwindow.CurrentColor);
         }
-        //      gr_moveto(vx+1,vy);
         return Val_unit;
 }
 
-CAMLprim value gr_moveto(value vx, value vy)
+CAMLprim value caml_gr_moveto(value vx, value vy)
 {
         grwindow.grx = Int_val(vx);
         grwindow.gry = Int_val(vy);
@@ -61,17 +58,17 @@ CAMLprim value gr_moveto(value vx, value vy)
         return Val_unit;
 }
 
-CAMLprim value gr_current_x(void)
+CAMLprim value caml_gr_current_x(void)
 {
         return Val_int(grwindow.grx);
 }
 
-CAMLprim value gr_current_y(void)
+CAMLprim value caml_gr_current_y(void)
 {
         return Val_int(grwindow.gry);
 }
 
-CAMLprim value gr_lineto(value vx, value vy)
+CAMLprim value caml_gr_lineto(value vx, value vy)
 {
         int x = Int_val(vx);
         int y = Int_val(vy);
@@ -87,43 +84,20 @@ CAMLprim value gr_lineto(value vx, value vy)
         return Val_unit;
 }
 
-CAMLprim value gr_draw_rect(value vx, value vy, value vw, value vh)
+CAMLprim value caml_gr_draw_rect(value vx, value vy, value vw, value vh)
 {
-#if 0
-        int x = Int_val(vx);
-        int y = Int_val(vy);
-        int w = Int_val(vw);
-        int h = Int_val(vh);
-
-        gr_check_open();
-        if(grdisplay_mode) {
-                Rectangle(grwindow.gc,x, Wcvt(y) , x+w, Wcvt(y+h));
-        }
-        if(grremember_mode) {
-                Rectangle(grwindow.gcBitmap,x, Wcvt(y), x+w, Wcvt(h+y));
-        }
-        return Val_unit;
-#else
-        // (x,y)=top bottom
         int     x, y, w, h;
         POINT pt[5];
         x=Int_val(vx);
-        y=Int_val(vy);
+        y=Wcvt(Int_val(vy));
         w=Int_val(vw);
         h=Int_val(vh);
 
-        //                      pt[0].x = x;   pt[0].y = UD(y+h-1);
-        pt[0].x = x;
-        pt[0].y = Wcvt(y-1);
-        pt[1].x = x+w;
-        pt[1].y = pt[0].y;
-        //                      pt[2].x = pt[1].x;  pt[2].y = UD(y-1);
-        pt[2].x = pt[1].x;
-        pt[2].y = Wcvt(y+h-1);
-        pt[3].x = pt[0].x;
-        pt[3].y = pt[2].y;
-        pt[4].x = pt[0].x;
-        pt[4].y = pt[0].y;
+        pt[0].x = x;         pt[0].y = y - h;
+	pt[1].x = x + w;     pt[1].y = y - h;
+	pt[2].x = x + w;     pt[2].y = y;
+	pt[3].x = x;         pt[3].y = y;
+	pt[4].x = x;         pt[4].y = y - h;
         if (grremember_mode) {
                 Polyline(grwindow.gcBitmap,pt, 5);
         }
@@ -131,10 +105,9 @@ CAMLprim value gr_draw_rect(value vx, value vy, value vw, value vh)
                 Polyline(grwindow.gc,pt, 5);
         }
         return Val_unit;
-#endif
 }
 
-CAMLprim value gr_draw_text(value text,value x)
+CAMLprim value caml_gr_draw_text(value text,value x)
 {
         POINT pt;
         int oldmode = SetBkMode(grwindow.gc,TRANSPARENT);
@@ -155,7 +128,7 @@ CAMLprim value gr_draw_text(value text,value x)
         return Val_unit;
 }
 
-CAMLprim value gr_fill_rect(value vx, value vy, value vw, value vh)
+CAMLprim value caml_gr_fill_rect(value vx, value vy, value vw, value vh)
 {
         int x = Int_val(vx);
         int y = Int_val(vy);
@@ -175,13 +148,13 @@ CAMLprim value gr_fill_rect(value vx, value vy, value vw, value vh)
         return Val_unit;
 }
 
-CAMLprim value gr_sound(value freq, value vdur)
+CAMLprim value caml_gr_sound(value freq, value vdur)
 {
         Beep(freq,vdur);
         return Val_unit;
 }
 
-CAMLprim value gr_point_color(value vx, value vy)
+CAMLprim value caml_gr_point_color(value vx, value vy)
 {
         int x = Int_val(vx);
         int y = Int_val(vy);
@@ -196,7 +169,7 @@ CAMLprim value gr_point_color(value vx, value vy)
         return Val_long((r<<16) + (g<<8) + b);
 }
 
-CAMLprim value gr_circle(value x,value y,value radius)
+CAMLprim value caml_gr_circle(value x,value y,value radius)
 {
         int left,top,right,bottom;
 
@@ -209,24 +182,24 @@ CAMLprim value gr_circle(value x,value y,value radius)
         return Val_unit;
 }
 
-CAMLprim value gr_set_window_title(value text)
+CAMLprim value caml_gr_set_window_title(value text)
 {
         SetWindowText(grwindow.hwnd,(char *)text);
         return Val_unit;
 }
 
-CAMLprim value gr_draw_arc(value *argv, int argc)
+CAMLprim value caml_gr_draw_arc(value *argv, int argc)
 {
   return gr_draw_or_fill_arc(argv[0], argv[1], argv[2], argv[3],
 			     argv[4], argv[5], FALSE);
 }
 
-CAMLprim value gr_draw_arc_nat(vx, vy, vrx, vry, vstart, vend)
+CAMLprim value caml_gr_draw_arc_nat(vx, vy, vrx, vry, vstart, vend)
 {
   return gr_draw_or_fill_arc(vx, vy, vrx, vry, vstart, vend, FALSE);
 }
 
-CAMLprim value gr_set_line_width(value vwidth)
+CAMLprim value caml_gr_set_line_width(value vwidth)
 {
         int width = Int_val(vwidth);
         HPEN oldPen,newPen;
@@ -241,7 +214,7 @@ CAMLprim value gr_set_line_width(value vwidth)
         return Val_unit;
 }
 
-CAMLprim value gr_set_color(value vcolor)
+CAMLprim value caml_gr_set_color(value vcolor)
 {
         HBRUSH oldBrush, newBrush;
         LOGBRUSH lb;
@@ -265,7 +238,6 @@ CAMLprim value gr_set_color(value vcolor)
         grwindow.CurrentPen = newPen;
         SetTextColor(grwindow.gc,c);
         SetTextColor(grwindow.gcBitmap,c);
-        //      printf("gr_set_color, vcolor = %x, lbColor = %x\n", Int_val(vcolor), lb.lbColor);
         oldBrush = grwindow.CurrentBrush;
         lb.lbStyle = BS_SOLID;
         lb.lbColor = c;
@@ -312,26 +284,26 @@ static value gr_draw_or_fill_arc(value vx, value vy, value vrx, value vry,
                 SelectObject(grwindow.gcBitmap,grwindow.CurrentPen);
                 SelectObject(grwindow.gcBitmap,grwindow.CurrentBrush);
                 if( fill )
-                        Pie(grwindow.gcBitmap,x1, UD(y1), x2, UD(y2),
-                                x3, UD(y3), x4, UD(y4));
+                        Pie(grwindow.gcBitmap,x1, Wcvt(y1), x2, Wcvt(y2),
+                                x3, Wcvt(y3), x4, Wcvt(y4));
                 else
-                        Arc(grwindow.gcBitmap,x1, UD(y1), x2, UD(y2),
-                                x3, UD(y3), x4, UD(y4));
+                        Arc(grwindow.gcBitmap,x1, Wcvt(y1), x2, Wcvt(y2),
+                                x3, Wcvt(y3), x4, Wcvt(y4));
         }
         if( grdisplay_mode ) {
                 SelectObject(grwindow.gc,grwindow.CurrentPen);
                 SelectObject(grwindow.gc,grwindow.CurrentBrush);
                 if (fill)
-                        Pie(grwindow.gc,x1, UD(y1), x2, UD(y2),
-                                x3, UD(y3), x4, UD(y4));
+                        Pie(grwindow.gc,x1, Wcvt(y1), x2, Wcvt(y2),
+                                x3, Wcvt(y3), x4, Wcvt(y4));
                 else
-                        Arc(grwindow.gc,x1, UD(y1), x2, UD(y2),
-                                x3, UD(y3), x4, UD(y4));
+                        Arc(grwindow.gc,x1, Wcvt(y1), x2, Wcvt(y2),
+                                x3, Wcvt(y3), x4, Wcvt(y4));
         }
         return Val_unit;
 }
 
-CAMLprim value gr_show_bitmap(value filename,int x,int y)
+CAMLprim value caml_gr_show_bitmap(value filename,int x,int y)
 {
         AfficheBitmap(filename,grwindow.gcBitmap,x,Wcvt(y));
         AfficheBitmap(filename,grwindow.gc,x,Wcvt(y));
@@ -340,7 +312,7 @@ CAMLprim value gr_show_bitmap(value filename,int x,int y)
 
 
 
-CAMLprim value gr_get_mousex(void)
+CAMLprim value caml_gr_get_mousex(void)
 {
         POINT pt;
         GetCursorPos(&pt);
@@ -348,7 +320,7 @@ CAMLprim value gr_get_mousex(void)
         return pt.x;
 }
 
-CAMLprim value gr_get_mousey(void)
+CAMLprim value caml_gr_get_mousey(void)
 {
         POINT pt;
         GetCursorPos(&pt);
@@ -369,35 +341,35 @@ static void gr_font(char *fontname)
         }
 }
 
-CAMLprim value gr_set_font(value fontname)
+CAMLprim value caml_gr_set_font(value fontname)
 {
         gr_check_open();
         gr_font(String_val(fontname));
         return Val_unit;
 }
 
-CAMLprim value gr_set_text_size (value sz)
+CAMLprim value caml_gr_set_text_size (value sz)
 {
         return Val_unit;
 }
 
-CAMLprim value gr_draw_char(value chr)
+CAMLprim value caml_gr_draw_char(value chr)
 {
         char str[1];
         gr_check_open();
         str[0] = Int_val(chr);
-        gr_draw_text((value)str, 1);
+        caml_gr_draw_text((value)str, 1);
         return Val_unit;
 }
 
-CAMLprim value gr_draw_string(value str)
+CAMLprim value caml_gr_draw_string(value str)
 {
         gr_check_open();
-        gr_draw_text(str, string_length(str));
+        caml_gr_draw_text(str, string_length(str));
         return Val_unit;
 }
 
-CAMLprim value gr_text_size(value str)
+CAMLprim value caml_gr_text_size(value str)
 {
         SIZE extent;
         value res;
@@ -414,125 +386,7 @@ CAMLprim value gr_text_size(value str)
         return res;
 }
 
-#if 0
-static unsigned char gr_queue[SIZE_QUEUE];
-static int gr_head = 0;       /* position of next read */
-static int gr_tail = 0;       /* position of next write */
-
-#define QueueIsEmpty (gr_head == gr_tail)
-#define QueueIsFull  (gr_head == gr_tail + 1)
-
-void gr_enqueue_char(unsigned char c)
-{
-        if (QueueIsFull) return;
-        gr_queue[gr_tail] = c;
-        gr_tail++;
-        if (gr_tail >= SIZE_QUEUE) gr_tail = 0;
-}
-#endif
-
-#define Button_down             1
-#define Button_up               2
-#define Key_pressed             4
-#define Mouse_motion    8
-#define Poll                    16
-MSG * InspectMessages = NULL;
-
-CAMLprim value gr_wait_event(value eventlist)
-{
-        value res;
-        int mask;
-        BOOL poll;
-        int mouse_x, mouse_y, button, key;
-        int root_x, root_y, win_x, win_y;
-        int r,i,stop;
-        unsigned int modifiers;
-        POINT pt;
-        MSG msg;
-
-        gr_check_open();
-        mask = 0;
-        poll = FALSE;
-        while (eventlist != Val_int(0)) {
-                switch (Int_val(Field(eventlist,0))) {
-                case 0:                     /* Button_down */
-                        mask |= Button_down;
-                        break;
-                case 1:                     /* Button_up */
-                        mask |= Button_up;
-                        break;
-                case 2:                     /* Key_pressed */
-                        mask |= Key_pressed;
-                        break;
-                case 3:                     /* Mouse_motion */
-                        mask |= Mouse_motion;
-                        break;
-                case 4:                     /* Poll */
-                        poll = TRUE;
-                        break;
-                }
-                eventlist = Field(eventlist,1);
-        }
-        mouse_x = -1;
-        mouse_y = -1;
-        button = 0;
-        key = -1;
-
-        if (poll) {
-                // Poll uses info on last event stored in global variables
-                mouse_x = MouseLastX;
-                mouse_y = MouseLastY;
-                button = MouseLbuttonDown | MouseMbuttonDown | MouseRbuttonDown;
-                key = LastKey;
-        }
-        else { // Not polled. Block for a message
-                InspectMessages = &msg;
-                while (1) {
-                        WaitForSingleObject(EventHandle,INFINITE);
-                        stop = 0;
-                        switch (msg.message) {
-                        case WM_LBUTTONDOWN:
-                        case WM_MBUTTONDOWN:
-                        case WM_RBUTTONDOWN:
-                                button = 1;
-                                if (mask&Button_down) stop = 1;
-                                break;
-                        case WM_LBUTTONUP:
-                        case WM_MBUTTONUP:
-                        case WM_RBUTTONUP:
-                                button = 0;
-                                if (mask&Button_up) stop = 1;
-                                break;
-                        case WM_MOUSEMOVE:
-                                if (mask&Mouse_motion) stop = 1;
-                                break;
-                        case WM_CHAR:
-                                key = msg.wParam & 0xFF;
-                                if (mask&Key_pressed) stop = 1;
-                                break;
-                        }
-                        if (stop) {
-                                pt = msg.pt;
-                                MapWindowPoints(HWND_DESKTOP,grwindow.hwnd,&pt,1);
-                                mouse_x = pt.x;
-                                mouse_y = grwindow.height- 1 - pt.y;
-                                break;
-                        }
-                        if (msg.message == WM_CLOSE)
-                                break;
-                }
-                InspectMessages = NULL;
-        }
-        res = alloc_small(5, 0);
-        Field(res, 0) = Val_int(mouse_x);
-        Field(res, 1) = Val_int(mouse_y);
-        Field(res, 2) = Val_bool(button);
-        Field(res, 3) = Val_bool(key != -1);
-        Field(res, 4) = Val_int(key & 0xFF);
-        return res;
-}
-
-CAMLprim value gr_fill_poly(value vect)
+CAMLprim value caml_gr_fill_poly(value vect)
 {
         int n_points, i;
         POINT   *p,*poly;
@@ -545,7 +399,7 @@ CAMLprim value gr_fill_poly(value vect)
         p = poly;
         for( i = 0; i < n_points; i++ ){
                 p->x = Int_val(Field(Field(vect,i),0));
-                p->y = UD(Int_val(Field(Field(vect,i),1)));
+                p->y = Wcvt(Int_val(Field(Field(vect,i),1)));
                 p++;
         }
         if (grremember_mode) {
@@ -561,13 +415,13 @@ CAMLprim value gr_fill_poly(value vect)
         return Val_unit;
 }
 
-CAMLprim value gr_fill_arc(value *argv, int argc)
+CAMLprim value caml_gr_fill_arc(value *argv, int argc)
 {
   return gr_draw_or_fill_arc(argv[0], argv[1], argv[2], argv[3],
 			     argv[4], argv[5], TRUE);
 }
 
-CAMLprim value gr_fill_arc_nat(vx, vy, vrx, vry, vstart, vend)
+CAMLprim value caml_gr_fill_arc_nat(vx, vy, vrx, vry, vstart, vend)
 {
   return gr_draw_or_fill_arc(vx, vy, vrx, vry, vstart, vend, TRUE);
 }
@@ -584,13 +438,12 @@ struct image {
 #define Height(i) (((struct image *)Data_custom_val(i))->h)
 #define Data(i) (((struct image *)Data_custom_val(i))->data)
 #define Mask(i) (((struct image *)Data_custom_val(i))->mask)
-//(1280x1024)
-#define Max_image_mem 53000000 
+#define Max_image_mem 500000 
 
 static void finalize_image (value i)
 {
-        free (Data(i));
-        if (Mask(i) != NULL) free(Mask(i));
+        DeleteObject (Data(i));
+        if (Mask(i) != NULL) DeleteObject(Mask(i));
 }
 
 static struct custom_operations image_ops = {
@@ -602,31 +455,35 @@ static struct custom_operations image_ops = {
         custom_deserialize_default
 };
 
-CAMLprim value gr_create_image(value w,value h)
+CAMLprim value caml_gr_create_image(value vw, value vh)
 {
         HBITMAP cbm;
         value res;
+	int w = Int_val(vw);
+	int h = Int_val(vh);
 
-        if (Int_val (w) < 0 || Int_val (h) < 0)
+        if (w < 0 || h < 0)
                 gr_fail("create_image: width and height must be positive",0);
 
-        cbm = CreateCompatibleBitmap(grwindow.gcBitmap, Int_val(w), Int_val(h));
+        cbm = CreateCompatibleBitmap(grwindow.gc, w, h);
+	if (cbm == NULL)
+	        gr_fail("create_image: cannot create bitmap", 0);
         res = alloc_custom(&image_ops, sizeof(struct image),
                 w * h, Max_image_mem);
         if (res) {
-                Width (res) = Int_val(w);
-                Height (res) = Int_val(h);
+		Width (res) = w;
+                Height (res) = h;
                 Data (res) = cbm;
                 Mask (res) = NULL;
         }
         return res;
 }
 
-CAMLprim value gr_blit_image (value i, value x, value y)
+CAMLprim value caml_gr_blit_image (value i, value x, value y)
 {
         HBITMAP oldBmp = SelectObject(grwindow.tempDC,Data(i));
         int xsrc = Int_val(x);
-        int ysrc = UD(Int_val(y) + Height(i) - 1);
+        int ysrc = Wcvt(Int_val(y) + Height(i) - 1);
         BitBlt(grwindow.tempDC,0, 0, Width(i), Height(i),
                 grwindow.gcBitmap, xsrc, ysrc, SRCCOPY);
         SelectObject(grwindow.tempDC,oldBmp);
@@ -634,12 +491,11 @@ CAMLprim value gr_blit_image (value i, value x, value y)
 }
 
 
-CAMLprim value gr_draw_image(value i, value x, value y)
+CAMLprim value caml_gr_draw_image(value i, value x, value y)
 {
         HBITMAP oldBmp;
 
         int xdst = Int_val(x);
-//      int ydst = UD(Int_val(y) + Height(i) - 1);
         int ydst = Wcvt(Int_val(y)+Height(i)-1);
         if (Mask(i) == NULL) {
                 if (grremember_mode) {
@@ -679,7 +535,7 @@ CAMLprim value gr_draw_image(value i, value x, value y)
         return Val_unit;
 }
 
-CAMLprim value gr_make_image(value matrix)
+CAMLprim value caml_gr_make_image(value matrix)
 {
         int width, height,has_transp,i,j;
         value img;
@@ -696,7 +552,7 @@ CAMLprim value gr_make_image(value matrix)
                 }
         }
         Begin_roots1(matrix)
-                img = gr_create_image(Val_int(width), Val_int(height));
+                img = caml_gr_create_image(Val_int(width), Val_int(height));
         End_roots();
         has_transp = 0;
         oldBmp = SelectObject(grwindow.tempDC,Data(img));
@@ -750,7 +606,7 @@ static value alloc_int_vect(mlsize_t size)
         return res;
 }
 
-CAMLprim value gr_dump_image (value img)
+CAMLprim value caml_gr_dump_image (value img)
 {
         int height = Height(img);
         int width = Width(img);
