@@ -11,7 +11,7 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: libgraph.h,v 1.5 2002/07/26 08:37:51 xleroy Exp $ */
+/* $Id: libgraph.h,v 1.8.2.1 2004/06/21 15:31:58 xleroy Exp $ */
 
 #include <stdio.h>
 #include <windows.h>
@@ -24,13 +24,6 @@ struct canvas {
 };
 
 extern HWND grdisplay;     /* The display connection */
-//extern int grscreen;            /* The screen number */
-//extern Colormap grcolormap;     /* The color map */
-//extern struct canvas grwindow;  /* The graphics window */
-//extern struct canvas grbstore;  /* The pixmap used for backing store */
-//extern int grwhite, grblack;    /* Black and white pixels for X */
-//extern int grbackground;        /* Background color for X
-//                                   (used for CAML color -1) */
 extern COLORREF grbackground;
 extern BOOL grdisplay_mode;     /* Display-mode flag */
 extern BOOL grremember_mode;    /* Remember-mode flag */
@@ -46,35 +39,17 @@ extern int bits_per_pixel;
 #define Wcvt(y) (grwindow.height - 1 - (y))
 #define Bcvt(y) (grwindow.height - 1 - (y))
 #define WtoB(y) ((y) + WindowRect.bottom - grwindow.h)
-//#define BtoW(y) ((y) + WindowRect.bottom - grbstore.h)
 
 #define DEFAULT_SCREEN_WIDTH 1024
 #define DEFAULT_SCREEN_HEIGHT 768
 #define BORDER_WIDTH 2
 #define WINDOW_NAME "Caml graphics"
 #define ICON_NAME "Caml graphics"
-#define DEFAULT_EVENT_MASK \
-          (ExposureMask | KeyPressMask | StructureNotifyMask)
-#define DEFAULT_FONT "fixed"
 #define SIZE_QUEUE 256
 
-/* To handle events asynchronously */
-#ifdef HAS_ASYNC_IO
-#define USE_ASYNC_IO
-#define EVENT_SIGNAL SIGIO
-#else
-#ifdef HAS_SETITIMER
-#define USE_INTERVAL_TIMER
-#define EVENT_SIGNAL SIGALRM
-#else
-#define USE_ALARM
-#define EVENT_SIGNAL SIGALRM
-#endif
-#endif
-
 void gr_fail(char *fmt, char *arg);
 void gr_check_open(void);
-CAMLprim value gr_set_color(value vcolor);
+CAMLprim value caml_gr_set_color(value vcolor);
 
 // Windows specific definitions
 extern RECT WindowRect;
@@ -99,10 +74,5 @@ typedef struct tagWindow {
 
 extern GR_WINDOW grwindow;
 HFONT CreationFont(char *name);
-extern int MouseLbuttonDown,MouseMbuttonDown,MouseRbuttonDown;
-extern HANDLE EventHandle;
-extern MSG * InspectMessages;
-extern int MouseLbuttonDown,MouseMbuttonDown,MouseRbuttonDown;
-extern int MouseLastX, MouseLastY;
-extern int LastKey;
-
+extern void caml_gr_init_event_queue(void);
+extern void caml_gr_handle_event(UINT msg, WPARAM wParam, LPARAM lParam);
