@@ -11,7 +11,7 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: accept.c,v 1.17 2002/06/07 09:49:41 xleroy Exp $ */
+/* $Id: accept.c,v 1.18.6.1 2005/02/02 15:40:14 xleroy Exp $ */
 
 #include <mlvalues.h>
 #include <alloc.h>
@@ -29,6 +29,7 @@ CAMLprim value unix_accept(sock)
   int oldvalue, oldvaluelen, newvalue, retcode;
   union sock_addr_union addr;
   socklen_param_type addr_len;
+  int errcode = 0;
 
   oldvaluelen = sizeof(oldvalue);
   retcode = getsockopt(INVALID_SOCKET, SOL_SOCKET, SO_OPENTYPE,
@@ -43,18 +44,20 @@ CAMLprim value unix_accept(sock)
   enter_blocking_section();
   snew = accept(sconn, &addr.s_gen, &addr_len);
   leave_blocking_section();
+  if( snew == INVALID_SOCKET )
+    errcode = WSAGetLastError ();
   if (retcode == 0) {
     /* Restore initial mode */
     setsockopt(INVALID_SOCKET, SOL_SOCKET, SO_OPENTYPE, 
                (char *) &oldvalue, oldvaluelen);
   }
   if (snew == INVALID_SOCKET) {
-    win32_maperr(WSAGetLastError());
+    win32_maperr(errcode);
     uerror("accept", Nothing);
   }
   Begin_roots2 (fd, adr)
     fd = win_alloc_socket(snew);
-    adr = alloc_sockaddr(&addr, addr_len);
+    adr = alloc_sockaddr(&addr, addr_len, snew);
     res = alloc_small(2, 0);
     Field(res, 0) = fd;
     Field(res, 1) = adr;
