@@ -11,7 +11,7 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: bind.c,v 1.9 2001/12/07 13:40:43 xleroy Exp $ */
+/* $Id: bind.c,v 1.10 2002/04/30 15:00:45 xleroy Exp $ */
 
 #include <mlvalues.h>
 #include "unixsupport.h"
@@ -25,7 +25,7 @@ CAMLprim value unix_bind(socket, address)
   socklen_param_type addr_len;
 
   get_sockaddr(address, &addr, &addr_len);
-  ret = bind((SOCKET) Handle_val(socket), &addr.s_gen, addr_len);
+  ret = bind(Socket_val(socket), &addr.s_gen, addr_len);
   if (ret == -1) {
     win32_maperr(WSAGetLastError());
     uerror("bind", Nothing);
