@@ -11,16 +11,23 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: close.c,v 1.4 2001/12/07 13:40:43 xleroy Exp $ */
+/* $Id: close.c,v 1.5 2002/04/30 15:00:46 xleroy Exp $ */
 
 #include <mlvalues.h>
 #include "unixsupport.h"
 
 CAMLprim value unix_close(value fd)
 {
-  if (! CloseHandle(Handle_val(fd))) {
-    win32_maperr(GetLastError());
-    uerror("close", Nothing);
+  if (Descr_kind_val(fd) == KIND_SOCKET) {
+    if (closesocket(Socket_val(fd)) != 0) {
+      win32_maperr(WSAGetLastError());
+      uerror("close", Nothing);
+    }
+  } else {
+    if (! CloseHandle(Handle_val(fd))) {
+      win32_maperr(GetLastError());
+      uerror("close", Nothing);
+    }
   }
   return Val_unit;
 }
