@@ -11,16 +11,17 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: connect.c,v 1.9 2001/12/07 13:40:43 xleroy Exp $ */
+/* $Id: connect.c,v 1.11 2002/06/07 09:49:41 xleroy Exp $ */
 
 #include <mlvalues.h>
+#include <signals.h>
 #include "unixsupport.h"
 #include "socketaddr.h"
 
 CAMLprim value unix_connect(socket, address)
      value socket, address;
 {
-  SOCKET s = (SOCKET) Handle_val(socket);
+  SOCKET s = Socket_val(socket);
   int retcode;
   union sock_addr_union addr;
   socklen_param_type addr_len;
