@@ -11,7 +11,7 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: getpeername.c,v 1.8 2001/12/07 13:40:44 xleroy Exp $ */
+/* $Id: getpeername.c,v 1.9.6.1 2005/02/02 15:40:14 xleroy Exp $ */
 
 #include <mlvalues.h>
 #include "unixsupport.h"
@@ -25,11 +25,11 @@ CAMLprim value unix_getpeername(sock)
   socklen_param_type addr_len;
 
   addr_len = sizeof(sock_addr);
-  retcode = getpeername((SOCKET) Handle_val(sock),
+  retcode = getpeername(Socket_val(sock),
                         &addr.s_gen, &addr_len);
   if (retcode == -1) {
     win32_maperr(WSAGetLastError());
     uerror("getpeername", Nothing);
   }
-  return alloc_sockaddr(&addr, addr_len);
+  return alloc_sockaddr(&addr, addr_len, -1);
 }
