@@ -11,7 +11,7 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: getsockname.c,v 1.6 2001/12/07 13:40:44 xleroy Exp $ */
+/* $Id: getsockname.c,v 1.7.6.1 2005/02/02 15:40:14 xleroy Exp $ */
 
 #include <mlvalues.h>
 #include "unixsupport.h"
@@ -25,8 +25,8 @@ CAMLprim value unix_getsockname(sock)
   socklen_param_type addr_len;
 
   addr_len = sizeof(sock_addr);
-  retcode = getsockname((SOCKET) Handle_val(sock),
+  retcode = getsockname(Socket_val(sock),
                         &addr.s_gen, &addr_len);
   if (retcode == -1) uerror("getsockname", Nothing);
-  return alloc_sockaddr(&addr, addr_len);
+  return alloc_sockaddr(&addr, addr_len, -1);
 }
