@@ -6,11 +6,12 @@
 /*                                                                     */
 /*  Copyright 1996 Institut National de Recherche en Informatique et   */
 /*  en Automatique.  All rights reserved.  This file is distributed    */
-/*  under the terms of the GNU Library General Public License.         */
+/*  under the terms of the GNU Library General Public License, with    */
+/*  the special exception on linking described in file ../../LICENSE.  */
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: pipe.c,v 1.5 1999/11/17 18:58:14 xleroy Exp $ */
+/* $Id: pipe.c,v 1.7 2001/12/07 13:40:45 xleroy Exp $ */
 
 #include <mlvalues.h>
 #include <memory.h>
@@ -20,7 +21,7 @@
 
 #define SIZEBUF 1024
 
-value unix_pipe(value unit)                /* ML */
+CAMLprim value unix_pipe(value unit)
 {
   SECURITY_ATTRIBUTES attr;
   HANDLE readh, writeh;
@@ -30,7 +31,7 @@ value unix_pipe(value unit)                /* ML */
   attr.lpSecurityDescriptor = NULL;
   attr.bInheritHandle = TRUE;
   if (! CreatePipe(&readh, &writeh, &attr, SIZEBUF)) {
-    _dosmaperr(GetLastError());
+    win32_maperr(GetLastError());
     uerror("pipe", Nothing);
   }
   Begin_roots2(readfd, writefd)
