@@ -11,7 +11,7 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: sendrecv.c,v 1.16 2002/06/07 09:49:41 xleroy Exp $ */
+/* $Id: sendrecv.c,v 1.16.6.1 2005/02/02 15:40:14 xleroy Exp $ */
 
 #include <mlvalues.h>
 #include <alloc.h>
@@ -71,7 +71,7 @@ CAMLprim value unix_recvfrom(value sock, value buff, value ofs, value len, value
       uerror("recvfrom", Nothing);
     }
     memmove (&Byte(buff, Long_val(ofs)), iobuf, ret);
-    adr = alloc_sockaddr(&addr, addr_len);
+    adr = alloc_sockaddr(&addr, addr_len, -1);
     res = alloc_small(2, 0);
     Field(res, 0) = Val_int(ret);
     Field(res, 1) = adr;
