@@ -11,11 +11,11 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: sleep.c,v 1.4 2001/12/07 13:40:46 xleroy Exp $ */
+/* $Id: sleep.c,v 1.5 2002/06/07 09:49:41 xleroy Exp $ */
 
 #include <mlvalues.h>
+#include <signals.h>
 #include "unixsupport.h"
-#include <windows.h>
 
 CAMLprim value unix_sleep(t)
      value t;
