@@ -11,11 +11,9 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: socketaddr.h,v 1.5 2001/12/07 13:40:46 xleroy Exp $ */
+/* $Id: socketaddr.h,v 1.7.2.1 2005/02/02 15:40:14 xleroy Exp $ */
 
 #include <misc.h>
-#include <sys/types.h>
-#include <winsock.h>
 
 union sock_addr_union {
   struct sockaddr s_gen;
@@ -30,11 +28,10 @@ typedef socklen_t socklen_param_type;
 typedef int socklen_param_type;
 #endif
 
-void get_sockaddr (value mladdr,
-                   union sock_addr_union * addr /*out*/,
-                   socklen_param_type * addr_len /*out*/);
-value alloc_sockaddr (union sock_addr_union * addr /*in*/,
-                      socklen_param_type addr_len);
-value alloc_inet_addr (uint32 inaddr);
-
-#define GET_INET_ADDR(v) (*((uint32 *) (v)))
+extern void get_sockaddr (value mladdr,
+                          union sock_addr_union * addr /*out*/,
+                          socklen_param_type * addr_len /*out*/);
+CAMLprim value alloc_sockaddr (union sock_addr_union * addr /*in*/,
+                      socklen_param_type addr_len, int close_on_error);
+CAMLprim value alloc_inet_addr (struct in_addr * inaddr);
+#define GET_INET_ADDR(v) (*((struct in_addr *) (v)))
