@@ -11,7 +11,7 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: unixsupport.h,v 1.14 2002/08/07 11:49:10 xleroy Exp $ */
+/* $Id: unixsupport.h,v 1.15 2003/01/06 14:52:57 xleroy Exp $ */
 
 #define WIN32_LEAN_AND_MEAN
 #include <wtypes.h>
@@ -31,16 +31,19 @@ struct filedescr {
     SOCKET socket;
   } fd;
   enum { KIND_HANDLE, KIND_SOCKET } kind;
+  int crt_fd;
 };
 
 #define Handle_val(v) (((struct filedescr *) Data_custom_val(v))->fd.handle)
 #define Socket_val(v) (((struct filedescr *) Data_custom_val(v))->fd.socket)
 #define Descr_kind_val(v) (((struct filedescr *) Data_custom_val(v))->kind)
+#define CRT_fd_val(v) (((struct filedescr *) Data_custom_val(v))->crt_fd)
 
 extern value win_alloc_handle_or_socket(HANDLE);
 extern value win_alloc_handle(HANDLE);
 extern value win_alloc_socket(SOCKET);
 
+#define NO_CRT_FD (-1)
 #define Nothing ((value) 0)
 
 extern void win32_maperr(unsigned long errcode);
