@@ -11,7 +11,7 @@
 /*                                                                     */
 /***********************************************************************/
 
-/* $Id: winwait.c,v 1.13 2001/12/07 13:40:47 xleroy Exp $ */
+/* $Id: winwait.c,v 1.14 2002/06/07 09:49:41 xleroy Exp $ */
 
 #include <windows.h>
 #include <mlvalues.h>
@@ -40,7 +40,8 @@ static int wait_flag_table[] = { CAML_WNOHANG, CAML_WUNTRACED };
 
 CAMLprim value win_waitpid(value vflags, value vpid_req)
 {
-  int status, flags;
+  int flags;
+  DWORD status;
   HANDLE pid_req = (HANDLE) Long_val(vpid_req);
 
   flags = convert_flag_list(vflags, wait_flag_table);
