@@ -10,29 +10,58 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: location.ml,v 1.40 2002/06/26 14:51:03 xleroy Exp $ *)
+(* $Id: location.ml,v 1.44.6.1 2005/01/12 17:01:58 doligez Exp $ *)
 
 open Lexing
 
-type t =
-  { loc_start: int; loc_end: int; loc_ghost: bool }
+type t = { loc_start: position; loc_end: position; loc_ghost: bool };;
 
-let none = { loc_start = -1; loc_end = -1; loc_ghost = true }
+let none = { loc_start = dummy_pos; loc_end = dummy_pos; loc_ghost = true };;
 
-let symbol_rloc () =
-  { loc_start = Parsing.symbol_start(); loc_end = Parsing.symbol_end();
-    loc_ghost = false }
+let in_file name =
+  let loc = {
+    pos_fname = name;
+    pos_lnum = 1;
+    pos_bol = 0;
+    pos_cnum = -1;
+  } in
+  { loc_start = loc; loc_end = loc; loc_ghost = true }
+;;
 
-let symbol_gloc () =
-  { loc_start = Parsing.symbol_start(); loc_end = Parsing.symbol_end();
-    loc_ghost = true }
+let curr lexbuf = {
+  loc_start = lexbuf.lex_start_p;
+  loc_end = lexbuf.lex_curr_p;
+  loc_ghost = false
+};;
 
-let rhs_loc n =
-  { loc_start = Parsing.rhs_start n; loc_end = Parsing.rhs_end n;
-    loc_ghost = false }
+let init lexbuf fname =
+  lexbuf.lex_curr_p <- {
+    pos_fname = fname;
+    pos_lnum = 1;
+    pos_bol = 0;
+    pos_cnum = 0;
+  }
+;;
+
+let symbol_rloc () = {
+  loc_start = Parsing.symbol_start_pos ();
+  loc_end = Parsing.symbol_end_pos ();
+  loc_ghost = false;
+};;
+
+let symbol_gloc () = {
+  loc_start = Parsing.symbol_start_pos ();
+  loc_end = Parsing.symbol_end_pos ();
+  loc_ghost = true;
+};;
+
+let rhs_loc n = {
+  loc_start = Parsing.rhs_start_pos n;
+  loc_end = Parsing.rhs_end_pos n;
+  loc_ghost = false;
+};;
 
 let input_name = ref ""
-
 let input_lexbuf = ref (None : lexbuf option)
 
 (* Terminal info *)
@@ -62,9 +91,9 @@ let highlight_terminfo ppf num_lines lb loc1 loc2 =
   print_string "# ";
   for pos = 0 to lb.lex_buffer_len - pos0 - 1 do
     if !bol then (print_string "  "; bol := false);
-    if pos = loc1.loc_start || pos = loc2.loc_start then
+    if pos = loc1.loc_start.pos_cnum || pos = loc2.loc_start.pos_cnum then
       Terminfo.standout true;
-    if pos = loc1.loc_end || pos = loc2.loc_end then
+    if pos = loc1.loc_end.pos_cnum || pos = loc2.loc_end.pos_cnum then
       Terminfo.standout false;
     let c = lb.lex_buffer.[pos + pos0] in
     print_char c;
@@ -88,12 +117,13 @@ let highlight_dumb ppf lb loc =
   let line_start = ref 0 and line_end = ref 0 in
   for pos = 0 to end_pos do
     if lb.lex_buffer.[pos + pos0] = '\n' then begin
-      if loc.loc_start > pos then incr line_start;
-      if loc.loc_end   > pos then incr line_end
+      if loc.loc_start.pos_cnum > pos then incr line_start;
+      if loc.loc_end.pos_cnum   > pos then incr line_end;
     end
   done;
   (* Print character location (useful for Emacs) *)
-  Format.fprintf ppf "Characters %i-%i:@." loc.loc_start loc.loc_end;
+  Format.fprintf ppf "Characters %i-%i:@."
+                 loc.loc_start.pos_cnum loc.loc_end.pos_cnum;
   (* Print the input, underlining the location *)
   print_string "  ";
   let line = ref 0 in
@@ -106,12 +136,12 @@ let highlight_dumb ppf lb loc =
         print_char c
       else if !line = !line_start then
         (* first line of multiline loc: print ... before loc_start *)
-        if pos < loc.loc_start
+        if pos < loc.loc_start.pos_cnum
         then print_char '.'
         else print_char c
       else if !line = !line_end then
         (* last line of multiline loc: print ... after loc_end *)
-        if pos < loc.loc_end
+        if pos < loc.loc_end.pos_cnum
         then print_char c
         else print_char '.'
       else if !line > !line_start && !line < !line_end then
@@ -121,16 +151,16 @@ let highlight_dumb ppf lb loc =
       if !line = !line_start && !line = !line_end then begin
         (* loc is on one line: underline location *)
         print_string "\n  ";
-        for i = !pos_at_bol to loc.loc_start - 1 do
+        for i = !pos_at_bol to loc.loc_start.pos_cnum - 1 do
           print_char ' '
         done;
-        for i = loc.loc_start to loc.loc_end - 1 do
+        for i = loc.loc_start.pos_cnum to loc.loc_end.pos_cnum - 1 do
           print_char '^'
         done
       end;
       if !line >= !line_start && !line <= !line_end then begin
         print_char '\n';
-        if pos < loc.loc_end then print_string "  "
+        if pos < loc.loc_end.pos_cnum then print_string "  "
       end;
       incr line;
       pos_at_bol := pos + 1;
@@ -169,21 +199,35 @@ let reset () =
   num_loc_lines := 0
 
 let (msg_file, msg_line, msg_chars, msg_to, msg_colon, msg_head) =
-  match Sys.os_type with
-  | "MacOS" -> ("File \"", "\"; line ", "; characters ", " to ", "", "### ")
-  | _ -> ("File \"", "\", line ", ", characters ", "-", ":", "")
+  ("File \"", "\", line ", ", characters ", "-", ":", "")
+
+(* return file, line, char from the given position *)
+let get_pos_info pos =
+  let (filename, linenum, linebeg) =
+    if pos.pos_fname = "" && !input_name = "" then
+      ("", -1, 0)
+    else if pos.pos_fname = "" then
+      Linenum.for_position !input_name pos.pos_cnum
+    else
+      (pos.pos_fname, pos.pos_lnum, pos.pos_bol)
+  in
+  (filename, linenum, pos.pos_cnum - linebeg)
+;;
 
 let print ppf loc =
-  if String.length !input_name = 0 then
+  let (file, line, startchar) = get_pos_info loc.loc_start in
+  let endchar = loc.loc_end.pos_cnum - loc.loc_start.pos_cnum + startchar in
+  let (startchar, endchar) =
+    if startchar < 0 then (0, 1) else (startchar, endchar)
+  in
+  if file = "" then begin
     if highlight_locations ppf loc none then () else
-      fprintf ppf "Characters %i-%i:@." loc.loc_start loc.loc_end
-  else begin
-    let (filename, linenum, linebeg) =
-            Linenum.for_position !input_name loc.loc_start in
-    fprintf ppf "%s%s%s%i" msg_file filename msg_line linenum;
-    fprintf ppf "%s%i" msg_chars (loc.loc_start - linebeg);
-    fprintf ppf "%s%i%s@.%s"
-     msg_to (loc.loc_end - linebeg) msg_colon msg_head;
+      fprintf ppf "Characters %i-%i:@."
+              loc.loc_start.pos_cnum loc.loc_end.pos_cnum
+  end else begin
+    fprintf ppf "%s%s%s%i" msg_file file msg_line line;
+    fprintf ppf "%s%i" msg_chars startchar;
+    fprintf ppf "%s%i%s@.%s" msg_to endchar msg_colon msg_head;
   end
 
 let print_warning loc ppf w =
