@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: parsetree.mli,v 1.35 2002/04/18 07:27:41 garrigue Exp $ *)
+(* $Id: parsetree.mli,v 1.40 2003/11/25 08:46:45 garrigue Exp $ *)
 
 (* Abstract syntax tree produced by parsing *)
 
@@ -110,6 +110,7 @@ and expression_desc =
   | Pexp_assertfalse
   | Pexp_lazy of expression
   | Pexp_poly of expression * core_type option
+  | Pexp_object of class_structure
 
 (* Value descriptions *)
 
@@ -129,8 +130,8 @@ and type_declaration =
 
 and type_kind =
     Ptype_abstract
-  | Ptype_variant of (string * core_type list) list
-  | Ptype_record of (string * mutable_flag * core_type) list
+  | Ptype_variant of (string * core_type list) list * private_flag
+  | Ptype_record of (string * mutable_flag * core_type) list * private_flag
 
 and exception_declaration = core_type list
 
@@ -208,6 +209,7 @@ and signature_item_desc =
   | Psig_type of (string * type_declaration) list
   | Psig_exception of string * exception_declaration
   | Psig_module of string * module_type
+  | Psig_recmodule of (string * module_type) list
   | Psig_modtype of string * modtype_declaration
   | Psig_open of Longident.t
   | Psig_include of module_type
@@ -249,6 +251,7 @@ and structure_item_desc =
   | Pstr_exception of string * exception_declaration
   | Pstr_exn_rebind of string * Longident.t
   | Pstr_module of string * module_expr
+  | Pstr_recmodule of (string * module_type * module_expr) list
   | Pstr_modtype of string * module_type
   | Pstr_open of Longident.t
   | Pstr_class of class_declaration list
