@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: syntaxerr.ml,v 1.7 2000/04/12 09:54:53 garrigue Exp $ *)
+(* $Id: syntaxerr.ml,v 1.8 2002/04/18 08:50:43 garrigue Exp $ *)
 
 (* Auxiliary type for reporting syntax errors *)
 
@@ -26,7 +26,7 @@ exception Escape_error
 let report_error ppf = function
   | Unclosed(opening_loc, opening, closing_loc, closing) ->
       if String.length !Location.input_name = 0
-      && Location.highlight_locations opening_loc closing_loc
+      && Location.highlight_locations ppf opening_loc closing_loc
       then fprintf ppf "Syntax error: '%s' expected, \
                    the highlighted '%s' might be unmatched" closing opening
       else begin
