@@ -11,7 +11,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: array.ml,v 1.21 2002/07/12 09:47:53 xleroy Exp $ *)
+(* $Id: array.ml,v 1.23 2003/12/31 14:20:39 doligez Exp $ *)
 
 (* Array operations *)
 
@@ -20,8 +20,8 @@ external get: 'a array -> int -> 'a = "%array_safe_get"
 external set: 'a array -> int -> 'a -> unit = "%array_safe_set"
 external unsafe_get: 'a array -> int -> 'a = "%array_unsafe_get"
 external unsafe_set: 'a array -> int -> 'a -> unit = "%array_unsafe_set"
-external make: int -> 'a -> 'a array = "make_vect"
-external create: int -> 'a -> 'a array = "make_vect"
+external make: int -> 'a -> 'a array = "caml_make_vect"
+external create: int -> 'a -> 'a array = "caml_make_vect"
 
 let init l f =
   if l = 0 then [||] else
@@ -200,7 +200,7 @@ let sort cmp a =
   let rec bubbledown l i =
     let j = maxson l i in
     set a i (get a j);
-    bubbledown l j;
+    bubbledown l j
   in
   let bubble l i = try bubbledown l i with Bottom i -> i in
   let rec trickleup i e =
