@@ -6,15 +6,17 @@
 (*                                                                     *)
 (*  Copyright 1996 Institut National de Recherche en Informatique et   *)
 (*  en Automatique.  All rights reserved.  This file is distributed    *)
-(*  under the terms of the GNU Library General Public License.         *)
+(*  under the terms of the GNU Library General Public License, with    *)
+(*  the special exception on linking described in file ../LICENSE.     *)
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: callback.ml,v 1.3 1999/12/08 08:21:56 garrigue Exp $ *)
+(* $Id: callback.ml,v 1.5 2003/12/31 14:20:39 doligez Exp $ *)
 
 (* Registering Caml values with the C runtime for later callbacks *)
 
-external register_named_value: string -> Obj.t -> unit = "register_named_value"
+external register_named_value : string -> Obj.t -> unit
+                              = "caml_register_named_value"
 
 let register name v =
   register_named_value name (Obj.repr v)
