@@ -11,7 +11,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: char.mli,v 1.15 2001/12/07 13:40:50 xleroy Exp $ *)
+(* $Id: char.mli,v 1.16 2002/06/26 09:13:58 xleroy Exp $ *)
 
 (** Character operations. *)
 
@@ -34,6 +34,15 @@ val lowercase : char -> char
 val uppercase : char -> char
 (** Convert the given character to its equivalent uppercase character. *)
 
+type t = char
+(** An alias for the type of characters. *)
+
+val compare: t -> t -> int
+(** The comparison function for characters, with the same specification as
+    {!Pervasives.compare}.  Along with the type [t], this function [compare]
+    allows the module [Char] to be passed as argument to the functors
+    {!Set.Make} and {!Map.Make}. *)
+
 (**/**)
 
 external unsafe_chr : int -> char = "%identity"
