@@ -11,7 +11,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: obj.mli,v 1.23 2001/12/07 13:40:56 xleroy Exp $ *)
+(* $Id: obj.mli,v 1.27 2004/01/01 16:42:40 doligez Exp $ *)
 
 (** Operations on internal representations of values.
 
@@ -23,27 +23,35 @@ type t
 external repr : 'a -> t = "%identity"
 external obj : t -> 'a = "%identity"
 external magic : 'a -> 'b = "%identity"
-external is_block : t -> bool = "obj_is_block"
+external is_block : t -> bool = "caml_obj_is_block"
 external is_int : t -> bool = "%obj_is_int"
-external tag : t -> int = "obj_tag"
+external tag : t -> int = "caml_obj_tag"
+external set_tag : t -> int -> unit = "caml_obj_set_tag"
 external size : t -> int = "%obj_size"
+external truncate : t -> int -> unit = "caml_obj_truncate"
 external field : t -> int -> t = "%obj_field"
 external set_field : t -> int -> t -> unit = "%obj_set_field"
-external new_block : int -> int -> t = "obj_block"
-external dup : t -> t = "obj_dup"
-external truncate : t -> int -> unit = "obj_truncate"
+external new_block : int -> int -> t = "caml_obj_block"
+external dup : t -> t = "caml_obj_dup"
 
-val no_scan_tag : int
+val lazy_tag : int
 val closure_tag : int
-val infix_tag : int
 val object_tag : int
+val infix_tag : int
+val forward_tag : int
+val no_scan_tag : int
 val abstract_tag : int
 val string_tag : int
 val double_tag : int
 val double_array_tag : int
-val final_tag : int
+val custom_tag : int
+val final_tag : int  (* DEPRECATED *)
 
-(** The following two functions are deprecated.  Use module {!Marshal} instead. *)
+val int_tag : int
+val out_of_heap_tag : int
+
+(** The following two functions are deprecated.  Use module {!Marshal}
+    instead. *)
 
 val marshal : t -> string
 val unmarshal : string -> int -> t * int
