@@ -11,7 +11,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: ocamlprof.ml,v 1.30 2002/04/18 07:27:43 garrigue Exp $ *)
+(* $Id: ocamlprof.ml,v 1.37.2.1 2004/11/18 23:52:08 doligez Exp $ *)
 
 open Printf
 
@@ -21,9 +21,9 @@ open Location
 open Misc
 open Parsetree
 
-(* User programs must not use identifiers that start with this prefix. *)
-let idprefix = "__ocaml_prof";;
-
+(* User programs must not use identifiers that start with these prefixes. *)
+let idprefix = "__ocaml_prof_";;
+let modprefix = "OCAML__prof_";;
 
 (* Errors specific to the profiler *)
 exception Profiler of string
@@ -39,9 +39,6 @@ let cur_point = ref 0
 and inchan = ref stdin
 and outchan = ref stdout
 
-(* In case we forgot something *)
-(*exception Inversion of int * int*)
-
 (* To copy source fragments *)
 let copy_buffer = String.create 256
 
@@ -66,15 +63,11 @@ let copy_chars =
   | _       -> copy_chars_unix
 
 let copy next =
-  if next < !cur_point then begin
-    (*raise (Inversion(!cur_point, next));*)
-    (*fprintf stderr "warning: inversion at %d, %d\n" !cur_point next;*)
-    assert false
-  end else begin
-    seek_in !inchan !cur_point;
-    copy_chars (next - !cur_point);
-    cur_point := next;
-  end
+  assert (next >= !cur_point);
+  seek_in !inchan !cur_point;
+  copy_chars (next - !cur_point);
+  cur_point := next;
+;;
 
 let prof_counter = ref 0;;
 
@@ -91,12 +84,11 @@ let insert_action st en =
 let add_incr_counter modul (kind,pos) =
    copy pos;
    match kind with
-   | Close -> fprintf !outchan ")";
    | Open ->
-         fprintf !outchan
-                 "(%s_cnt_%s_.(%d) <- Pervasives.succ %s_cnt_%s_.(%d); "
-                 idprefix modul !prof_counter idprefix modul !prof_counter;
+         fprintf !outchan "(%sProfiling.incr %s%s_cnt %d; "
+                 modprefix idprefix modul !prof_counter;
          incr prof_counter;
+   | Close -> fprintf !outchan ")";
 ;;
 
 let counters = ref (Array.create 0 0)
@@ -116,8 +108,8 @@ let add_val_counter (kind,pos) =
 (* ************* rewrite ************* *)
 
 let insert_profile rw_exp ex =
-  let st = ex.pexp_loc.loc_start
-  and en = ex.pexp_loc.loc_end
+  let st = ex.pexp_loc.loc_start.Lexing.pos_cnum
+  and en = ex.pexp_loc.loc_end.Lexing.pos_cnum
   and gh = ex.pexp_loc.loc_ghost
   in
   if gh || st = en then
@@ -134,12 +126,13 @@ let pos_len = ref 0
 let init_rewrite modes mod_name =
   cur_point := 0;
   if !instr_mode then begin
-    fprintf !outchan "let %s_cnt_%s_ = Array.create 0000000" idprefix mod_name;
+    fprintf !outchan "module %sProfiling = Profiling;; " modprefix;
+    fprintf !outchan "let %s%s_cnt = Array.create 000000000" idprefix mod_name;
     pos_len := pos_out !outchan;
     fprintf !outchan 
             " 0;; Profiling.counters := \
-              (\"%s\", (\"%s\", %s_cnt_%s_)) :: !Profiling.counters;; "
-            mod_name modes idprefix mod_name
+              (\"%s\", (\"%s\", %s%s_cnt)) :: !Profiling.counters;; "
+            mod_name modes idprefix mod_name;
   end
 
 let final_rewrite add_function =
@@ -149,7 +142,7 @@ let final_rewrite add_function =
   copy (in_channel_length !inchan);
   if !instr_mode then begin
     let len = string_of_int !prof_counter in
-    if String.length len > 7 then raise (Profiler "too many counters");
+    if String.length len > 9 then raise (Profiler "too many counters");
     seek_out !outchan (!pos_len - String.length len);
     output_string !outchan len
   end;
@@ -185,7 +178,7 @@ and rw_exp iflag sexp =
     rewrite_exp iflag sbody
 
   | Pexp_function (_, _, caselist) ->
-    if !instr_fun && not sexp.pexp_loc.loc_ghost then
+    if !instr_fun then
       rewrite_function iflag caselist
     else
       rewrite_patlexp_list iflag caselist
@@ -289,6 +282,9 @@ and rw_exp iflag sexp =
 
   | Pexp_poly (sexp, _) -> rewrite_exp iflag sexp
 
+  | Pexp_object (_, fieldl) ->
+      List.iter (rewrite_class_field iflag) fieldl
+
 and rewrite_ifbody iflag ghost sifbody =
   if !instr_if && not ghost then
     insert_profile rw_exp sifbody
@@ -377,6 +373,7 @@ let rewrite_file srcfile add_function =
   inchan := open_in_bin srcfile;
   let lb = Lexing.from_channel !inchan in
   Location.input_name := srcfile;
+  Location.init lb srcfile;
   List.iter (rewrite_str_item false) (Parse.implementation lb);
   final_rewrite add_function;
   close_in !inchan
@@ -410,11 +407,11 @@ let dumpfile = ref "ocamlprof.dump"
 
 (* Process a file *)
 
-let process_file filename =
-  if not (Filename.check_suffix filename ".ml") then
-    null_rewrite filename
-  else
-   let modname = Filename.basename(Filename.chop_suffix filename ".ml") in
+let process_intf_file filename = null_rewrite filename;;
+
+let process_impl_file filename =
+   let modname = Filename.basename(Filename.chop_extension filename) in
+       (* FIXME should let modname = String.capitalize modname *)
    if !instr_mode then begin
      (* Instrumentation mode *)
      set_flags !modes;
@@ -437,6 +434,14 @@ let process_file filename =
      init_rewrite modes modname;
      rewrite_file filename add_val_counter;
    end
+;;
+
+let process_anon_file filename =
+  if Filename.check_suffix filename ".ml" then
+    process_impl_file filename
+  else
+    process_intf_file filename
+;;
 
 (* Main function *)
 
@@ -448,29 +453,27 @@ let main () =
   try
     Arg.parse [
        "-f", Arg.String (fun s -> dumpfile := s),
-             "<file>  Use <file> as dump file (default ocamlprof.dump)";
+             "<file>     Use <file> as dump file (default ocamlprof.dump)";
        "-F", Arg.String (fun s -> special_id := s),
-             "<s>  Insert string <s> with the counts";
-       "-instrument", Arg.Set instr_mode, " (undocumented)";
-       "-m", Arg.String (fun s -> modes := s), "<flags>  (undocumented)"
-      ] process_file usage;
+             "<s>        Insert string <s> with the counts";
+       "-impl", Arg.String process_impl_file,
+                "<file>  Process <file> as a .ml file";
+       "-instrument", Arg.Set instr_mode, "  (undocumented)";
+       "-intf", Arg.String process_intf_file,
+                "<file>  Process <file> as a .mli file";
+       "-m", Arg.String (fun s -> modes := s), "<flags>    (undocumented)"
+      ] process_anon_file usage;
     exit 0
   with x ->
     let report_error ppf = function
-    | Lexer.Error(err, start, stop) ->
+    | Lexer.Error(err, range) ->
         fprintf ppf "@[%a%a@]@."
-        Location.print {loc_start = start; loc_end = stop; loc_ghost = false}
-        Lexer.report_error err
+        Location.print range  Lexer.report_error err
     | Syntaxerr.Error err ->
         fprintf ppf "@[%a@]@."
         Syntaxerr.report_error err
     | Profiler msg ->
         fprintf ppf "@[%s@]@." msg
-(*
-    | Inversion(pos, next) ->
-        print_string "Internal error: inversion at char "; print_int pos;
-        print_string ", "; print_int next
-*)
     | Sys_error msg ->
         fprintf ppf "@[I/O error:@ %s@]@." msg
     | x -> raise x in
