@@ -7,17 +7,19 @@
 (*                                                                     *)
 (*  Copyright 1996 Institut National de Recherche en Informatique et   *)
 (*  en Automatique.  All rights reserved.  This file is distributed    *)
-(*  under the terms of the GNU Library General Public License.         *)
+(*  under the terms of the GNU Library General Public License, with    *)
+(*  the special exception on linking described in file ../LICENSE.     *)
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: profiling.ml,v 1.5 2000/12/28 13:06:48 weis Exp $ *)
+(* $Id: profiling.ml,v 1.6.10.1 2004/11/18 23:52:08 doligez Exp $ *)
 
 (* Run-time library for profiled programs *)
 
 type profiling_counters = (string * (string * int array)) list
 
-let counters = ref ([] : profiling_counters)
+let counters = ref ([] : profiling_counters);;
+let incr a i = a.(i) <- a.(i) + 1;;
 
 exception Bad_profile
 
