@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: topdirs.ml,v 1.58 2002/06/19 06:11:21 garrigue Exp $ *)
+(* $Id: topdirs.ml,v 1.62.2.1 2004/06/23 12:10:02 garrigue Exp $ *)
 
 (* Toplevel directives *)
 
@@ -37,8 +37,7 @@ let _ = Hashtbl.add directive_table "quit" (Directive_none dir_quit)
 let dir_directory s =
   let d = expand_directory Config.standard_library s in
   Config.load_path := d :: !Config.load_path;
-  Dll.add_path [d];
-  Env.reset_cache()
+  Dll.add_path [d]
 
 let _ = Hashtbl.add directive_table "directory" (Directive_string dir_directory)
 
@@ -52,23 +51,16 @@ let _ = Hashtbl.add directive_table "cd" (Directive_string dir_cd)
 
 exception Load_failed
 
-let check_consistency ppf filename compunit =
-  Bytelink.check_consistency filename compunit;
-  (* Check consistency of unit against its .cmi, if it can be found *)
+let check_consistency ppf filename cu =
   try
-    let crc_intf = Env.crc_of_unit compunit.cu_name in
-    let crc_impl =
-      try List.assoc compunit.cu_name compunit.cu_imports
-      with Not_found -> assert false in
-    if crc_intf <> crc_impl then begin
-      fprintf ppf "@[<hv 0>File %s is not up-to-date with respect to@ \
-                           interface %s@]@."
-              filename compunit.cu_name;
-        raise Load_failed
-      end
-  with Not_found ->
-    (* Couldn't find .cmi, ignore it (or should we print a warning?) *)
-    ()
+    List.iter
+      (fun (name, crc) -> Consistbl.check Env.crc_units name crc filename)
+      cu.cu_imports
+  with Consistbl.Inconsistency(name, user, auth) ->
+    fprintf ppf "@[<hv 0>The files %s@ and %s@ \
+                 disagree over interface %s@]@."
+            user auth name;
+    raise Load_failed
 
 let load_compunit ic filename ppf compunit =
   check_consistency ppf filename compunit;
@@ -110,12 +102,16 @@ let load_file ppf name =
         let toc_pos = input_binary_int ic in  (* Go to table of contents *)
         seek_in ic toc_pos;
         let lib = (input_value ic : library) in
-        begin try
-          Dll.open_dlls (List.map Dll.extract_dll_name lib.lib_dllibs)
-        with Failure reason ->
-          fprintf ppf "Cannot load required shared library: %s.@." reason;
-          raise Load_failed
-        end;
+        List.iter
+          (fun dllib ->
+            let name = Dll.extract_dll_name dllib in
+            try Dll.open_dlls [name]
+            with Failure reason ->
+              fprintf ppf
+                "Cannot load required shared library %s.@.Reason: %s.@."
+                name reason;
+              raise Load_failed)
+          lib.lib_dllibs;
         List.iter (load_compunit ic filename ppf) lib.lib_units;
         true
       end else begin
@@ -206,7 +202,7 @@ let _ = Hashtbl.add directive_table "remove_printer"
 
 (* The trace *)
 
-external current_environment: unit -> Obj.t = "get_current_environment"
+external current_environment: unit -> Obj.t = "caml_get_current_environment"
 
 let tracing_function_ptr =
   get_code_pointer
@@ -223,7 +219,9 @@ let dir_trace ppf lid =
     | _ ->
         let clos = eval_path path in
         (* Nothing to do if it's not a closure *)
-        if Obj.is_block clos && Obj.tag clos = Obj.closure_tag then begin
+        if Obj.is_block clos
+        && (Obj.tag clos = Obj.closure_tag || Obj.tag clos = Obj.infix_tag)
+        then begin
         match is_traced clos with
         | Some opath ->
             fprintf ppf "%a is already traced (under the name %a).@."
@@ -255,7 +253,7 @@ let dir_untrace ppf lid =
         []
     | f :: rem ->
         if Path.same f.path path then begin
-          set_code_pointer (eval_path path) f.actual_code;
+          set_code_pointer f.closure f.actual_code;
           fprintf ppf "%a is no longer traced.@." Printtyp.longident lid;
           rem
         end else f :: remove rem in
@@ -266,7 +264,7 @@ let dir_untrace ppf lid =
 let dir_untrace_all ppf () =
   List.iter
     (fun f ->
-      set_code_pointer (eval_path f.path) f.actual_code;
+      set_code_pointer f.closure f.actual_code;
       fprintf ppf "%a is no longer traced.@." Printtyp.path f.path)
     !traced_functions;
   traced_functions := []
