@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: topmain.ml,v 1.29 2002/04/24 08:02:51 xleroy Exp $ *)
+(* $Id: topmain.ml,v 1.34 2003/07/25 12:18:25 xleroy Exp $ *)
 
 open Clflags
 
@@ -20,7 +20,11 @@ let preload_objects = ref []
 
 let prepare ppf =
   Toploop.set_paths ();
-  try List.for_all (Topdirs.load_file ppf) (List.rev !preload_objects)
+  try
+    let res = 
+      List.for_all (Topdirs.load_file ppf) (List.rev !preload_objects) in
+    !Toploop.toplevel_startup_hook ();
+    res
   with x ->
     try Errors.report_error ppf x; false
     with x ->
@@ -31,8 +35,15 @@ let file_argument name =
   let ppf = Format.err_formatter in
   if Filename.check_suffix name ".cmo" || Filename.check_suffix name ".cma"
   then preload_objects := name :: !preload_objects
-  else exit
-      (if prepare ppf && Toploop.run_script ppf name Sys.argv then 0 else 2)
+  else
+    begin
+      let newargs = Array.sub Sys.argv !Arg.current
+                              (Array.length Sys.argv - !Arg.current)
+      in
+      if prepare ppf && Toploop.run_script ppf name newargs
+      then exit 0
+      else exit 2
+    end
 
 let main () =
   Arg.parse [
@@ -53,14 +64,17 @@ let main () =
        \032    A/a enable/disable all warnings\n\
        \032    C/c enable/disable suspicious comment\n\
        \032    D/d enable/disable deprecated features\n\
+       \032    E/e enable/disable fragile match\n\
        \032    F/f enable/disable partially applied function\n\
+       \032    L/l enable/disable labels omitted in application\n\
        \032    M/m enable/disable overriden method\n\
        \032    P/p enable/disable partial match\n\
        \032    S/s enable/disable non-unit statement\n\
        \032    U/u enable/disable unused match case\n\
        \032    V/v enable/disable hidden instance variable\n\
        \032    X/x enable/disable all other warnings\n\
-       \032    default setting is \"Al\" (all warnings but labels enabled)";
+       \032    default setting is \"Ale\"\n\
+       \032    (all warnings but labels and fragile match enabled)";
      "-warn-error" , Arg.String (Warnings.parse_options true),
        "<flags>  Enable or disable fatal warnings according to <flags>\n\
          \032    (see option -w for the list of flags)\n\
