@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: env.ml,v 1.48 2002/08/19 12:23:23 xleroy Exp $ *)
+(* $Id: env.ml,v 1.54 2004/06/12 08:55:47 xleroy Exp $ *)
 
 (* Environment handling *)
 
@@ -88,6 +88,20 @@ let empty = {
   cltypes = Ident.empty;
   summary = Env_empty }
 
+let diff_keys tbl1 tbl2 =
+  let keys2 = Ident.keys tbl2 in
+  List.filter
+    (fun id ->
+      match Ident.find_same id tbl2 with Pident _, _ ->
+        (try ignore (Ident.find_same id tbl1); false with Not_found -> true)
+      | _ -> false)
+    keys2
+
+let diff env1 env2 =
+  diff_keys env1.values env2.values @
+  diff_keys env1.modules env2.modules @
+  diff_keys env1.classes env2.classes
+
 (* Forward declarations *)
 
 let components_of_module' =
@@ -113,6 +127,20 @@ type pers_struct =
 let persistent_structures =
   (Hashtbl.create 17 : (string, pers_struct) Hashtbl.t)
 
+(* Consistency between persistent structures *)
+
+let crc_units = Consistbl.create()
+
+let check_consistency filename crcs =
+  try
+    List.iter
+      (fun (name, crc) -> Consistbl.check crc_units name crc filename)
+      crcs
+  with Consistbl.Inconsistency(name, source, auth) ->
+    raise(Error(Inconsistent_import(name, auth, source)))
+
+(* Reading persistent structures from .cmi files *)
+
 let read_pers_struct modname filename =
   let ic = open_in_bin filename in
   try
@@ -136,6 +164,7 @@ let read_pers_struct modname filename =
                ps_filename = filename } in
     if ps.ps_name <> modname then
       raise(Error(Illegal_renaming(ps.ps_name, filename)));
+    check_consistency filename ps.ps_crcs;
     Hashtbl.add persistent_structures modname ps;
     ps
   with End_of_file | Failure _ ->
@@ -149,7 +178,8 @@ let find_pers_struct name =
     read_pers_struct name (find_in_path_uncap !load_path (name ^ ".cmi"))
 
 let reset_cache() =
-  Hashtbl.clear persistent_structures
+  Hashtbl.clear persistent_structures;
+  Consistbl.clear crc_units
 
 (* Lookup by identifier *)
 
@@ -347,7 +377,7 @@ and lookup_class =
   lookup (fun env -> env.classes) (fun sc -> sc.comp_classes)
 and lookup_cltype =
   lookup (fun env -> env.cltypes) (fun sc -> sc.comp_cltypes)
-  
+
 (* Expand manifest module type names at the top of the given module type *)
 
 let rec scrape_modtype mty env =
@@ -364,21 +394,21 @@ let rec scrape_modtype mty env =
 
 let constructors_of_type ty_path decl =
   match decl.type_kind with
-    Type_variant cstrs ->
+    Type_variant(cstrs, priv) ->
       Datarepr.constructor_descrs
         (Btype.newgenty (Tconstr(ty_path, decl.type_params, ref Mnil)))
-        cstrs
-  | _ -> []
+        cstrs priv
+  | Type_record _ | Type_abstract -> []
 
 (* Compute label descriptions *)
 
 let labels_of_type ty_path decl =
   match decl.type_kind with
-    Type_record(labels, rep) ->
+    Type_record(labels, rep, priv) ->
       Datarepr.label_descrs
         (Btype.newgenty (Tconstr(ty_path, decl.type_params, ref Mnil)))
-        labels rep
-  | _ -> []
+        labels rep priv
+  | Type_variant _ | Type_abstract -> []
 
 (* Given a signature and a root path, prefix all idents in the signature
    by the root path and build the corresponding substitution. *)
@@ -390,7 +420,7 @@ let rec prefix_idents root pos sub = function
       let nextpos = match decl.val_kind with Val_prim _ -> pos | _ -> pos+1 in
       let (pl, final_sub) = prefix_idents root nextpos sub rem in
       (p::pl, final_sub)
-  | Tsig_type(id, decl) :: rem ->
+  | Tsig_type(id, decl, _) :: rem ->
       let p = Pdot(root, Ident.name id, nopos) in
       let (pl, final_sub) =
         prefix_idents root pos (Subst.add_type id p sub) rem in
@@ -399,7 +429,7 @@ let rec prefix_idents root pos sub = function
       let p = Pdot(root, Ident.name id, pos) in
       let (pl, final_sub) = prefix_idents root (pos+1) sub rem in
       (p::pl, final_sub)
-  | Tsig_module(id, mty) :: rem ->
+  | Tsig_module(id, mty, _) :: rem ->
       let p = Pdot(root, Ident.name id, pos) in
       let (pl, final_sub) =
         prefix_idents root (pos+1) (Subst.add_module id p sub) rem in
@@ -410,11 +440,11 @@ let rec prefix_idents root pos sub = function
         prefix_idents root pos
                       (Subst.add_modtype id (Tmty_ident p) sub) rem in
       (p::pl, final_sub)
-  | Tsig_class(id, decl) :: rem ->
+  | Tsig_class(id, decl, _) :: rem ->
       let p = Pdot(root, Ident.name id, pos) in
       let (pl, final_sub) = prefix_idents root (pos + 1) sub rem in
       (p::pl, final_sub)
-  | Tsig_cltype(id, decl) :: rem ->
+  | Tsig_cltype(id, decl, _) :: rem ->
       let p = Pdot(root, Ident.name id, nopos) in
       let (pl, final_sub) = prefix_idents root pos sub rem in
       (p::pl, final_sub)
@@ -442,7 +472,7 @@ let rec components_of_module env sub path mty =
             begin match decl.val_kind with
               Val_prim _ -> () | _ -> incr pos
             end
-        | Tsig_type(id, decl) ->
+        | Tsig_type(id, decl, _) ->
             let decl' = Subst.type_declaration sub decl in
             c.comp_types <-
               Tbl.add (Ident.name id) (decl', nopos) c.comp_types;
@@ -461,7 +491,7 @@ let rec components_of_module env sub path mty =
             c.comp_constrs <-
               Tbl.add (Ident.name id) (cstr, !pos) c.comp_constrs;
             incr pos
-        | Tsig_module(id, mty) ->
+        | Tsig_module(id, mty, _) ->
             let mty' = Subst.modtype sub mty in
             c.comp_modules <-
               Tbl.add (Ident.name id) (mty', !pos) c.comp_modules;
@@ -475,12 +505,12 @@ let rec components_of_module env sub path mty =
             c.comp_modtypes <-
               Tbl.add (Ident.name id) (decl', nopos) c.comp_modtypes;
             env := store_modtype id path decl !env
-        | Tsig_class(id, decl) ->
+        | Tsig_class(id, decl, _) ->
             let decl' = Subst.class_declaration sub decl in
             c.comp_classes <-
               Tbl.add (Ident.name id) (decl', !pos) c.comp_classes;
             incr pos
-        | Tsig_cltype(id, decl) ->
+        | Tsig_cltype(id, decl, _) ->
             let decl' = Subst.cltype_declaration sub decl in
             c.comp_cltypes <-
               Tbl.add (Ident.name id) (decl', !pos) c.comp_cltypes)
@@ -622,12 +652,12 @@ and enter_cltype = enter store_cltype
 let add_item comp env =
   match comp with
     Tsig_value(id, decl)     -> add_value id decl env
-  | Tsig_type(id, decl)      -> add_type id decl env
+  | Tsig_type(id, decl, _)   -> add_type id decl env
   | Tsig_exception(id, decl) -> add_exception id decl env
-  | Tsig_module(id, mty)     -> add_module id mty env
+  | Tsig_module(id, mty, _)  -> add_module id mty env
   | Tsig_modtype(id, decl)   -> add_modtype id decl env
-  | Tsig_class(id, decl)     -> add_class id decl env
-  | Tsig_cltype(id, decl)    -> add_cltype id decl env
+  | Tsig_class(id, decl, _)  -> add_class id decl env
+  | Tsig_cltype(id, decl, _) -> add_cltype id decl env
 
 let rec add_signature sg env =
   match sg with
@@ -647,21 +677,21 @@ let open_signature root sg env =
           Tsig_value(id, decl) ->
             store_value (Ident.hide id) p
                         (Subst.value_description sub decl) env
-        | Tsig_type(id, decl) ->
+        | Tsig_type(id, decl, _) ->
             store_type (Ident.hide id) p
                        (Subst.type_declaration sub decl) env
         | Tsig_exception(id, decl) ->
             store_exception (Ident.hide id) p
                             (Subst.exception_declaration sub decl) env
-        | Tsig_module(id, mty) ->
+        | Tsig_module(id, mty, _) ->
             store_module (Ident.hide id) p (Subst.modtype sub mty) env
         | Tsig_modtype(id, decl) ->
             store_modtype (Ident.hide id) p
                           (Subst.modtype_declaration sub decl) env
-        | Tsig_class(id, decl) ->
+        | Tsig_class(id, decl, _) ->
             store_class (Ident.hide id) p
                         (Subst.class_declaration sub decl) env
-        | Tsig_cltype(id, decl) ->
+        | Tsig_cltype(id, decl, _) ->
             store_cltype (Ident.hide id) p
                          (Subst.cltype_declaration sub decl) env)
       env sg pl in
@@ -690,22 +720,7 @@ let crc_of_unit name =
 (* Return the list of imported interfaces with their CRCs *)
 
 let imported_units() =
-  let imported_units =
-    ref ([] : (string * Digest.t) list) in
-  let units_xref =
-    (Hashtbl.create 13 : (string, Digest.t * string) Hashtbl.t) in
-  let add_unit source (name, crc) =
-    try
-      let (oldcrc, oldsource) = Hashtbl.find units_xref name in
-      if oldcrc <> crc then
-        raise(Error(Inconsistent_import(name, oldsource, source)))
-    with Not_found ->
-      Hashtbl.add units_xref name (crc, source);
-      imported_units := (name, crc) :: !imported_units in
-  Hashtbl.iter
-    (fun name ps -> List.iter (add_unit ps.ps_filename) ps.ps_crcs)
-    persistent_structures;
-  !imported_units
+  Consistbl.extract crc_units
 
 (* Save a signature to a file *)
 
@@ -733,7 +748,8 @@ let save_signature_with_imports sg modname filename imports =
         ps_comps = comps;
         ps_crcs = crcs;
         ps_filename = filename } in
-    Hashtbl.add persistent_structures modname ps
+    Hashtbl.add persistent_structures modname ps;
+    Consistbl.set crc_units modname crc filename
   with exn ->
     close_out oc;
     remove_file filename;
@@ -763,6 +779,6 @@ let report_error ppf = function
       "Wrong file naming: %s@ contains the compiled interface for@ %s"
       filename modname
   | Inconsistent_import(name, source1, source2) -> fprintf ppf
-      "@[<hov>The compiled interfaces %s@ and %s@ \
-              make inconsistent assumptions over interface %s@]"
-      source1 source2  name;;
+      "@[<hov>The files %s@ and %s@ \
+              make inconsistent assumptions@ over interface %s@]"
+      source1 source2 name
