@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: env.mli,v 1.26 2002/08/19 12:23:23 xleroy Exp $ *)
+(* $Id: env.mli,v 1.28 2003/11/25 09:20:42 garrigue Exp $ *)
 
 (* Environment handling *)
 
@@ -20,6 +20,7 @@ type t
 
 val empty: t
 val initial: t
+val diff: t -> t -> Ident.t list
 
 (* Lookup by paths *)
 
@@ -99,6 +100,10 @@ val crc_of_unit: string -> Digest.t
 
 val imported_units: unit -> (string * Digest.t) list
 
+(* Direct access to the table of imported compilation units with their CRC *)
+
+val crc_units: Consistbl.t
+
 (* Summaries -- compact representation of an environment, to be
    exported in debugging information. *)
 
