@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: ident.mli,v 1.13 2000/05/16 06:27:44 xleroy Exp $ *)
+(* $Id: ident.mli,v 1.17 2004/01/04 14:32:34 doligez Exp $ *)
 
 (* Identifiers (unique names) *)
 
@@ -18,9 +18,11 @@ type t
 
 val create: string -> t
 val create_persistent: string -> t
+val create_predef_exn: string -> t
 val rename: t -> t
 val name: t -> string
 val unique_name: t -> string
+val unique_toplevel_name: t -> string
 val persistent: t -> bool
 val equal: t -> t -> bool
         (* Compare identifiers by name. *)      
@@ -32,16 +34,18 @@ val same: t -> t -> bool
            name. *)
 val hide: t -> t
         (* Return an identifier with same name as the given identifier,
-           but stamp different from any stamp returns by new.
+           but stamp different from any stamp returned by new.
            When put in a 'a tbl, this identifier can only be looked
            up by name. *)
 
 val make_global: t -> unit
 val global: t -> bool
+val is_predef_exn: t -> bool
 
 val binding_time: t -> int
 val current_time: unit -> int
 val set_current_time: int -> unit
+val reinit: unit -> unit
 
 val print: Format.formatter -> t -> unit
 
@@ -52,3 +56,4 @@ val empty: 'a tbl
 val add: t -> 'a -> 'a tbl -> 'a tbl
 val find_same: t -> 'a tbl -> 'a
 val find_name: string -> 'a tbl -> 'a
+val keys: 'a tbl -> t list
