@@ -10,10 +10,11 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: predef.ml,v 1.23 2002/04/18 07:27:45 garrigue Exp $ *)
+(* $Id: predef.ml,v 1.29 2004/01/04 14:32:34 doligez Exp $ *)
 
 (* Predefined type constructors (with special typing rules in typecore) *)
 
+open Asttypes
 open Path
 open Types
 open Btype
@@ -27,7 +28,7 @@ and ident_unit = Ident.create "unit"
 and ident_exn = Ident.create "exn"
 and ident_array = Ident.create "array"
 and ident_list = Ident.create "list"
-and ident_format = Ident.create "format"
+and ident_format4 = Ident.create "format4"
 and ident_option = Ident.create "option"
 and ident_nativeint = Ident.create "nativeint"
 and ident_int32 = Ident.create "int32"
@@ -43,7 +44,7 @@ and path_unit = Pident ident_unit
 and path_exn = Pident ident_exn
 and path_array = Pident ident_array
 and path_list = Pident ident_list
-and path_format = Pident ident_format
+and path_format4 = Pident ident_format4
 and path_option = Pident ident_option
 and path_nativeint = Pident ident_nativeint
 and path_int32 = Pident ident_int32
@@ -65,20 +66,23 @@ and type_int32 = newgenty (Tconstr(path_int32, [], ref Mnil))
 and type_int64 = newgenty (Tconstr(path_int64, [], ref Mnil))
 and type_lazy_t t = newgenty (Tconstr(path_lazy_t, [t], ref Mnil))
 
-let ident_match_failure = Ident.create "Match_failure"
-and ident_out_of_memory = Ident.create "Out_of_memory"
-and ident_invalid_argument = Ident.create "Invalid_argument"
-and ident_failure = Ident.create "Failure"
-and ident_not_found = Ident.create "Not_found"
-and ident_sys_error = Ident.create "Sys_error"
-and ident_end_of_file = Ident.create "End_of_file"
-and ident_division_by_zero = Ident.create "Division_by_zero"
-and ident_stack_overflow = Ident.create "Stack_overflow"
-and ident_sys_blocked_io = Ident.create "Sys_blocked_io"
-and ident_assert_failure = Ident.create "Assert_failure"
+let ident_match_failure = Ident.create_predef_exn "Match_failure"
+and ident_out_of_memory = Ident.create_predef_exn "Out_of_memory"
+and ident_invalid_argument = Ident.create_predef_exn "Invalid_argument"
+and ident_failure = Ident.create_predef_exn "Failure"
+and ident_not_found = Ident.create_predef_exn "Not_found"
+and ident_sys_error = Ident.create_predef_exn "Sys_error"
+and ident_end_of_file = Ident.create_predef_exn "End_of_file"
+and ident_division_by_zero = Ident.create_predef_exn "Division_by_zero"
+and ident_stack_overflow = Ident.create_predef_exn "Stack_overflow"
+and ident_sys_blocked_io = Ident.create_predef_exn "Sys_blocked_io"
+and ident_assert_failure = Ident.create_predef_exn "Assert_failure"
+and ident_undefined_recursive_module =
+        Ident.create_predef_exn "Undefined_recursive_module"
 
 let path_match_failure = Pident ident_match_failure
 and path_assert_failure = Pident ident_assert_failure
+and path_undefined_recursive_module = Pident ident_undefined_recursive_module
 
 let build_initial_env add_type add_exception empty_env =
   let decl_abstr =
@@ -90,19 +94,19 @@ let build_initial_env add_type add_exception empty_env =
   and decl_bool =
     {type_params = [];
      type_arity = 0;
-     type_kind = Type_variant["false",[]; "true",[]];
+     type_kind = Type_variant(["false",[]; "true",[]], Public);
      type_manifest = None;
      type_variance = []}
   and decl_unit =
     {type_params = []; 
      type_arity = 0;
-     type_kind = Type_variant["()",[]];
+     type_kind = Type_variant(["()",[]], Public);
      type_manifest = None;
      type_variance = []}
   and decl_exn =
     {type_params = [];
      type_arity = 0;
-     type_kind = Type_variant [];
+     type_kind = Type_variant([], Public);
      type_manifest = None;
      type_variance = []}
   and decl_array =
@@ -111,34 +115,36 @@ let build_initial_env add_type add_exception empty_env =
      type_arity = 1;
      type_kind = Type_abstract;
      type_manifest = None;
-     type_variance = [true, true]}
+     type_variance = [true, true, true]}
   and decl_list =
     let tvar = newgenvar() in
     {type_params = [tvar];
      type_arity = 1;
-     type_kind = Type_variant["[]", []; "::", [tvar; type_list tvar]];
+     type_kind =
+       Type_variant(["[]", []; "::", [tvar; type_list tvar]], Public);
      type_manifest = None;
-     type_variance = [true, false]}
-  and decl_format =
-    {type_params = [newgenvar(); newgenvar(); newgenvar()];
-     type_arity = 3;
+     type_variance = [true, false, false]}
+  and decl_format4 =
+    {type_params = [newgenvar(); newgenvar(); newgenvar(); newgenvar()];
+     type_arity = 4;
      type_kind = Type_abstract;
      type_manifest = None;
-     type_variance = [true, true; true, true; true, true]}
+     type_variance = [true, true, true; true, true, true;
+		      true, true, true; true, true, true]}
   and decl_option =
     let tvar = newgenvar() in
     {type_params = [tvar];
      type_arity = 1;
-     type_kind = Type_variant["None", []; "Some", [tvar]];
+     type_kind = Type_variant(["None", []; "Some", [tvar]], Public);
      type_manifest = None;
-     type_variance = [true, false]}
+     type_variance = [true, false, false]}
   and decl_lazy_t =
     let tvar = newgenvar() in
     {type_params = [tvar];
      type_arity = 1;
      type_kind = Type_abstract;
      type_manifest = None;
-     type_variance = [true, false]}
+     type_variance = [true, false, false]}
   in
 
   add_exception ident_match_failure
@@ -154,12 +160,14 @@ let build_initial_env add_type add_exception empty_env =
   add_exception ident_division_by_zero [] (
   add_exception ident_assert_failure
                          [newgenty (Ttuple[type_string; type_int; type_int])] (
+  add_exception ident_undefined_recursive_module
+                         [newgenty (Ttuple[type_string; type_int; type_int])] (
   add_type ident_int64 decl_abstr (
   add_type ident_int32 decl_abstr (
   add_type ident_nativeint decl_abstr (
   add_type ident_lazy_t decl_lazy_t (
   add_type ident_option decl_option (
-  add_type ident_format decl_format (
+  add_type ident_format4 decl_format4 (
   add_type ident_list decl_list (
   add_type ident_array decl_array (
   add_type ident_exn decl_exn (
@@ -169,7 +177,7 @@ let build_initial_env add_type add_exception empty_env =
   add_type ident_string decl_abstr (
   add_type ident_char decl_abstr (
   add_type ident_int decl_abstr (
-    empty_env))))))))))))))))))))))))))
+    empty_env)))))))))))))))))))))))))))
 
 let builtin_values =
   List.map (fun id -> Ident.make_global id; (Ident.name id, id))
@@ -177,4 +185,4 @@ let builtin_values =
        ident_invalid_argument;
        ident_failure; ident_not_found; ident_sys_error; ident_end_of_file;
        ident_division_by_zero; ident_sys_blocked_io;
-       ident_assert_failure ]
+       ident_assert_failure; ident_undefined_recursive_module ]
