@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: primitive.mli,v 1.6 2000/03/13 16:49:01 weis Exp $ *)
+(* $Id: primitive.mli,v 1.7 2001/08/06 12:28:49 ddr Exp $ *)
 
 (* Description of primitive functions *)
 
@@ -23,6 +23,4 @@ type description =
 
 val parse_declaration: int -> string list -> description
 
-open Format
-
-val print_description: formatter -> description -> unit
+val description_list: description -> string list
