@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: subst.ml,v 1.37 2002/04/18 07:27:45 garrigue Exp $ *)
+(* $Id: subst.ml,v 1.46 2004/06/12 08:55:48 xleroy Exp $ *)
 
 (* Substitutions *)
 
@@ -66,8 +66,10 @@ let rec typexp s ty =
   let ty = repr ty in
   match ty.desc with
     Tvar | Tunivar ->
-      if s.for_saving then
-        let ty' = newpersty ty.desc in
+      if s.for_saving || ty.id < 0 then
+        let ty' =
+          if s.for_saving then newpersty ty.desc else newty2 ty.level ty.desc
+        in
         save_desc ty ty.desc; ty.desc <- Tsubst ty'; ty'
       else ty
   | Tsubst ty ->
@@ -96,26 +98,28 @@ let rec typexp s ty =
           let row = row_repr row in
           let more = repr row.row_more in
           (* We must substitute in a subtle way *)
+          (* Tsubst takes a tuple containing the row var and the variant *)
           begin match more.desc with
-            Tsubst ({desc=Tvariant _} as ty2) ->
+            Tsubst {desc = Ttuple [_;ty2]} ->
               (* This variant type has been already copied *)
               ty.desc <- Tsubst ty2; (* avoid Tlink in the new type *)
               Tlink ty2
           | _ ->
-              let static = static_row row in
+              let dup =
+                s.for_saving || more.level = generic_level || static_row row in
               (* Various cases for the row variable *)
               let more' =
                 match more.desc with Tsubst ty -> ty
                 | _ ->
+                    save_desc more more.desc;
                     if s.for_saving then newpersty more.desc else
-                    if static then newgenvar () else more
+                    if dup && more.desc <> Tunivar then newgenvar () else more
               in
               (* Register new type first for recursion *)
-              save_desc more more.desc;
-              more.desc <- ty.desc;
+              more.desc <- Tsubst(newgenty(Ttuple[more';ty']));
               (* Return a new copy *)
               let row =
-                copy_row (typexp s) true row (not s.for_saving) more' in
+                copy_row (typexp s) true row (not dup) more' in
               let row =
                 if s.for_saving then {row with row_bound = []} else row in
               match row.row_name with
@@ -154,15 +158,16 @@ let type_declaration s decl =
       type_kind =
         begin match decl.type_kind with
           Type_abstract -> Type_abstract
-        | Type_variant cstrs ->
+        | Type_variant (cstrs, priv) ->
             Type_variant(
               List.map (fun (n, args) -> (n, List.map (typexp s) args))
-                       cstrs)
-        | Type_record(lbls, rep) ->
+                       cstrs,
+              priv)
+        | Type_record(lbls, rep, priv) ->
             Type_record(
               List.map (fun (n, mut, arg) -> (n, mut, typexp s arg))
                        lbls,
-              rep)
+              rep, priv)
         end;
       type_manifest =
         begin match decl.type_manifest with
@@ -178,7 +183,11 @@ let type_declaration s decl =
 let class_signature s sign =
   { cty_self = typexp s sign.cty_self;
     cty_vars = Vars.map (function (m, t) -> (m, typexp s t)) sign.cty_vars;
-    cty_concr = sign.cty_concr }
+    cty_concr = sign.cty_concr;
+    cty_inher =
+      List.map (fun (p, tl) -> (type_path s p, List.map (typexp s) tl))
+        sign.cty_inher
+  }
 
 let rec class_type s =
   function
@@ -210,8 +219,8 @@ let cltype_declaration s decl =
       clty_type = class_type s decl.clty_type;
       clty_path = type_path s decl.clty_path }
   in
-  (* Do not clean up if saving: next is type_declaration *)
-  if not s.for_saving then cleanup_types ();
+  (* Do clean up even if saving: type_declaration may be recursive *)
+  cleanup_types ();
   decl
 
 let class_type s cty =
@@ -228,10 +237,10 @@ let exception_declaration s tyl =
 
 let rec rename_bound_idents s idents = function
     [] -> (List.rev idents, s)
-  | Tsig_type(id, d) :: sg ->
+  | Tsig_type(id, d, _) :: sg ->
       let id' = Ident.rename id in
       rename_bound_idents (add_type id (Pident id') s) (id' :: idents) sg
-  | Tsig_module(id, mty) :: sg ->
+  | Tsig_module(id, mty, _) :: sg ->
       let id' = Ident.rename id in
       rename_bound_idents (add_module id (Pident id') s) (id' :: idents) sg
   | Tsig_modtype(id, d) :: sg ->
@@ -239,7 +248,7 @@ let rec rename_bound_idents s idents = function
       rename_bound_idents (add_modtype id (Tmty_ident(Pident id')) s)
                           (id' :: idents) sg
   | (Tsig_value(id, _) | Tsig_exception(id, _) | 
-     Tsig_class(id, _) | Tsig_cltype(id, _)) :: sg ->
+     Tsig_class(id, _, _) | Tsig_cltype(id, _, _)) :: sg ->
       let id' = Ident.rename id in
       rename_bound_idents s (id' :: idents) sg
 
@@ -272,18 +281,18 @@ and signature_component s comp newid =
   match comp with
     Tsig_value(id, d) ->
       Tsig_value(newid, value_description s d)
-  | Tsig_type(id, d) ->
-      Tsig_type(newid, type_declaration s d)
+  | Tsig_type(id, d, rs) ->
+      Tsig_type(newid, type_declaration s d, rs)
   | Tsig_exception(id, d) ->
       Tsig_exception(newid, exception_declaration s d)
-  | Tsig_module(id, mty) ->
-      Tsig_module(newid, modtype s mty)
+  | Tsig_module(id, mty, rs) ->
+      Tsig_module(newid, modtype s mty, rs)
   | Tsig_modtype(id, d) ->
       Tsig_modtype(newid, modtype_declaration s d)
-  | Tsig_class(id, d) ->
-      Tsig_class(newid, class_declaration s d)
-  | Tsig_cltype(id, d) ->
-      Tsig_cltype(newid, cltype_declaration s d)
+  | Tsig_class(id, d, rs) ->
+      Tsig_class(newid, class_declaration s d, rs)
+  | Tsig_cltype(id, d, rs) ->
+      Tsig_cltype(newid, cltype_declaration s d, rs)
 
 and modtype_declaration s = function
     Tmodtype_abstract -> Tmodtype_abstract
