@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: typedecl.ml,v 1.52 2002/06/12 09:52:08 garrigue Exp $ *)
+(* $Id: typedecl.ml,v 1.67.10.2 2004/12/09 10:15:43 garrigue Exp $ *)
 
 (**** Typing of type definitions ****)
 
@@ -32,7 +32,7 @@ type error =
   | Constraint_failed of type_expr * type_expr
   | Unconsistent_constraint of (type_expr * type_expr) list
   | Type_clash of (type_expr * type_expr) list
-  | Parameters_differ of type_expr * type_expr
+  | Parameters_differ of Path.t * type_expr * type_expr
   | Null_arity_external
   | Missing_native_external
   | Unbound_type_var
@@ -54,7 +54,7 @@ let enter_type env (name, sdecl) id =
       type_manifest =
         begin match sdecl.ptype_manifest with None -> None
         | Some _ -> Some(Ctype.newvar ()) end;
-      type_variance = List.map (fun _ -> true, true) sdecl.ptype_params;
+      type_variance = List.map (fun _ -> true, true, true) sdecl.ptype_params;
     }
   in
   Env.add_type id decl env
@@ -89,7 +89,7 @@ let transl_declaration env (name, sdecl) id =
   reset_type_variables();
   Ctype.begin_def ();
   let params =
-    try List.map (enter_type_variable true) sdecl.ptype_params
+    try List.map (enter_type_variable true sdecl.ptype_loc) sdecl.ptype_params
     with Already_bound ->
       raise(Error(sdecl.ptype_loc, Repeated_parameter))
   in
@@ -106,7 +106,7 @@ let transl_declaration env (name, sdecl) id =
         begin match sdecl.ptype_kind with
           Ptype_abstract ->
             Type_abstract
-        | Ptype_variant cstrs ->
+        | Ptype_variant (cstrs, priv) ->
             let all_constrs = ref StringSet.empty in
             List.iter
               (fun (name, args) ->
@@ -120,8 +120,8 @@ let transl_declaration env (name, sdecl) id =
             Type_variant(List.map
               (fun (name, args) ->
                       (name, List.map (transl_simple_type env true) args))
-              cstrs)
-        | Ptype_record lbls ->
+              cstrs, priv)
+        | Ptype_record (lbls, priv) ->
             let all_labels = ref StringSet.empty in
             List.iter
               (fun (name, mut, arg) ->
@@ -139,7 +139,7 @@ let transl_declaration env (name, sdecl) id =
               if List.for_all (fun (name, mut, arg) -> is_float env arg) lbls'
               then Record_float
               else Record_regular in
-            Type_record(lbls', rep)
+            Type_record(lbls', rep, priv)
         end;
       type_manifest =
         begin match sdecl.ptype_manifest with
@@ -150,7 +150,7 @@ let transl_declaration env (name, sdecl) id =
               raise(Error(sdecl.ptype_loc, Recursive_abbrev name));
             Some ty
         end;
-      type_variance = List.map (fun _ -> true, true) params;
+      type_variance = List.map (fun _ -> true, true, true) params;
     } in
 
   (* Check constraints *)
@@ -170,13 +170,13 @@ let generalize_decl decl =
   begin match decl.type_kind with
     Type_abstract ->
       ()
-  | Type_variant v ->
+  | Type_variant (v, priv) ->
       List.iter (fun (_, tyl) -> List.iter Ctype.generalize tyl) v
-  | Type_record(r, rep) ->
+  | Type_record(r, rep, priv) ->
       List.iter (fun (_, _, ty) -> Ctype.generalize ty) r
   end;
   begin match decl.type_manifest with
-    None    -> ()
+  | None    -> ()
   | Some ty -> Ctype.generalize ty
   end
 
@@ -195,16 +195,13 @@ let rec check_constraints_rec env loc visited ty =
   visited := TypeSet.add ty !visited;
   match ty.desc with
   | Tconstr (path, args, _) ->
-      Ctype.begin_def ();
       let args' = List.map (fun _ -> Ctype.newvar ()) args in
       let ty' = Ctype.newconstr path args' in
       begin try Ctype.enforce_constraints env ty'
       with Ctype.Unify _ -> assert false
       | Not_found -> raise (Error(loc, Unavailable_type_constructor path))
       end;
-      Ctype.end_def ();
-      Ctype.generalize ty';
-      if not (List.for_all2 (Ctype.moregeneral env false) args' args) then
+      if not (Ctype.matches env ty ty') then
         raise (Error(loc, Constraint_failed (ty, ty')));
       List.iter (check_constraints_rec env loc visited) args
   | Tpoly (ty, tl) ->
@@ -218,21 +215,26 @@ let check_constraints env (_, sdecl) (_, decl) =
   let visited = ref TypeSet.empty in
   begin match decl.type_kind with
   | Type_abstract -> ()
-  | Type_variant l ->
-      let pl =
-        match sdecl.ptype_kind with Ptype_variant pl -> pl | _ -> assert false
+  | Type_variant (l, _) ->
+      let rec find_pl = function
+          Ptype_variant(pl, _) -> pl
+        | Ptype_record _ | Ptype_abstract -> assert false
       in
+      let pl = find_pl sdecl.ptype_kind in
       List.iter
         (fun (name, tyl) ->
           let styl = try List.assoc name pl with Not_found -> assert false in
           List.iter2
-            (fun sty ty -> check_constraints_rec env sty.ptyp_loc visited ty)
+            (fun sty ty ->
+              check_constraints_rec env sty.ptyp_loc visited ty)
             styl tyl)
         l
-  | Type_record (l, _) ->
-      let pl =
-        match sdecl.ptype_kind with Ptype_record pl -> pl | _ -> assert false
+  | Type_record (l, _, _) ->
+      let rec find_pl = function
+          Ptype_record(pl, _) -> pl
+        | Ptype_variant _ | Ptype_abstract -> assert false
       in
+      let pl = find_pl sdecl.ptype_kind in
       let rec get_loc name = function
           [] -> assert false
         | (name', _, sty) :: tl ->
@@ -281,148 +283,208 @@ let check_abbrev env (_, sdecl) (id, decl) =
 
 (* Check for ill-defined abbrevs *)
 
-(* Occur check *)
-let check_recursive_abbrev env (name, sdecl) (id, decl) =
-  match decl.type_manifest with
-    Some ty ->
-      begin try Ctype.correct_abbrev env id decl.type_params ty with
-        Ctype.Recursive_abbrev ->
-          raise(Error(sdecl.ptype_loc, Recursive_abbrev name))
-      end
-  | _ ->
-      ()
+let check_recursion env loc path decl to_check =
+  (* to_check is true for potentially mutually recursive paths.
+     (path, decl) is the type declaration to be checked. *)
 
-(* Recursive expansion check *)
+  let visited = ref [] in
 
-let rec check_expansion_rec env id args loc id_check_list visited ty =
-  let ty = Ctype.repr ty in
-  if List.memq ty visited then () else
-  let visited = ty :: visited in
-  begin match ty.desc with
-  | Tconstr(Path.Pident id' as path, args', _) ->
-      if Ident.same id id' then begin
-        if not (Ctype.equal env false args args') then
-          raise (Error(loc, Parameters_differ(ty, Ctype.newconstr path args)))
-      end else begin try
-        let (loc, checked) = List.assoc id' id_check_list in
-        if List.exists (Ctype.equal env false args') !checked then () else
-        begin
-          checked := args' :: !checked;
-          let id_check_list = List.remove_assoc id' id_check_list in
-          let (params, body) = Env.find_type_expansion path env in
-          let (params, body) = Ctype.instance_parameterized_type params body in
-          begin
-            try List.iter2 (Ctype.unify env) params args'
-            with Ctype.Unify _ -> assert false
+  let rec check_regular cpath args prev_exp ty =
+    let ty = Ctype.repr ty in
+    if not (List.memq ty !visited) then begin
+      visited := ty :: !visited;
+      match ty.desc with
+      | Tconstr(path', args', _) ->
+          if Path.same path path' then begin
+            if not (Ctype.equal env false args args') then
+              raise (Error(loc, 
+                     Parameters_differ(cpath, ty, Ctype.newconstr path args)))
+          end
+          (* Attempt to expand a type abbreviation if:
+              1- [to_check path'] holds
+                 (otherwise the expansion cannot involve [path]);
+              2- we haven't expanded this type constructor before
+                 (otherwise we could loop if [path'] is itself 
+                 a non-regular abbreviation). *)
+          else if to_check path' && not (List.mem path' prev_exp) then begin
+            try
+              (* Attempt expansion *)
+              let (params0, body0) = Env.find_type_expansion path' env in
+              let (params, body) = 
+                Ctype.instance_parameterized_type params0 body0 in
+              begin
+                try List.iter2 (Ctype.unify env) params args'
+                with Ctype.Unify _ ->
+                  raise (Error(loc, Constraint_failed
+                                 (ty, Ctype.newconstr path' params0)));
+              end;
+              check_regular path' args (path' :: prev_exp) body
+            with Not_found -> ()
           end;
-          check_expansion_rec env id args loc id_check_list visited body
-        end
-      with Not_found -> ()
-      end
-  | _ -> ()
-  end;
-  Btype.iter_type_expr
-    (check_expansion_rec env id args loc id_check_list visited) ty
+          List.iter (check_regular cpath args prev_exp) args'
+      | Tpoly (ty, tl) ->
+          let (_, ty) = Ctype.instance_poly false tl ty in
+          check_regular cpath args prev_exp ty
+      | _ ->
+          Btype.iter_type_expr (check_regular cpath args prev_exp) ty
+    end in
 
-let check_expansion env id_loc_list (id, decl) =
-  if decl.type_params = [] then () else
   match decl.type_manifest with
   | None -> ()
   | Some body ->
+      (* Check that recursion is well-founded *)
+      begin try
+        Ctype.correct_abbrev env path decl.type_params body
+      with Ctype.Recursive_abbrev ->
+        raise(Error(loc, Recursive_abbrev (Path.name path)))
+      end;
+      (* Check that recursion is regular *)
+      if decl.type_params = [] then () else
       let (args, body) =
         Ctype.instance_parameterized_type decl.type_params body in
-      let id_check_list =
-        List.map (fun (id, loc) -> (id, (loc, ref []))) id_loc_list in
-      check_expansion_rec env id args
-        (List.assoc id id_loc_list) id_check_list [] body
+      check_regular path args [] body
+
+let check_abbrev_recursion env id_loc_list (id, decl) =
+  check_recursion env (List.assoc id id_loc_list) (Path.Pident id) decl
+    (function Path.Pident id -> List.mem_assoc id id_loc_list | _ -> false)
 
 (* Compute variance *)
-let compute_variance env tvl nega posi ty =
+
+let compute_variance env tvl nega posi cntr ty =
   let pvisited = ref TypeSet.empty
-  and nvisited = ref TypeSet.empty in
-  let rec compute_variance_rec posi nega ty =
+  and nvisited = ref TypeSet.empty
+  and cvisited = ref TypeSet.empty in
+  let rec compute_variance_rec posi nega cntr ty =
     let ty = Ctype.repr ty in
     if (not posi || TypeSet.mem ty !pvisited)
-    && (not nega || TypeSet.mem ty !nvisited) then
+    && (not nega || TypeSet.mem ty !nvisited)
+    && (not cntr || TypeSet.mem ty !cvisited) then
       ()
     else begin
       if posi then pvisited := TypeSet.add ty !pvisited;
       if nega then nvisited := TypeSet.add ty !nvisited;
+      if cntr then cvisited := TypeSet.add ty !cvisited;
+      let compute_same = compute_variance_rec posi nega cntr in
       match ty.desc with
         Tarrow (_, ty1, ty2, _) ->
-          compute_variance_rec nega posi ty1;
-          compute_variance_rec posi nega ty2
+          compute_variance_rec nega posi true ty1;
+          compute_same ty2
       | Ttuple tl ->
-          List.iter (compute_variance_rec posi nega) tl
+          List.iter compute_same tl
       | Tconstr (path, tl, _) ->
           if tl = [] then () else begin
             try
               let decl = Env.find_type path env in
               List.iter2
-                (fun ty (co,cn) ->
+                (fun ty (co,cn,ct) ->
                   compute_variance_rec
                     (posi && co || nega && cn)
                     (posi && cn || nega && co)
+		    (cntr || ct)
                     ty)
                 tl decl.type_variance
             with Not_found ->
-              List.iter (compute_variance_rec true true) tl
+              List.iter (compute_variance_rec true true true) tl
           end
       | Tobject (ty, _) ->
-          compute_variance_rec posi nega ty
+          compute_same ty
       | Tfield (_, _, ty1, ty2) ->
-          compute_variance_rec posi nega ty1;
-          compute_variance_rec posi nega ty2
+          compute_same ty1;
+          compute_same ty2
       | Tsubst ty ->
-          compute_variance_rec posi nega ty
+          compute_same ty
       | Tvariant row ->
           List.iter
             (fun (_,f) ->
               match Btype.row_field_repr f with
                 Rpresent (Some ty) ->
-                  compute_variance_rec posi nega ty
+                  compute_same ty
               | Reither (_, tyl, _, _) ->
-                  List.iter (compute_variance_rec posi nega) tyl
+                  List.iter compute_same tyl
               | _ -> ())
             (Btype.row_repr row).row_fields
       | Tpoly (ty, _) ->
-          compute_variance_rec posi nega ty
+          compute_same ty
       | Tvar | Tnil | Tlink _ | Tunivar -> ()
     end
   in
-  compute_variance_rec nega posi ty;
+  compute_variance_rec nega posi cntr ty;
   List.iter
-    (fun (ty, covar, convar) ->
+    (fun (ty, covar, convar, ctvar) ->
       if TypeSet.mem ty !pvisited then covar := true;
-      if TypeSet.mem ty !nvisited then convar := true)
+      if TypeSet.mem ty !nvisited then convar := true;
+      if TypeSet.mem ty !cvisited then ctvar := true)
     tvl
 
-let compute_variance_decl env decl (required, loc) =
+let make_variance ty = (ty, ref false, ref false, ref false)
+let whole_type decl =
+  match decl.type_kind with
+    Type_variant (tll,_) ->
+      Btype.newgenty
+        (Ttuple (List.map (fun (_, tl) -> Btype.newgenty (Ttuple tl)) tll))
+  | Type_record (ftl, _, _) ->
+      Btype.newgenty
+        (Ttuple (List.map (fun (_, _, ty) -> ty) ftl))
+  | Type_abstract ->
+      match decl.type_manifest with
+        Some ty -> ty
+      | _ -> Btype.newgenty (Ttuple [])
+
+let compute_variance_decl env sharp decl (required, loc) =
   if decl.type_kind = Type_abstract && decl.type_manifest = None then
-    List.map (fun (c, n) -> if c || n then (c, n) else (true, true)) required
+    List.map (fun (c, n) -> if c || n then (c, n, n) else (true, true, true))
+      required
   else
-  let tvl = List.map (fun ty -> (Btype.repr ty, ref false, ref false))
-      decl.type_params in
+  let params = List.map Btype.repr decl.type_params in
+  let tvl0 = List.map make_variance params in
+  let fvl = Ctype.free_variables (whole_type decl) in
+  let fvl = List.filter (fun v -> not (List.memq v params)) fvl in
+  let tvl1 = List.map make_variance fvl in
+  let tvl2 = List.map make_variance fvl in
+  let tvl = tvl0 @ tvl1 in
   begin match decl.type_kind with
     Type_abstract ->
       begin match decl.type_manifest with
         None -> assert false
-      | Some ty -> compute_variance env tvl true false ty
+      | Some ty -> compute_variance env tvl true false false ty
       end
-  | Type_variant tll ->
+  | Type_variant (tll, _) ->
       List.iter
-        (fun (_,tl) -> List.iter (compute_variance env tvl true false) tl)
+        (fun (_,tl) ->
+	  List.iter (compute_variance env tvl true false false) tl)
         tll
-  | Type_record (ftl, _) ->
+  | Type_record (ftl, _, _) ->
       List.iter
-        (fun (_, mut, ty) -> compute_variance env tvl true (mut = Mutable) ty)
+        (fun (_, mut, ty) ->
+	  let cn = (mut = Mutable) in
+	  compute_variance env tvl true cn cn ty)
         ftl
   end;
+  List.iter2
+    (fun (ty, co, cn, ct) (c, n) ->
+      if ty.desc <> Tvar then begin
+        let (c, n) = if c || n then (c, n) else (true, true) in
+        co := c; cn := n; ct := n;
+        compute_variance env tvl2 c n n ty
+      end)
+    tvl0 required;
+  if not sharp then
+    List.iter2
+      (fun (_, c1, n1, t1) (_, c2, n2, t2) ->
+        if !c1 && not !c2 || !n1 && not !n2 ||
+           !t1 && not !t2 && decl.type_kind = Type_abstract
+        then raise (Error(loc, Bad_variance)))
+      tvl1 tvl2;
   List.map2
-    (fun (_, co, cn) (c, n) ->
+    (fun (_, co, cn, ct) (c, n) ->
       if c && !cn || n && !co then raise (Error(loc, Bad_variance));
-      (!co, !cn))
-    tvl required
+      let ct = if decl.type_kind = Type_abstract then ct else cn in
+      (!co, !cn, !ct))
+    tvl0 required
+
+let is_sharp id =
+  let s = Ident.name id in
+  String.length s > 0 && s.[0] = '#'
 
 let rec compute_variance_fixpoint env decls required variances =
   let new_decls =
@@ -435,22 +497,26 @@ let rec compute_variance_fixpoint env decls required variances =
       new_decls env
   in
   let new_variances =
-    List.map2 (fun (_, decl) -> compute_variance_decl new_env decl)
+    List.map2
+      (fun (id, decl) -> compute_variance_decl new_env (is_sharp id) decl)
       new_decls required
   in
   let new_variances =
-    List.map2 (List.map2 (fun (c1,n1) (c2,n2) -> (c1||c2), (n1||n2)))
+    List.map2
+      (List.map2 (fun (c1,n1,t1) (c2,n2,t2) -> c1||c2, n1||n2, t1||t2))
       new_variances variances in
   if new_variances = variances then
     new_decls, new_env
   else
     compute_variance_fixpoint env decls required new_variances
 
+let init_variance (id, decl) =
+  List.map (fun _ -> (false, false, false)) decl.type_params
+
 (* for typeclass.ml *)
 let compute_variance_decls env decls =
   let decls, required = List.split decls in
-  let variances =
-    List.map (fun (l,_) -> List.map (fun _ -> false, false) l) required in
+  let variances = List.map init_variance decls in
   fst (compute_variance_fixpoint env decls required variances)
 
 (* Translate a set of mutually recursive type declarations *)
@@ -485,8 +551,12 @@ let transl_type_decl env name_sdecl_list =
   (* Generalize type declarations. *)
   Ctype.end_def();
   List.iter (fun (_, decl) -> generalize_decl decl) decls;
-  (* Check for recursive abbrevs *)
-  List.iter2 (check_recursive_abbrev newenv) name_sdecl_list decls;
+  (* Check for ill-formed abbrevs *)
+  let id_loc_list =
+    List.map2 (fun id (_,sdecl) -> (id, sdecl.ptype_loc))
+      id_list name_sdecl_list
+  in
+  List.iter (check_abbrev_recursion newenv id_loc_list) decls;
   (* Check that all type variable are closed *)
   List.iter2
     (fun (_, sdecl) (id, decl) ->
@@ -498,25 +568,14 @@ let transl_type_decl env name_sdecl_list =
   List.iter2 (check_abbrev newenv) name_sdecl_list decls;
   (* Check that constraints are enforced *)
   List.iter2 (check_constraints newenv) name_sdecl_list decls;
-  (* Check that abbreviations have same parameters *)
-  let id_loc_list =
-    List.map2
-      (fun id (_,sdecl) ->
-        match sdecl.ptype_manifest with None -> []
-        | Some {ptyp_loc=loc} -> [id, loc])
-      id_list name_sdecl_list
-  in
-  List.iter (check_expansion newenv (List.flatten id_loc_list)) decls;
   (* Add variances to the environment *)
   let required =
     List.map (fun (_, sdecl) -> sdecl.ptype_variance, sdecl.ptype_loc)
       name_sdecl_list
   in
   let final_decls, final_env =
-    compute_variance_fixpoint env decls required
-      (List.map
-         (fun (_,decl) -> List.map (fun _ -> (false, false)) decl.type_params)
-         decls) in
+    compute_variance_fixpoint env decls required (List.map init_variance decls)
+  in
   (* Done *)
   (final_decls, final_env)
 
@@ -564,7 +623,7 @@ let transl_with_constraint env sdecl =
   Ctype.begin_def();
   let params =
     try
-      List.map (enter_type_variable true) sdecl.ptype_params
+      List.map (enter_type_variable true sdecl.ptype_loc) sdecl.ptype_params
     with Already_bound ->
       raise(Error(sdecl.ptype_loc, Repeated_parameter)) in
   List.iter
@@ -591,11 +650,45 @@ let transl_with_constraint env sdecl =
     raise(Error(sdecl.ptype_loc, Unbound_type_var));
   let decl =
     {decl with type_variance =
-     compute_variance_decl env decl (sdecl.ptype_variance, sdecl.ptype_loc)} in
+     compute_variance_decl env false decl
+       (sdecl.ptype_variance, sdecl.ptype_loc)} in
   Ctype.end_def();
   generalize_decl decl;
   decl
 
+(* Approximate a type declaration: just make all types abstract *)
+
+let abstract_type_decl arity =
+  let rec make_params n =
+    if n <= 0 then [] else Ctype.newvar() :: make_params (n-1) in
+  Ctype.begin_def();
+  let decl =
+    { type_params = make_params arity;
+      type_arity = arity;
+      type_kind = Type_abstract;
+      type_manifest = None;
+      type_variance = replicate_list (true, true, true) arity } in
+  Ctype.end_def();
+  generalize_decl decl;
+  decl
+
+let approx_type_decl env name_sdecl_list =
+  List.map
+    (fun (name, sdecl) -> 
+      (Ident.create name,
+       abstract_type_decl (List.length sdecl.ptype_params)))
+    name_sdecl_list
+
+(* Variant of check_abbrev_recursion to check the well-formedness
+   conditions on type abbreviations defined within recursive modules. *)
+
+let check_recmod_typedecl env loc recmod_ids path decl =
+  (* recmod_ids is the list of recursively-defined module idents.
+     (path, decl) is the type declaration to be checked. *)
+  check_recursion env loc path decl
+    (fun path -> List.mem (Path.head path) recmod_ids)
+
+
 (**** Error report ****)
 
 open Format
@@ -624,12 +717,12 @@ let report_error ppf = function
       Printtyp.mark_loops ty';
       fprintf ppf "@[<hv>Type@ %a@ should be an instance of@ %a@]"
         Printtyp.type_expr ty Printtyp.type_expr ty'
-  | Parameters_differ (ty, ty') ->
+  | Parameters_differ (path, ty, ty') ->
       Printtyp.reset_and_mark_loops ty;
       Printtyp.mark_loops ty';
       fprintf ppf
-        "@[<hv>In this definition, type@ %a@ should be@ %a@]"
-        Printtyp.type_expr ty Printtyp.type_expr ty'
+        "@[<hv>In the definition of %s, type@ %a@ should be@ %a@]"
+        (Path.name path) Printtyp.type_expr ty Printtyp.type_expr ty'
   | Unconsistent_constraint trace ->
       fprintf ppf "The type constraints are not consistent.@.";
       Printtyp.report_unification_error ppf trace
