@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: types.ml,v 1.17 2002/04/18 07:27:46 garrigue Exp $ *)
+(* $Id: types.ml,v 1.24 2004/06/12 08:55:49 xleroy Exp $ *)
 
 (* Representation of types and declarations *)
 
@@ -91,7 +91,7 @@ and value_kind =
   | Val_ivar of mutable_flag * string   (* Instance variable (mutable ?) *)
   | Val_self of (Ident.t * type_expr) Meths.t ref *
                 (Ident.t * Asttypes.mutable_flag * type_expr) Vars.t ref *
-                string
+                string * type_expr
                                         (* Self *)
   | Val_anc of (string * Ident.t) list * string
                                         (* Ancestor *)
@@ -105,7 +105,8 @@ type constructor_description =
     cstr_arity: int;                    (* Number of arguments *)
     cstr_tag: constructor_tag;          (* Tag for heap blocks *)
     cstr_consts: int;                   (* Number of constant constructors *)
-    cstr_nonconsts: int }               (* Number of non-const constructors *)
+    cstr_nonconsts: int;                (* Number of non-const constructors *)
+    cstr_private: private_flag }        (* Read-only constructor? *)
 
 and constructor_tag =
     Cstr_constant of int                (* Constant constructor (an int) *)
@@ -120,7 +121,8 @@ type label_description =
     lbl_mut: mutable_flag;              (* Is this a mutable field? *)
     lbl_pos: int;                       (* Position in block *)
     lbl_all: label_description array;   (* All the labels in this type *)
-    lbl_repres: record_representation } (* Representation for this record *)
+    lbl_repres: record_representation;  (* Representation for this record *)
+    lbl_private: private_flag }         (* Read-only field? *)
 
 and record_representation =
     Record_regular                      (* All fields are boxed / tagged *)
@@ -133,13 +135,13 @@ type type_declaration =
     type_arity: int;
     type_kind: type_kind;
     type_manifest: type_expr option;
-    type_variance: (bool * bool) list }
+    type_variance: (bool * bool * bool) list }
 
 and type_kind =
     Type_abstract
-  | Type_variant of (string * type_expr list) list
+  | Type_variant of (string * type_expr list) list * private_flag
   | Type_record of (string * mutable_flag * type_expr) list
-                 * record_representation
+                 * record_representation * private_flag
 
 type exception_declaration = type_expr list
 
@@ -155,7 +157,8 @@ type class_type =
 and class_signature =
   { cty_self: type_expr;
     cty_vars: (Asttypes.mutable_flag * type_expr) Vars.t;
-    cty_concr: Concr.t }
+    cty_concr: Concr.t;
+    cty_inher: (Path.t * type_expr list) list }
 
 type class_declaration =
   { cty_params: type_expr list;
@@ -179,13 +182,18 @@ and signature = signature_item list
 
 and signature_item =
     Tsig_value of Ident.t * value_description
-  | Tsig_type of Ident.t * type_declaration
+  | Tsig_type of Ident.t * type_declaration * rec_status
   | Tsig_exception of Ident.t * exception_declaration
-  | Tsig_module of Ident.t * module_type
+  | Tsig_module of Ident.t * module_type * rec_status
   | Tsig_modtype of Ident.t * modtype_declaration
-  | Tsig_class of Ident.t * class_declaration
-  | Tsig_cltype of Ident.t * cltype_declaration
+  | Tsig_class of Ident.t * class_declaration * rec_status
+  | Tsig_cltype of Ident.t * cltype_declaration * rec_status
 
 and modtype_declaration =
     Tmodtype_abstract
   | Tmodtype_manifest of module_type
+
+and rec_status =
+    Trec_not
+  | Trec_first
+  | Trec_next
