@@ -10,7 +10,7 @@
 (*                                                                     *)
 (***********************************************************************)
 
-(* $Id: ccomp.ml,v 1.15 2002/06/07 09:49:45 xleroy Exp $ *)
+(* $Id: ccomp.ml,v 1.17.2.1 2005/02/02 15:39:40 xleroy Exp $ *)
 
 (* Compiling C files and building C libraries *)
 
@@ -24,9 +24,27 @@ let command cmdline =
 
 let run_command cmdline = ignore(command cmdline)
 
+(* Build @responsefile to work around Windows limitations on 
+   command-line length *)
+let build_diversion lst =
+  let (responsefile, oc) = Filename.open_temp_file "camlresp" "" in
+  List.iter
+    (fun f ->
+      if f <> "" then begin
+        output_string oc (Filename.quote f); output_char oc '\n'
+      end)
+    lst;
+  close_out oc;
+  at_exit (fun () -> Misc.remove_file responsefile);
+  "@" ^ responsefile
+
 let quote_files lst =
-  String.concat " "
-    (List.map (fun f -> if f = "" then f else Filename.quote f) lst)
+  let s =
+    String.concat " "
+      (List.map (fun f -> if f = "" then f else Filename.quote f) lst) in
+  if Sys.os_type = "Win32" && String.length s >= 256
+  then build_diversion lst
+  else s
 
 let compile_file name =
   match Config.ccomp_type with
@@ -58,7 +76,7 @@ let create_archive archive file_list =
   let quoted_archive = Filename.quote archive in
   match Config.ccomp_type with
     "msvc" ->
-      command(Printf.sprintf "lib /nologo /debugtype:cv /out:%s %s"
+      command(Printf.sprintf "link /lib /nologo /debugtype:cv /out:%s %s"
                              quoted_archive (quote_files file_list))
   | _ ->
       let r1 =
