@@ -12,7 +12,7 @@
 
 /* Based on public-domain code from Berkeley Yacc */
 
-/* $Id: main.c,v 1.16 2001/07/12 12:54:23 doligez Exp $ */
+/* $Id: main.c,v 1.18 2004/04/21 23:26:05 doligez Exp $ */
 
 #include <signal.h>
 #include <string.h>
@@ -26,6 +26,7 @@ char lflag;
 char rflag;
 char tflag;
 char vflag;
+char qflag;
 char sflag;
 char big_endian;
 
@@ -38,6 +39,7 @@ char temp_form[] = "yacc.XXXXXXX";
 #endif
 
 int lineno;
+char *virtual_input_file_name = NULL;
 int outline;
 
 char *action_file_name;
@@ -134,7 +136,7 @@ void set_signals(void)
 
 void usage(void)
 {
-    fprintf(stderr, "usage: %s [-v] [-b file_prefix] filename\n",
+    fprintf(stderr, "usage: %s [-v] [-q] [-b file_prefix] filename\n",
             myname);
     exit(1);
 }
@@ -164,6 +166,10 @@ void getargs(int argc, char **argv)
             vflag = 1;
             break;
 
+        case 'q':
+            qflag = 1;
+            break;
+
         case 'b':
             if (*++s)
                  file_prefix = s;
@@ -188,6 +194,10 @@ void getargs(int argc, char **argv)
                 vflag = 1;
                 break;
 
+            case 'q':
+                qflag = 1;
+                break;
+
             default:
                 usage();
             }
